
DROP TABLE IF EXISTS `binaries`;
CREATE TABLE `binaries` (
		`ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
		`name` VARCHAR(255) NOT NULL DEFAULT '',
		`fromname` VARCHAR(255) NOT NULL DEFAULT '',
		`date` DATETIME DEFAULT NULL,
		`xref` VARCHAR(255) NOT NULL DEFAULT '',
		`totalParts` INT(11) UNSIGNED NOT NULL DEFAULT '0',
		`groupID` INT(11) UNSIGNED NOT NULL DEFAULT '0',
		`procstat` INT DEFAULT 0,
		`procattempts` INT DEFAULT 0,
		`categoryID` INT NULL,
		`regexID` INT NULL,
		`reqID` INT NULL,
		`relpart` INT DEFAULT 0,
		`reltotalpart` INT DEFAULT 0,
		`binaryhash` VARCHAR(255) NOT NULL,
		`relname` VARCHAR(255) NULL,
		`importname` VARCHAR(255) NULL,
		`releaseID` INT NULL,
		`dateadded` DATETIME DEFAULT NULL,
		PRIMARY KEY  (`ID`),
		KEY `fromname` (`fromname`),
		KEY `date` (`date`),
		KEY `groupID` (`groupID`)
		) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

CREATE INDEX ix_binary_relname ON binaries (relname);
CREATE INDEX ix_binary_procstat ON binaries (procstat);
CREATE INDEX ix_binary_releaseID ON binaries (releaseID);
CREATE INDEX ix_binary_dateadded ON binaries (dateadded);
CREATE INDEX ix_binary_binaryhash ON binaries (binaryhash);
CREATE INDEX ix_binary_groupID_procstat ON binaries (procstat, groupID);

DROP TABLE IF EXISTS `releases`;
CREATE TABLE `releases` 
(
`ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
`name` VARCHAR(255) NOT NULL DEFAULT '',
`searchname` VARCHAR(255) NOT NULL DEFAULT '',	
`totalpart` INT DEFAULT 0,	
`groupID` INT UNSIGNED NOT NULL DEFAULT '0',
`size` BIGINT UNSIGNED NOT NULL DEFAULT '0',
`postdate` DATETIME DEFAULT NULL,
`adddate` DATETIME DEFAULT NULL,
`guid` VARCHAR(50) NOT NULL,
`fromname` VARCHAR(255) NULL,
`completion` FLOAT NOT NULL DEFAULT '0',
`categoryID` INT DEFAULT 0,
`regexID` INT DEFAULT 0,
`rageID` INT NULL,
`seriesfull` VARCHAR(15) NULL,
`season` VARCHAR(10) NULL,
`episode` VARCHAR(10) NULL,
`tvtitle` varchar(255) null,
`tvairdate` datetime null,
`imdbID` MEDIUMINT(7) UNSIGNED ZEROFILL NULL,
`musicinfoID` INT NULL,
`consoleinfoID` INT NULL,
`reqID` INT NULL,
`grabs` INT UNSIGNED NOT NULL DEFAULT '0',
`comments` INT NOT NULL DEFAULT 0,
`passwordstatus` INT NOT NULL DEFAULT 0,
PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

CREATE INDEX ix_releases_adddate ON releases (`adddate`);
CREATE INDEX ix_releases_postdate ON releases (`postdate`);
CREATE INDEX ix_releases_categoryID ON releases (`categoryID`);
CREATE INDEX ix_releases_rageID ON releases (`rageID`);
CREATE INDEX ix_releases_imdbID ON releases (`imdbID`);
CREATE INDEX ix_releases_guid ON releases (`guid`);


DROP TABLE IF EXISTS `releasecomment`;
CREATE TABLE `releasecomment` 
(
`ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
`releaseID` INT(11) UNSIGNED NOT NULL,
`text` VARCHAR(2000) NOT NULL DEFAULT '',
`userID` INT(11) UNSIGNED NOT NULL,
`createddate` DATETIME DEFAULT NULL,
`host` VARCHAR(15) NULL,
PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

CREATE INDEX ix_releasecomment_releaseID ON releasecomment (`releaseID`);
CREATE INDEX ix_releasecomment_userID ON releasecomment (`userID`);


DROP TABLE IF EXISTS `menu`;
CREATE TABLE `menu` 
(
`ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
`href` VARCHAR(2000) NOT NULL DEFAULT '',
`title` VARCHAR(2000) NOT NULL DEFAULT '',
`tooltip` VARCHAR(2000) NOT NULL DEFAULT '',
`role` INT(11) UNSIGNED NOT NULL,
`ordinal` INT(11) UNSIGNED NOT NULL,
`menueval` VARCHAR(2000) NOT NULL DEFAULT '',
PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;


INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('search', 'Search', 
	'Search for Nzbs', 1, 10);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('browse', 'Browse', 
	'Browse for Nzbs', 1, 20);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('browsegroup', 'Browse Groups', 
	'Browse by Group', 1, 25);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('searchraw', 'Raw Search', 
	'Search for individual files', 1, 30);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('movies', 'Movies', 
	'Browse for Movies', 1, 40);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('music', 'Music', 
	'Browse for Music', 1, 45);
	
INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('console', 'Console', 
	'Browse for Games', 1, 48);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('admin', 'Admin', 
	'Admin', 2, 50);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('cart', 'My Cart', 
	'Your Nzb cart', 1, 60);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal`, `menueval` )
VALUES ('queue', 'My Queue', 
	'View Your Sabnzbd Queue', 1, 70, '{if $sabintegrated!="true"}-1{/if}');

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('forum', 'Forum', 
	'Browse Forum', 1, 70);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('profile', 'Profile', 
	'View your profile', 1, 80);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('logout', 'Logout', 
	'Logout', 1, 90);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('login', 'Login', 
	'Login', 0, 100);

INSERT INTO menu (`href`, `title`, `tooltip`, `role`, `ordinal` )
VALUES ('register', 'Register', 
	'Register', 0, 110);
	


DROP TABLE IF EXISTS `releasenfo`;
CREATE TABLE `releasenfo` (
  `ID` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `releaseID` int(11) UNSIGNED NOT NULL,
  `binaryID` int(11) UNSIGNED NOT NULL,
  `attempts` tinyint(1) UNSIGNED NOT NULL DEFAULT '0',
  `nfo` BLOB NULL DEFAULT NULL,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

CREATE UNIQUE INDEX ix_releasenfo_releaseID ON releasenfo (`releaseID`);

DROP TABLE IF EXISTS `binaryblacklist`;
CREATE TABLE `binaryblacklist` (
  `ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `groupname` VARCHAR(255) NULL,
  `regex` VARCHAR(2000) NOT NULL,
  `msgcol` INT(11) UNSIGNED NOT NULL DEFAULT 1,
  `optype` INT(11) UNSIGNED NOT NULL DEFAULT 1,
  `status` INT(11) UNSIGNED NOT NULL DEFAULT 1,
  `description` VARCHAR(1000) NULL,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=100000 ;

INSERT INTO `binaryblacklist` (`ID`, `groupname`, `regex`, `optype`, `status`, `description`) VALUES (100000, 'alt.binaries.boneless', 'usenet-4all|u4all|usenet4all', 2, 0, 'only allow u4all posts in boneless');

DROP TABLE IF EXISTS `releaseregex`;
CREATE TABLE `releaseregex` (
  `ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `groupname` VARCHAR(255) NULL,
  `regex` VARCHAR(2000) NOT NULL,
  `ordinal` INT(11) UNSIGNED NOT NULL,
  `status` INT(11) UNSIGNED NOT NULL DEFAULT 1,
  `description` VARCHAR(1000) NULL,
  `categoryID` INT NULL,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=100000 ;

INSERT INTO `releaseregex` (`ID`, `groupname`, `regex`, `ordinal`, `status`, `description`, `categoryID`) VALUES (1, NULL, '/^\\((?P<name>.*)\\).*?\\[(?P<parts>\\d{2,3}\\/\\d{2,3})/i', 52, 0, '', NULL);

DROP TABLE IF EXISTS `tvrage`;
CREATE TABLE `tvrage` 
(
`ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
`rageID` INT NOT NULL,
`tvdbID` INT NOT NULL,
`releasetitle` VARCHAR(255) NOT NULL DEFAULT '',
`description` VARCHAR(10000) NULL,
`genre` VARCHAR(64) NULL DEFAULT NULL,
`country` VARCHAR(2) NULL DEFAULT NULL,
`imgdata` longblob null,
`createddate` DATETIME DEFAULT NULL,
PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

CREATE INDEX ix_tvrage_rageID ON tvrage (`rageID`);

DROP TABLE IF EXISTS `forumpost`;
CREATE TABLE IF NOT EXISTS `forumpost` (
  `ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `forumID` INT(11) NOT NULL DEFAULT '1',
  `parentID` INT(11) NOT NULL DEFAULT '0',
  `userID` INT(11) UNSIGNED NOT NULL,
  `subject` VARCHAR(255) NOT NULL,
  `message` TEXT NOT NULL,
  `locked` TINYINT(1) UNSIGNED NOT NULL DEFAULT '0',
  `sticky` TINYINT(1) UNSIGNED NOT NULL DEFAULT '0',
  `replies` INT(11) UNSIGNED NOT NULL DEFAULT '0',
  `createddate` DATETIME NOT NULL,
  `updateddate` DATETIME NOT NULL,
  PRIMARY KEY  (`ID`),
  KEY `parentID` (`parentID`),
  KEY `userID` (`userID`),
  KEY `createddate` (`createddate`),
  KEY `updateddate` (`updateddate`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

INSERT INTO forumpost (`forumID`, `parentID`,  `userID`,  `subject`,  `message`, `locked`, `sticky`,`replies`,  `createddate`, `updateddate`)
VALUES (1,0,1,'Welcome to newznab','This install was brought to you by the newznab team @ http://www.newznab.com/',  0, 0, 0,  NOW(), NOW());


DROP TABLE IF EXISTS `movieinfo`;
CREATE TABLE `movieinfo` 
(
 `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `imdbID` mediumint(7) unsigned zerofill NOT NULL,
  `tmdbID` int(10) unsigned DEFAULT NULL,
  `title` varchar(128) NOT NULL,
  `tagline` VARCHAR(255) NOT NULL,
  `rating` varchar(4) NOT NULL,
  `plot` varchar(255) NOT NULL,
  `year` varchar(4) NOT NULL,
  `genre` varchar(64) NOT NULL,
  `director` VARCHAR(64) NOT NULL,
  `actors` VARCHAR(255) NOT NULL,
  `language` VARCHAR(64) NOT NULL,
  `cover` TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '0',
  `backdrop` TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '0',
  `createddate` datetime NOT NULL,
  `updateddate` datetime NOT NULL,
  PRIMARY KEY (`ID`),
  UNIQUE KEY `imdbID` (`imdbID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;


DROP TABLE IF EXISTS `groups`;
CREATE TABLE `groups` (
  `ID` INT(11) NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(255) NOT NULL DEFAULT '',
  `backfill_target` INT(4) NOT NULL DEFAULT '1',
  `first_record` BIGINT UNSIGNED NOT NULL DEFAULT '0',
  `first_record_postdate` DATETIME DEFAULT NULL,
  `last_record` BIGINT UNSIGNED NOT NULL DEFAULT '0',
  `last_record_postdate` DATETIME DEFAULT NULL,
  `last_updated` DATETIME DEFAULT NULL,
  `minfilestoformrelease` INT(4) NULL,
  `active` TINYINT(1) NOT NULL DEFAULT '0',
  `description` VARCHAR(255) NULL DEFAULT '',
  PRIMARY KEY  (`ID`),
  KEY `active` (`active`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

ALTER TABLE groups ADD UNIQUE (NAME);

INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.dvdr', 'This group contains DVD Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.boneless', 'This group contains XVID and X264 Movies. Some Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia', 'This group contains TV, Movies, and Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.movies.divx', 'This group contains XVID Movies');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.games.xbox', 'This group contains original XBOX Games.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.movies.xvid', 'This group contains XVID Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sony.psp', 'This group contains PSP Games.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.nintendo.ds', 'This group contains Nintendo DS Games');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.games.nintendods', 'This group contains Nintendo DS Games ');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.hdtv.x264', 'This group contains X264 Movies and HDTV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.games.xbox360', 'This group contains XBOX 360 Games and DLC.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.games.wii', 'This group contains Nintendo WII Games, WII-Ware, and VC.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.wmvhd', 'This group contains WMVHD Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.x264', 'This group contains X264 Movies and TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.wii', 'This group contains Nintendo WII Games, WII-Ware, and VC.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.moovee', 'This group contains XVID and X264 Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.inner-sanctum', 'This group contains PC and Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.warez.smartphone', 'This group contains Mobile Phone Apps.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.teevee', 'This group contains X264 and XVID TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.warez', 'This group contains PC 0DAY, PC ISO, and PC PHONE.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.complete_cd', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mpeg.video.music', 'This group contains a variety of Music Videos.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mp3', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mma', 'This group contains MMA/TNA Sport TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.classical', 'This group contains Classical Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mac', 'This group contains MAC/OSX Software.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.e-book', 'This group contains E-Books.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.warez.ibm-pc.0-day', 'This group contains PC-0Day.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.tvseries', 'This group contains X264 and XVID TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.ath', 'This group contains a variety of Music. Some Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.ftn', 'This group contains a variety of Music and TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.erotica', 'This group contains XXX.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.games', 'This group contains PC and Console Games.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.cores', 'This group contains a variety including Nintendo DS. Lots of Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.country.mp3', 'This group contains Country Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.1990s', 'This group contains Music from the 1990\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.console.ps3', 'This group contains PS3 Games.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.scary.exe.files', 'This group contains XVID and X264 Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.cd.image', 'This group contains PC-ISO.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.e-book.technical', 'This group contains E-Books.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.erotica.divx', 'This group contains XXX.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.test', 'This group contains a variety of content.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.x', 'This group contains a variety of content. Some Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.hou', 'This group contains a variety of content. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.pro-wrestling', 'This group contains WWE Sport TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.lossless', 'This group contains a variety of Lossless Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.comp', 'This group contains Warez. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.warez.quebec-hackers', 'This group contains PC-0day. Some Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.bluegrass', 'This group contains Bluegrass Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.radio.bbc', 'This group contains BBC Radio Music');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.e-book.flood', 'This group contains E-Books.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.movies.erotica', 'This group contains XXX');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.documentaries', 'This group contains Documentary Movies and TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.jazz', 'This group contains Jazz Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.erotica.amateur', 'This group contains XXX.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.1960s.mp3', 'This group contains Music from the 1960\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.1970s.mp3', 'This group contains Music from the 1970\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.comedy', 'This group contains Comedy Audio.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.2000s', 'This group contains Music from the 2000\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.christian', 'This group contains Christian Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.1950s', 'This group contains Music from the 1950\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.1970s', 'This group contains Music from the 1970\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.1980s', 'This group contains Music from the 1980\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mp3.bootlegs', 'This group contains Bootleg Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mp3.audiobooks', 'This group contains Audio Books.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.rap-hiphop.full-albums', 'This group contains Rap and Hip-Hop Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.full_albums', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.teen-idols', 'This group contains Music from Teen Idols');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.dance', 'This group contains Dance Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.warez.uk.mp3', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.heavy-metal', 'This group contains Heavy Metal Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.cartoons', 'This group contains Cartoon TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.sports', 'This group contains Sports TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.anime', 'This group contains Anime TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.lossless.classical', 'This group contains lossless Classical Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.disney', 'This group contains Disney TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.nospam', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.sitcoms', 'This group contains Sitcom TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.radio.british', 'This group contains British Radio Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.comedy.british', 'This group contains British Comedy TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.etc', 'This group contains a variety of items.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.misc', 'This group contains a variety of items.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.rock', 'This group contains Rock Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.dc', 'This group contains XVID and X264 Movies and TV. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.documentaries', 'This group contains Documentaries TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.cd.lossless', 'This group contains a variety of lossless Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.audiobooks.repost', 'This group contains Audiobooks.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.highspeed', 'This group contains XVID Movies. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.bloaf', 'This group contains a variety. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.big', 'This group contains XVID Movies. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.musicals', 'This group contains Musicals Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sound.mp3', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.jazz.vocals', 'This group contains Jazz Vocal Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.dvd.movies', 'This group contains DVDR Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.ebook', 'This group contains Ebook\'s.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.disco', 'This group contains Disco Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mp3.full_albums', 'This group contains a variety of Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.tv', 'This group contains XVID TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.lossless.country', 'This group contains lossless Country Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.uzenet', 'This group contains XXX. Some Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.mom', 'This group contains a variety. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.ijsklontje', 'This group contains XXX.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.lossless.1960s', 'This group contains lossless 1960\'s Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.emo', 'This group contains Emo Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.classic.tv.shows', 'This group contains Classic TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.dgma', 'This group contains XVID Movies. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.opera', 'This group contains Opera Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.ipod.videos', 'This group contains Mobile TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.music.opera', 'This group contains Opera Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.flac.jazz', 'This group contains lossless Jazz Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.tv', 'This group contains XVID and X264 TV.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.whitburn.pop', 'This group contains Pop Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sound.audiobooks', 'This group contains Audiobooks.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.acoustic', 'This group contains Accoustic Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.u-4all', 'This group contains XVID TV and Movies. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.sounds.mp3.progressive-country', 'This group contains Country Music.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.multimedia.classic-films', 'This group contains Classic TV and Movies.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.music.flac', 'This group contains a variety of lossless Music');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.ghosts', 'This group contains XVID TV and Movies. Mostly Foreign.');
INSERT INTO `groups` (`name`, `description`) VALUES ('alt.binaries.town', 'This group contains XVID TV and Movies. Mostly Foreign.');

DROP TABLE IF EXISTS `parts`;
CREATE TABLE `parts` (
  `ID` INT(16) UNSIGNED NOT NULL AUTO_INCREMENT,
  `binaryID` INT(11) UNSIGNED NOT NULL DEFAULT '0',
  `messageID` VARCHAR(255) NOT NULL DEFAULT '',
  `number` BIGINT UNSIGNED NOT NULL DEFAULT '0',
  `partnumber` INT UNSIGNED NOT NULL DEFAULT '0',
  `size` BIGINT UNSIGNED NOT NULL DEFAULT '0',
  `dateadded` DATETIME DEFAULT NULL,
  PRIMARY KEY  (`ID`),
  KEY `binaryID` (`binaryID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

CREATE INDEX ix_parts_dateadded ON parts (dateadded);
CREATE INDEX ix_parts_number ON parts (number);

DROP TABLE IF EXISTS `partrepair`;
CREATE TABLE `partrepair` (
  `ID` int(16) unsigned NOT NULL AUTO_INCREMENT,
  `numberID` BIGINT unsigned NOT NULL,
  `groupID` int(11) unsigned NOT NULL,
  `attempts` tinyint(1) NOT NULL default '0',
  PRIMARY KEY  (`ID`),
  UNIQUE KEY `ix_partrepair_numberID_groupID` (`numberID`,`groupID`)
) ENGINE=MyISAM  DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;


DROP TABLE IF EXISTS `category`;
CREATE TABLE category
(
`ID` INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
`title` VARCHAR(255) NOT NULL,
`parentID` INT NULL,
`status` INT NOT NULL DEFAULT '1',
`description` varchar(255) null
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=100000 ;

INSERT INTO category (ID, title) VALUES (1000, 'Console');
INSERT INTO category (ID, title) VALUES (2000, 'Movies');
INSERT INTO category (ID, title) VALUES (3000, 'Audio');
INSERT INTO category (ID, title) VALUES (4000, 'PC');
INSERT INTO category (ID, title) VALUES (5000, 'TV');
INSERT INTO category (ID, title) VALUES (6000, 'XXX');
INSERT INTO category (ID, title) VALUES (7000, 'Other');

INSERT INTO category (ID, title, parentID) VALUES (1010, 'NDS', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1020, 'PSP', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1030, 'Wii', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1040, 'Xbox', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1050, 'Xbox 360', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1060, 'WiiWare/VC', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1070, 'XBOX 360 DLC', 1000);
INSERT INTO category (ID, title, parentID) VALUES (1080, 'PS3', 1000);

INSERT INTO category (ID, title, parentID) VALUES (2010, 'Foreign', 2000);
INSERT INTO category (ID, title, parentID) VALUES (2020, 'Other', 2000);
INSERT INTO category (ID, title, parentID) VALUES (2030, 'SD', 2000);
INSERT INTO category (ID, title, parentID) VALUES (2040, 'HD', 2000);

INSERT INTO category (ID, title, parentID) VALUES (3010, 'MP3', 3000);
INSERT INTO category (ID, title, parentID) VALUES (3020, 'Video', 3000);
INSERT INTO category (ID, title, parentID) VALUES (3030, 'Audiobook', 3000);
INSERT INTO category (ID, title, parentID) VALUES (3040, 'Lossless', 3000);

INSERT INTO category (ID, title, parentID) VALUES (4010, '0day', 4000);
INSERT INTO category (ID, title, parentID) VALUES (4020, 'ISO', 4000);
INSERT INTO category (ID, title, parentID) VALUES (4030, 'Mac', 4000);
INSERT INTO category (ID, title, parentID) VALUES (4040, 'Phone', 4000);
INSERT INTO category (ID, title, parentID) VALUES (4050, 'Games', 4000);

INSERT INTO category (ID, title, parentID) VALUES (5020, 'Foreign', 5000);
INSERT INTO category (ID, title, parentID) VALUES (5030, 'SD', 5000);
INSERT INTO category (ID, title, parentID) VALUES (5040, 'HD', 5000);
INSERT INTO category (ID, title, parentID) VALUES (5050, 'Other', 5000);
INSERT INTO category (ID, title, parentID) VALUES (5060, 'Sport', 5000);

INSERT INTO category (ID, title, parentID) VALUES (6010, 'DVD', 6000);
INSERT INTO category (ID, title, parentID) VALUES (6020, 'WMV', 6000);
INSERT INTO category (ID, title, parentID) VALUES (6030, 'XviD', 6000);
INSERT INTO category (ID, title, parentID) VALUES (6040, 'x264', 6000);

INSERT INTO category (ID, title, parentID) VALUES (7010, 'Misc', 7000);
INSERT INTO category (ID, title, parentID) VALUES (7020, 'Ebook', 7000);
INSERT INTO category (ID, title, parentID) VALUES (7030, 'Comics', 7000);
INSERT INTO category (ID, title, parentID) VALUES (7040, 'Anime', 7000);


DROP TABLE IF EXISTS `users`;
CREATE TABLE `users` (
  `ID` INT(16) UNSIGNED NOT NULL AUTO_INCREMENT,
  `username` VARCHAR(50) NOT NULL,
  `email` VARCHAR(255) NOT NULL,
  `password` VARCHAR(255) NOT NULL,
  `role` INT NOT NULL DEFAULT 1,
  `host` VARCHAR(15) NULL,
  `grabs` INT NOT NULL DEFAULT 0,
  `rsstoken` VARCHAR(32) NOT NULL,
  `createddate` DATETIME NOT NULL,
  `resetguid` VARCHAR(50) NULL,
  `lastlogin` datetime default null,
  `apiaccess` datetime default null,
  `invites` int not null default 0,
  `invitedby` int null,
  `movieview` int not null default 1,
  `musicview` int not null default 1,
  `consoleview` int not null default 1,
  `userseed` VARCHAR(50) NOT NULL,
  PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

DROP TABLE IF EXISTS `usercart`;
CREATE TABLE `usercart` (
  `ID` INT(16) UNSIGNED NOT NULL AUTO_INCREMENT,
  `userID` INT NOT NULL ,
  `releaseID` INT NOT NULL,
  `createddate` DATETIME NOT NULL,
  PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;
CREATE UNIQUE INDEX ix_usercart_userrelease ON usercart (userID, releaseID);

DROP TABLE IF EXISTS `userexcat`;
CREATE TABLE `userexcat` (
  `ID` INT(16) UNSIGNED NOT NULL AUTO_INCREMENT,
  `userID` INT NOT NULL ,
  `categoryID` INT NOT NULL,
  `createddate` DATETIME NOT NULL,
  PRIMARY KEY  (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;
CREATE UNIQUE INDEX ix_userexcat_usercat ON userexcat (userID, categoryID);

DROP TABLE IF EXISTS `userinvite`;
CREATE TABLE `userinvite` (
  `ID` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `guid` varchar(50) NOT NULL,
  `userID` int(11) UNSIGNED NOT NULL,
  `createddate` datetime not null,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

DROP TABLE IF EXISTS `content`;
CREATE TABLE content
(
`id` INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
`title` VARCHAR(255) NOT NULL,
`url` VARCHAR(2000) NULL,
`body` TEXT NULL,
`metadescription` VARCHAR(1000) NOT NULL,
`metakeywords` VARCHAR(1000) NOT NULL,
`contenttype` INT NOT NULL,
`showinmenu` INT NOT NULL,
`status` INT NOT NULL,
`ordinal` INT NULL,
`role` INT NOT NULL DEFAULT 0
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

INSERT INTO content (title, body, contenttype, STATUS, metadescription, metakeywords, showinmenu)
VALUES ('welcome to newznab', '<p>A usenet indexing community site thats easy to configure.</p><p>Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p><p>Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p>', 3, 1, '', '', 0);

INSERT INTO content (title, url, body, contenttype, STATUS, showinmenu, metadescription, metakeywords) 
VALUES ('example content', '/great/seo/content/page/', '<p>this is an example content page</p><p>Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p><p>Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p>', 2, 1, 1, '', '');

INSERT INTO content (title, url, body, contenttype, STATUS, showinmenu, metadescription, metakeywords)
VALUES ('another example', '/another/great/seo/content/page/', '<p>this is another example content page</p><p>Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p><p>Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p>', 2, 1, 1, '', '');

DROP TABLE IF EXISTS `site`;
CREATE TABLE site (
`id` INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
`code` VARCHAR(255) NOT NULL,
`title` VARCHAR(1000) NOT NULL,
`strapline` VARCHAR(1000) NOT NULL,
`metatitle` VARCHAR(1000) NOT NULL,
`metadescription` VARCHAR(1000) NOT NULL,
`metakeywords` VARCHAR(1000) NOT NULL,
`footer` VARCHAR(2000) NOT NULL,
`email` VARCHAR(1000) NOT NULL,
`lastupdate` DATETIME NOT NULL,
`google_adsense_search` VARCHAR(255) NULL,
`google_adsense_sidepanel` VARCHAR(255) NULL,
`google_analytics_acc` VARCHAR(255) NULL,
`google_adsense_acc` VARCHAR(255) NULL,
`siteseed` VARCHAR(50) NOT NULL,
`tandc` VARCHAR(5000) NOT NULL,
`registerstatus` INT NOT NULL DEFAULT 0,
`style` VARCHAR(50) NULL,
`menuposition` INT NOT NULL DEFAULT 1,
`dereferrer_link` VARCHAR(255) NULL,
`nzbpath` VARCHAR(500) NOT NULL,
`rawretentiondays` INT NOT NULL DEFAULT 3,
`attemptgroupbindays` INT NOT NULL DEFAULT 2,
`lookuptvrage` INT NOT NULL DEFAULT 1,
`lookupimdb` INT NOT NULL DEFAULT 1,
`lookupnfo` INT NOT NULL DEFAULT 1,
`lookupmusic` INT NOT NULL DEFAULT 1,
`lookupgames` INT NOT NULL DEFAULT 1,
`amazonpubkey` VARCHAR(255) NULL,
`amazonprivkey` VARCHAR(255) NULL,
`tmdbkey` VARCHAR(255) NULL,
`compressedheaders` INT NOT NULL DEFAULT 0,
`maxmssgs` INT NOT NULL DEFAULT 20000,
`newgroupscanmethod` INT NOT NULL DEFAULT 0,
`newgroupdaystoscan` INT NOT NULL DEFAULT 3,
`newgroupmsgstoscan` INT NOT NULL DEFAULT 50000,
`storeuserips` INT NOT NULL DEFAULT 0,
`minfilestoformrelease` INT NOT NULL DEFAULT 1,
`reqidurl` VARCHAR(1000) NOT NULL DEFAULT 'http://allfilled.newznab.com/query.php?t=[GROUP]&reqid=[REQID]',
`latestregexurl` VARCHAR(1000) NOT NULL DEFAULT 'http://www.newznab.com/getregex.php',
`latestregexrevision` INT NOT NULL DEFAULT 0,
`releaseretentiondays` INT NOT NULL DEFAULT 0,
`checkpasswordedrar` INT NOT NULL DEFAULT 0,
`showpasswordedrelease` INT NOT NULL DEFAULT 0
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;


INSERT INTO `site`
	(`id`, 
	`code`, 
	`title`, 
	`strapline`, 
	`metatitle`, 
	`metadescription`,
	`metakeywords`,
	`footer`, 
	`email`, 
	`lastupdate`, 
	`google_adsense_search`, 
	`google_adsense_sidepanel`, 
	`google_analytics_acc`,
	`siteseed`,
	`tandc`,
	`registerstatus`,
	`style`,
	`dereferrer_link`,
	`nzbpath`,
	`rawretentiondays`,
	`attemptgroupbindays`,
	`amazonpubkey`,
	`amazonprivkey`,
	`tmdbkey`
	)
	VALUES
	(NULL, 'newznab', 'Newznab', 'A great usenet indexer', 'Newznab - A great usenet indexer', 
	'Newznab a usenet indexing website with community features', 'usenet,nzbs,newznab,cms,community', 
	'newznab is designed to be a simple usenet indexing site that is easy to configure as a community website.',
	'info@newznab.com', 
	NOW(), NULL, NULL, NULL, MD5(UUID()), '<p>All information within this database is indexed by an automated process, without any human intervention. It is obtained from global Usenet newsgroups over which this site has no control. We cannot prevent that you might find obscene or objectionable material by using this service. If you do come across obscene, incorrect or objectionable results, let us know by using the contact form.</p>',0, '', 
	'http://www.dereferer.ws/?', '/your/path/to/nzbs/', 2, 2, 'AKIAIPDNG5EU7LB4AD3Q', 'B58mVwyj+T/MEucxWugJ3GQ0CcW2kQq16qq/1WpS', '9a4e16adddcd1e86da19bcaf5ff3c2a3');

DROP TABLE IF EXISTS `consoleinfo`;
CREATE TABLE `consoleinfo` (
  `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `title` varchar(128) COLLATE utf8_unicode_ci NOT NULL,
  `asin` varchar(128) COLLATE utf8_unicode_ci DEFAULT NULL,
  `url` varchar(1000) COLLATE utf8_unicode_ci DEFAULT NULL,
  `salesrank` int(10) unsigned DEFAULT NULL,
  `platform` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
  `publisher` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
  `genreID` INT( 10 ) NULL DEFAULT NULL,
  `esrb` VARCHAR( 255 ) NULL DEFAULT NULL,
  `releasedate` datetime DEFAULT NULL,
  `review` varchar(2000) COLLATE utf8_unicode_ci DEFAULT NULL,
  `cover` tinyint(1) unsigned NOT NULL DEFAULT '0',
  `createddate` datetime NOT NULL,
  `updateddate` datetime NOT NULL,
  PRIMARY KEY (`ID`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci AUTO_INCREMENT=1 ;

DROP TABLE IF EXISTS `musicinfo`;
CREATE TABLE `musicinfo` 
(
  `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `title` varchar(128) NOT NULL,
  `asin` varchar(128) NULL,
  `url` varchar(1000) NULL,
  `salesrank` int(10) unsigned NULL,
  `artist` varchar(255) NULL,
  `publisher` varchar(255) NULL,
  `releasedate` datetime NULL,
  `review` varchar(2000) NULL,
  `year` varchar(4) NOT NULL,
  `genreID` int(10) NULL,
  `tracks` varchar(2000) NULL,
  `cover` TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '0',
  `createddate` datetime NOT NULL,
  `updateddate` datetime NOT NULL,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;



DROP TABLE IF EXISTS `genres`;
CREATE TABLE `genres` 
(
  `ID` int NOT NULL AUTO_INCREMENT,
  `title` varchar(255) NOT NULL,
  `type` INT( 4 ) NULL DEFAULT NULL,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1 ;

INSERT INTO `genres` 
(
  `title`, `type`
) VALUES
  ('Blues', 3000),
  ('Classic Rock', 3000),
  ('Country', 3000),
  ('Dance', 3000),
  ('Disco', 3000),
  ('Funk', 3000),
  ('Grunge', 3000),
  ('Hip-Hop', 3000),
  ('Jazz', 3000),
  ('Metal', 3000),
  ('New Age', 3000),
  ('Oldies', 3000),
  ('Other', 3000),
  ('Pop', 3000),
  ('R&B', 3000),
  ('Rap', 3000),
  ('Reggae', 3000),
  ('Rock', 3000),
  ('Techno', 3000),
  ('Industrial', 3000),
  ('Alternative', 3000),
  ('Ska', 3000),
  ('Death Metal', 3000),
  ('Pranks', 3000),
  ('Soundtrack', 3000),
  ('Euro-Techno', 3000),
  ('Ambient', 3000),
  ('Trip-Hop', 3000),
  ('Vocal', 3000),
  ('Jazz+Funk', 3000),
  ('Fusion', 3000),
  ('Trance', 3000),
  ('Classical', 3000),
  ('Instrumental', 3000),
  ('Acid', 3000),
  ('House', 3000),
  ('Game', 3000),
  ('Sound Clip', 3000),
  ('Gospel', 3000),
  ('Noise', 3000),
  ('Alternative Rock', 3000),
  ('Bass', 3000),
  ('Soul', 3000),
  ('Punk', 3000),
  ('Space', 3000),
  ('Meditative', 3000),
  ('Instrumental Pop', 3000),
  ('Instrumental Rock', 3000),
  ('Ethnic', 3000),
  ('Gothic', 3000),
  ('Darkwave', 3000),
  ('Techno-Industrial', 3000),
  ('Electronic', 3000),
  ('Pop-Folk', 3000),
  ('Eurodance', 3000),
  ('Dream', 3000),
  ('Southern Rock', 3000),
  ('Comedy', 3000),
  ('Cult', 3000),
  ('Gangsta', 3000),
  ('Top 40', 3000),
  ('Christian Rap', 3000),
  ('Pop/Funk', 3000),
  ('Jungle', 3000),
  ('Native US', 3000),
  ('Cabaret', 3000),
  ('New Wave', 3000),
  ('Psychadelic', 3000),
  ('Rave', 3000),
  ('Showtunes', 3000),
  ('Trailer', 3000),
  ('Lo-Fi', 3000),
  ('Tribal', 3000),
  ('Acid Punk', 3000),
  ('Acid Jazz', 3000),
  ('Polka', 3000),
  ('Retro', 3000),
  ('Musical', 3000),
  ('Rock & Roll', 3000),
  ('Hard Rock', 3000),
  ('Folk', 3000),
  ('Folk-Rock', 3000),
  ('National Folk', 3000),
  ('Swing', 3000),
  ('Fast Fusion', 3000),
  ('Bebob', 3000),
  ('Latin', 3000),
  ('Revival', 3000),
  ('Celtic', 3000),
  ('Bluegrass', 3000),
  ('Avantgarde', 3000),
  ('Gothic Rock', 3000),
  ('Progressive Rock', 3000),
  ('Psychedelic Rock', 3000),
  ('Symphonic Rock', 3000),
  ('Slow Rock', 3000),
  ('Big Band', 3000),
  ('Chorus', 3000),
  ('Easy Listening', 3000),
  ('Acoustic', 3000),
  ('Humour', 3000),
  ('Speech', 3000),
  ('Chanson', 3000),
  ('Opera', 3000),
  ('Chamber Music', 3000),
  ('Sonata', 3000),
  ('Symphony', 3000),
  ('Booty Bass', 3000),
  ('Primus', 3000),
  ('Porn Groove', 3000),
  ('Satire', 3000),
  ('Slow Jam', 3000),
  ('Club', 3000),
  ('Tango', 3000),
  ('Samba', 3000),
  ('Folklore', 3000),
  ('Ballad', 3000),
  ('Power Ballad', 3000),
  ('Rhytmic Soul', 3000),
  ('Freestyle', 3000),
  ('Duet', 3000),
  ('Punk Rock', 3000),
  ('Drum Solo', 3000),
  ('Acapella', 3000),
  ('Euro-House', 3000),
  ('Dance Hall', 3000),
  ('Goa', 3000),
  ('Drum & Bass', 3000),
  ('Club-House', 3000),
  ('Hardcore', 3000),
  ('Terror', 3000),
  ('Indie', 3000),
  ('BritPop', 3000),
  ('Negerpunk', 3000),
  ('Polsk Punk', 3000),
  ('Beat', 3000),
  ('Christian Gangsta', 3000),
  ('Heavy Metal', 3000),
  ('Black Metal', 3000),
  ('Crossover', 3000),
  ('Contemporary C', 3000),
  ('Christian Rock', 3000),
  ('Merengue', 3000),
  ('Salsa', 3000),
  ('Thrash Metal', 3000),
  ('Anime', 3000),
  ('JPop', 3000),
  ('SynthPop', 3000),
  ('Electronica', 3000);













INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(1,6318,'The Young and the Restless',NULL,now(),NULL,70328),
(2,3557,'Flash Gordon (1954)',NULL,now(),NULL,70331),
(3,77,'Jesse Hawkes',NULL,now(),NULL,70332),
(4,9814,'The Jack Paar Tonight Show',NULL,now(),NULL,70333),
(5,6246,'The Tonight Show Starring Johnny Carson',NULL,now(),NULL,70334),
(6,12038,'He and She',NULL,now(),NULL,70335),
(7,6247,'The Tonight Show with Jay Leno',NULL,now(),NULL,70336),
(8,502,'Hazell',NULL,now(),NULL,70337),
(9,503,'No Strings',NULL,now(),NULL,70338),
(10,8698,'Cuffy',NULL,now(),NULL,70339),
(11,504,'The Shillingbury Tales',NULL,now(),NULL,70340),
(12,2732,'Bay City Blues',NULL,now(),NULL,70341),
(13,7498,'Newsnight',NULL,now(),NULL,70342),
(14,3320,'Dirty Dancing',NULL,now(),NULL,70345),
(15,10431,'Dark Towers',NULL,now(),NULL,70346),
(16,5207,'Sirens',NULL,now(),NULL,70347),
(17,21242,'Life Moments',NULL,now(),NULL,70348),
(18,507,'Two Faces West',NULL,now(),NULL,70349),
(19,508,'About Face',NULL,now(),NULL,70351),
(20,4805,'Perversions of Science',NULL,now(),NULL,70352),
(21,6652,'WWE Pay-Per-View',NULL,now(),NULL,70353),
(22,7193,'Foxbusters',NULL,now(),NULL,70354),
(23,6352,'ThunderCats',NULL,now(),NULL,70355),
(24,509,'Hootenanny',NULL,now(),NULL,70356),
(25,510,'Mike and Angelo',NULL,now(),NULL,70357),
(26,2985,'Captain Z-RO',NULL,now(),NULL,70358),
(27,5158,'Shadow Chasers',NULL,now(),NULL,70359),
(28,5720,'The Day Today',NULL,now(),NULL,70360),
(29,9625,'The Mary Whitehouse Experience',NULL,now(),NULL,70361),
(30,15948,'Here\'s Boomer',NULL,now(),NULL,70362),
(31,8485,'The Enigma Files',NULL,now(),NULL,70363),
(32,3256,'Days of our Lives',NULL,now(),NULL,70366),
(33,4533,'Movie Stars',NULL,now(),NULL,70367),
(34,5438,'Sylvester & Tweety Mysteries',NULL,now(),NULL,70368),
(35,2512,'Against All Odds',NULL,now(),NULL,70371),
(36,512,'The Young Lawyers',NULL,now(),NULL,70373),
(37,513,'The Interns',NULL,now(),NULL,70374),
(38,4394,'Matt Helm',NULL,now(),NULL,70375),
(39,514,'Division 4',NULL,now(),NULL,70376),
(40,515,'A Kind of Loving',NULL,now(),NULL,70377),
(41,8626,'Trouble in Mind',NULL,now(),NULL,70378),
(42,6055,'The O\'Keefes',NULL,now(),NULL,70379),
(43,4924,'Queens Supreme',NULL,now(),NULL,70380),
(44,4913,'Pulaski',NULL,now(),NULL,70381),
(45,4664,'Night Gallery',NULL,now(),NULL,70382),
(46,516,'Roger Roger',NULL,now(),NULL,70383),
(47,517,'Union Pacific',NULL,now(),NULL,70384),
(48,6288,'The Westerner',NULL,now(),NULL,70385),
(49,3666,'Get Smart',NULL,now(),NULL,70386),
(50,3202,'Dad',NULL,now(),NULL,70388),
(51,3212,'Dalton\'s Code of Vengeance',NULL,now(),NULL,70389),
(52,3224,'Danger Island',NULL,now(),NULL,70390),
(53,518,'Dangerous Assignment',NULL,now(),NULL,70391),
(54,3265,'Dead of Night',NULL,now(),NULL,70392),
(55,3141,'Coronet Blue',NULL,now(),NULL,70393),
(56,3334,'Dog City',NULL,now(),NULL,70394),
(57,3341,'Don Coyote and Sancho Panda',NULL,now(),NULL,70395),
(58,3356,'Downtown (1986)',NULL,now(),NULL,70396),
(59,3357,'Downwardly Mobile',NULL,now(),NULL,70397),
(60,520,'The Dream Team (1999)',NULL,now(),NULL,70398),
(61,3388,'Droids',NULL,now(),NULL,70399),
(62,3395,'Dudley',NULL,now(),NULL,70400),
(63,5739,'The Duke',NULL,now(),NULL,70401),
(64,3428,'Electra Woman and Dyna Girl',NULL,now(),NULL,70402),
(65,6291,'The Wide Country',NULL,now(),NULL,70403),
(66,5752,'The Enid Blyton Adventure Series',NULL,now(),NULL,70404),
(67,3447,'Enos',NULL,now(),NULL,70406),
(68,3452,'Erasmus Microman',NULL,now(),NULL,70407),
(69,3463,'Ever Decreasing Circles',NULL,now(),NULL,70408),
(70,5756,'The Evil Touch',NULL,now(),NULL,70409),
(71,521,'Extreme Dinosaurs',NULL,now(),NULL,70410),
(72,3488,'Fairly Secret Army',NULL,now(),NULL,70411),
(73,5770,'The Fanelli Boys',NULL,now(),NULL,70413),
(74,2915,'Bronco',NULL,now(),NULL,70414),
(75,5775,'The Fast Show',NULL,now(),NULL,70415),
(76,3524,'Father Brown',NULL,now(),NULL,70416),
(77,5787,'The Flying Doctor',NULL,now(),NULL,70418),
(78,3589,'Foxy Lady',NULL,now(),NULL,70419),
(79,5804,'The Gay Cavalier',NULL,now(),NULL,70420),
(80,3656,'George',NULL,now(),NULL,70421),
(81,3671,'Get Well Soon',NULL,now(),NULL,70422),
(82,3831,'Hennesey',NULL,now(),NULL,70423),
(83,3675,'Ghosts of Motley Hall',NULL,now(),NULL,70424),
(84,3734,'Great Ghost Tales',NULL,now(),NULL,70425),
(85,3743,'Grim Tales',NULL,now(),NULL,70426),
(86,5834,'The Groovy Fellers',NULL,now(),NULL,70427),
(87,5836,'The Guardians',NULL,now(),NULL,70428),
(88,3754,'Guinness World Records: Primetime',NULL,now(),NULL,70429),
(89,3778,'Hamish Macbeth',NULL,now(),NULL,70431),
(90,3806,'Hawaiian Eye',NULL,now(),NULL,70432),
(91,3780,'Hammer House of Mystery & Suspense',NULL,now(),NULL,70433),
(92,5840,'The Hardy Boys',NULL,now(),NULL,70434),
(93,3824,'Heat of the Sun',NULL,now(),NULL,70435),
(94,3855,'High Sierra Search and Rescue',NULL,now(),NULL,70436),
(95,3851,'Highlander: The Animated Series',NULL,now(),NULL,70437),
(96,3867,'Hippies',NULL,now(),NULL,70438),
(97,3880,'Hollywood Safari',NULL,now(),NULL,70439),
(98,5862,'The House of Windsor',NULL,now(),NULL,70440),
(99,3936,'I, Lovett',NULL,now(),NULL,70441),
(100,4854,'Police Story',NULL,now(),NULL,70442),
(101,4399,'Maverick',NULL,now(),NULL,70443),
(102,5869,'The Immortal (1970)',NULL,now(),NULL,70444),
(103,3963,'In Search of Happiness',NULL,now(),NULL,70445),
(104,3975,'Internet Slutts',NULL,now(),NULL,70446),
(105,3982,'Into the Labyrinth',NULL,now(),NULL,70447),
(106,3998,'Is It Bill Bailey?',NULL,now(),NULL,70448),
(107,522,'It\'s Only TV But I Like It',NULL,now(),NULL,70449),
(108,4022,'Jack and Jeremy\'s Real Lives',NULL,now(),NULL,70450),
(109,4099,'Kama Sutra',NULL,now(),NULL,70451),
(110,4118,'Kevin Spencer',NULL,now(),NULL,70452),
(111,4131,'King Arthur and the Knights of Justice',NULL,now(),NULL,70453),
(112,5001,'Richard Diamond, Private Detective',NULL,now(),NULL,70454),
(113,4137,'Kinvig',NULL,now(),NULL,70455),
(114,5912,'The Last Machine',NULL,now(),NULL,70456),
(115,5913,'The Last Salute',NULL,now(),NULL,70457),
(116,4206,'Law and Disorder',NULL,now(),NULL,70458),
(117,4210,'Lazarus and Dingwall',NULL,now(),NULL,70459),
(118,5928,'The Lenny Henry Show (1987)',NULL,now(),NULL,70460),
(119,4220,'Lenny Henry Tonite',NULL,now(),NULL,70461),
(120,4243,'Lightning Force',NULL,now(),NULL,70462),
(121,5942,'The Lone Ranger (1966)',NULL,now(),NULL,70463),
(122,78,'Lock-Up',NULL,now(),NULL,70464),
(123,523,'The Magician (1999)',NULL,now(),NULL,70465),
(124,4347,'Making News',NULL,now(),NULL,70466),
(125,4377,'Married for Life',NULL,now(),NULL,70467),
(126,5972,'The Martin Short Show (1999)',NULL,now(),NULL,70468),
(127,4404,'Max Monroe: Loose Cannon',NULL,now(),NULL,70469),
(128,4410,'May to December',NULL,now(),NULL,70470),
(129,4412,'McCallum',NULL,now(),NULL,70471),
(130,524,'The Micro Ventures',NULL,now(),NULL,70472),
(131,4470,'Mighty Orbots',NULL,now(),NULL,70473),
(132,5623,'The Bill',NULL,now(),NULL,70474),
(133,4509,'Mona the Vampire',NULL,now(),NULL,70475),
(134,4515,'Monkey Magic (1998)',NULL,now(),NULL,70476),
(135,525,'The Monroes (1995)',NULL,now(),NULL,70477),
(136,4516,'Monster by Mistake',NULL,now(),NULL,70478),
(137,4524,'Moonbase 3',NULL,now(),NULL,70479),
(138,4534,'Mowgli: The New Adventures of the Jungle Book',NULL,now(),NULL,70480),
(139,4550,'Mulberry',NULL,now(),NULL,70481),
(140,4582,'My Life and Times',NULL,now(),NULL,70482),
(141,79,'Thieftakers',NULL,now(),NULL,70483),
(142,4606,'Mythic Warriors',NULL,now(),NULL,70484),
(143,526,'Wichita Town',NULL,now(),NULL,70485),
(144,527,'The Wrangler',NULL,now(),NULL,70486),
(145,3845,'Hidden Hills',NULL,now(),NULL,70487),
(146,2509,'African Skies',NULL,now(),NULL,70488),
(147,4622,'NASCAR Racers',NULL,now(),NULL,70489),
(148,4643,'Neverwhere',NULL,now(),NULL,70490),
(149,6050,'The Nicholas Craig Masterclass',NULL,now(),NULL,70491),
(150,528,'Nightingales (1990)',NULL,now(),NULL,70492),
(151,529,'Office Gossip',NULL,now(),NULL,70493),
(152,6071,'The Outsider',NULL,now(),NULL,70494),
(153,4771,'Paradise Postponed',NULL,now(),NULL,70495),
(154,4803,'Perfect World',NULL,now(),NULL,70496),
(155,4818,'Philip Marlowe',NULL,now(),NULL,70497),
(156,4859,'Pond Life',NULL,now(),NULL,70498),
(157,4869,'Porkpie',NULL,now(),NULL,70499),
(158,3622,'Full House',NULL,now(),NULL,70500),
(159,530,'P.R.',NULL,now(),NULL,70501),
(160,6106,'The Preston Episodes',NULL,now(),NULL,70502),
(161,531,'Preston Front',NULL,now(),NULL,70503),
(162,6116,'The Quest (1982)',NULL,now(),NULL,70504),
(163,17841,'Heroes and Villains',NULL,now(),NULL,70505),
(164,2728,'Battle of the Network Stars',NULL,now(),NULL,70506),
(165,6165,'The Scarlet Pimpernel',NULL,now(),NULL,70507),
(166,4983,'Rescue Heroes',NULL,now(),NULL,70508),
(167,4990,'Return to the Planet of the Apes',NULL,now(),NULL,70509),
(168,5002,'Richard the Lionheart',NULL,now(),NULL,70510),
(169,5010,'Ripley\'s Believe It or Not!',NULL,now(),NULL,70511),
(170,5025,'RoboCop: Alpha Commando',NULL,now(),NULL,70512),
(171,5026,'RoboCop: Prime Directives',NULL,now(),NULL,70513),
(172,5036,'Rock Follies',NULL,now(),NULL,70514),
(173,5042,'Rocky Jones, Space Ranger',NULL,now(),NULL,70515),
(174,21892,'Ronin Warriors',NULL,now(),NULL,70516),
(175,5062,'Route 66 (1993)',NULL,now(),NULL,70517),
(176,6466,'UltraForce',NULL,now(),NULL,70518),
(177,6488,'Unsub',NULL,now(),NULL,70519),
(178,6265,'The Untouchables (1993)',NULL,now(),NULL,70520),
(179,532,'Valley of the Dinosaurs',NULL,now(),NULL,70521),
(180,6497,'Van-pires',NULL,now(),NULL,70523),
(181,6164,'The Scarlet Pimpernel (1956)',NULL,now(),NULL,70526),
(182,533,'Scotland Yard',NULL,now(),NULL,70527),
(183,5124,'Sealab 2020',NULL,now(),NULL,70528),
(184,6173,'The Secret Diary of Adrian Mole Aged 13 3/4',NULL,now(),NULL,70529),
(185,5150,'Serpico',NULL,now(),NULL,70530),
(186,5165,'Shannon\'s Deal',NULL,now(),NULL,70531),
(187,4414,'McCloud',NULL,now(),NULL,70532),
(188,5177,'Sherlock Holmes in the 22nd Century',NULL,now(),NULL,70534),
(189,5208,'Sir Francis Drake',NULL,now(),NULL,70535),
(190,5218,'Skysurfer Strike Force',NULL,now(),NULL,70536),
(191,5225,'Slinger\'s Day',NULL,now(),NULL,70537),
(192,5236,'Snoops (1989)',NULL,now(),NULL,70538),
(193,5269,'So You Think You\'ve Got Troubles',NULL,now(),NULL,70539),
(194,5239,'Sob Sisters',NULL,now(),NULL,70540),
(195,3922,'How To Marry A Millionaire',NULL,now(),NULL,70541),
(196,5259,'Sorry!',NULL,now(),NULL,70542),
(197,5393,'Sunnyside Farm',NULL,now(),NULL,70543),
(198,5425,'Swat Kats: The Radical Squadron',NULL,now(),NULL,70545),
(199,5435,'Sword of Freedom',NULL,now(),NULL,70546),
(200,5453,'Tales of the Vikings',NULL,now(),NULL,70547),
(201,5460,'Tama and Friends',NULL,now(),NULL,70548),
(202,534,'Tandarra',NULL,now(),NULL,70549),
(203,5465,'Tarzan (1966)',NULL,now(),NULL,70550),
(204,80,'Harper Valley',NULL,now(),NULL,70551),
(205,5467,'Tarzan (1991)',NULL,now(),NULL,70552),
(206,5469,'Tarzan Lord of the Jungle',NULL,now(),NULL,70553),
(207,5470,'Tarzan The Epic Adventures',NULL,now(),NULL,70554),
(208,10859,'Telly Addicts',NULL,now(),NULL,70555),
(209,5509,'The 10%ers',NULL,now(),NULL,70556),
(210,6234,'The Third Man',NULL,now(),NULL,70557),
(211,6235,'The Three Musketeers',NULL,now(),NULL,70558),
(212,6381,'Tom Stone',NULL,now(),NULL,70560),
(213,2894,'Brand New Life',NULL,now(),NULL,70561),
(214,6249,'Top Secret Life of Edgar Briggs',NULL,now(),NULL,70562),
(215,6387,'Totally Circus',NULL,now(),NULL,70563),
(216,535,'Totally Hoops',NULL,now(),NULL,70564),
(217,6393,'Touching Evil (UK)',NULL,now(),NULL,70565),
(218,6419,'Treasure',NULL,now(),NULL,70567),
(219,6427,'Tripper\'s Day',NULL,now(),NULL,70568),
(220,6437,'Trying Times',NULL,now(),NULL,70569),
(221,5539,'The Adventures of Robin Hood',NULL,now(),NULL,70571),
(222,536,'Circus of the Stars',NULL,now(),NULL,70573),
(223,81,'West Point Story',NULL,now(),NULL,70574),
(224,4197,'Later with Bob Costas',NULL,now(),NULL,70575),
(225,3268,'Dean Martin Celebrity Roast',NULL,now(),NULL,70577),
(226,23896,'Highway Patrol',NULL,now(),NULL,70578),
(227,5935,'The Little Lulu Show',NULL,now(),NULL,70579),
(228,82,'The Millionaire',NULL,now(),NULL,70580),
(229,5473,'Tattooed Teenage Alien Fighters from Beverly Hills',NULL,now(),NULL,70581),
(230,13332,'Shirley\'s World',NULL,now(),NULL,70582),
(231,10046,'Casting Couch',NULL,now(),NULL,70583),
(232,3944,'I Love Lucy',NULL,now(),NULL,70584),
(233,9221,'Late Lunch',NULL,now(),NULL,70585),
(234,8299,'Lotsa Luck',NULL,now(),NULL,70586),
(235,12112,'Roger Ramjet',NULL,now(),NULL,70587),
(236,9958,'Bright Promise',NULL,now(),NULL,70588),
(237,6057,'The Odd Couple',NULL,now(),NULL,70591),
(238,2500,'Adventures in Paradise',NULL,now(),NULL,70592),
(239,11028,'Saints and Sinners',NULL,now(),NULL,70593),
(240,6274,'The Virginian',NULL,now(),NULL,70594),
(241,5061,'Route 66',NULL,now(),NULL,70595),
(242,2900,'Breaker High',NULL,now(),NULL,70596),
(243,539,'Jane',NULL,now(),NULL,70597),
(244,16212,'Heartland',NULL,now(),NULL,70598),
(245,3093,'Clueless',NULL,now(),NULL,70599),
(246,7131,'Dateline NBC',NULL,now(),NULL,70600),
(247,4309,'Lucy Sullivan is Getting Married',NULL,now(),NULL,70602),
(248,6074,'The Parent \'Hood',NULL,now(),NULL,70603),
(249,541,'McCready and Daughter',NULL,now(),NULL,70604),
(250,5209,'Sister, Sister',NULL,now(),NULL,70605),
(251,4291,'Love, American Style',NULL,now(),NULL,70608),
(252,5102,'Saved by the Bell: The New Class',NULL,now(),NULL,70609),
(253,3712,'Goode Behavior',NULL,now(),NULL,70610),
(254,5656,'The Brothers Grunt',NULL,now(),NULL,70612),
(255,5536,'The Adventures of Paddington Bear',NULL,now(),NULL,70613),
(256,4063,'The Client',NULL,now(),NULL,70614),
(257,84,'Cain\'s Hundred',NULL,now(),NULL,70615),
(258,5463,'Target: The Corruptors',NULL,now(),NULL,70616),
(259,6245,'The Tonight Show',NULL,now(),NULL,70617),
(260,15890,'Star Search',NULL,now(),NULL,70618),
(261,542,'Harold and the Purple Crayon',NULL,now(),NULL,70619),
(262,1098,'Tony Orlando And Dawn',NULL,now(),NULL,70620),
(263,6508,'V Graham Norton',NULL,now(),NULL,70621),
(264,2738,'Bear in the Big Blue House',NULL,now(),NULL,70622),
(265,5900,'The Jonathan Winters Show',NULL,now(),NULL,70624),
(266,7025,'Nickelodeon GUTS',NULL,now(),NULL,70625),
(267,7969,'This is Your Life',NULL,now(),NULL,70627),
(268,543,'Your All American College Show',NULL,now(),NULL,70628),
(269,4158,'Kraft Music Hall',NULL,now(),NULL,70629),
(270,6297,'The Wizard',NULL,now(),NULL,70630),
(271,544,'Tiswas',NULL,now(),NULL,70631),
(272,4478,'Miracles',NULL,now(),NULL,70632),
(273,3772,'Hack',NULL,now(),NULL,70633),
(274,5219,'Slattery\'s People',NULL,now(),NULL,70634),
(275,4948,'Rawhide',NULL,now(),NULL,70635),
(276,545,'Moon and Son',NULL,now(),NULL,70636),
(277,546,'On Common Ground',NULL,now(),NULL,70637),
(278,16754,'Sonny Spoon',NULL,now(),NULL,70638),
(279,3448,'Entertainment Tonight',NULL,now(),NULL,70640),
(280,547,'The Experiment',NULL,now(),NULL,70641),
(281,5020,'Robbery Homicide Division',NULL,now(),NULL,70642),
(282,85,'Talk Of The Town',NULL,now(),NULL,70644),
(283,6657,'WWE Velocity',NULL,now(),NULL,70649),
(284,549,'A.J.\'s Time Travelers',NULL,now(),NULL,70650),
(285,550,'Bug Juice',NULL,now(),NULL,70651),
(286,551,'Tall Tales and Legends',NULL,now(),NULL,70652),
(287,552,'Under the Umbrella Tree',NULL,now(),NULL,70654),
(288,6977,'The O\'Reilly Factor',NULL,now(),NULL,70656),
(289,5670,'The Carol Burnett Show',NULL,now(),NULL,70657),
(290,11,'USA High',NULL,now(),NULL,70658),
(291,6198,'The Smothers Brothers Comedy Hour',NULL,now(),NULL,70659),
(292,8707,'George and the Dragon',NULL,now(),NULL,70661),
(293,8943,'Pardon the Expression!',NULL,now(),NULL,70662),
(294,5011,'Ripping Yarns',NULL,now(),NULL,70663),
(295,553,'The Morecambe & Wise Show',NULL,now(),NULL,70664),
(296,7332,'Xuxa',NULL,now(),NULL,70665),
(297,12205,'Crook and Chase',NULL,now(),NULL,70666),
(298,16718,'Open All Night',NULL,now(),NULL,70670),
(299,555,'Washingtoon',NULL,now(),NULL,70671),
(300,6616,'Wimzie\'s House',NULL,now(),NULL,70672),
(301,9337,'The Frankie Howerd Show',NULL,now(),NULL,70673),
(302,7441,'Whoops Baghdad',NULL,now(),NULL,70674),
(303,11170,'Then Churchill Said to Me',NULL,now(),NULL,70675),
(304,2502,'Adventures in Wonderland',NULL,now(),NULL,70676),
(305,3952,'In-Laws',NULL,now(),NULL,70677),
(306,2598,'American Dreams',NULL,now(),NULL,70678),
(307,556,'Brass Eye',NULL,now(),NULL,70679),
(308,557,'Big Train',NULL,now(),NULL,70680),
(309,558,'The Railway Children',NULL,now(),NULL,70681),
(310,5507,'That Was Then',NULL,now(),NULL,70683),
(311,3010,'Cedric the Entertainer Presents',NULL,now(),NULL,70684),
(312,4918,'Push, Nevada',NULL,now(),NULL,70686),
(313,11143,'The Blue Angels',NULL,now(),NULL,70687),
(314,4547,'M Squad',NULL,now(),NULL,70688),
(315,3535,'Feds',NULL,now(),NULL,70689),
(316,1097,'Caesar\'s Hour',NULL,now(),NULL,70691),
(317,2892,'Bram & Alice',NULL,now(),NULL,70693),
(318,5956,'The Lucy Show',NULL,now(),NULL,70695),
(319,562,'The Eddie Cantor Comedy Theatre',NULL,now(),NULL,70696),
(320,563,'The Magic Basket',NULL,now(),NULL,70697),
(321,2863,'Body and Soul',NULL,now(),NULL,70698),
(322,3836,'Here\'s Lucy',NULL,now(),NULL,70699),
(323,86,'Life with Lucy',NULL,now(),NULL,70700),
(324,15606,'The Johnny Carson Show',NULL,now(),NULL,70701),
(325,3775,'Half & Half',NULL,now(),NULL,70702),
(326,564,'Kath & Kim',NULL,now(),NULL,70703),
(327,3789,'Hardball (1989)',NULL,now(),NULL,70705),
(328,22274,'Rab C Nesbitt',NULL,now(),NULL,70706),
(329,15638,'Hot Pursuit',NULL,now(),NULL,70707),
(330,6241,'The Today Show',NULL,now(),NULL,70708),
(331,566,'Victoria Wood',NULL,now(),NULL,70709),
(332,3719,'Good Morning America',NULL,now(),NULL,70710),
(333,567,'WWE Confidential',NULL,now(),NULL,70711),
(334,17989,'Lifestyles of the Rich And Famous',NULL,now(),NULL,70712),
(335,5894,'The Jim Henson Hour',NULL,now(),NULL,70713),
(336,568,'Campus Cops',NULL,now(),NULL,70714),
(337,6423,'Trigger Happy TV',NULL,now(),NULL,70715),
(338,12868,'Starlight Theatre',NULL,now(),NULL,70716),
(339,569,'That Peter Kay Thing',NULL,now(),NULL,70717),
(340,14947,'Kaz',NULL,now(),NULL,70718),
(341,570,'The Tomorrow People (1992)',NULL,now(),NULL,70719),
(342,13093,'The Morning Show',NULL,now(),NULL,70720),
(343,571,'Out of Tune',NULL,now(),NULL,70721),
(344,16681,'Joe and Sons',NULL,now(),NULL,70723),
(345,3575,'For Better or for Worse',NULL,now(),NULL,70724),
(346,572,'Harry Enfield\'s Brand Spanking New Show',NULL,now(),NULL,70725),
(347,3889,'Homicide: Life On The Street',NULL,now(),NULL,70727),
(348,8386,'Tears Before Bedtime',NULL,now(),NULL,70728),
(349,4295,'Loved by You',NULL,now(),NULL,70729),
(350,6874,'Biography',NULL,now(),NULL,70730),
(351,3827,'Helen West',NULL,now(),NULL,70732),
(352,7247,'The 11 O\'Clock Show',NULL,now(),NULL,70733),
(353,14443,'Extra',NULL,now(),NULL,70734),
(354,574,'RI:SE',NULL,now(),NULL,70735),
(355,575,'WWE The Bottom Line',NULL,now(),NULL,70736),
(356,576,'WWE After Burn',NULL,now(),NULL,70737),
(357,6133,'The Rerun Show',NULL,now(),NULL,70738),
(358,8618,'Time After Time',NULL,now(),NULL,70739),
(359,14036,'The Match Game/Hollywood Squares Hour',NULL,now(),NULL,70741),
(360,14338,'In Suspicious Circumstances',NULL,now(),NULL,70742),
(361,8327,'Lytton\'s Diary',NULL,now(),NULL,70744),
(362,9295,'The Eamonn Andrews Show',NULL,now(),NULL,70745),
(363,18543,'The Big Show',NULL,now(),NULL,70747),
(364,13360,'Growing Rich',NULL,now(),NULL,70748),
(365,578,'Supply and Demand',NULL,now(),NULL,70749),
(366,579,'Mister Roberts',NULL,now(),NULL,70751),
(367,15624,'O\'Hara, U.S. Treasury',NULL,now(),NULL,70752),
(368,3155,'Covington Cross',NULL,now(),NULL,70753),
(369,9775,'Rita Rudner',NULL,now(),NULL,70755),
(370,580,'The Last Resort',NULL,now(),NULL,70756),
(371,3120,'Comic Relief',NULL,now(),NULL,70757),
(372,6992,'Hollywood Greats',NULL,now(),NULL,70759),
(373,3012,'Celebrity Big Brother',NULL,now(),NULL,70760),
(374,4185,'Lassie',NULL,now(),NULL,70762),
(375,4065,'Johnny Bago',NULL,now(),NULL,70763),
(376,12019,'Side Street',NULL,now(),NULL,70764),
(377,6076,'The Partners (1971)',NULL,now(),NULL,70765),
(378,4003,'It\'s Always Jan',NULL,now(),NULL,70766),
(379,87,'Broken Arrow',NULL,now(),NULL,70767),
(380,3383,'Dressing for Breakfast',NULL,now(),NULL,70769),
(381,8739,'Hope It Rains',NULL,now(),NULL,70770),
(382,8867,'Double First',NULL,now(),NULL,70771),
(383,581,'The Martha Raye Show',NULL,now(),NULL,70772),
(384,5506,'That Girl',NULL,now(),NULL,70773),
(385,10362,'God\'s Gift',NULL,now(),NULL,70775),
(386,10846,'Streetmate',NULL,now(),NULL,70776),
(387,88,'Whispering Smith',NULL,now(),NULL,70778),
(388,13849,'Oblivious',NULL,now(),NULL,70779),
(389,4181,'Laramie',NULL,now(),NULL,70780),
(390,582,'Late World with Zach',NULL,now(),NULL,70783),
(391,4761,'Ozzy & Drix',NULL,now(),NULL,70784),
(392,2681,'Avengers (1999)',NULL,now(),NULL,70785),
(393,2750,'Beech is Back',NULL,now(),NULL,70786),
(394,5415,'SurfSide 6',NULL,now(),NULL,70787),
(395,11345,'Thank Your Lucky Stars',NULL,now(),NULL,70788),
(396,585,'Ready Steady Go!',NULL,now(),NULL,70789),
(397,5810,'The George Burns and Gracie Allen Show',NULL,now(),NULL,70790),
(398,16647,'Mickey',NULL,now(),NULL,70792),
(399,5729,'The Don Knotts Show',NULL,now(),NULL,70793),
(400,6131,'The Red Skelton Show',NULL,now(),NULL,70794),
(401,5774,'The Farmer\'s Daughter',NULL,now(),NULL,70795),
(402,6629,'Wizards and Warriors',NULL,now(),NULL,70796),
(403,5032,'Robot Wars: Grand Champions',NULL,now(),NULL,70797),
(404,4978,'Renford Rejects',NULL,now(),NULL,70798),
(405,2780,'Beyblade',NULL,now(),NULL,70799),
(406,4816,'Phantom Investigators',NULL,now(),NULL,70800),
(407,5789,'The Flying Nun',NULL,now(),NULL,70801),
(408,8310,'Love Street',NULL,now(),NULL,70803),
(409,4023,'Jack & Jill',NULL,now(),NULL,70804),
(410,587,'The Big Comfy Couch',NULL,now(),NULL,70805),
(411,6161,'The Sandy Duncan Show',NULL,now(),NULL,70806),
(412,89,'Search for Tomorrow',NULL,now(),NULL,70807),
(413,588,'I Love Mummy',NULL,now(),NULL,70808),
(414,7847,'Polka Dot Door',NULL,now(),NULL,70809),
(415,90,'The Edge of Night',NULL,now(),NULL,70810),
(416,4663,'Night Court',NULL,now(),NULL,70812),
(417,4870,'Port Charles',NULL,now(),NULL,70813),
(418,2601,'American Idol',NULL,now(),NULL,70814),
(419,91,'Texas',NULL,now(),NULL,70816),
(420,3781,'Hamtaro',NULL,now(),NULL,70817),
(421,590,'Captain Scarlet and the Mysterons',NULL,now(),NULL,70818),
(422,5568,'The Ambassador',NULL,now(),NULL,70819),
(423,5322,'Star Fleet',NULL,now(),NULL,70820),
(424,592,'Party Wagon',NULL,now(),NULL,70821),
(425,6159,'The Sally Jessy Raphael Show',NULL,now(),NULL,70822),
(426,92,'Password',NULL,now(),NULL,70823),
(427,23951,'The Force',NULL,now(),NULL,70826),
(428,594,'Powerstone',NULL,now(),NULL,70827),
(429,595,'Fireball XL5',NULL,now(),NULL,70828),
(430,2480,'Access Hollywood',NULL,now(),NULL,70829),
(431,5627,'Black Sash',NULL,now(),NULL,70832),
(432,7825,'XPM',NULL,now(),NULL,70833),
(433,596,'Rollerjam',NULL,now(),NULL,70834),
(434,8300,'Harry\'s Girls',NULL,now(),NULL,70837),
(435,4548,'MTV Cribs',NULL,now(),NULL,70839),
(436,4420,'MDA',NULL,now(),NULL,70840),
(437,10704,'Prickly Heat',NULL,now(),NULL,70841),
(438,6956,'Super Sloppy Double Dare',NULL,now(),NULL,70842),
(439,4109,'Keen Eddie',NULL,now(),NULL,70843),
(440,4623,'Nash Bridges',NULL,now(),NULL,70845),
(441,598,'Dark Season',NULL,now(),NULL,70848),
(442,599,'Bulman',NULL,now(),NULL,70849),
(443,6536,'Wanda at Large',NULL,now(),NULL,70850),
(444,5039,'Rock Profile',NULL,now(),NULL,70853),
(445,12109,'Joe\'s World',NULL,now(),NULL,70854),
(446,2992,'Carol Burnett & Company',NULL,now(),NULL,70855),
(447,4746,'Outlaws (1986)',NULL,now(),NULL,70857),
(448,600,'Mataku',NULL,now(),NULL,70858),
(449,12209,'Vicki',NULL,now(),NULL,70859),
(450,4296,'Love Hina',NULL,now(),NULL,70861),
(451,13009,'Vampire Princess Miyu',NULL,now(),NULL,70862),
(452,12434,'This is Tom Jones',NULL,now(),NULL,70864),
(453,602,'X',NULL,now(),NULL,70865),
(454,19142,'Away We Go',NULL,now(),NULL,70866),
(455,5995,'The Mr. Potato Head Show',NULL,now(),NULL,70867),
(456,603,'Three\'s a Crowd (2000)',NULL,now(),NULL,70868),
(457,5593,'The Bachelor',NULL,now(),NULL,70869),
(458,6126,'The Real World/Road Rules Challenge',NULL,now(),NULL,70870),
(459,606,'Hollywood Showdown',NULL,now(),NULL,70871),
(460,607,'Greed',NULL,now(),NULL,70872),
(461,4937,'Raffles',NULL,now(),NULL,70873),
(462,5572,'The Ananda Lewis Show',NULL,now(),NULL,70875),
(463,4308,'Lucky',NULL,now(),NULL,70876),
(464,3409,'E! True Hollywood Story',NULL,now(),NULL,70877),
(465,4991,'Return to Treasure Island',NULL,now(),NULL,70879),
(466,7458,'Lingo',NULL,now(),NULL,70880),
(467,16719,'Fitz and Bones',NULL,now(),NULL,70881),
(468,93,'The Mike Douglas Show',NULL,now(),NULL,70882),
(469,609,'The Bobby Heenan Show',NULL,now(),NULL,70883),
(470,6661,'WWF Shotgun Saturday Night',NULL,now(),NULL,70884),
(471,3229,'Danger UXB',NULL,now(),NULL,70885),
(472,19082,'The Jerry Lewis Show',NULL,now(),NULL,70886),
(473,3662,'Get Along Gang',NULL,now(),NULL,70887),
(474,3370,'Dragon\'s Lair',NULL,now(),NULL,70889),
(475,610,'Robotix',NULL,now(),NULL,70890),
(476,611,'Sectaurs',NULL,now(),NULL,70891),
(477,4426,'Medic',NULL,now(),NULL,70892),
(478,6201,'The Smothers Brothers Show (1975)',NULL,now(),NULL,70893),
(479,6199,'The Smothers Brothers Comedy Hour (1988)',NULL,now(),NULL,70894),
(480,4030,'Jackson 5ive',NULL,now(),NULL,70895),
(481,612,'Beyond the Glory',NULL,now(),NULL,70896),
(482,3128,'Common Law',NULL,now(),NULL,70898),
(483,613,'Father Dear Father',NULL,now(),NULL,70899),
(484,615,'Great Pretenders',NULL,now(),NULL,70901),
(485,616,'Looking for Love: Bachelorettes in Alaska',NULL,now(),NULL,70902),
(486,7143,'This Week in Baseball',NULL,now(),NULL,70904),
(487,3150,'Coupling (US)',NULL,now(),NULL,70905),
(488,5075,'Ryan\'s Hope',NULL,now(),NULL,70906),
(489,22408,'The John Davidson Show',NULL,now(),NULL,70907),
(490,5762,'The Fall and Rise of Reginald Perrin',NULL,now(),NULL,70908),
(491,6015,'The New Adventures of Batman',NULL,now(),NULL,70909),
(492,617,'Legends of the Super-Heroes',NULL,now(),NULL,70910),
(493,5468,'Tarzan and the Super 7',NULL,now(),NULL,70911),
(494,4184,'Larry King Live',NULL,now(),NULL,70912),
(495,5405,'SuperFriends: The Legendary Super Powers Show',NULL,now(),NULL,70913),
(496,6227,'The Super Powers Team: Galactic Guardians',NULL,now(),NULL,70914),
(497,18289,'The Last Precinct',NULL,now(),NULL,70915),
(498,3177,'Crossroads (1992)',NULL,now(),NULL,70916);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(499,4721,'One On One',NULL,now(),NULL,70919),
(500,618,'Animal Crackers',NULL,now(),NULL,70920),
(501,9300,'The Estate Agents',NULL,now(),NULL,70921),
(502,619,'The Hunted',NULL,now(),NULL,70923),
(503,5258,'Sorority Life',NULL,now(),NULL,70924),
(504,22124,'History vs. Hollywood',NULL,now(),NULL,70925),
(505,11696,'Hard Copy',NULL,now(),NULL,70926),
(506,620,'Celeb',NULL,now(),NULL,70927),
(507,9051,'There\'s a Lot of It About',NULL,now(),NULL,70928),
(508,6975,'Get the Picture',NULL,now(),NULL,70929),
(509,5476,'Team Knight Rider',NULL,now(),NULL,70930),
(510,7076,'History IQ',NULL,now(),NULL,70931),
(511,23876,'Pick Your Brain',NULL,now(),NULL,70932),
(512,622,'What Would You Do?',NULL,now(),NULL,70933),
(513,8384,'Take a Letter Mr Jones',NULL,now(),NULL,70935),
(514,623,'Terrahawks',NULL,now(),NULL,70936),
(515,624,'Mike, Lu & Og',NULL,now(),NULL,70937),
(516,625,'The Andy Williams Show (1962)',NULL,now(),NULL,70938),
(517,626,'The Andy Williams Show (1969)',NULL,now(),NULL,70939),
(518,628,'Rockface',NULL,now(),NULL,70941),
(519,4785,'Paths to Freedom',NULL,now(),NULL,70942),
(520,9805,'Saturday Live (1996)',NULL,now(),NULL,70944),
(521,2937,'Bump in the Night',NULL,now(),NULL,70945),
(522,4844,'Point Blank',NULL,now(),NULL,70946),
(523,6014,'The New Addams Family',NULL,now(),NULL,70947),
(524,629,'Movie House',NULL,now(),NULL,70949),
(525,95,'Bare Essence',NULL,now(),NULL,70950),
(526,19000,'Manhunt (1970)',NULL,now(),NULL,70952),
(527,632,'Look at the State We\'re In!',NULL,now(),NULL,70954),
(528,8661,'Broaden Your Mind',NULL,now(),NULL,70955),
(529,633,'At Last the 1948 Show',NULL,now(),NULL,70956),
(530,9025,'Sez Les',NULL,now(),NULL,70957),
(531,4463,'Mickey Mouse Works',NULL,now(),NULL,70958),
(532,636,'Old Tom',NULL,now(),NULL,70960),
(533,3784,'Hannay',NULL,now(),NULL,70961),
(534,4249,'Linc\'s',NULL,now(),NULL,70962),
(535,637,'Sam and Max Freelance Police',NULL,now(),NULL,70963),
(536,6310,'The Wuzzles',NULL,now(),NULL,70964),
(537,638,'Totally in Tune',NULL,now(),NULL,70965),
(538,639,'Sir Bernard\'s Stately Homes',NULL,now(),NULL,70966),
(539,640,'Lady Chatterley\'s Stories',NULL,now(),NULL,70968),
(540,2749,'Bedtime Stories',NULL,now(),NULL,70969),
(541,7219,'This Morning',NULL,now(),NULL,70970),
(542,641,'Castle Haven',NULL,now(),NULL,70971),
(543,3521,'Fast Track',NULL,now(),NULL,70972),
(544,645,'TVography',NULL,now(),NULL,70974),
(545,6447,'TV Tales',NULL,now(),NULL,70975),
(546,6955,'Family Double Dare',NULL,now(),NULL,70977),
(547,3933,'Hyperion Bay',NULL,now(),NULL,70979),
(548,96,'Lillie',NULL,now(),NULL,70980),
(549,649,'The Legacy of Reginald Perrin',NULL,now(),NULL,70981),
(550,650,'Edward and Mrs Simpson',NULL,now(),NULL,70982),
(551,8962,'Rich Tea and Sympathy',NULL,now(),NULL,70983),
(552,12374,'Jemima Shore Investigates',NULL,now(),NULL,70984),
(553,8736,'Holding the Fort',NULL,now(),NULL,70985),
(554,5666,'The Cabbage Patch',NULL,now(),NULL,70986),
(555,651,'Psychos',NULL,now(),NULL,70987),
(556,23184,'Disraeli',NULL,now(),NULL,70989),
(557,5285,'Special Squad',NULL,now(),NULL,70990),
(558,19879,'Elizabeth R',NULL,now(),NULL,70991),
(559,7661,'Cop Shop',NULL,now(),NULL,70992),
(560,13230,'It\'s Dark Outside',NULL,now(),NULL,70993),
(561,2699,'Baddiel & Skinner Unplanned',NULL,now(),NULL,70995),
(562,8834,'The Other \' Arf',NULL,now(),NULL,70996),
(563,653,'The Wind in the Willows',NULL,now(),NULL,70997),
(564,97,'The Adventurer',NULL,now(),NULL,70998),
(565,3346,'Doomwatch',NULL,now(),NULL,71001),
(566,2826,'Bless Me Father',NULL,now(),NULL,71002),
(567,8723,'Hell\'s Bells',NULL,now(),NULL,71003),
(568,8316,'Life Begins At Forty',NULL,now(),NULL,71004),
(569,8431,'Oh Father!',NULL,now(),NULL,71005),
(570,8508,'Oh Brother!',NULL,now(),NULL,71006),
(571,9040,'Sorry I\'m Single',NULL,now(),NULL,71007),
(572,8652,'Blanding\'s Castle',NULL,now(),NULL,71008),
(573,3289,'Department S',NULL,now(),NULL,71009),
(574,654,'All Gas and Gaiters',NULL,now(),NULL,71010),
(575,8640,'The Bed-Sit Girl',NULL,now(),NULL,71011),
(576,6305,'The World of Wooster',NULL,now(),NULL,71012),
(577,6083,'The Pebbles and Bamm-Bamm Show',NULL,now(),NULL,71013),
(578,4139,'Kirby: Right Back at Ya!',NULL,now(),NULL,71014),
(579,4750,'Out of the Unknown',NULL,now(),NULL,71015),
(580,655,'I Bet You Will',NULL,now(),NULL,71016),
(581,657,'WXO Wrestling',NULL,now(),NULL,71017),
(582,8714,'Good Girl',NULL,now(),NULL,71018),
(583,20400,'Ruby (2002)',NULL,now(),NULL,71019),
(584,5591,'The Avengers',NULL,now(),NULL,71020),
(585,658,'Bramwell',NULL,now(),NULL,71021),
(586,659,'Hot Metal',NULL,now(),NULL,71022),
(587,660,'Strange Report',NULL,now(),NULL,71023),
(588,661,'Young Lions',NULL,now(),NULL,71024),
(589,662,'AWF Warriors of Wrestling',NULL,now(),NULL,71025),
(590,663,'Junior High',NULL,now(),NULL,71026),
(591,8977,'Rutland Weekend Television',NULL,now(),NULL,71027),
(592,2725,'BattleBots',NULL,now(),NULL,71028),
(593,98,'Capitol',NULL,now(),NULL,71029),
(594,665,'The Huggabug Club',NULL,now(),NULL,71030),
(595,19227,'Madame\'s Place',NULL,now(),NULL,71031),
(596,5482,'Teenage Mutant Ninja Turtles (2003)',NULL,now(),NULL,71032),
(597,666,'Check It Out',NULL,now(),NULL,71033),
(598,2600,'American High',NULL,now(),NULL,71034),
(599,2511,'AfterMASH',NULL,now(),NULL,71036),
(600,99,'Here Come the Brides',NULL,now(),NULL,71037),
(601,8572,'The Pulse',NULL,now(),NULL,71038),
(602,23955,'Erotic Confessions',NULL,now(),NULL,71039),
(603,3681,'Gidget',NULL,now(),NULL,71041),
(604,2905,'Bremner, Bird and Fortune',NULL,now(),NULL,71042),
(605,10147,'Underworld',NULL,now(),NULL,71043),
(606,4619,'Nanny and the Professor',NULL,now(),NULL,71044),
(607,668,'Poldark',NULL,now(),NULL,71045),
(608,669,'I, Claudius',NULL,now(),NULL,71046),
(609,670,'Other People\'s Children',NULL,now(),NULL,71048),
(610,12227,'Code R',NULL,now(),NULL,71049),
(611,6501,'Vegas',NULL,now(),NULL,71051),
(612,5371,'Street Justice',NULL,now(),NULL,71052),
(613,3544,'Filthy, Rich and Catflap',NULL,now(),NULL,71054),
(614,671,'Exposure',NULL,now(),NULL,71055),
(615,672,'Rank',NULL,now(),NULL,71056),
(616,16644,'No Time for Sergeants',NULL,now(),NULL,71057),
(617,10664,'Riker',NULL,now(),NULL,71058),
(618,674,'Meet the Marks',NULL,now(),NULL,71059),
(619,675,'Forever',NULL,now(),NULL,71060),
(620,4305,'Loving',NULL,now(),NULL,71061),
(621,6549,'WCW Saturday Night',NULL,now(),NULL,71062),
(622,6099,'The Pleasure Zone',NULL,now(),NULL,71063),
(623,100,'Love of Life',NULL,now(),NULL,71064),
(624,676,'My Dad\'s the Prime Minister',NULL,now(),NULL,71065),
(625,4783,'Passions',NULL,now(),NULL,71066),
(626,677,'I\'m a Celebrity: Get Me Out of Here!',NULL,now(),NULL,71067),
(627,12738,'Play for Tomorrow',NULL,now(),NULL,71068),
(628,5395,'Sunset Beach',NULL,now(),NULL,71069),
(629,7714,'Glenview High',NULL,now(),NULL,71070),
(630,16948,'Condo',NULL,now(),NULL,71071),
(631,16209,'Hearts Are Wild',NULL,now(),NULL,71073),
(632,5929,'The Lenny Henry Show (1995)',NULL,now(),NULL,71074),
(633,678,'Believe Nothing',NULL,now(),NULL,71076),
(634,679,'Hot Wheels',NULL,now(),NULL,71078),
(635,6327,'The Zoo Gang',NULL,now(),NULL,71079),
(636,680,'Lenny Henry in Pieces',NULL,now(),NULL,71080),
(637,5613,'The Best Sex Ever',NULL,now(),NULL,71081),
(638,21240,'Thrills',NULL,now(),NULL,71082),
(639,681,'National Velvet',NULL,now(),NULL,71083),
(640,682,'The Head',NULL,now(),NULL,71084),
(641,4344,'Major Dad',NULL,now(),NULL,71085),
(642,683,'Joe 90',NULL,now(),NULL,71086),
(643,685,'WCW WorldWide',NULL,now(),NULL,71091),
(644,6545,'WCW Clash of the Champions',NULL,now(),NULL,71092),
(645,16699,'Harris and Company',NULL,now(),NULL,71093),
(646,3253,'Da Vinci\'s Inquest',NULL,now(),NULL,71094),
(647,5195,'Sifl & Olly Show',NULL,now(),NULL,71095),
(648,3980,'Intimate Sessions',NULL,now(),NULL,71096),
(649,687,'Walking with Dinosaurs',NULL,now(),NULL,71097),
(650,5637,'The Book of Pooh',NULL,now(),NULL,71099),
(651,5946,'The Losers',NULL,now(),NULL,71100),
(652,688,'The Secret Service',NULL,now(),NULL,71101),
(653,689,'Kratts\' Creatures',NULL,now(),NULL,71102),
(654,690,'Timeline',NULL,now(),NULL,71103),
(655,691,'In Search of Ancient Ireland',NULL,now(),NULL,71104),
(656,20517,'In the Footsteps of Alexander the Great',NULL,now(),NULL,71105),
(657,21207,'Conquistadors',NULL,now(),NULL,71106),
(658,3569,'Fonz and the Happy Days Gang',NULL,now(),NULL,71109),
(659,692,'Let\'s Bowl',NULL,now(),NULL,71110),
(660,4591,'My Parents Are Aliens',NULL,now(),NULL,71111),
(661,16795,'The Cop & The Kid',NULL,now(),NULL,71112),
(662,16679,'Sierra',NULL,now(),NULL,71113),
(663,4692,'Now and Again',NULL,now(),NULL,71115),
(664,693,'Joe Bash',NULL,now(),NULL,71116),
(665,4434,'Meet My Folks',NULL,now(),NULL,71118),
(666,4269,'Lizzie McGuire',NULL,now(),NULL,71119),
(667,694,'The Ranch',NULL,now(),NULL,71120),
(668,695,'The House That Jack Built',NULL,now(),NULL,71121),
(669,22217,'Beyond with James Van Praagh',NULL,now(),NULL,71122),
(670,696,'30 Seconds to Fame',NULL,now(),NULL,71123),
(671,19183,'The Baxters',NULL,now(),NULL,71124),
(672,4726,'One World',NULL,now(),NULL,71125),
(673,697,'Rockpoint P.D.',NULL,now(),NULL,71127),
(674,698,'Littl\' Bits',NULL,now(),NULL,71128),
(675,11080,'The Intrepids',NULL,now(),NULL,71129),
(676,18111,'Weapon Masters',NULL,now(),NULL,81258),
(677,6667,'Xcalibur',NULL,now(),NULL,71132),
(678,101,'The Doctors (1963)',NULL,now(),NULL,71133),
(679,6889,'The People\'s Court',NULL,now(),NULL,71134),
(680,6228,'The Surreal Life',NULL,now(),NULL,71135),
(681,699,'One Hit Wonders',NULL,now(),NULL,71136),
(682,102,'Generations',NULL,now(),NULL,71137),
(683,6587,'While You Were Out',NULL,now(),NULL,71138),
(684,15838,'Stir Crazy',NULL,now(),NULL,71139),
(685,9133,'Boyz Unlimited',NULL,now(),NULL,71140),
(686,9802,'S and M',NULL,now(),NULL,71141),
(687,701,'Unwrapped',NULL,now(),NULL,71142),
(688,702,'The Biz',NULL,now(),NULL,71145),
(689,5030,'Robot Wars',NULL,now(),NULL,71146),
(690,7309,'You Lie Like A Dog',NULL,now(),NULL,71147),
(691,2557,'All Grown Up',NULL,now(),NULL,71148),
(692,5578,'The Anna Nicole Show',NULL,now(),NULL,71149),
(693,4285,'Lost at Home',NULL,now(),NULL,71150),
(694,703,'Up & Coming',NULL,now(),NULL,71151),
(695,704,'Freewheelers',NULL,now(),NULL,71152),
(696,8658,'The Bright Side',NULL,now(),NULL,71153),
(697,705,'The Greatest',NULL,now(),NULL,71155),
(698,103,'Arli$$',NULL,now(),NULL,71157),
(699,5445,'TailDaters',NULL,now(),NULL,71159),
(700,5408,'Superman (1988)',NULL,now(),NULL,71161),
(701,706,'The Chinese Detective',NULL,now(),NULL,71163),
(702,707,'Being April',NULL,now(),NULL,71164),
(703,5246,'Somerset',NULL,now(),NULL,71165),
(704,7444,'Wide Awake Club',NULL,now(),NULL,71166),
(705,5132,'Second Chances',NULL,now(),NULL,71167),
(706,6666,'X-Men: Evolution',NULL,now(),NULL,71168),
(707,23209,'No. 73',NULL,now(),NULL,71171),
(708,9011,'Grand Ole Opry',NULL,now(),NULL,71174),
(709,2778,'Beware of Dog',NULL,now(),NULL,71175),
(710,711,'Making the Cut',NULL,now(),NULL,71176),
(711,4179,'Land of the Lost',NULL,now(),NULL,71177),
(712,712,'River City',NULL,now(),NULL,71178),
(713,3325,'Dixon of Dock Green',NULL,now(),NULL,71180),
(714,4464,'Middle Ages',NULL,now(),NULL,71181),
(715,2902,'Breakfast with Frost',NULL,now(),NULL,71182),
(716,5842,'The Hardy Boys/Nancy Drew Mysteries',NULL,now(),NULL,71183),
(717,714,'Popstars: The Rivals',NULL,now(),NULL,71184),
(718,5594,'The Bachelorette',NULL,now(),NULL,71187),
(719,717,'Fist of Fun',NULL,now(),NULL,71188),
(720,2560,'All is Forgiven',NULL,now(),NULL,71189),
(721,104,'Breaking Away',NULL,now(),NULL,71190),
(722,718,'The Toxic Crusaders',NULL,now(),NULL,71191),
(723,719,'Mad Jack the Pirate',NULL,now(),NULL,71192),
(724,720,'Invasion of the Hidden Cameras',NULL,now(),NULL,71193),
(725,3154,'Cover Me: Based on the True Life of an FBI Family',NULL,now(),NULL,71194),
(726,11038,'Twenty-One',NULL,now(),NULL,71195),
(727,5019,'Roar',NULL,now(),NULL,71196),
(728,3263,'Deadly Games',NULL,now(),NULL,71197),
(729,5709,'The Cut (MTV)',NULL,now(),NULL,71198),
(730,16635,'The Slap Maxwell Story',NULL,now(),NULL,71199),
(731,3901,'Hotel',NULL,now(),NULL,71200),
(732,4843,'Please Don\'t Eat the Daisies',NULL,now(),NULL,71201),
(733,16808,'The Boys of Twilight',NULL,now(),NULL,71202),
(734,721,'The Brothers (1972)',NULL,now(),NULL,71203),
(735,722,'The Adventures of Spin and Marty',NULL,now(),NULL,71204),
(736,5799,'The Further Adventures of Spin and Marty',NULL,now(),NULL,71205),
(737,6022,'The New Adventures of Spin and Marty',NULL,now(),NULL,71206),
(738,105,'Bat Masterson',NULL,now(),NULL,71207),
(739,4331,'Maggie',NULL,now(),NULL,71208),
(740,6674,'Yancy Derringer',NULL,now(),NULL,71209),
(741,4207,'Lawman',NULL,now(),NULL,71211),
(742,723,'The Last Resort (1987)',NULL,now(),NULL,71212),
(743,2446,'240-Robert',NULL,now(),NULL,71213),
(744,3919,'How the West Was Won',NULL,now(),NULL,71214),
(745,724,'Play with Me Sesame',NULL,now(),NULL,71215),
(746,4273,'Logan\'s Run',NULL,now(),NULL,71216),
(747,2483,'Ace Crawford Private Eye',NULL,now(),NULL,71217),
(748,15709,'Treasury Men in Action',NULL,now(),NULL,71218),
(749,4349,'Making the Band 2',NULL,now(),NULL,71220),
(750,7960,'Judge Judy',NULL,now(),NULL,71221),
(751,13047,'Shop Til\' You Drop',NULL,now(),NULL,71222),
(752,3807,'Hawaii Five-O',NULL,now(),NULL,71223),
(753,6218,'The Streets of San Francisco',NULL,now(),NULL,71224),
(754,725,'Lost on Earth',NULL,now(),NULL,71225),
(755,726,'Love, Hate and Joy',NULL,now(),NULL,71227),
(756,6641,'Wonder Woman',NULL,now(),NULL,71228),
(757,4368,'Mannix',NULL,now(),NULL,71229),
(758,7986,'Fun House',NULL,now(),NULL,71230),
(759,10077,'Double Up',NULL,now(),NULL,71231),
(760,5305,'Sport Billy',NULL,now(),NULL,71233),
(761,727,'Contest Searchlight',NULL,now(),NULL,71234),
(762,728,'Heart and Soul',NULL,now(),NULL,71236),
(763,729,'LOL: Laugh out Loud',NULL,now(),NULL,71237),
(764,730,'Olliver\'s Adventures',NULL,now(),NULL,71238),
(765,4086,'Jungle Jim',NULL,now(),NULL,71239),
(766,4381,'Martial Law',NULL,now(),NULL,71240),
(767,106,'Sigmund and the Sea Monsters',NULL,now(),NULL,71241),
(768,12368,'G-Force: Gaurdians of Space',NULL,now(),NULL,71242),
(769,18999,'The Man Called X',NULL,now(),NULL,71243),
(770,731,'Comic Remix',NULL,now(),NULL,71244),
(771,7717,'Class Of \'74',NULL,now(),NULL,71246),
(772,732,'The Young Doctors',NULL,now(),NULL,71247),
(773,4271,'Local Heroes',NULL,now(),NULL,71248),
(774,5314,'Square Pegs',NULL,now(),NULL,71249),
(775,107,'Buddies',NULL,now(),NULL,71250),
(776,6481,'Union Square',NULL,now(),NULL,71251),
(777,733,'Tom Snyder',NULL,now(),NULL,71252),
(778,17774,'Married: The First Year',NULL,now(),NULL,71253),
(779,6005,'The Naked Truth',NULL,now(),NULL,71255),
(780,5714,'The Daily Show',NULL,now(),NULL,71256),
(781,12882,'Harbor Command',NULL,now(),NULL,71257),
(782,7465,'Kaiser Aluminum Hour',NULL,now(),NULL,71258),
(783,2907,'Brewster Place',NULL,now(),NULL,71260),
(784,3250,'Davey and Goliath',NULL,now(),NULL,71261),
(785,6420,'Tremors',NULL,now(),NULL,71262),
(786,734,'Tightrope',NULL,now(),NULL,71263),
(787,3445,'Endurance',NULL,now(),NULL,71264),
(788,6687,'You Rang, M\'Lord',NULL,now(),NULL,71265),
(789,735,'Out of the Blue (UK)',NULL,now(),NULL,71266),
(790,13416,'Mitch',NULL,now(),NULL,71267),
(791,736,'Noel\'s House Party',NULL,now(),NULL,71268),
(792,7250,'13 East',NULL,now(),NULL,71270),
(793,2435,'106 and Park',NULL,now(),NULL,71271),
(794,10465,'Mr. and Mrs.',NULL,now(),NULL,71272),
(795,15859,'Sure as Fate',NULL,now(),NULL,71273),
(796,738,'Me and My Girl',NULL,now(),NULL,71274),
(797,4007,'It\'s Garry Shandling\'s Show',NULL,now(),NULL,71275),
(798,5614,'The Betty Hutton Show',NULL,now(),NULL,71276),
(799,109,'Stoney Burke',NULL,now(),NULL,71277),
(800,743,'The Irresponsible Captain Tylor',NULL,now(),NULL,71279),
(801,745,'Tribes',NULL,now(),NULL,71280),
(802,746,'Commander Cork',NULL,now(),NULL,71284),
(803,747,'Maktar',NULL,now(),NULL,71285),
(804,748,'Longhair & Doubledome',NULL,now(),NULL,71286),
(805,749,'Comic Groove',NULL,now(),NULL,71287),
(806,5971,'The Martin Short Show',NULL,now(),NULL,71288),
(807,750,'Coronado 9',NULL,now(),NULL,71289),
(808,7242,'The 1/2 Hour Comedy Hour',NULL,now(),NULL,71290),
(809,751,'Riding High',NULL,now(),NULL,71291),
(810,3864,'Hiller and Diller',NULL,now(),NULL,71293),
(811,14687,'The Baseball Bunch (2005)',NULL,now(),NULL,71294),
(812,752,'Bowler',NULL,now(),NULL,71296),
(813,12002,'Russell Coight\'s All Aussie Adventures',NULL,now(),NULL,71297),
(814,6591,'Whiz Kids',NULL,now(),NULL,71298),
(815,755,'Date with the Angels',NULL,now(),NULL,71299),
(816,6369,'Toad Patrol',NULL,now(),NULL,71300),
(817,2695,'Baby Looney Tunes',NULL,now(),NULL,71301),
(818,10440,'It\'s A Knockout',NULL,now(),NULL,71302),
(819,2906,'Bret Maverick',NULL,now(),NULL,71304),
(820,3962,'In Search of.....',NULL,now(),NULL,71309),
(821,3583,'For Love or Money',NULL,now(),NULL,71310),
(822,758,'The Saturday Night Armistice',NULL,now(),NULL,71311),
(823,4382,'Martian Successor Nadesico',NULL,now(),NULL,71313),
(824,3599,'Fraud Squad',NULL,now(),NULL,71315),
(825,4717,'One Day at a Time',NULL,now(),NULL,71316),
(826,759,'Grady',NULL,now(),NULL,71317),
(827,760,'WMAC Masters',NULL,now(),NULL,71318),
(828,7377,'Wally\'s Workshop',NULL,now(),NULL,71320),
(829,13038,'Blockbusters',NULL,now(),NULL,71321),
(830,18920,'Mindreaders',NULL,now(),NULL,71322),
(831,7734,'Play Your Hunch',NULL,now(),NULL,71323),
(832,761,'Do Not Adjust Your Set',NULL,now(),NULL,71324),
(833,5866,'The Huntress',NULL,now(),NULL,71325),
(834,2432,'100 Centre Street',NULL,now(),NULL,71327),
(835,22986,'Children\'s Ward',NULL,now(),NULL,71329),
(836,762,'Saturday Supercade',NULL,now(),NULL,71331),
(837,3833,'Henry\'s World',NULL,now(),NULL,71332),
(838,11584,'Leave It To The Girls',NULL,now(),NULL,71333),
(839,763,'Legends Of The Hidden Temple',NULL,now(),NULL,71335),
(840,13417,'Smuggler',NULL,now(),NULL,71336),
(841,3308,'Dick Turpin',NULL,now(),NULL,71337),
(842,8620,'Tom, Dick and Harriet',NULL,now(),NULL,71338),
(843,2523,'Airline',NULL,now(),NULL,71339),
(844,5861,'The House of Eliott',NULL,now(),NULL,71340),
(845,764,'A for Andromeda',NULL,now(),NULL,71341),
(846,7466,'Kangaroo Creek Gang',NULL,now(),NULL,71342),
(847,766,'Tracey McBean',NULL,now(),NULL,71343),
(848,767,'Phoenix Nights',NULL,now(),NULL,71344),
(849,8916,'Lame Ducks',NULL,now(),NULL,71345),
(850,6499,'Van Dyke and Company',NULL,now(),NULL,71347),
(851,768,'Ocean Avenue',NULL,now(),NULL,71348),
(852,10448,'Laugh Line',NULL,now(),NULL,71349),
(853,769,'Fort Boyard',NULL,now(),NULL,71350),
(854,770,'The Paradise Club',NULL,now(),NULL,71352),
(855,12140,'The Jewel in the Crown',NULL,now(),NULL,71353),
(856,19704,'Top Ten',NULL,now(),NULL,71354),
(857,5737,'The Drew Carey Show',NULL,now(),NULL,71355),
(858,10247,'Pantomime Quiz',NULL,now(),NULL,71356),
(859,6475,'Under One Roof (2002)',NULL,now(),NULL,71357),
(860,771,'2000 Acres Of Sky',NULL,now(),NULL,71358),
(861,10449,'Liar',NULL,now(),NULL,71360),
(862,9901,'Mama',NULL,now(),NULL,71365),
(863,10133,'Jamie',NULL,now(),NULL,71366),
(864,110,'Rituals',NULL,now(),NULL,71367),
(865,772,'Banzai',NULL,now(),NULL,71368),
(866,4024,'Jack\'s Place',NULL,now(),NULL,71369),
(867,3903,'Hotel Malibu',NULL,now(),NULL,71370),
(868,13390,'Love For Lydia',NULL,now(),NULL,71372),
(869,6395,'Tour of Duty',NULL,now(),NULL,71373),
(870,773,'The Plucky Duck Show',NULL,now(),NULL,71374),
(871,6042,'The New Scooby-Doo Mysteries',NULL,now(),NULL,71375),
(872,775,'BattleTech',NULL,now(),NULL,71376),
(873,776,'Saturday Disney',NULL,now(),NULL,71377),
(874,11281,'Worst Day of My Life',NULL,now(),NULL,71378),
(875,3696,'Glory Days (1990)',NULL,now(),NULL,71379),
(876,5483,'Teen Angel',NULL,now(),NULL,71380),
(877,10216,'Pyramid',NULL,now(),NULL,71381),
(878,7323,'Your Face or Mine?',NULL,now(),NULL,71382),
(879,6615,'Will the Real Jerry Lewis Please Sit Down?',NULL,now(),NULL,71384),
(880,6449,'Twice in a Lifetime',NULL,now(),NULL,71385),
(881,3328,'Doctor, Doctor',NULL,now(),NULL,71386),
(882,3385,'Drexell\'s Class',NULL,now(),NULL,71387),
(883,12455,'Cowboy in Africa',NULL,now(),NULL,71388),
(884,777,'Wacko',NULL,now(),NULL,71389),
(885,778,'Moville Mysteries',NULL,now(),NULL,71390),
(886,6436,'Truth or Scare',NULL,now(),NULL,71391),
(887,8116,'Nightline',NULL,now(),NULL,71392),
(888,23084,'The McLaughlin Group',NULL,now(),NULL,71393),
(889,6336,'This Just In',NULL,now(),NULL,71395),
(890,6986,'Think Fast',NULL,now(),NULL,71396),
(891,12041,'This Week',NULL,now(),NULL,71397),
(892,4939,'Rainbow Brite',NULL,now(),NULL,71398),
(893,8308,'Loose Women',NULL,now(),NULL,71399),
(894,3816,'Head of the Class',NULL,now(),NULL,71400),
(895,780,'Bracken',NULL,now(),NULL,71401),
(896,111,'The City',NULL,now(),NULL,71402),
(897,17066,'Nurse',NULL,now(),NULL,71403),
(898,4608,'My Three Sons',NULL,now(),NULL,71404),
(899,781,'The Steven Banks Show',NULL,now(),NULL,71405),
(900,15330,'Say When!',NULL,now(),NULL,71406),
(901,6001,'The Music Scene',NULL,now(),NULL,71407),
(902,6972,'Soccer AM',NULL,now(),NULL,71409),
(903,10453,'Loves Like a Dog',NULL,now(),NULL,71412),
(904,5064,'Rowan & Martin\'s Laugh-In',NULL,now(),NULL,71413),
(905,782,'Brideshead Revisited',NULL,now(),NULL,71414),
(906,783,'Forensic Files',NULL,now(),NULL,71415),
(907,5961,'The Magic School Bus',NULL,now(),NULL,71416),
(908,7241,'1, 2, 3 Go',NULL,now(),NULL,71418),
(909,6464,'Ultimate Muscle: The Kinnikuman Legacy',NULL,now(),NULL,71419),
(910,15770,'Medallion Theatre',NULL,now(),NULL,71420),
(911,16897,'The Philip Morris Playhouse',NULL,now(),NULL,71422),
(912,15703,'Center Stage',NULL,now(),NULL,71423),
(913,5735,'The Dr. Phil Show',NULL,now(),NULL,71424),
(914,4251,'Line of Fire',NULL,now(),NULL,71425),
(915,788,'Maggie... At Home With',NULL,now(),NULL,71427),
(916,789,'Elvira\'s Movie Macabre',NULL,now(),NULL,71433),
(917,2474,'ABC Stage 67',NULL,now(),NULL,71434),
(918,3382,'Dream On',NULL,now(),NULL,71436),
(919,14864,'Pursuit',NULL,now(),NULL,71437),
(920,4569,'My Big Fat Greek Life',NULL,now(),NULL,71438),
(921,7372,'Wake, Rattle & Roll',NULL,now(),NULL,71439),
(922,16751,'Dink, The Little Dinosaur',NULL,now(),NULL,71440),
(923,13114,'The Barchester Chronicles',NULL,now(),NULL,71441),
(924,4731,'On the Spot',NULL,now(),NULL,71442),
(925,791,'I\'m a Celebrity, Get Me Out of Here! (US)',NULL,now(),NULL,71443),
(926,792,'Fushigi Yuugi',NULL,now(),NULL,71444),
(927,2467,'A.E.S. Hudson Street',NULL,now(),NULL,71446),
(928,18641,'American Chronicles',NULL,now(),NULL,71447),
(929,794,'Out of Sight',NULL,now(),NULL,71448),
(930,795,'Operation Junkyard',NULL,now(),NULL,71449),
(931,16998,'The Ted Knight Show',NULL,now(),NULL,71450),
(932,6023,'The New Adventures of Superman',NULL,now(),NULL,71451),
(933,3515,'Fantastic Voyage',NULL,now(),NULL,71452),
(934,6225,'The Superman/Aquaman Hour of Adventure',NULL,now(),NULL,71453),
(935,796,'Journey to the Center of the Earth',NULL,now(),NULL,71454),
(936,5402,'SuperFriends',NULL,now(),NULL,71455),
(937,4187,'Lassie\'s Rescue Rangers',NULL,now(),NULL,71456),
(938,7660,'Bellamy',NULL,now(),NULL,71457),
(939,798,'Maple Town Story',NULL,now(),NULL,71458),
(940,799,'Wedding Peach',NULL,now(),NULL,71459),
(941,800,'Bachelors Walk',NULL,now(),NULL,71460),
(942,13425,'On Home Ground',NULL,now(),NULL,71461),
(943,801,'Sharpe',NULL,now(),NULL,71462),
(944,19159,'Joey & Dad',NULL,now(),NULL,71463),
(945,15883,'The New Steve Allen Show',NULL,now(),NULL,71465),
(946,7420,'What a Country',NULL,now(),NULL,71467),
(947,10984,'Rosetti and Ryan',NULL,now(),NULL,71469),
(948,5615,'The Beverly Hillbillies',NULL,now(),NULL,71471),
(949,8487,'Gamesmaster',NULL,now(),NULL,71472),
(950,5235,'Snoops',NULL,now(),NULL,71473),
(951,802,'How to Survive a Marriage',NULL,now(),NULL,71474),
(952,6412,'Transformers: The Headmasters',NULL,now(),NULL,71475),
(953,2920,'Brothers',NULL,now(),NULL,71477),
(954,11736,'Glad Rags',NULL,now(),NULL,71478),
(955,4138,'Kipper',NULL,now(),NULL,71479),
(956,2492,'The New Adam 12',NULL,now(),NULL,71480),
(957,3014,'Celebrity Bootcamp',NULL,now(),NULL,71481),
(958,803,'American Gladiators',NULL,now(),NULL,71483),
(959,804,'Chiller',NULL,now(),NULL,71484),
(960,4451,'Mercy Point',NULL,now(),NULL,71485),
(961,3669,'Getting By',NULL,now(),NULL,71486),
(962,805,'McDonald\'s Young Entertainers',NULL,now(),NULL,71488),
(963,6546,'WCW Main Event',NULL,now(),NULL,71491),
(964,2470,'Aaahh!!! Real Monsters',NULL,now(),NULL,71492),
(965,3900,'Horatio Hornblower',NULL,now(),NULL,71493),
(966,8593,'Split Ends',NULL,now(),NULL,71494),
(967,5126,'Sean\'s Show',NULL,now(),NULL,71495),
(968,3303,'Diary',NULL,now(),NULL,71496),
(969,4555,'Murder Call',NULL,now(),NULL,71497),
(970,807,'Cannonball',NULL,now(),NULL,71498),
(971,8105,'Eddie Dodd',NULL,now(),NULL,71499),
(972,5790,'The Footy Show',NULL,now(),NULL,71500),
(973,5354,'Stingers',NULL,now(),NULL,71501),
(974,6586,'Where the Heart Is',NULL,now(),NULL,71502),
(975,5485,'Teen Titans',NULL,now(),NULL,71503),
(976,5901,'The Journey of Allen Strange',NULL,now(),NULL,71504),
(977,809,'The Eleventh Hour (2002)',NULL,now(),NULL,71505),
(978,10213,'Hidden Faces',NULL,now(),NULL,71506),
(979,810,'Sharman',NULL,now(),NULL,71507),
(980,4748,'Out of Order',NULL,now(),NULL,71508),
(981,12,'The Colbys',NULL,now(),NULL,71509),
(982,811,'Horace and Tina',NULL,now(),NULL,71510),
(983,812,'Custer\'s Last Stand-Up',NULL,now(),NULL,71512),
(984,813,'The Riordans',NULL,now(),NULL,71513),
(985,815,'Outside Edge',NULL,now(),NULL,71514),
(986,9491,'Josie',NULL,now(),NULL,71515),
(987,12788,'Runaways',NULL,now(),NULL,71516),
(988,7970,'Conflict (1994)',NULL,now(),NULL,71518),
(989,7094,'Good Day Live',NULL,now(),NULL,71519),
(990,10262,'The Hamptons',NULL,now(),NULL,71520),
(991,8106,'Hale & Pace',NULL,now(),NULL,71524),
(992,816,'Bootleg',NULL,now(),NULL,71525),
(993,5016,'Road Rovers',NULL,now(),NULL,71526),
(994,5474,'Taxi',NULL,now(),NULL,71527),
(995,2779,'Bewitched',NULL,now(),NULL,71528),
(996,3423,'Edge of Darkness',NULL,now(),NULL,71529);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(997,1091,'The Dick Clark Show',NULL,now(),NULL,71530),
(998,15499,'Hollywood Beat',NULL,now(),NULL,71533),
(999,817,'Love Thy Neighbour',NULL,now(),NULL,71534),
(1000,4600,'Mystery Hunters',NULL,now(),NULL,71537),
(1001,818,'Run, Buddy, Run',NULL,now(),NULL,71539),
(1002,3290,'Designing For The Sexes',NULL,now(),NULL,71540),
(1003,3038,'Charlie Lawrence',NULL,now(),NULL,71541),
(1004,2908,'Brighton Belles',NULL,now(),NULL,71542),
(1005,15774,'The Chevrolet Tele-Theatre',NULL,now(),NULL,71543),
(1006,16345,'Down You Go',NULL,now(),NULL,71544),
(1007,5444,'Taggart',NULL,now(),NULL,71545),
(1008,820,'The Sanford Arms',NULL,now(),NULL,71546),
(1009,4716,'One by One',NULL,now(),NULL,71547),
(1010,822,'Little Muppet Monsters',NULL,now(),NULL,71548),
(1011,13357,'The London Embassy',NULL,now(),NULL,71549),
(1012,4952,'Reba',NULL,now(),NULL,71550),
(1013,6413,'Transformers: Victory',NULL,now(),NULL,71551),
(1014,13,'The Betty White Show',NULL,now(),NULL,71553),
(1015,4945,'Randall and Hopkirk (Deceased) (1969)',NULL,now(),NULL,71555),
(1016,112,'That Was the Week That Was',NULL,now(),NULL,71556),
(1017,6680,'You Can\'t Take It With You',NULL,now(),NULL,71557),
(1018,15649,'Lucky Luke',NULL,now(),NULL,71558),
(1019,827,'Chaser Non-Stop News Network (CNNNN)',NULL,now(),NULL,71559),
(1020,7318,'You\'ve Been Framed',NULL,now(),NULL,71560),
(1021,113,'Shelley',NULL,now(),NULL,71561),
(1022,3140,'Coronation Street',NULL,now(),NULL,71565),
(1023,7520,'The Center',NULL,now(),NULL,71566),
(1024,9393,'High Hopes',NULL,now(),NULL,71567),
(1025,828,'Strange Paradise',NULL,now(),NULL,71568),
(1026,4813,'Petticoat Junction',NULL,now(),NULL,71569),
(1027,114,'Brookside',NULL,now(),NULL,71570),
(1028,829,'The Super',NULL,now(),NULL,71572),
(1029,15085,'The Phil Donahue Show',NULL,now(),NULL,71573),
(1030,4213,'Leave It to Beaver',NULL,now(),NULL,71575),
(1031,6153,'The Royal Family',NULL,now(),NULL,71577),
(1032,19211,'The Girls',NULL,now(),NULL,71578),
(1033,115,'Dolly',NULL,now(),NULL,71579),
(1034,4810,'Pet Psychic',NULL,now(),NULL,71580),
(1035,4171,'Ladies Man',NULL,now(),NULL,71581),
(1036,17542,'The Six O\' Clock Follies',NULL,now(),NULL,71582),
(1037,831,'Come Outside',NULL,now(),NULL,71583),
(1038,6441,'Turn A Gundam',NULL,now(),NULL,71585),
(1039,4354,'Malcolm & Eddie',NULL,now(),NULL,71586),
(1040,5628,'The Blue Knight',NULL,now(),NULL,71587),
(1041,4912,'Public Morals',NULL,now(),NULL,71588),
(1042,6324,'The Young Riders',NULL,now(),NULL,71589),
(1043,4152,'Kojak',NULL,now(),NULL,71590),
(1044,4011,'It Had to Be You',NULL,now(),NULL,71591),
(1045,832,'The Dreamstone',NULL,now(),NULL,71593),
(1046,17025,'One of the Boys',NULL,now(),NULL,71594),
(1047,833,'Marsupilami',NULL,now(),NULL,71595),
(1048,18205,'Paris 7000',NULL,now(),NULL,71596),
(1049,834,'Uh-oh!',NULL,now(),NULL,71597),
(1050,5535,'The Adventures of Ozzie & Harriet',NULL,now(),NULL,71598),
(1051,835,'Carry on Laughing',NULL,now(),NULL,71599),
(1052,836,'Sykes',NULL,now(),NULL,71600),
(1053,5983,'The Mike O\'Malley Show',NULL,now(),NULL,71601),
(1054,6401,'Trackside at...',NULL,now(),NULL,71604),
(1055,6463,'Ultimate Force',NULL,now(),NULL,71606),
(1056,837,'Ultraman: The Ultimate Hero',NULL,now(),NULL,71607),
(1057,4390,'Match Game 90/91',NULL,now(),NULL,71608),
(1058,5611,'The Best Damn Sports Show Period',NULL,now(),NULL,71609),
(1059,6456,'Two of a Kind',NULL,now(),NULL,71610),
(1060,838,'Theodore Tugboat',NULL,now(),NULL,71612),
(1061,3174,'Crossbow',NULL,now(),NULL,71614),
(1062,4793,'Pee-Wee\'s Playhouse',NULL,now(),NULL,71615),
(1063,840,'Korg: 70,000 B.C.',NULL,now(),NULL,71616),
(1064,14,'The Andy Williams Show',NULL,now(),NULL,71617),
(1065,842,'Hyper Police',NULL,now(),NULL,71618),
(1066,3490,'Faking It',NULL,now(),NULL,71619),
(1067,4542,'Mr. Rhodes',NULL,now(),NULL,71621),
(1068,4241,'Life With Elizabeth',NULL,now(),NULL,71622),
(1069,843,'Battle Force: Andromeda',NULL,now(),NULL,71623),
(1070,3324,'Disneyland',NULL,now(),NULL,71625),
(1071,6891,'Kukla, Fran and Ollie',NULL,now(),NULL,71626),
(1072,5813,'The George Wendt Show',NULL,now(),NULL,71627),
(1073,4075,'Jonny Quest (1986)',NULL,now(),NULL,71629),
(1074,9964,'Person to Person',NULL,now(),NULL,71630),
(1075,9813,'Jack Hanna\'s Animal Adventures',NULL,now(),NULL,71631),
(1076,4707,'Oh Baby',NULL,now(),NULL,71632),
(1077,845,'Magic Knight Rayearth',NULL,now(),NULL,71634),
(1078,6037,'The New Monkees',NULL,now(),NULL,71636),
(1079,18193,'The American Experience',NULL,now(),NULL,71638),
(1080,7308,'You Don\'t Say!',NULL,now(),NULL,71639),
(1081,12464,'Cross-Wits',NULL,now(),NULL,71640),
(1082,18856,'Bullseye',NULL,now(),NULL,71643),
(1083,18908,'Scrabble',NULL,now(),NULL,71644),
(1084,18897,'Blackout',NULL,now(),NULL,71645),
(1085,2674,'Austin City Limits',NULL,now(),NULL,71649),
(1086,846,'The Righteous Apples',NULL,now(),NULL,71650),
(1087,3881,'Holmes and Yoyo',NULL,now(),NULL,71651),
(1088,3707,'Gomer Pyle, U.S.M.C.',NULL,now(),NULL,71653),
(1089,116,'Ben Casey',NULL,now(),NULL,71655),
(1090,849,'Peep Show',NULL,now(),NULL,71656),
(1091,5814,'The Ghost and Mrs. Muir',NULL,now(),NULL,71657),
(1092,5733,'The Doris Day Show',NULL,now(),NULL,71658),
(1093,850,'Runaway Bay',NULL,now(),NULL,71659),
(1094,851,'My Living Doll',NULL,now(),NULL,71660),
(1095,853,'Roman Holidays',NULL,now(),NULL,71661),
(1096,854,'Avenger Penguins',NULL,now(),NULL,71662),
(1097,4389,'Match Game 73',NULL,now(),NULL,71664),
(1098,4769,'Pandamonium',NULL,now(),NULL,71666),
(1099,6430,'Trollkins',NULL,now(),NULL,71667),
(1100,5560,'The All-New SuperFriends Hour',NULL,now(),NULL,71668),
(1101,6027,'The New Bill Cosby Show',NULL,now(),NULL,71670),
(1102,6304,'The World\'s Greatest SuperFriends',NULL,now(),NULL,71671),
(1103,117,'Match Game PM',NULL,now(),NULL,71672),
(1104,856,'Vets in Practice',NULL,now(),NULL,71673),
(1105,7148,'Family Feud',NULL,now(),NULL,71674),
(1106,3431,'elimiDATE',NULL,now(),NULL,71675),
(1107,11776,'The New Family Feud',NULL,now(),NULL,71676),
(1108,16690,'Sugar Time!',NULL,now(),NULL,71677),
(1109,15698,'The Steve Allen Show',NULL,now(),NULL,71678),
(1110,11476,'Punishment',NULL,now(),NULL,71680),
(1111,860,'TV Land Legends: The 60 Minutes Interviews',NULL,now(),NULL,71684),
(1112,3100,'Codename: Kids Next Door',NULL,now(),NULL,71685),
(1113,861,'Raw Toonage',NULL,now(),NULL,71686),
(1114,862,'Inhumanoids',NULL,now(),NULL,71687),
(1115,863,'A Flame in the Wind',NULL,now(),NULL,71688),
(1116,864,'Blobheads',NULL,now(),NULL,71689),
(1117,11037,'I\'ve Got a Secret',NULL,now(),NULL,71690),
(1118,11069,'Big Eddie',NULL,now(),NULL,71692),
(1119,10307,'First Love',NULL,now(),NULL,71693),
(1120,5472,'Tattletales',NULL,now(),NULL,71694),
(1121,7304,'You Bet Your Life',NULL,now(),NULL,71696),
(1122,6070,'The Outer Limits',NULL,now(),NULL,71699),
(1123,867,'Bless This House (UK)',NULL,now(),NULL,71701),
(1124,5078,'S.W.A.T.',NULL,now(),NULL,71702),
(1125,6143,'The Rob Nelson Show',NULL,now(),NULL,71703),
(1126,8302,'Blacke\'s Magic',NULL,now(),NULL,71704),
(1127,14747,'Adventures On Kythera',NULL,now(),NULL,71705),
(1128,8959,'Relative Strangers',NULL,now(),NULL,71706),
(1129,3164,'Crazy Like A Fox',NULL,now(),NULL,71707),
(1130,5679,'The Charmings',NULL,now(),NULL,71709),
(1131,2793,'Big Brother Jake',NULL,now(),NULL,71711),
(1132,2451,'3 South',NULL,now(),NULL,71712),
(1133,4821,'Phil of the Future',NULL,now(),NULL,71714),
(1134,868,'El Hazard',NULL,now(),NULL,71715),
(1135,4408,'Mayberry R.F.D.',NULL,now(),NULL,71716),
(1136,18893,'Winning Lines',NULL,now(),NULL,71717),
(1137,5499,'That\'s Incredible',NULL,now(),NULL,71718),
(1138,869,'Inside TV Land',NULL,now(),NULL,71719),
(1139,6602,'Widows',NULL,now(),NULL,71720),
(1140,2589,'America\'s Next Top Model',NULL,now(),NULL,71721),
(1141,871,'Bad Cop, Bad Cop',NULL,now(),NULL,71722),
(1142,872,'The River',NULL,now(),NULL,71723),
(1143,3721,'Goodnight, Beantown',NULL,now(),NULL,71724),
(1144,17043,'The Entertainers',NULL,now(),NULL,71725),
(1145,118,'Love, Sidney',NULL,now(),NULL,71726),
(1146,8257,'The Tony Randall Show',NULL,now(),NULL,71727),
(1147,6301,'The Wonderful World of Disney (1997)',NULL,now(),NULL,71730),
(1148,5329,'Star Search (2003)',NULL,now(),NULL,71731),
(1149,12206,'One Man\'s Family',NULL,now(),NULL,71732),
(1150,12065,'Griff',NULL,now(),NULL,71733),
(1151,873,'Cutting It',NULL,now(),NULL,71734),
(1152,874,'Is Harry On The Boat',NULL,now(),NULL,71735),
(1153,5004,'Richie Brockelman, Private Eye',NULL,now(),NULL,71736),
(1154,119,'The Debbie Reynolds Show',NULL,now(),NULL,71738),
(1155,120,'All That Glitters',NULL,now(),NULL,71739),
(1156,878,'The Magic Roundabout',NULL,now(),NULL,71740),
(1157,879,'Max and Ruby',NULL,now(),NULL,71741),
(1158,880,'Boogiepop Phantom',NULL,now(),NULL,71742),
(1159,121,'Hail To the Chief',NULL,now(),NULL,71743),
(1160,3086,'Clerks',NULL,now(),NULL,71745),
(1161,5870,'The Incredible Hulk',NULL,now(),NULL,71746),
(1162,882,'tlc',NULL,now(),NULL,71747),
(1163,4298,'Love Hurts',NULL,now(),NULL,71748),
(1164,7292,'The Yeagers',NULL,now(),NULL,71749),
(1165,6617,'Win Ben Stein\'s Money',NULL,now(),NULL,71751),
(1166,885,'The Queens Nose',NULL,now(),NULL,71752),
(1167,3418,'EastEnders',NULL,now(),NULL,71753),
(1168,886,'The Wild House',NULL,now(),NULL,71754),
(1169,8982,'The Asphalt Jungle',NULL,now(),NULL,71755),
(1170,3005,'Casualty',NULL,now(),NULL,71756),
(1171,122,'Watching',NULL,now(),NULL,71757),
(1172,19092,'Storybook Squares',NULL,now(),NULL,71758),
(1173,8109,'All Star Blitz',NULL,now(),NULL,71759),
(1174,15,'The Courtship of Eddie\'s Father',NULL,now(),NULL,71761),
(1175,4824,'Pie in the Sky',NULL,now(),NULL,71762),
(1176,4699,'O2Be',NULL,now(),NULL,71763),
(1177,4301,'Lovejoy',NULL,now(),NULL,71764),
(1178,3854,'High School Reunion',NULL,now(),NULL,71765),
(1179,13033,'Chain Reaction',NULL,now(),NULL,71766),
(1180,3797,'Harsh Realm',NULL,now(),NULL,71768),
(1181,891,'Cutter to Houston',NULL,now(),NULL,71771),
(1182,892,'Born Sloppy',NULL,now(),NULL,71772),
(1183,4477,'Minor Adjustments',NULL,now(),NULL,71774),
(1184,894,'Popstars: Ireland',NULL,now(),NULL,71775),
(1185,4342,'Maison Ikkoku',NULL,now(),NULL,71776),
(1186,2918,'Brother\'s Keeper',NULL,now(),NULL,71777),
(1187,4575,'My Favorite Martian',NULL,now(),NULL,71778),
(1188,4528,'Mork & Mindy',NULL,now(),NULL,71779),
(1189,895,'The Vacant Lot',NULL,now(),NULL,71781),
(1190,9931,'TV Offal',NULL,now(),NULL,71782),
(1191,896,'Suspicion',NULL,now(),NULL,71783),
(1192,897,'Almost Live!',NULL,now(),NULL,71785),
(1193,6120,'The Real Adventures of Jonny Quest',NULL,now(),NULL,71786),
(1194,13455,'Love Connection',NULL,now(),NULL,71787),
(1195,5407,'Superman: The Animated Series',NULL,now(),NULL,71788),
(1196,5085,'Safe Harbor',NULL,now(),NULL,71789),
(1197,3182,'Crusade',NULL,now(),NULL,71790),
(1198,5515,'Thea',NULL,now(),NULL,71791),
(1199,898,'Aquila',NULL,now(),NULL,71792),
(1200,214,'Friday Night Videos',NULL,now(),NULL,71793),
(1201,4585,'My Life is a Sitcom',NULL,now(),NULL,71794),
(1202,4060,'Joe Millionaire',NULL,now(),NULL,71795),
(1203,899,'Crossroads (2003)',NULL,now(),NULL,71796),
(1204,16805,'The Bob Crane Show',NULL,now(),NULL,71797),
(1205,3457,'Essence of Emeril',NULL,now(),NULL,71798),
(1206,3437,'Emeril Live',NULL,now(),NULL,71799),
(1207,16352,'Pebble Mill at One',NULL,now(),NULL,71800),
(1208,12085,'Jim\'ll Fix It',NULL,now(),NULL,71801),
(1209,12229,'The Tomorrow Show',NULL,now(),NULL,71802),
(1210,11516,'Nationwide',NULL,now(),NULL,71803),
(1211,9729,'Pirates',NULL,now(),NULL,71806),
(1212,900,'Sesame Park',NULL,now(),NULL,71807),
(1213,901,'Groundling Marsh',NULL,now(),NULL,71809),
(1214,3078,'Clarissa Explains It All',NULL,now(),NULL,71810),
(1215,5083,'Sabrina the Animated Series',NULL,now(),NULL,71811),
(1216,6688,'You Wish',NULL,now(),NULL,71812),
(1217,2564,'All Saints',NULL,now(),NULL,71815),
(1218,13465,'The Law and Mr. Jones',NULL,now(),NULL,71816),
(1219,5501,'That\'s Life (1968)',NULL,now(),NULL,71818),
(1220,124,'Love is a Four Letter Word',NULL,now(),NULL,71819),
(1221,126,'Water Rats',NULL,now(),NULL,71820),
(1222,127,'Baby Boom',NULL,now(),NULL,71821),
(1223,8142,'I Married Dora',NULL,now(),NULL,71822),
(1224,15626,'How\'d They Do That?',NULL,now(),NULL,71823),
(1225,10695,'Perfect Match',NULL,now(),NULL,71825),
(1226,904,'Solid Gold Hits',NULL,now(),NULL,71826),
(1227,3928,'Hull High',NULL,now(),NULL,71827),
(1228,128,'The Leslie Uggams Show',NULL,now(),NULL,71828),
(1229,6200,'The Smothers Brothers Show',NULL,now(),NULL,71829),
(1230,905,'The Big Arvo',NULL,now(),NULL,71830),
(1231,20915,'The District Nurse',NULL,now(),NULL,71831),
(1232,9799,'The Roy Hudd Show',NULL,now(),NULL,71833),
(1233,11381,'Sword of Honour',NULL,now(),NULL,71834),
(1234,908,'Play of the Month',NULL,now(),NULL,71835),
(1235,909,'The Wednesday Play',NULL,now(),NULL,71836),
(1236,910,'In Loving Memory',NULL,now(),NULL,71837),
(1237,9495,'Just William',NULL,now(),NULL,71838),
(1238,911,'The Maxx',NULL,now(),NULL,71839),
(1239,3273,'Deepwater Black',NULL,now(),NULL,71840),
(1240,912,'The DuPont Show of the Week',NULL,now(),NULL,71841),
(1241,18851,'Now You See It',NULL,now(),NULL,71842),
(1242,915,'Star of the Family',NULL,now(),NULL,71846),
(1243,18464,'Half Nelson',NULL,now(),NULL,71847),
(1244,22266,'Newton\'s Apple',NULL,now(),NULL,71848),
(1245,16954,'Dance Fever',NULL,now(),NULL,71849),
(1246,16777,'Hanging In',NULL,now(),NULL,71850),
(1247,7392,'The Waverly Wonders',NULL,now(),NULL,71851),
(1248,3338,'Dolphin Cove',NULL,now(),NULL,71852),
(1249,4474,'Mile High',NULL,now(),NULL,71853),
(1250,3248,'Dave\'s World',NULL,now(),NULL,71854),
(1251,18886,'Split Second',NULL,now(),NULL,71855),
(1252,916,'Weinerville',NULL,now(),NULL,71856),
(1253,6299,'The Wonderful Wizard of Oz',NULL,now(),NULL,71857),
(1254,2675,'Austin Stories',NULL,now(),NULL,71858),
(1255,917,'Please Sir',NULL,now(),NULL,71860),
(1256,11947,'The Better Sex',NULL,now(),NULL,71861),
(1257,3031,'Chappelle\'s Show',NULL,now(),NULL,71862),
(1258,918,'Gadget Boy and Heather',NULL,now(),NULL,71864),
(1259,16731,'For Love and Honor',NULL,now(),NULL,71865),
(1260,5764,'The Family Man',NULL,now(),NULL,71866),
(1261,3254,'Davis Rules',NULL,now(),NULL,71868),
(1262,129,'Good Sports',NULL,now(),NULL,71869),
(1263,919,'Chuck Finn',NULL,now(),NULL,71870),
(1264,12625,'The Double Life of Henry Phyfe',NULL,now(),NULL,71871),
(1265,2785,'Big Apple',NULL,now(),NULL,71872),
(1266,920,'Having It Off',NULL,now(),NULL,71873),
(1267,6841,'Call For Help',NULL,now(),NULL,71874),
(1268,6169,'The Screen Savers',NULL,now(),NULL,71875),
(1269,16727,'Mama Malone',NULL,now(),NULL,71876),
(1270,15500,'The Oldest Rookie',NULL,now(),NULL,71877),
(1271,921,'Clochemerle',NULL,now(),NULL,71878),
(1272,922,'Thirty-Minute Theatre',NULL,now(),NULL,71879),
(1273,5554,'The Alan Young Show',NULL,now(),NULL,71881),
(1274,12757,'Five Fingers',NULL,now(),NULL,71882),
(1275,15699,'The NBC Comedy Hour',NULL,now(),NULL,71883),
(1276,9342,'Galloping Galaxies!',NULL,now(),NULL,71884),
(1277,7437,'Whizzkid\'s Guide',NULL,now(),NULL,71886),
(1278,9525,'The Kenneth Williams Show',NULL,now(),NULL,71888),
(1279,2887,'Boy Meets World',NULL,now(),NULL,71889),
(1280,3882,'Home and Away',NULL,now(),NULL,71890),
(1281,3306,'Dick and the Duchess',NULL,now(),NULL,71893),
(1282,923,'Starting Out',NULL,now(),NULL,71895),
(1283,6468,'Uncle Buck',NULL,now(),NULL,71896),
(1284,5279,'Space Patrol',NULL,now(),NULL,71897),
(1285,7327,'Your Lucky Clue',NULL,now(),NULL,71898),
(1286,5583,'The Arlene Francis Show',NULL,now(),NULL,71900),
(1287,7438,'Who\'s Whose',NULL,now(),NULL,71901),
(1288,15328,'The Lazarus Man',NULL,now(),NULL,71902),
(1289,16891,'Nash Airflyte Theatre',NULL,now(),NULL,71903),
(1290,925,'Chewin\' the Fat',NULL,now(),NULL,71904),
(1291,6084,'The People Next Door',NULL,now(),NULL,71905),
(1292,3085,'Cleopatra 2525',NULL,now(),NULL,71906),
(1293,131,'Normal Life',NULL,now(),NULL,71907),
(1294,4082,'Judging Amy',NULL,now(),NULL,71909),
(1295,4630,'Nearly Departed',NULL,now(),NULL,71910),
(1296,4836,'Platinum',NULL,now(),NULL,71912),
(1297,3112,'Colonial House',NULL,now(),NULL,71913),
(1298,4797,'Pensacola: Wings of Gold',NULL,now(),NULL,71914),
(1299,3243,'Dark Skies',NULL,now(),NULL,71915),
(1300,2622,'Anna and the King',NULL,now(),NULL,71916),
(1301,10291,'The Best of Everything',NULL,now(),NULL,71917),
(1302,10142,'Paradise Bay',NULL,now(),NULL,71918),
(1303,10178,'Portia Faces Life',NULL,now(),NULL,71919),
(1304,9959,'The Catlins',NULL,now(),NULL,71920),
(1305,132,'Beyond Chance',NULL,now(),NULL,71921),
(1306,8945,'Pig in the Middle',NULL,now(),NULL,71922),
(1307,8965,'Roll Over Beethoven',NULL,now(),NULL,71923),
(1308,6642,'Woof!',NULL,now(),NULL,71924),
(1309,928,'Bernard\'s Watch',NULL,now(),NULL,71925),
(1310,929,'Comedy Playhouse',NULL,now(),NULL,71927),
(1311,2713,'Baretta',NULL,now(),NULL,71928),
(1312,930,'Eco-Challenge',NULL,now(),NULL,71929),
(1313,3432,'Elimidate Deluxe',NULL,now(),NULL,71930),
(1314,17921,'The Andros Targets',NULL,now(),NULL,71931),
(1315,9186,'Calucci\'s Department',NULL,now(),NULL,71933),
(1316,931,'Code Name: Foxfire',NULL,now(),NULL,71934),
(1317,2770,'Bette',NULL,now(),NULL,71935),
(1318,4095,'Just Shoot Me',NULL,now(),NULL,71936),
(1319,2438,'18 Wheels of Justice',NULL,now(),NULL,71937),
(1320,932,'Detective School',NULL,now(),NULL,71938),
(1321,17314,'Dog and Cat',NULL,now(),NULL,71939),
(1322,17072,'Fortune Dane',NULL,now(),NULL,71940),
(1323,11649,'I Witness Video',NULL,now(),NULL,71941),
(1324,933,'Kingston: Confidential',NULL,now(),NULL,71942),
(1325,16665,'Mr. Deeds Goes to Town',NULL,now(),NULL,71943),
(1326,16680,'The Texas Wheelers',NULL,now(),NULL,71945),
(1327,4815,'Phantom 2040',NULL,now(),NULL,71946),
(1328,7333,'The XYY Man',NULL,now(),NULL,71947),
(1329,935,'Absolutely',NULL,now(),NULL,71948),
(1330,5556,'The Aldrich Family',NULL,now(),NULL,71951),
(1331,2661,'Astro Boy (1963)',NULL,now(),NULL,71952),
(1332,937,'History Bites',NULL,now(),NULL,71953),
(1333,11070,'Big Hawaii',NULL,now(),NULL,71955),
(1334,17873,'Concrete Cowboys',NULL,now(),NULL,71956),
(1335,938,'The Contender (1980)',NULL,now(),NULL,71957),
(1336,16760,'The Trials of Rosie O\'Neill',NULL,now(),NULL,71958),
(1337,16588,'Dante',NULL,now(),NULL,71959),
(1338,4798,'People Like Us',NULL,now(),NULL,71961),
(1339,6621,'Wings',NULL,now(),NULL,71962),
(1340,3104,'Cold Feet (US)',NULL,now(),NULL,71963),
(1341,6426,'Trinity',NULL,now(),NULL,71964),
(1342,4973,'Remote Control',NULL,now(),NULL,71965),
(1343,19593,'I Had Three Wives',NULL,now(),NULL,71967),
(1344,940,'Ivan the Terrible',NULL,now(),NULL,71968),
(1345,5309,'Sports Night',NULL,now(),NULL,71969),
(1346,11197,'Willo the Wisp',NULL,now(),NULL,71970),
(1347,10134,'The Kallikaks',NULL,now(),NULL,71971),
(1348,941,'Legmen',NULL,now(),NULL,71972),
(1349,3945,'I Love The 80\'s',NULL,now(),NULL,71973),
(1350,133,'The Paul Williams Comedy Show',NULL,now(),NULL,71974),
(1351,16708,'The McLean Stevenson Show',NULL,now(),NULL,71975),
(1352,5206,'Sir Arthur Conan Doyle\'s Sherlock Holmes',NULL,now(),NULL,71976),
(1353,942,'Hey Monie!',NULL,now(),NULL,71977),
(1354,943,'The Onedin Line',NULL,now(),NULL,71978),
(1355,6580,'When Things Were Rotten',NULL,now(),NULL,71979),
(1356,4497,'Mobile One',NULL,now(),NULL,71980),
(1357,9835,'Spin-Off',NULL,now(),NULL,71983),
(1358,2658,'As The World Turns',NULL,now(),NULL,71984),
(1359,17035,'Taboo',NULL,now(),NULL,71985),
(1360,3716,'Good Heavens',NULL,now(),NULL,71986),
(1361,4392,'Matlock Police',NULL,now(),NULL,71992),
(1362,7663,'The Link Men',NULL,now(),NULL,71993),
(1363,945,'Ryan',NULL,now(),NULL,71994),
(1364,2995,'Carson\'s Law',NULL,now(),NULL,71995),
(1365,10290,'The Tim Conway Comedy Hour',NULL,now(),NULL,71996),
(1366,13575,'Cram',NULL,now(),NULL,71997),
(1367,4055,'Jimmy Kimmel Live',NULL,now(),NULL,71998),
(1368,2607,'Amos \'n\' Andy',NULL,now(),NULL,71999),
(1369,8778,'Me Mammy',NULL,now(),NULL,72000),
(1370,946,'The Karen Dunbar Show',NULL,now(),NULL,72002),
(1371,13289,'Bat Out of Hell',NULL,now(),NULL,72004),
(1372,3842,'Hey Arnold!',NULL,now(),NULL,72005),
(1373,4642,'Never Mind the Quality, Feel the Width',NULL,now(),NULL,72007),
(1374,947,'Almost There!',NULL,now(),NULL,72008),
(1375,2931,'Buffy the Animated Series',NULL,now(),NULL,72009),
(1376,8409,'Attitudes',NULL,now(),NULL,72011),
(1377,4713,'Once and Again',NULL,now(),NULL,72012),
(1378,16687,'Fay',NULL,now(),NULL,72014),
(1379,948,'Pongwiffy',NULL,now(),NULL,72015),
(1380,134,'Loves Me, Loves Me Not',NULL,now(),NULL,72016),
(1381,18217,'Broadway Television Theatre',NULL,now(),NULL,72017),
(1382,18879,'Double Talk',NULL,now(),NULL,72018),
(1383,4332,'Maggie (1981)',NULL,now(),NULL,72020),
(1384,3917,'Howdy Doody',NULL,now(),NULL,72021),
(1385,3670,'Getting Personal',NULL,now(),NULL,72022),
(1386,950,'Escape from Experiment Island',NULL,now(),NULL,72024),
(1387,10501,'Here\'s Duffy',NULL,now(),NULL,72026),
(1388,6147,'The Rookies',NULL,now(),NULL,72027),
(1389,8430,'No Place Like Home (1983)',NULL,now(),NULL,72030),
(1390,16845,'Ethel and Albert',NULL,now(),NULL,72031),
(1391,13339,'Motorola Television Hour',NULL,now(),NULL,72032),
(1392,5327,'Stark Raving Mad',NULL,now(),NULL,72033),
(1393,5565,'The Amazing Dunninger',NULL,now(),NULL,72034),
(1394,10462,'Mastermind',NULL,now(),NULL,72037),
(1395,6152,'The Royal',NULL,now(),NULL,72038),
(1396,6435,'Trust',NULL,now(),NULL,72039),
(1397,4193,'LateLine',NULL,now(),NULL,72042),
(1398,135,'Jennifer Slept Here',NULL,now(),NULL,72044),
(1399,10312,'Ann Jillian',NULL,now(),NULL,72046),
(1400,11387,'Silent Number',NULL,now(),NULL,72047),
(1401,4056,'Joanie Loves Chachi',NULL,now(),NULL,72048),
(1402,11422,'Seven Little Australians',NULL,now(),NULL,72049),
(1403,136,'You Take the Kids',NULL,now(),NULL,72050),
(1404,4766,'Pacific Station',NULL,now(),NULL,72051),
(1405,18903,'Eye Guess',NULL,now(),NULL,72052),
(1406,954,'Mr. Bogus',NULL,now(),NULL,72053),
(1407,4376,'Married by America',NULL,now(),NULL,72054),
(1408,6382,'Toonheads',NULL,now(),NULL,72055),
(1409,3392,'Duckman',NULL,now(),NULL,72056),
(1410,5263,'South Central',NULL,now(),NULL,72057),
(1411,955,'13 Demon Street',NULL,now(),NULL,72058),
(1412,5941,'The Lone Ranger',NULL,now(),NULL,72059),
(1413,11470,'The Power, The Passion',NULL,now(),NULL,72060),
(1414,2448,'2gether',NULL,now(),NULL,72062),
(1415,956,'Patrol Boat',NULL,now(),NULL,72063),
(1416,4311,'Luke\'s Kingdom',NULL,now(),NULL,72064),
(1417,7301,'You Asked For It',NULL,now(),NULL,72065),
(1418,957,'Marion and Geoff',NULL,now(),NULL,72066),
(1419,3251,'David Cassidy - Man Undercover',NULL,now(),NULL,72067),
(1420,958,'Telephone Time',NULL,now(),NULL,72068),
(1421,11507,'Detective',NULL,now(),NULL,72069),
(1422,2717,'Barrier Reef',NULL,now(),NULL,72071),
(1423,960,'Bobby Dazzler',NULL,now(),NULL,72072),
(1424,22182,'On the Record with Bob Costas',NULL,now(),NULL,72074),
(1425,18857,'Inside The NFL',NULL,now(),NULL,72075),
(1426,961,'Rentaghost',NULL,now(),NULL,72076),
(1427,4356,'Malibu, CA',NULL,now(),NULL,72077),
(1428,963,'Strassman',NULL,now(),NULL,72078),
(1429,5403,'SuperFriends (1978)',NULL,now(),NULL,72079),
(1430,5404,'SuperFriends (1980)',NULL,now(),NULL,72080),
(1431,8309,'Liquid News',NULL,now(),NULL,72082),
(1432,6115,'The Quest',NULL,now(),NULL,72083),
(1433,12468,'Celebrity Sweepstakes',NULL,now(),NULL,72084),
(1434,964,'Boarding House: North Shore',NULL,now(),NULL,72086),
(1435,965,'The Awful Truth',NULL,now(),NULL,72087),
(1436,6138,'The Rifleman',NULL,now(),NULL,72089),
(1437,4010,'It\'s Your Move',NULL,now(),NULL,72090),
(1438,5513,'The 20th Century-Fox Hour',NULL,now(),NULL,72091),
(1439,137,'Climax',NULL,now(),NULL,72092),
(1440,5847,'The Help',NULL,now(),NULL,72093),
(1441,13668,'He\'s The Mayor',NULL,now(),NULL,72094),
(1442,967,'Thriller (1973)',NULL,now(),NULL,72095),
(1443,968,'Kraft Television Theatre (1953)',NULL,now(),NULL,72096),
(1444,3803,'Have Gun, Will Travel',NULL,now(),NULL,72097),
(1445,969,'Dance Fever (2003)',NULL,now(),NULL,72099),
(1446,4310,'Luis',NULL,now(),NULL,72100),
(1447,5958,'The Lyon\'s Den',NULL,now(),NULL,72101),
(1448,6343,'Threat Matrix',NULL,now(),NULL,72102),
(1449,6253,'The Tracy Morgan Show',NULL,now(),NULL,72103),
(1450,6273,'The View',NULL,now(),NULL,72104),
(1451,4103,'Karen Sisco',NULL,now(),NULL,72105),
(1452,4105,'Kate Brasher',NULL,now(),NULL,72106),
(1453,4378,'Married to the Kellys',NULL,now(),NULL,72107),
(1454,5990,'The Monkees',NULL,now(),NULL,72109),
(1455,14556,'The Ant and Dec Show',NULL,now(),NULL,72111),
(1456,2862,'Bodies of Evidence',NULL,now(),NULL,72112),
(1457,14989,'Re:covered',NULL,now(),NULL,72113),
(1458,970,'The Cambridge Spies',NULL,now(),NULL,72114),
(1459,2581,'Amazing Grace',NULL,now(),NULL,72117),
(1460,7346,'Made in Heaven',NULL,now(),NULL,72118),
(1461,8679,'Can We Get On Now, Please?',NULL,now(),NULL,72119),
(1462,4214,'Leaving L.A.',NULL,now(),NULL,72120),
(1463,3160,'Cracking Up',NULL,now(),NULL,72121),
(1464,2827,'Bless This House',NULL,now(),NULL,72122),
(1465,4380,'Martha Stewart Living',NULL,now(),NULL,72123),
(1466,6565,'What-A-Mess',NULL,now(),NULL,72124),
(1467,972,'Teknoman',NULL,now(),NULL,72125),
(1468,6066,'The Osmonds',NULL,now(),NULL,72127),
(1469,7380,'War and Peace',NULL,now(),NULL,72128),
(1470,973,'Whirlybirds',NULL,now(),NULL,72130),
(1471,6149,'The Rosie O\'Donnell Show',NULL,now(),NULL,72132),
(1472,2967,'Canadian Idol',NULL,now(),NULL,72133),
(1473,2984,'Captain Video And His Video Rangers',NULL,now(),NULL,72134),
(1474,975,'20 Things to Do Before You\'re 30',NULL,now(),NULL,72137),
(1475,12235,'The Late Late Show with Tom Snyder',NULL,now(),NULL,72138),
(1476,976,'Beulah',NULL,now(),NULL,72140),
(1477,4245,'Like Family',NULL,now(),NULL,72142),
(1478,5071,'Run of the House',NULL,now(),NULL,72143),
(1479,5464,'Tarzan',NULL,now(),NULL,72144),
(1480,8948,'Poor Little Rich Girls',NULL,now(),NULL,72145),
(1481,4263,'Live with Regis and Kelly',NULL,now(),NULL,72146),
(1482,7396,'The Way It Was',NULL,now(),NULL,72148),
(1483,5291,'Spenser: For Hire',NULL,now(),NULL,72149),
(1484,4196,'Later... with Jools Holland',NULL,now(),NULL,72150),
(1485,977,'Big Urban Myth Show',NULL,now(),NULL,72151),
(1486,978,'Matinee Theater',NULL,now(),NULL,72152),
(1487,5351,'Steve Harvey\'s Big Time Challenge',NULL,now(),NULL,72157),
(1488,6325,'The Zack Files',NULL,now(),NULL,72159),
(1489,21027,'Elvis',NULL,now(),NULL,72160),
(1490,13949,'Live with.. Chris Moyles',NULL,now(),NULL,72161),
(1491,12619,'Oh Boy!',NULL,now(),NULL,72162),
(1492,3200,'Da Ali G Show',NULL,now(),NULL,72165),
(1493,8332,'Today with Des and Mel',NULL,now(),NULL,72166),
(1494,18436,'Andra Avenyn',NULL,now(),NULL,80703);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(1495,3024,'Century City',NULL,now(),NULL,72168),
(1496,23145,'The Mayor of Casterbridge',NULL,now(),NULL,72169),
(1497,5755,'The Equalizer',NULL,now(),NULL,72170),
(1498,4440,'MegaMan: NT Warrior',NULL,now(),NULL,72172),
(1499,5392,'Sunday Night Heat',NULL,now(),NULL,72174),
(1500,3897,'Hope & Faith',NULL,now(),NULL,72175),
(1501,6259,'The Twilight Zone (1985)',NULL,now(),NULL,72176),
(1502,979,'Hunter (2003)',NULL,now(),NULL,72177),
(1503,980,'To Serve Them All My Days',NULL,now(),NULL,72179),
(1504,2715,'Barney and Friends',NULL,now(),NULL,72180),
(1505,4850,'Pole Position',NULL,now(),NULL,72182),
(1506,3315,'Dino-Riders',NULL,now(),NULL,72183),
(1507,982,'SpaceCats',NULL,now(),NULL,72184),
(1508,6581,'Where\'s Waldo?',NULL,now(),NULL,72185),
(1509,19123,'Goldie Gold and Action Jack',NULL,now(),NULL,72188),
(1510,6599,'Who Wants To Marry My Dad?',NULL,now(),NULL,72189),
(1511,17283,'The Harlem Globetrotters Popcorn Machine',NULL,now(),NULL,72191),
(1512,2762,'Ben Vereen... Comin\' at Ya',NULL,now(),NULL,72192),
(1513,3076,'Civil Wars',NULL,now(),NULL,72193),
(1514,5750,'The Ellen DeGeneres Show',NULL,now(),NULL,72194),
(1515,984,'The Immigrants',NULL,now(),NULL,72195),
(1516,985,'Harry Enfield\'s Television Programme',NULL,now(),NULL,72196),
(1517,2916,'Brooklyn Bridge',NULL,now(),NULL,72197),
(1518,4490,'Miss Match',NULL,now(),NULL,72198),
(1519,3137,'Cop Rock',NULL,now(),NULL,72199),
(1520,987,'You\'re On!',NULL,now(),NULL,72200),
(1521,6973,'Hannity & Colmes',NULL,now(),NULL,72202),
(1522,3344,'Doogie Howser, M.D.',NULL,now(),NULL,72203),
(1523,16758,'Top of the Hill',NULL,now(),NULL,72204),
(1524,18141,'General Electric Summer Originals',NULL,now(),NULL,72205),
(1525,5228,'Small Wonder',NULL,now(),NULL,72206),
(1526,6583,'Where in the World is Carmen Sandiego?',NULL,now(),NULL,72207),
(1527,2563,'All of Us',NULL,now(),NULL,72208),
(1528,3786,'Happy Family',NULL,now(),NULL,72209),
(1529,3352,'Doug',NULL,now(),NULL,72210),
(1530,6987,'Make the Grade',NULL,now(),NULL,72211),
(1531,138,'Checking In',NULL,now(),NULL,72213),
(1532,139,'Fish Police',NULL,now(),NULL,72214),
(1533,5871,'The Incredible Hulk (1982)',NULL,now(),NULL,72215),
(1534,5772,'The Fantastic Four (1967)',NULL,now(),NULL,72216),
(1535,989,'Thunderbirds 2086',NULL,now(),NULL,72217),
(1536,3598,'Fraternity Life',NULL,now(),NULL,72220),
(1537,9928,'Trouble for Two',NULL,now(),NULL,72221),
(1538,12536,'Compact',NULL,now(),NULL,72222),
(1539,7366,'Armchair Theatre',NULL,now(),NULL,72223),
(1540,991,'Dr. Finlay\'s Casebook',NULL,now(),NULL,72225),
(1541,992,'No Bananas',NULL,now(),NULL,72226),
(1542,10936,'Judge Mathis',NULL,now(),NULL,72228),
(1543,5034,'Roc',NULL,now(),NULL,72230),
(1544,4950,'Real Time With Bill Maher',NULL,now(),NULL,72231),
(1545,11496,'The Outsiders (1976)',NULL,now(),NULL,72232),
(1546,4438,'Mega Babies',NULL,now(),NULL,72234),
(1547,4034,'Jake and the Fatman',NULL,now(),NULL,72235),
(1548,5654,'The Brotherhood of Poland, N.H.',NULL,now(),NULL,72236),
(1549,3767,'Paradise',NULL,now(),NULL,72238),
(1550,2703,'Bakersfield, P.D.',NULL,now(),NULL,72240),
(1551,5978,'The Michael Essany Show',NULL,now(),NULL,72242),
(1552,5166,'Sharon, Lois & Bram\'s Elephant Show',NULL,now(),NULL,72243),
(1553,6670,'Xiaolin Showdown',NULL,now(),NULL,72245),
(1554,8588,'A Sharp Intake of Breath',NULL,now(),NULL,72246),
(1555,13351,'Shakespeare: The Animated Tales',NULL,now(),NULL,72247),
(1556,16867,'The NBC Follies',NULL,now(),NULL,72251),
(1557,7093,'Mystery!',NULL,now(),NULL,72252),
(1558,15730,'Sammy & Company',NULL,now(),NULL,72253),
(1559,998,'Phoenix (1992)',NULL,now(),NULL,72254),
(1560,5996,'The Mullets',NULL,now(),NULL,72255),
(1561,6681,'You Must Be the Husband',NULL,now(),NULL,72256),
(1562,4048,'Jeopardy (2002)',NULL,now(),NULL,72257),
(1563,4833,'Pirate Islands',NULL,now(),NULL,72258),
(1564,999,'The Peter Principle',NULL,now(),NULL,72259),
(1565,4318,'M.Y.O.B.',NULL,now(),NULL,72260),
(1566,7410,'Weekend',NULL,now(),NULL,72261),
(1567,1000,'Our World',NULL,now(),NULL,72263),
(1568,5096,'Sanford',NULL,now(),NULL,72264),
(1569,6080,'The Patty Duke Show',NULL,now(),NULL,72265),
(1570,5534,'The Adventures of McGee and Me',NULL,now(),NULL,72267),
(1571,4914,'Punk\'d',NULL,now(),NULL,72268),
(1572,1001,'Too Young to Go Steady',NULL,now(),NULL,72270),
(1573,1002,'Strange But True',NULL,now(),NULL,72271),
(1574,4452,'Merge',NULL,now(),NULL,72272),
(1575,3939,'I\'m With Her',NULL,now(),NULL,72273),
(1576,23280,'Reborn in the USA',NULL,now(),NULL,72274),
(1577,5037,'Rock Me Baby',NULL,now(),NULL,72275),
(1578,2604,'A Minute With Stan Hooper',NULL,now(),NULL,72276),
(1579,19066,'Dinah\'s Place',NULL,now(),NULL,72278),
(1580,3119,'Come to Papa',NULL,now(),NULL,72279),
(1581,3142,'Corrector Yui',NULL,now(),NULL,72280),
(1582,18853,'To Say The Least',NULL,now(),NULL,72282),
(1583,3133,'Conrad Bloom',NULL,now(),NULL,72283),
(1584,4195,'Late Night With David Letterman',NULL,now(),NULL,72284),
(1585,6428,'Tripping the Rift',NULL,now(),NULL,72285),
(1586,8969,'Room at the Bottom',NULL,now(),NULL,72288),
(1587,7256,'20/20',NULL,now(),NULL,72289),
(1588,10963,'Rodeo Drive',NULL,now(),NULL,72290),
(1589,1003,'Ed Stone Is Dead',NULL,now(),NULL,72293),
(1590,5619,'The Big House (2004)',NULL,now(),NULL,72294),
(1591,4002,'It\'s All Relative',NULL,now(),NULL,72295),
(1592,5380,'Stuart Little',NULL,now(),NULL,72296),
(1593,4774,'Pardon the Interruption',NULL,now(),NULL,72298),
(1594,4789,'Peacemakers',NULL,now(),NULL,72299),
(1595,3460,'Eve',NULL,now(),NULL,72300),
(1596,4795,'Penn & Teller: Bullshit!',NULL,now(),NULL,72301),
(1597,5856,'The Hollywood Palace',NULL,now(),NULL,72302),
(1598,141,'Hullabaloo!',NULL,now(),NULL,72304),
(1599,16710,'Tales of the 77th Bengal Lancers',NULL,now(),NULL,72305),
(1600,6270,'The Venture Bros.',NULL,now(),NULL,72306),
(1601,6397,'Townies',NULL,now(),NULL,72307),
(1602,6025,'The New Andy Griffith Show',NULL,now(),NULL,72308),
(1603,15729,'Celebrity Bowling',NULL,now(),NULL,72310),
(1604,4812,'Pet Star',NULL,now(),NULL,72311),
(1605,1006,'Hardware',NULL,now(),NULL,72313),
(1606,6045,'The New Three Stooges',NULL,now(),NULL,72314),
(1607,6268,'The Van Dyke Show',NULL,now(),NULL,72315),
(1608,5519,'The Accidental Family',NULL,now(),NULL,72316),
(1609,2460,'704 Hauser',NULL,now(),NULL,72318),
(1610,15891,'Cavalcade of Stars',NULL,now(),NULL,72319),
(1611,22238,'My First Time',NULL,now(),NULL,72320),
(1612,5048,'Romeo!',NULL,now(),NULL,72321),
(1613,5881,'The Jack Benny Program',NULL,now(),NULL,72322),
(1614,12363,'The Space Sentinels',NULL,now(),NULL,72323),
(1615,1009,'Offenders',NULL,now(),NULL,72325),
(1616,142,'Pryor\'s Place',NULL,now(),NULL,72326),
(1617,5786,'The Flip Wilson Show',NULL,now(),NULL,72327),
(1618,3710,'Good Company',NULL,now(),NULL,72328),
(1619,6009,'The Nat King Cole Show',NULL,now(),NULL,72329),
(1620,5527,'The Adventures of Gulliver',NULL,now(),NULL,72330),
(1621,1010,'Happening \'68',NULL,now(),NULL,72331),
(1622,7998,'The Arsenio Hall Show',NULL,now(),NULL,72332),
(1623,12118,'Out',NULL,now(),NULL,72333),
(1624,3062,'ChuckleVision',NULL,now(),NULL,72334),
(1625,1012,'Rubbadubbers',NULL,now(),NULL,72335),
(1626,4979,'Reno 911!',NULL,now(),NULL,72336),
(1627,1013,'Miami Sands',NULL,now(),NULL,72337),
(1628,2437,'15 Storeys High',NULL,now(),NULL,72339),
(1629,7254,'2 Minute Drill',NULL,now(),NULL,72340),
(1630,6694,'Zane Grey Theater',NULL,now(),NULL,72341),
(1631,1015,'American Muscle Car',NULL,now(),NULL,72342),
(1632,2994,'Caroline in the City',NULL,now(),NULL,72345),
(1633,1016,'Gold Trails and Ghost Towns',NULL,now(),NULL,72346),
(1634,1018,'Macross 7',NULL,now(),NULL,72347),
(1635,4975,'Renegade',NULL,now(),NULL,72348),
(1636,1020,'Mushrambo',NULL,now(),NULL,72350),
(1637,143,'Hee Haw',NULL,now(),NULL,72351),
(1638,4838,'Playboy After Dark',NULL,now(),NULL,72353),
(1639,4217,'Legend of Prince Valiant',NULL,now(),NULL,72354),
(1640,4651,'NewsRadio',NULL,now(),NULL,72355),
(1641,1023,'Burn It',NULL,now(),NULL,72357),
(1642,2466,'9 to 5',NULL,now(),NULL,72360),
(1643,3084,'Cleghorne!',NULL,now(),NULL,72362),
(1644,4480,'Misery Loves Company',NULL,now(),NULL,72363),
(1645,3621,'Fudge',NULL,now(),NULL,72365),
(1646,4801,'Pepsi Smash',NULL,now(),NULL,72366),
(1647,6676,'Yes, Dear',NULL,now(),NULL,72367),
(1648,4202,'Law & Order',NULL,now(),NULL,72368),
(1649,5938,'The Lively Ones',NULL,now(),NULL,72369),
(1650,1026,'Fresh Fields',NULL,now(),NULL,72370),
(1651,1027,'Comedy Inc.',NULL,now(),NULL,72371),
(1652,3007,'Catweazle',NULL,now(),NULL,72372),
(1653,13220,'Mr Palfrey of Westminster',NULL,now(),NULL,72373),
(1654,3902,'Hotel Erotica',NULL,now(),NULL,72374),
(1655,19005,'Juvenile Jury',NULL,now(),NULL,72376),
(1656,15837,'Twilight Theater',NULL,now(),NULL,72377),
(1657,2966,'Canada: A People\'s History',NULL,now(),NULL,72378),
(1658,11942,'Ben Hall',NULL,now(),NULL,72379),
(1659,2946,'Buzzkill',NULL,now(),NULL,72380),
(1660,3028,'Chambers',NULL,now(),NULL,72381),
(1661,10402,'Three Steps to Heaven',NULL,now(),NULL,72383),
(1662,1029,'The Secret Life of Toys',NULL,now(),NULL,72384),
(1663,4753,'Outside the Rules',NULL,now(),NULL,72385),
(1664,16969,'Manhunt (2001)',NULL,now(),NULL,72388),
(1665,2454,'3rd Rock from the Sun',NULL,now(),NULL,72389),
(1666,10093,'The Clear Horizon',NULL,now(),NULL,72390),
(1667,3834,'Herbie the Love Bug',NULL,now(),NULL,72391),
(1668,6424,'Trigger Happy TV (US)',NULL,now(),NULL,72392),
(1669,1031,'Project Arms',NULL,now(),NULL,72394),
(1670,4019,'I Was a Sixth Grade Alien',NULL,now(),NULL,72395),
(1671,1032,'Mr. Personality',NULL,now(),NULL,72396),
(1672,9782,'Rory Bremner, Who Else?',NULL,now(),NULL,72398),
(1673,10716,'The Big Revue',NULL,now(),NULL,72399),
(1674,1033,'Hey, Joel!',NULL,now(),NULL,72400),
(1675,3405,'Dusty\'s Trail',NULL,now(),NULL,72402),
(1676,1034,'Lavender Castle',NULL,now(),NULL,72403),
(1677,3518,'Far Out Space Nuts',NULL,now(),NULL,72405),
(1678,2777,'Beverly Hills Buntz',NULL,now(),NULL,72407),
(1679,3450,'Equal Justice',NULL,now(),NULL,72410),
(1680,19063,'Crime and Punishment (1993)',NULL,now(),NULL,72411),
(1681,1038,'Shadow Skill',NULL,now(),NULL,72412),
(1682,4159,'Kristin',NULL,now(),NULL,72413),
(1683,3422,'Edgemont',NULL,now(),NULL,72414),
(1684,5888,'The Jeff Foxworthy Show',NULL,now(),NULL,72415),
(1685,4341,'Mail Call',NULL,now(),NULL,72417),
(1686,10969,'The Roller Girls',NULL,now(),NULL,72418),
(1687,4170,'L.A. Law',NULL,now(),NULL,72419),
(1688,12249,'Karen\'s Song',NULL,now(),NULL,72420),
(1689,4374,'Marmalade Boy',NULL,now(),NULL,72421),
(1690,4678,'No, Honestly',NULL,now(),NULL,72422),
(1691,5926,'The Legends of Treasure Island',NULL,now(),NULL,72423),
(1692,5390,'Sunday Dinner',NULL,now(),NULL,72424),
(1693,10108,'The Secret Storm',NULL,now(),NULL,72425),
(1694,4708,'Oh Grow Up',NULL,now(),NULL,72426),
(1695,1040,'Stingray (1964)',NULL,now(),NULL,72428),
(1696,5489,'Temporarily Yours',NULL,now(),NULL,72429),
(1697,4891,'Primus',NULL,now(),NULL,72430),
(1698,1041,'The Murder Game',NULL,now(),NULL,72431),
(1699,1044,'Haunted Junction',NULL,now(),NULL,72432),
(1700,5953,'The Love Boat: The Next Wave',NULL,now(),NULL,72434),
(1701,1047,'Virus',NULL,now(),NULL,72436),
(1702,1048,'Paradise Hotel',NULL,now(),NULL,72437),
(1703,3911,'House of Dreams',NULL,now(),NULL,72438),
(1704,1050,'Racket Squad',NULL,now(),NULL,72440),
(1705,1051,'IMX',NULL,now(),NULL,72441),
(1706,10539,'Dirty Pair',NULL,now(),NULL,72442),
(1707,1053,'His and Her Circumstances',NULL,now(),NULL,72444),
(1708,5257,'Sorcerer Hunters',NULL,now(),NULL,72445),
(1709,1054,'Legend of Basara',NULL,now(),NULL,72448),
(1710,2433,'100 Deeds for Eddie McDowd',NULL,now(),NULL,72450),
(1711,4165,'KYTV',NULL,now(),NULL,72453),
(1712,144,'The Demon Headmaster',NULL,now(),NULL,72455),
(1713,3966,'Inside Fame',NULL,now(),NULL,72456),
(1714,21924,'Music In High Places',NULL,now(),NULL,72457),
(1715,6243,'The Tomorrow People',NULL,now(),NULL,72458),
(1716,5252,'Sonic X',NULL,now(),NULL,72459),
(1717,19847,'The Strip',NULL,now(),NULL,72460),
(1718,10978,'Excel Saga',NULL,now(),NULL,72462),
(1719,9876,'The Telegoons',NULL,now(),NULL,72463),
(1720,9908,'The World of Beachcomber',NULL,now(),NULL,72464),
(1721,8702,'Curry and Chips',NULL,now(),NULL,72465),
(1722,5540,'The Adventures of Shirley Holmes',NULL,now(),NULL,72466),
(1723,1057,'D*A*A*S Kapital',NULL,now(),NULL,72467),
(1724,1058,'Hand Maid May',NULL,now(),NULL,72470),
(1725,4811,'Petrocelli',NULL,now(),NULL,72472),
(1726,5067,'Rude Awakening',NULL,now(),NULL,72473),
(1727,2947,'Buzz Lightyear of Star Command',NULL,now(),NULL,72474),
(1728,1064,'Vandread: The Second Stage',NULL,now(),NULL,72475),
(1729,4862,'Popetown',NULL,now(),NULL,72478),
(1730,12629,'Get It Together',NULL,now(),NULL,72479),
(1731,7031,'Countdown',NULL,now(),NULL,72480),
(1732,1065,'Noozles',NULL,now(),NULL,72481),
(1733,16811,'Sunset Beat',NULL,now(),NULL,72482),
(1734,5546,'The Adventures of Superboy (1966)',NULL,now(),NULL,72483),
(1735,3301,'Diagnosis Murder',NULL,now(),NULL,72484),
(1736,11068,'Biff Baker USA',NULL,now(),NULL,72485),
(1737,2683,'Average Joe',NULL,now(),NULL,72486),
(1738,3210,'Dalgliesh',NULL,now(),NULL,72487),
(1739,3475,'Extreme Makeover',NULL,now(),NULL,72488),
(1740,1067,'Dynomutt, Dog Wonder',NULL,now(),NULL,72489),
(1741,1068,'Slam Dunk',NULL,now(),NULL,72491),
(1742,5918,'The League of Gentlemen',NULL,now(),NULL,72492),
(1743,1069,'Silent Mobius',NULL,now(),NULL,72493),
(1744,4275,'Lonesome Dove: The Outlaw Years',NULL,now(),NULL,72494),
(1745,4326,'Madison',NULL,now(),NULL,72495),
(1746,8229,'Julie',NULL,now(),NULL,72496),
(1747,3835,'Hercules: The Legendary Journeys',NULL,now(),NULL,72497),
(1748,6290,'The White Shadow',NULL,now(),NULL,72498),
(1749,1072,'Knife Edge',NULL,now(),NULL,72500),
(1750,9318,'For 4 Tonight',NULL,now(),NULL,72501),
(1751,5609,'The Berenstain Bears',NULL,now(),NULL,72502),
(1752,7411,'Weekend Break',NULL,now(),NULL,72504),
(1753,13383,'Wokenwell',NULL,now(),NULL,72505),
(1754,13060,'Backup',NULL,now(),NULL,72506),
(1755,4369,'Manor House',NULL,now(),NULL,72507),
(1756,1073,'The Court of Last Resort',NULL,now(),NULL,72508),
(1757,1074,'The Ruff & Ready Show',NULL,now(),NULL,72509),
(1758,9430,'It\'s Magic!',NULL,now(),NULL,72513),
(1759,1075,'Mr Benn',NULL,now(),NULL,72516),
(1760,1076,'King Rollo',NULL,now(),NULL,72517),
(1761,6183,'The Shari Lewis Show',NULL,now(),NULL,72520),
(1762,4091,'Just For Fun!',NULL,now(),NULL,72522),
(1763,7398,'Way Out Games!',NULL,now(),NULL,72523),
(1764,1079,'The Chuck McCann Show',NULL,now(),NULL,72533),
(1765,3790,'Hardcastle & McCormick',NULL,now(),NULL,72534),
(1766,18800,'Junior Almost Anything Goes',NULL,now(),NULL,72535),
(1767,1080,'The Cops',NULL,now(),NULL,72536),
(1768,15918,'Love Thy Neighbor',NULL,now(),NULL,72537),
(1769,11022,'Capital News',NULL,now(),NULL,72539),
(1770,6048,'The New Zoo Revue',NULL,now(),NULL,72540),
(1771,3169,'Crime Traveller',NULL,now(),NULL,72541),
(1772,4405,'Max Steel',NULL,now(),NULL,72543),
(1773,5890,'The Jerry Springer Show',NULL,now(),NULL,72548),
(1774,2792,'Big Brother (UK)',NULL,now(),NULL,72549),
(1775,1082,'The Cartoon Zoo',NULL,now(),NULL,72550),
(1776,4715,'One Big Family',NULL,now(),NULL,72551),
(1777,4877,'Power Rangers',NULL,now(),NULL,72553),
(1778,2646,'Ark II',NULL,now(),NULL,72555),
(1779,4738,'Otherworld',NULL,now(),NULL,72556),
(1780,146,'Chained',NULL,now(),NULL,72559),
(1781,147,'Trust Me',NULL,now(),NULL,72560),
(1782,6317,'The Yogi Bear Show',NULL,now(),NULL,72562),
(1783,3472,'Extreme',NULL,now(),NULL,72563),
(1784,148,'Burnside',NULL,now(),NULL,72564),
(1785,13545,'The Chief',NULL,now(),NULL,72567),
(1786,6404,'Trading Spaces: Boys vs. Girls',NULL,now(),NULL,72568),
(1787,1086,'It Ain\'t Half Hot Mum',NULL,now(),NULL,72569),
(1788,10396,'Action In The Afternoon',NULL,now(),NULL,72572),
(1789,8032,'Mr.Wizard\'s World',NULL,now(),NULL,72573),
(1790,6093,'The Pink Panther Show',NULL,now(),NULL,72575),
(1791,19037,'Sing Along With Mitch',NULL,now(),NULL,72577),
(1792,1087,'When the Boat Comes In',NULL,now(),NULL,72580),
(1793,149,'87th Precinct',NULL,now(),NULL,72581),
(1794,6448,'Twelve O\'Clock High',NULL,now(),NULL,72582),
(1795,13406,'The Stars Look Down',NULL,now(),NULL,72583),
(1796,5968,'The Man Who Never Was',NULL,now(),NULL,72585),
(1797,2678,'Automan',NULL,now(),NULL,72589),
(1798,15623,'The Hudson Brothers Razzle Dazzle Show',NULL,now(),NULL,72592),
(1799,17376,'People Are Funny',NULL,now(),NULL,72593),
(1800,7258,'21',NULL,now(),NULL,72597),
(1801,1089,'In Melbourne Tonight',NULL,now(),NULL,72598),
(1802,11762,'Funky Squad',NULL,now(),NULL,72599),
(1803,12695,'Recovery',NULL,now(),NULL,72600),
(1804,7675,'Burke\'s Backyard',NULL,now(),NULL,72601),
(1805,1090,'Fractured Flickers',NULL,now(),NULL,72602),
(1806,8990,'Bank on the Stars',NULL,now(),NULL,72603),
(1807,4559,'Murder Investigation Team',NULL,now(),NULL,72605),
(1808,3060,'Christy',NULL,now(),NULL,72606),
(1809,14898,'Ernie In Kovacsland',NULL,now(),NULL,72609),
(1810,2771,'Better Days',NULL,now(),NULL,72610),
(1811,14899,'The Ernie Kovacs Show',NULL,now(),NULL,72611),
(1812,5363,'Strange Luck',NULL,now(),NULL,72612),
(1813,3968,'Inside the Actors Studio',NULL,now(),NULL,72614),
(1814,1094,'The Aunty Jack Show',NULL,now(),NULL,72615),
(1815,11275,'Wollongong the Brave',NULL,now(),NULL,72616),
(1816,150,'Tribeca',NULL,now(),NULL,72618),
(1817,1095,'Harry Hill',NULL,now(),NULL,72620),
(1818,1096,'Fruit Fancies',NULL,now(),NULL,72621),
(1819,20597,'Fantasy',NULL,now(),NULL,72622),
(1820,3645,'Garfield Goose & Friends',NULL,now(),NULL,72623),
(1821,4860,'Poochini',NULL,now(),NULL,72625),
(1822,18871,'Get The Message',NULL,now(),NULL,72626),
(1823,15635,'Hour Glass',NULL,now(),NULL,72627),
(1824,1099,'The Singing Detective',NULL,now(),NULL,72628),
(1825,4126,'Kikaider',NULL,now(),NULL,72629),
(1826,5022,'Robin\'s Nest',NULL,now(),NULL,72630),
(1827,1100,'The Big Impression',NULL,now(),NULL,72631),
(1828,21222,'Livin\' Large',NULL,now(),NULL,72635),
(1829,2707,'Bananas In Pajamas',NULL,now(),NULL,72638),
(1830,1101,'The Basil Brush Show',NULL,now(),NULL,72639),
(1831,10608,'Popworld',NULL,now(),NULL,72641),
(1832,2479,'Acapulco H.E.A.T.',NULL,now(),NULL,72642),
(1833,1103,'Kingswood Country',NULL,now(),NULL,72643),
(1834,8648,'Shine on Harvey Moon',NULL,now(),NULL,72644),
(1835,11196,'Jackanory',NULL,now(),NULL,72645),
(1836,5839,'The Handler',NULL,now(),NULL,72646),
(1837,9808,'Scotch & Wry',NULL,now(),NULL,72647),
(1838,5927,'The Lenny Henry Show (1984)',NULL,now(),NULL,72648),
(1839,10054,'Change of Heart',NULL,now(),NULL,72649),
(1840,4403,'Max Liebman Presents',NULL,now(),NULL,72650),
(1841,1105,'The Belzer Connection',NULL,now(),NULL,72651),
(1842,9690,'The Galaxy Goof-Ups',NULL,now(),NULL,72652),
(1843,151,'Space Cadets',NULL,now(),NULL,72654),
(1844,7445,'The Wil Shriner Show',NULL,now(),NULL,72656),
(1845,16,'Forever Fernwood',NULL,now(),NULL,72657),
(1846,5274,'Spaced',NULL,now(),NULL,72658),
(1847,7253,'199 Park Lane',NULL,now(),NULL,72659),
(1848,3063,'Chuck Woolery: Naturally Stoned',NULL,now(),NULL,72660),
(1849,3087,'Clifford\'s Puppy Days',NULL,now(),NULL,72661),
(1850,5487,'Teletubbies',NULL,now(),NULL,72662),
(1851,2482,'Accused',NULL,now(),NULL,72663),
(1852,8604,'The George Jessel Show',NULL,now(),NULL,72664),
(1853,1107,'Train 48',NULL,now(),NULL,72665),
(1854,5667,'The California Raisin Show',NULL,now(),NULL,72666),
(1855,4898,'Probe',NULL,now(),NULL,72667),
(1856,6913,'Famous Jury Trials',NULL,now(),NULL,72669),
(1857,21544,'Destination Stardom',NULL,now(),NULL,72671),
(1858,12881,'Navy Log',NULL,now(),NULL,72672),
(1859,4189,'Last Comic Standing',NULL,now(),NULL,72673),
(1860,3132,'Conquest',NULL,now(),NULL,72674),
(1861,4289,'Love & Money',NULL,now(),NULL,72676),
(1862,2549,'All\'s Fair',NULL,now(),NULL,72677),
(1863,1113,'Birdz',NULL,now(),NULL,72678),
(1864,3530,'Father of the Pride',NULL,now(),NULL,72681),
(1865,1115,'French Fields',NULL,now(),NULL,72682),
(1866,1117,'Trash To Cash',NULL,now(),NULL,72683),
(1867,6124,'The Real Roseanne Show',NULL,now(),NULL,72684),
(1868,1118,'Unscrewed with Martin Sargent',NULL,now(),NULL,72686),
(1869,1120,'The Adventures of T-Rex',NULL,now(),NULL,72687),
(1870,1123,'Freaky Stories',NULL,now(),NULL,72688),
(1871,13288,'The Wilde Alliance',NULL,now(),NULL,72689),
(1872,1124,'The Metric Marvels',NULL,now(),NULL,72690),
(1873,9543,'Live Floor Show',NULL,now(),NULL,72691),
(1874,6349,'Thriller',NULL,now(),NULL,72692),
(1875,4531,'MXC',NULL,now(),NULL,72693),
(1876,1126,'Champions of the Wild',NULL,now(),NULL,72694),
(1877,4745,'Outlaws',NULL,now(),NULL,72695),
(1878,8366,'Don\'t Wait Up',NULL,now(),NULL,72696),
(1879,7282,'Zardip\'s Search for Healthy Wellness',NULL,now(),NULL,72698),
(1880,2893,'Branded',NULL,now(),NULL,72699),
(1881,6698,'Zixx: Level One',NULL,now(),NULL,72700),
(1882,5991,'The Monroes',NULL,now(),NULL,72701),
(1883,1128,'Extreme Engineering',NULL,now(),NULL,72702),
(1884,4008,'It\'s Good to be ...',NULL,now(),NULL,72703),
(1885,5846,'The Heights',NULL,now(),NULL,72704),
(1886,11828,'Open Air',NULL,now(),NULL,72705),
(1887,1129,'Wizadora',NULL,now(),NULL,72706),
(1888,5992,'The Montel Williams Show',NULL,now(),NULL,72707),
(1889,3101,'Code Red',NULL,now(),NULL,72708),
(1890,4097,'K9 and Company',NULL,now(),NULL,72710),
(1891,5697,'The Cosby Mysteries',NULL,now(),NULL,72711),
(1892,4567,'Musical Comedy Time',NULL,now(),NULL,72712),
(1893,4732,'Oobi',NULL,now(),NULL,72713),
(1894,5430,'Swift Justice',NULL,now(),NULL,72714),
(1895,4920,'QI',NULL,now(),NULL,72716),
(1896,4986,'Resurrection Blvd.',NULL,now(),NULL,72717),
(1897,1132,'Micallef Tonight',NULL,now(),NULL,72718),
(1898,4827,'Pilot Candidate',NULL,now(),NULL,72719),
(1899,3969,'Inside Victor Lewis-Smith',NULL,now(),NULL,72721),
(1900,6276,'The Voice of Firestone',NULL,now(),NULL,72722),
(1901,6134,'The Restaurant',NULL,now(),NULL,72723),
(1902,3938,'I\'m With Busey',NULL,now(),NULL,72725),
(1903,1136,'Gasaraki',NULL,now(),NULL,72726),
(1904,3121,'Comic Relief Does Fame Academy',NULL,now(),NULL,72727),
(1905,8321,'Crimewatch UK',NULL,now(),NULL,72728),
(1906,1137,'HeartBeat (1988)',NULL,now(),NULL,72729),
(1907,4842,'Playmakers',NULL,now(),NULL,72731),
(1908,3538,'Ferris Bueller',NULL,now(),NULL,72732),
(1909,9360,'The Galton and Simpson Playhouse',NULL,now(),NULL,72733),
(1910,15854,'The Islanders',NULL,now(),NULL,72734),
(1911,1143,'Monkey Dust',NULL,now(),NULL,72735),
(1912,3441,'Empire',NULL,now(),NULL,72736),
(1913,7281,'Z Games',NULL,now(),NULL,72737),
(1914,4767,'Page to Screen',NULL,now(),NULL,72738),
(1915,5295,'Spider-Man (1967)',NULL,now(),NULL,72739),
(1916,7297,'You & Me Kid',NULL,now(),NULL,72740),
(1917,2559,'All in the Family',NULL,now(),NULL,72741),
(1918,1146,'Wish You Were Here (1990)',NULL,now(),NULL,72743),
(1919,8314,'Love Bytes',NULL,now(),NULL,72744),
(1920,3197,'Cyborg 009',NULL,now(),NULL,72745),
(1921,17861,'AWA All-Star Wrestling',NULL,now(),NULL,72747),
(1922,4502,'Moby Dick and the Mighty Mightor',NULL,now(),NULL,72748),
(1923,1149,'Shazzan!',NULL,now(),NULL,72749),
(1924,4989,'Return To Jupiter',NULL,now(),NULL,72750),
(1925,13635,'Tomorrow\'s World',NULL,now(),NULL,72752),
(1926,1152,'Wally Gator',NULL,now(),NULL,72753),
(1927,1154,'Hillbilly Bears',NULL,now(),NULL,72755),
(1928,17,'Seven Brides for Seven Brothers',NULL,now(),NULL,72756),
(1929,4510,'Monchhichis',NULL,now(),NULL,72759),
(1930,13234,'The New Adventures of Charlie Chan',NULL,now(),NULL,72760),
(1931,11505,'The Orlando Jones Show',NULL,now(),NULL,72761),
(1932,3368,'Dragnet (1951)',NULL,now(),NULL,72763),
(1933,1156,'Peter Potamus and His Magic Flying Balloon',NULL,now(),NULL,72765),
(1934,1157,'Lippy the Lion',NULL,now(),NULL,72766),
(1935,7424,'What\'s In a Word',NULL,now(),NULL,72767),
(1936,6101,'The Porky Pig Show',NULL,now(),NULL,72768),
(1937,2789,'Big Brother (Africa)',NULL,now(),NULL,72769),
(1938,5910,'The Last Detective',NULL,now(),NULL,72770),
(1939,1158,'Boys and Girls',NULL,now(),NULL,72771),
(1940,4384,'Martin Mystery',NULL,now(),NULL,72773),
(1941,4435,'Meet My Folks (UK)',NULL,now(),NULL,72774),
(1942,6415,'Transformers Energon',NULL,now(),NULL,72776),
(1943,1160,'Love in the 21st Century',NULL,now(),NULL,72778),
(1944,3979,'In the House',NULL,now(),NULL,72779),
(1945,3011,'Celebrities Uncensored',NULL,now(),NULL,72780),
(1946,6331,'thirtysomething',NULL,now(),NULL,72781),
(1947,6405,'Trading Spaces: Family',NULL,now(),NULL,72782),
(1948,1162,'The Lionhearts',NULL,now(),NULL,72783),
(1949,1164,'Lancelot Link, Secret Chimp',NULL,now(),NULL,72784),
(1950,9048,'Tales from the Lazy Acre',NULL,now(),NULL,72785),
(1951,2590,'American Bandstand',NULL,now(),NULL,72786),
(1952,152,'The Glen Campbell Goodtime Hour',NULL,now(),NULL,72787),
(1953,5193,'Sidewalks: Video Nite',NULL,now(),NULL,72788),
(1954,18913,'Stumpers',NULL,now(),NULL,72789),
(1955,3162,'Crash Nebula',NULL,now(),NULL,72790),
(1956,1167,'I My Me! Strawberry Eggs',NULL,now(),NULL,72792),
(1957,1169,'Stark',NULL,now(),NULL,72795),
(1958,1171,'Battle Athletes Victory',NULL,now(),NULL,72796),
(1959,2573,'Al TV',NULL,now(),NULL,72797),
(1960,10033,'Britain\'s Brainiest',NULL,now(),NULL,72798),
(1961,1173,'The Jasper Carrott Trial',NULL,now(),NULL,72799),
(1962,1174,'Canned Carrott',NULL,now(),NULL,72800),
(1963,1175,'Forever Green',NULL,now(),NULL,72802),
(1964,2909,'Brimstone',NULL,now(),NULL,72803),
(1965,3292,'Design on a Dime',NULL,now(),NULL,72804),
(1966,1176,'Look Around You',NULL,now(),NULL,72805),
(1967,6840,'X-Play',NULL,now(),NULL,72806),
(1968,3017,'Celebrity Fit Club',NULL,now(),NULL,72808),
(1969,1178,'Evil Con Carne',NULL,now(),NULL,72809),
(1970,3270,'Dear John (UK)',NULL,now(),NULL,72810),
(1971,11502,'Takin\' Over the Asylum',NULL,now(),NULL,72812),
(1972,1179,'Hi-de-hi',NULL,now(),NULL,72813),
(1973,1180,'Redcap (1964)',NULL,now(),NULL,72816),
(1974,1181,'It\'s The Wolf',NULL,now(),NULL,72818),
(1975,1182,'Trevor\'s World of Sport',NULL,now(),NULL,72819),
(1976,5315,'St. Elsewhere',NULL,now(),NULL,72821),
(1977,4834,'Pixie & Dixie',NULL,now(),NULL,72822),
(1978,4955,'Rebus',NULL,now(),NULL,72823),
(1979,1183,'The Adventurer (1986)',NULL,now(),NULL,72824),
(1980,4121,'Kid Notorious',NULL,now(),NULL,72827),
(1981,1185,'Timeblazers',NULL,now(),NULL,72828),
(1982,5580,'The Apprentice',NULL,now(),NULL,72829),
(1983,4871,'Postcards from Buster',NULL,now(),NULL,72830),
(1984,17146,'Flatbush',NULL,now(),NULL,72833),
(1985,1188,'Rainbow',NULL,now(),NULL,72834),
(1986,4161,'K Street',NULL,now(),NULL,72835),
(1987,1189,'Mina and the Count',NULL,now(),NULL,72836),
(1988,1190,'Beg, Borrow & Deal',NULL,now(),NULL,72837),
(1989,4518,'Monster House',NULL,now(),NULL,72838),
(1990,1191,'The Savage Dragon',NULL,now(),NULL,72839),
(1991,5648,'The Brendan Leonard Show',NULL,now(),NULL,72840),
(1992,153,'Scorch',NULL,now(),NULL,72841);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(1993,3706,'Gokudo',NULL,now(),NULL,72843),
(1994,1196,'Zenki',NULL,now(),NULL,72844),
(1995,1197,'Cyber Team in Akihabara',NULL,now(),NULL,72845),
(1996,3758,'Gun',NULL,now(),NULL,72846),
(1997,3495,'Fame',NULL,now(),NULL,72847),
(1998,17123,'Kid N\' Play',NULL,now(),NULL,72849),
(1999,1202,'The Tweenies',NULL,now(),NULL,72851),
(2000,3848,'Higher Ground',NULL,now(),NULL,72852),
(2001,4322,'Made',NULL,now(),NULL,72853),
(2002,4225,'Lexx',NULL,now(),NULL,72854),
(2003,15725,'He Said, She Said',NULL,now(),NULL,72855),
(2004,4519,'Monster Rancher',NULL,now(),NULL,72858),
(2005,10280,'Adam And Joe Go Tokyo',NULL,now(),NULL,82755),
(2006,3080,'Class of \'96',NULL,now(),NULL,72863),
(2007,3879,'Hollywood Off Ramp',NULL,now(),NULL,72867),
(2008,4402,'Maximum Exposure',NULL,now(),NULL,72868),
(2009,8328,'Laura the Prairie Girl',NULL,now(),NULL,72869),
(2010,154,'Chromiumblue.com',NULL,now(),NULL,72872),
(2011,1209,'I Hate This House',NULL,now(),NULL,72874),
(2012,12821,'Southern Cross',NULL,now(),NULL,72876),
(2013,2527,'Akazukin ChaCha',NULL,now(),NULL,72878),
(2014,1215,'Bo\' Selecta!',NULL,now(),NULL,72881),
(2015,1217,'Tijuana Toads',NULL,now(),NULL,72883),
(2016,3838,'Here Comes the Grump',NULL,now(),NULL,72886),
(2017,10651,'Coming Attractions',NULL,now(),NULL,72887),
(2018,6302,'The Wonder Years',NULL,now(),NULL,72888),
(2019,3678,'Ghostwriter',NULL,now(),NULL,72889),
(2020,6577,'What Not To Wear',NULL,now(),NULL,72890),
(2021,1223,'Posh Nosh',NULL,now(),NULL,72891),
(2022,2921,'Brum',NULL,now(),NULL,72892),
(2023,4936,'Radio Free Roscoe',NULL,now(),NULL,72893),
(2024,9682,'Gambit',NULL,now(),NULL,72894),
(2025,19041,'Las Vegas Gambit',NULL,now(),NULL,72895),
(2026,155,'Happily Ever After: Fairy Tales for Every Child',NULL,now(),NULL,72897),
(2027,1226,'Super Doll Licca-chan',NULL,now(),NULL,72898),
(2028,4154,'Kokoro Library',NULL,now(),NULL,72900),
(2029,1227,'Orphen',NULL,now(),NULL,72901),
(2030,10981,'Last Exile',NULL,now(),NULL,72902),
(2031,1230,'Bomberman Bidaman Bakugaiden',NULL,now(),NULL,72904),
(2032,3863,'Hikaru no Go',NULL,now(),NULL,72905),
(2033,1233,'Lost (2001)',NULL,now(),NULL,72906),
(2034,7337,'Queen for a Day',NULL,now(),NULL,72907),
(2035,1234,'Prospects',NULL,now(),NULL,72908),
(2036,4778,'Partners',NULL,now(),NULL,72909),
(2037,1235,'Eyes Down',NULL,now(),NULL,72910),
(2038,1236,'Uranium',NULL,now(),NULL,72911),
(2039,7341,'A Question of Sport',NULL,now(),NULL,72914),
(2040,7608,'Enough Rope with Andrew Denton',NULL,now(),NULL,72915),
(2041,16973,'The Front Page',NULL,now(),NULL,72919),
(2042,1401,'Weiss kreuz',NULL,now(),NULL,72922),
(2043,4286,'Lost in Space',NULL,now(),NULL,72923),
(2044,16748,'Turbo Teen',NULL,now(),NULL,72924),
(2045,1242,'Killer Net',NULL,now(),NULL,72925),
(2046,18,'Fallen Angels',NULL,now(),NULL,72927),
(2047,1243,'Blue Murder (UK)',NULL,now(),NULL,72928),
(2048,1244,'The Beany and Cecil Show',NULL,now(),NULL,72929),
(2049,1245,'The Bob Clampett Show',NULL,now(),NULL,72930),
(2050,19149,'Operation: Entertainment',NULL,now(),NULL,72932),
(2051,6390,'Total Security',NULL,now(),NULL,72935),
(2052,1247,'The Tex Avery Show',NULL,now(),NULL,72936),
(2053,5816,'The Girl with Something Extra',NULL,now(),NULL,72937),
(2054,1248,'The New Yankee Workshop',NULL,now(),NULL,72938),
(2055,8502,'Dirt',NULL,now(),NULL,79679),
(2056,2988,'Cardcaptors',NULL,now(),NULL,72940),
(2057,2436,'15/Love',NULL,now(),NULL,72941),
(2058,5705,'The Crouches',NULL,now(),NULL,72942),
(2059,4115,'Kerching',NULL,now(),NULL,72943),
(2060,5681,'The Chris Isaak Show',NULL,now(),NULL,72944),
(2061,9897,'Make the Connection',NULL,now(),NULL,72945),
(2062,1250,'The Ant and The Aardvark',NULL,now(),NULL,72947),
(2063,18847,'Winner Take All',NULL,now(),NULL,72948),
(2064,10149,'What\'s It Worth',NULL,now(),NULL,72949),
(2065,16864,'Mayor of the Town',NULL,now(),NULL,72953),
(2066,3055,'China Beach',NULL,now(),NULL,72954),
(2067,3167,'Crime Stories',NULL,now(),NULL,72955),
(2068,1251,'The Expert',NULL,now(),NULL,72957),
(2069,5745,'The Education of Max Bickford',NULL,now(),NULL,72958),
(2070,1252,'Thursday\'s Child',NULL,now(),NULL,72960),
(2071,1253,'The Witch\'s Daughter',NULL,now(),NULL,72961),
(2072,3907,'Hot Wheels Highway 35',NULL,now(),NULL,72962),
(2073,3435,'Emergency!',NULL,now(),NULL,72963),
(2074,1256,'Blott on the Landscape',NULL,now(),NULL,72964),
(2075,10696,'Perfect Match (Australia)',NULL,now(),NULL,72966),
(2076,7427,'Wheel of Fortune (Australia)',NULL,now(),NULL,72967),
(2077,1257,'Enchanted Musical Playhouse',NULL,now(),NULL,72968),
(2078,1258,'Storybook Cinema',NULL,now(),NULL,72970),
(2079,1259,'1MMs',NULL,now(),NULL,72971),
(2080,1260,'Mental Block',NULL,now(),NULL,72972),
(2081,5491,'Tenacious D',NULL,now(),NULL,72974),
(2082,1261,'Karaoke',NULL,now(),NULL,72976),
(2083,1263,'I Love the 70s',NULL,now(),NULL,72978),
(2084,4880,'Premium Blend',NULL,now(),NULL,72979),
(2085,1265,'Tying the Knot',NULL,now(),NULL,72980),
(2086,156,'Grange Hill',NULL,now(),NULL,72981),
(2087,7296,'Yellowthread Street',NULL,now(),NULL,72982),
(2088,12443,'TRL (UK)',NULL,now(),NULL,72983),
(2089,1266,'Head 2 Toe',NULL,now(),NULL,72984),
(2090,1267,'Revolver',NULL,now(),NULL,72985),
(2091,18293,'Shell Game',NULL,now(),NULL,72986),
(2092,5653,'The Bronx Zoo',NULL,now(),NULL,72987),
(2093,12102,'Fresh Gear',NULL,now(),NULL,72988),
(2094,11733,'Getaway',NULL,now(),NULL,72989),
(2095,3718,'Good Morning, Miss Bliss',NULL,now(),NULL,72990),
(2096,1269,'Annette',NULL,now(),NULL,72991),
(2097,157,'Minder',NULL,now(),NULL,72993),
(2098,21502,'Tech Live',NULL,now(),NULL,72994),
(2099,158,'Rumpole of the Bailey',NULL,now(),NULL,72995),
(2100,5841,'The Hardy Boys (1956)',NULL,now(),NULL,72996),
(2101,17837,'Dickens of London',NULL,now(),NULL,72997),
(2102,4350,'Making the Grade',NULL,now(),NULL,72998),
(2103,19707,'Brunch',NULL,now(),NULL,73000),
(2104,3187,'Cupid (2003)',NULL,now(),NULL,73001),
(2105,4258,'Little Men',NULL,now(),NULL,73002),
(2106,10225,'Lonesome Dove',NULL,now(),NULL,80305),
(2107,9095,'The All New Alexei Sayle Show',NULL,now(),NULL,73004),
(2108,6097,'The Plastic Man Comedy Adventure Show',NULL,now(),NULL,73005),
(2109,4233,'Life Begins',NULL,now(),NULL,73006),
(2110,5909,'The Larry Sanders Show',NULL,now(),NULL,73007),
(2111,3068,'Citizen Baines',NULL,now(),NULL,73008),
(2112,1271,'BBC Television Shakespeare',NULL,now(),NULL,73009),
(2113,3950,'Immortal Grand Prix',NULL,now(),NULL,73011),
(2114,1272,'Pure 24',NULL,now(),NULL,73012),
(2115,1273,'The Final Solution',NULL,now(),NULL,73013),
(2116,8484,'The Crystal Maze',NULL,now(),NULL,79817),
(2117,5169,'She-Ra: Princess of Power',NULL,now(),NULL,73015),
(2118,4041,'Jayce and the Wheeled Warriors',NULL,now(),NULL,73016),
(2119,1275,'The Henderson Kids II',NULL,now(),NULL,73017),
(2120,1276,'Crashburn',NULL,now(),NULL,73019),
(2121,1277,'The Baldy Man',NULL,now(),NULL,73020),
(2122,4718,'One Foot in the Grave',NULL,now(),NULL,73021),
(2123,1278,'Pig City',NULL,now(),NULL,73022),
(2124,4954,'ReBoot',NULL,now(),NULL,73025),
(2125,1279,'Harry & Cosh',NULL,now(),NULL,73026),
(2126,1280,'Street Sharks',NULL,now(),NULL,73027),
(2127,5882,'The Jackie Thomas Show',NULL,now(),NULL,73030),
(2128,15653,'Willy',NULL,now(),NULL,73032),
(2129,1283,'Rosemary and Thyme',NULL,now(),NULL,73033),
(2130,5617,'The Big Easy',NULL,now(),NULL,73034),
(2131,3446,'Enemy at the Door',NULL,now(),NULL,73036),
(2132,1285,'Performing As',NULL,now(),NULL,73037),
(2133,1287,'Macross',NULL,now(),NULL,73038),
(2134,6442,'Tutenstein',NULL,now(),NULL,73039),
(2135,4079,'Judge Joe Brown',NULL,now(),NULL,73040),
(2136,14938,'Tinseltown TV',NULL,now(),NULL,73042),
(2137,6506,'Veronica Clare',NULL,now(),NULL,73043),
(2138,1289,'You\'re A Star',NULL,now(),NULL,73044),
(2139,1290,'Eyes of Amyphysto',NULL,now(),NULL,73045),
(2140,7895,'All Aboard',NULL,now(),NULL,73046),
(2141,9057,'Turn Out the Lights',NULL,now(),NULL,73048),
(2142,8770,'It\'s Murder, but Is It Art?',NULL,now(),NULL,73049),
(2143,5849,'The Hidden Room',NULL,now(),NULL,73050),
(2144,8950,'Potter',NULL,now(),NULL,73052),
(2145,8047,'A. J. Wentworth, B. A.',NULL,now(),NULL,73053),
(2146,159,'The Joey Bishop Show (1961)',NULL,now(),NULL,73056),
(2147,1294,'Terri McIntyre - Classy Bitch',NULL,now(),NULL,73057),
(2148,4365,'Manimal',NULL,now(),NULL,73058),
(2149,4367,'Mann & Machine',NULL,now(),NULL,73059),
(2150,160,'She Wolf of London',NULL,now(),NULL,73060),
(2151,4639,'Nero Wolfe (1981)',NULL,now(),NULL,73061),
(2152,1295,'Kenny vs. Spenny',NULL,now(),NULL,73062),
(2153,2453,'3 Non Blondes',NULL,now(),NULL,73063),
(2154,4391,'Matlock',NULL,now(),NULL,73064),
(2155,2639,'Archer',NULL,now(),NULL,73065),
(2156,1296,'Mischief City',NULL,now(),NULL,73066),
(2157,1297,'Good Eats',NULL,now(),NULL,73067),
(2158,161,'Nakia',NULL,now(),NULL,73070),
(2159,162,'The Cowboys',NULL,now(),NULL,73071),
(2160,3810,'Hawkins',NULL,now(),NULL,73072),
(2161,1298,'Canterbury Tales',NULL,now(),NULL,73074),
(2162,4021,'Jack & Bobby',NULL,now(),NULL,73075),
(2163,2952,'Cade\'s County',NULL,now(),NULL,73077),
(2164,1299,'Shake, Rattle, and Roll',NULL,now(),NULL,73079),
(2165,4517,'Monster Garage',NULL,now(),NULL,73080),
(2166,5051,'Room 222',NULL,now(),NULL,73081),
(2167,3508,'Family Man (1988)',NULL,now(),NULL,73083),
(2168,10283,'Player$',NULL,now(),NULL,73084),
(2169,8499,'Jackpot',NULL,now(),NULL,73085),
(2170,1301,'Starcade',NULL,now(),NULL,73086),
(2171,163,'The Weakest Link (US)',NULL,now(),NULL,73087),
(2172,18914,'Shoot for the Stars',NULL,now(),NULL,73089),
(2173,18899,'Hollywood\'s Talking',NULL,now(),NULL,73090),
(2174,7180,'The Colgate Comedy Hour',NULL,now(),NULL,73091),
(2175,15704,'The Name\'s the Same',NULL,now(),NULL,73092),
(2176,5443,'Tactical to Practical',NULL,now(),NULL,73093),
(2177,10069,'Dealer\'s Choice',NULL,now(),NULL,73094),
(2178,8551,'Time Team',NULL,now(),NULL,73096),
(2179,3578,'Ford Theatre',NULL,now(),NULL,73097),
(2180,8646,'Big Jim and the Figaro Club',NULL,now(),NULL,73098),
(2181,5629,'The Bob Cummings Show',NULL,now(),NULL,73099),
(2182,164,'The Ford Show',NULL,now(),NULL,73100),
(2183,12626,'The Jimmy Dean Show',NULL,now(),NULL,73101),
(2184,10347,'Face Is Familiar',NULL,now(),NULL,73102),
(2185,3946,'I Love the 80s Strikes Back',NULL,now(),NULL,73105),
(2186,1302,'The Letter People',NULL,now(),NULL,73106),
(2187,6489,'U Pick Live',NULL,now(),NULL,73108),
(2188,4682,'No Job for a Lady',NULL,now(),NULL,73110),
(2189,4840,'Playhouse 90',NULL,now(),NULL,73114),
(2190,165,'The Life of Riley (1953)',NULL,now(),NULL,73115),
(2191,20205,'Pop Idol Extra',NULL,now(),NULL,73116),
(2192,19678,'Music City Tonight',NULL,now(),NULL,73119),
(2193,8600,'Art Linkletter\'s House Party',NULL,now(),NULL,73121),
(2194,8603,'Arthur Murray Party',NULL,now(),NULL,73123),
(2195,9617,'Milligan in ...',NULL,now(),NULL,73124),
(2196,9677,'Oh in Colour',NULL,now(),NULL,73125),
(2197,9423,'Idiot Weekly, Price 2d',NULL,now(),NULL,73126),
(2198,1303,'Wild West C.O.W. Boys of Moo Mesa',NULL,now(),NULL,73127),
(2199,12661,'The George Gobel Show',NULL,now(),NULL,73128),
(2200,22945,'Shower of Stars',NULL,now(),NULL,73129),
(2201,8298,'Lux Playhouse',NULL,now(),NULL,73133),
(2202,15736,'Theatre 625',NULL,now(),NULL,73135),
(2203,1304,'Celebrities Under Pressure',NULL,now(),NULL,73136),
(2204,5740,'The Dukes',NULL,now(),NULL,73138),
(2205,22418,'Pat Boone Chevy Showroom',NULL,now(),NULL,73139),
(2206,1305,'The Pilot Show',NULL,now(),NULL,73140),
(2207,7208,'Victor and Hugo, Bunglers in Crime',NULL,now(),NULL,73143),
(2208,1307,'The Vanishing Man',NULL,now(),NULL,73144),
(2209,10048,'The Celebrity Game',NULL,now(),NULL,73146),
(2210,166,'Shindig',NULL,now(),NULL,73148),
(2211,6477,'Unexplained Mysteries',NULL,now(),NULL,73149),
(2212,3620,'F Troop',NULL,now(),NULL,73154),
(2213,1311,'Mike\'s Super-Short Show',NULL,now(),NULL,73155),
(2214,17937,'Arena (1975)',NULL,now(),NULL,73156),
(2215,1316,'Rude Dog and the Dweebs',NULL,now(),NULL,73158),
(2216,8046,'The Tycoon',NULL,now(),NULL,73159),
(2217,2889,'Bracken\'s World',NULL,now(),NULL,73160),
(2218,5688,'The Collector',NULL,now(),NULL,73161),
(2219,1318,'The Secret Of Lost Creek',NULL,now(),NULL,73162),
(2220,3165,'Creature Comforts',NULL,now(),NULL,73163),
(2221,167,'Supertrain',NULL,now(),NULL,73166),
(2222,1322,'Bodger and Badger',NULL,now(),NULL,73167),
(2223,8509,'A Perfect State',NULL,now(),NULL,73168),
(2224,11250,'Empire Road',NULL,now(),NULL,73169),
(2225,19144,'The Dom DeLuise Show',NULL,now(),NULL,73172),
(2226,1323,'Never the Twain',NULL,now(),NULL,73173),
(2227,15866,'Life and Times',NULL,now(),NULL,73175),
(2228,9364,'The Gravy Train',NULL,now(),NULL,73176),
(2229,6079,'The Pat Sajak Show',NULL,now(),NULL,73177),
(2230,168,'City (1990)',NULL,now(),NULL,73179),
(2231,5602,'The Batman',NULL,now(),NULL,73180),
(2232,3910,'House Hunters',NULL,now(),NULL,73182),
(2233,1328,'Celebrity Treasure Island',NULL,now(),NULL,73183),
(2234,1329,'Popstars (New Zealand)',NULL,now(),NULL,73184),
(2235,1330,'We\'ll Meet Again',NULL,now(),NULL,73186),
(2236,6512,'VH1 Goes Inside',NULL,now(),NULL,73187),
(2237,3943,'I Love 1990\'s',NULL,now(),NULL,73189),
(2238,5484,'Teen Big Brother',NULL,now(),NULL,73190),
(2239,6520,'Viva La Bam',NULL,now(),NULL,73193),
(2240,1332,'Chilly Beach',NULL,now(),NULL,73194),
(2241,10308,'The Howard Stern Show',NULL,now(),NULL,73195),
(2242,1334,'Ultra Maniac',NULL,now(),NULL,73196),
(2243,4851,'Police Academy: The Animated Series',NULL,now(),NULL,73197),
(2244,3955,'Incredible Crash Dummies',NULL,now(),NULL,73198),
(2245,4922,'Quark',NULL,now(),NULL,73199),
(2246,15662,'Hollywood Screen Test',NULL,now(),NULL,73201),
(2247,14971,'I\'m the Answer',NULL,now(),NULL,73203),
(2248,4684,'Norm',NULL,now(),NULL,73204),
(2249,4938,'Rags to Riches',NULL,now(),NULL,73205),
(2250,1338,'Beverly Hills Vet',NULL,now(),NULL,73206),
(2251,9887,'The Magnificent Marble Machine',NULL,now(),NULL,73207),
(2252,7439,'The Who, What or Where Game',NULL,now(),NULL,73208),
(2253,2769,'Be the Creature',NULL,now(),NULL,73209),
(2254,4362,'Manhattan, AZ',NULL,now(),NULL,73212),
(2255,3588,'Four Corners',NULL,now(),NULL,73214),
(2256,5159,'Shaft',NULL,now(),NULL,73215),
(2257,169,'Savannah',NULL,now(),NULL,73216),
(2258,6028,'The New Casper Cartoon Show',NULL,now(),NULL,73218),
(2259,4826,'Pig Sty',NULL,now(),NULL,73219),
(2260,8879,'Fancy Wanders',NULL,now(),NULL,73220),
(2261,6476,'Under Suspicion',NULL,now(),NULL,73221),
(2262,3106,'Cold Pizza',NULL,now(),NULL,73222),
(2263,1341,'The Clinic',NULL,now(),NULL,73223),
(2264,23061,'The Terry and Gaby Show',NULL,now(),NULL,73224),
(2265,6053,'Nutt House',NULL,now(),NULL,73227),
(2266,10868,'Treasure Hunt (1983)',NULL,now(),NULL,73228),
(2267,4886,'Prey',NULL,now(),NULL,73229),
(2268,6188,'The Simple Life',NULL,now(),NULL,73230),
(2269,2468,'A.K.A. Pablo',NULL,now(),NULL,73231),
(2270,10437,'Interceptor',NULL,now(),NULL,73232),
(2271,6484,'University Hospital',NULL,now(),NULL,73235),
(2272,1344,'Rising Stars',NULL,now(),NULL,73238),
(2273,4132,'King of Bandit Jing',NULL,now(),NULL,73239),
(2274,6452,'Twitch City',NULL,now(),NULL,73240),
(2275,15586,'Hunter (1977)',NULL,now(),NULL,73241),
(2276,15585,'The Hunter (1952)',NULL,now(),NULL,73242),
(2277,16706,'The Last Resort (1979)',NULL,now(),NULL,73243),
(2278,4532,'Most Haunted',NULL,now(),NULL,73245),
(2279,2452,'30 Minute Meals',NULL,now(),NULL,73246),
(2280,2925,'Bubblegum Crisis',NULL,now(),NULL,73247),
(2281,5277,'Space Goofs',NULL,now(),NULL,73248),
(2282,4491,'Miss Spider\'s Sunny Patch Friends',NULL,now(),NULL,73249),
(2283,5009,'Rik Mayall Presents',NULL,now(),NULL,73250),
(2284,13128,'The Rat Catchers',NULL,now(),NULL,73251),
(2285,12535,'The Newcomers',NULL,now(),NULL,73252),
(2286,3478,'Eyes',NULL,now(),NULL,73253),
(2287,2830,'Blind Men',NULL,now(),NULL,73254),
(2288,5873,'The Inside',NULL,now(),NULL,73256),
(2289,4635,'Nelson\'s Column',NULL,now(),NULL,73257),
(2290,4130,'King',NULL,now(),NULL,73258),
(2291,6704,'Zoids Fuzors',NULL,now(),NULL,73259),
(2292,5660,'The Bugs Bunny and Tweety Show',NULL,now(),NULL,73260),
(2293,3920,'How to be a Little Sod',NULL,now(),NULL,73261),
(2294,19,'Another Day',NULL,now(),NULL,73262),
(2295,7985,'Top of the Pops (US)',NULL,now(),NULL,73263),
(2296,16555,'Don Kirshner\'s Rock Concert',NULL,now(),NULL,73264),
(2297,22300,'Frost on Sunday',NULL,now(),NULL,73267),
(2298,5792,'The Four Just Men',NULL,now(),NULL,73269),
(2299,4573,'My Dad the Rock Star',NULL,now(),NULL,73270),
(2300,1348,'Sunshine',NULL,now(),NULL,73271),
(2301,17899,'CBS Summer Playhouse',NULL,now(),NULL,73272),
(2302,6513,'VH1 ILL-ustrated',NULL,now(),NULL,73274),
(2303,7317,'You\'re Putting Me On',NULL,now(),NULL,73275),
(2304,4345,'Makeover Mamas',NULL,now(),NULL,73276),
(2305,3020,'Celebrity Poker Showdown',NULL,now(),NULL,73277),
(2306,2577,'A Man Called Hawk',NULL,now(),NULL,73280),
(2307,3166,'Crime & Punishment',NULL,now(),NULL,73281),
(2308,20,'The Defenders',NULL,now(),NULL,73282),
(2309,6334,'This is Wonderland',NULL,now(),NULL,73283),
(2310,22933,'Pop-Up Video',NULL,now(),NULL,73284),
(2311,4820,'Philly',NULL,now(),NULL,73285),
(2312,4974,'Ren and Stimpy Adult Party Cartoon',NULL,now(),NULL,73287),
(2313,10668,'Eggheads',NULL,now(),NULL,73288),
(2314,5503,'That\'s My Bush!',NULL,now(),NULL,73289),
(2315,7101,'60 Minutes',NULL,now(),NULL,73290),
(2316,4787,'Pauly',NULL,now(),NULL,73291),
(2317,7417,'West 57th',NULL,now(),NULL,73292),
(2318,1349,'Meteor Garden',NULL,now(),NULL,73293),
(2319,1350,'Paris',NULL,now(),NULL,73295),
(2320,7553,'A Current Affair (Australia)',NULL,now(),NULL,73296),
(2321,5702,'The Crew',NULL,now(),NULL,73297),
(2322,1351,'Two\'s Company',NULL,now(),NULL,73298),
(2323,6056,'The Oblongs',NULL,now(),NULL,73302),
(2324,9185,'Call My Bluff (US)',NULL,now(),NULL,73304),
(2325,3847,'Higglytown Heroes',NULL,now(),NULL,73305),
(2326,1352,'Essential Poems',NULL,now(),NULL,73307),
(2327,8785,'Misleading Cases',NULL,now(),NULL,73309),
(2328,170,'Filthy Rich',NULL,now(),NULL,73310),
(2329,4647,'Newlyweds (1993)',NULL,now(),NULL,73313),
(2330,2477,'Absolutely Fabulous',NULL,now(),NULL,73314),
(2331,6702,'ZOIDS: Chaotic Century',NULL,now(),NULL,73315),
(2332,4113,'Kenny the Shark',NULL,now(),NULL,73316),
(2333,4230,'Life\'s Work',NULL,now(),NULL,73317),
(2334,1356,'Rick Mercer Report',NULL,now(),NULL,73319),
(2335,1357,'Shirt Tales',NULL,now(),NULL,73321),
(2336,2545,'Aliens in the Family',NULL,now(),NULL,73322),
(2337,1359,'Mix It Up',NULL,now(),NULL,73323),
(2338,2663,'Astronauts (1981)',NULL,now(),NULL,73325),
(2339,7319,'Young Dr. Malone',NULL,now(),NULL,73326),
(2340,171,'Girls on Top',NULL,now(),NULL,73327),
(2341,8576,'Beat the Clock (1950)',NULL,now(),NULL,73328),
(2342,19044,'Beat the Clock (1969)',NULL,now(),NULL,73330),
(2343,4117,'Kevin Hill',NULL,now(),NULL,73331),
(2344,6399,'Tracey Takes On...',NULL,now(),NULL,73332),
(2345,6519,'Visionaries: Knights of the Magical Light',NULL,now(),NULL,73333),
(2346,1360,'TUGS',NULL,now(),NULL,73335),
(2347,1361,'Billy the Cat',NULL,now(),NULL,73336),
(2348,6703,'ZOIDS: Guardian Force',NULL,now(),NULL,73337),
(2349,1363,'House Rules (1998)',NULL,now(),NULL,73338),
(2350,1364,'Sutherland\'s Law',NULL,now(),NULL,73340),
(2351,5587,'The Associates (1979)',NULL,now(),NULL,73341),
(2352,1365,'Lady Lovelylocks and the Pixietails',NULL,now(),NULL,73342),
(2353,5703,'The Critic',NULL,now(),NULL,73344),
(2354,15616,'My Friend Tony',NULL,now(),NULL,73345),
(2355,4229,'Life. . .and Stuff',NULL,now(),NULL,73346),
(2356,4122,'Kid Paddle',NULL,now(),NULL,73347),
(2357,3081,'Claude\'s Crib',NULL,now(),NULL,73349),
(2358,6963,'My Two Wives',NULL,now(),NULL,73351),
(2359,3526,'Fatherhood',NULL,now(),NULL,73352),
(2360,1371,'Living in TV Land',NULL,now(),NULL,73353),
(2361,5053,'Room Raiders',NULL,now(),NULL,73354),
(2362,12275,'Bustin\' Loose',NULL,now(),NULL,73355),
(2363,19035,'Dream Girl of \'67',NULL,now(),NULL,73356),
(2364,16730,'Mr. Smith',NULL,now(),NULL,73359),
(2365,22473,'By Popular Demand',NULL,now(),NULL,73362),
(2366,12472,'Blind Date (1949)',NULL,now(),NULL,73364),
(2367,8597,'Answer Yes or No',NULL,now(),NULL,73367),
(2368,1373,'Coming to the Stage',NULL,now(),NULL,73368),
(2369,7322,'Your Cheatin\' Heart',NULL,now(),NULL,73369),
(2370,4406,'Maya & Miguel',NULL,now(),NULL,73370),
(2371,5282,'Sparks',NULL,now(),NULL,73371),
(2372,10628,'Douglas Fairbanks Jr Presents',NULL,now(),NULL,73372),
(2373,12759,'The Man Behind the Badge',NULL,now(),NULL,73373),
(2374,6673,'Yakkity Yak',NULL,now(),NULL,73374),
(2375,12543,'DuPont Show with June Allyson',NULL,now(),NULL,73376),
(2376,2978,'Captain Nice',NULL,now(),NULL,73379),
(2377,7419,'What a Cartoon',NULL,now(),NULL,73380),
(2378,22411,'The Bobby Goldsboro Show',NULL,now(),NULL,73381),
(2379,10970,'Rollin\' on the River',NULL,now(),NULL,73382),
(2380,172,'The Mary Tyler Moore Hour',NULL,now(),NULL,73383),
(2381,2461,'77 Sunset Strip',NULL,now(),NULL,73384),
(2382,4231,'life as we know it',NULL,now(),NULL,73385),
(2383,2885,'Bourbon Street Beat',NULL,now(),NULL,73386),
(2384,5914,'The Late Late Show with Craig Ferguson',NULL,now(),NULL,73387),
(2385,4883,'Pretty Guardian Sailor Moon: Live Action',NULL,now(),NULL,73389),
(2386,173,'The Marshall Chronicles',NULL,now(),NULL,73390),
(2387,10360,'The Generation Game',NULL,now(),NULL,73391),
(2388,1376,'11 Somerset',NULL,now(),NULL,73392),
(2389,4398,'The Maury Show',NULL,now(),NULL,73393),
(2390,4779,'Party Girl',NULL,now(),NULL,73394),
(2391,3912,'House Rules',NULL,now(),NULL,73395),
(2392,3443,'Encore! Encore!',NULL,now(),NULL,73396),
(2393,1377,'Between the Sheets',NULL,now(),NULL,73397),
(2394,8427,'Brotherly Love (2000)',NULL,now(),NULL,73398),
(2395,2538,'Alias Smith and Jones',NULL,now(),NULL,73399),
(2396,5502,'That\'s Life (1998)',NULL,now(),NULL,73400),
(2397,1379,'Legacy of the Silver Shadow',NULL,now(),NULL,73401),
(2398,16668,'The Funny Side',NULL,now(),NULL,73402),
(2399,6210,'The Starlet',NULL,now(),NULL,73403),
(2400,1380,'How the Stars Get Hot',NULL,now(),NULL,73404),
(2401,5908,'The Krofft Supershow',NULL,now(),NULL,73405),
(2402,3568,'Foley Square',NULL,now(),NULL,73406),
(2403,15873,'Hippodrome',NULL,now(),NULL,73407),
(2404,10659,'The Revlon Revue',NULL,now(),NULL,73409),
(2405,9815,'The Jack Paar Show',NULL,now(),NULL,73410),
(2406,18463,'Foot in the Door',NULL,now(),NULL,73412),
(2407,3241,'Darkroom',NULL,now(),NULL,73413),
(2408,22452,'The Big Party',NULL,now(),NULL,73414),
(2409,1381,'Glynis',NULL,now(),NULL,73415),
(2410,4752,'Out of This World',NULL,now(),NULL,73417),
(2411,4253,'Listen Up',NULL,now(),NULL,73418),
(2412,3022,'Center of the Universe',NULL,now(),NULL,73419),
(2413,3546,'Finder of Lost Loves',NULL,now(),NULL,73420),
(2414,1384,'Billy (1979)',NULL,now(),NULL,73422),
(2415,3462,'Even Stevens',NULL,now(),NULL,73424),
(2416,17738,'The Joey Bishop Show',NULL,now(),NULL,73426),
(2417,3424,'Eerie, Indiana',NULL,now(),NULL,73429),
(2418,5979,'The Michael Richards Show',NULL,now(),NULL,73430),
(2419,10127,'Dan August',NULL,now(),NULL,73431),
(2420,3924,'Hudson Street',NULL,now(),NULL,73432),
(2421,4702,'Ocean Star',NULL,now(),NULL,73436),
(2422,4526,'Moon Over Miami',NULL,now(),NULL,73437),
(2423,4743,'Out All Night',NULL,now(),NULL,73438),
(2424,6256,'The Troubleshooters (1965)',NULL,now(),NULL,73439),
(2425,6282,'The Wayans Bros.',NULL,now(),NULL,73440),
(2426,5749,'The Ellen Burstyn Show',NULL,now(),NULL,73441),
(2427,3434,'Elly & Jools',NULL,now(),NULL,73442),
(2428,1389,'Charles II: The Power and the Passion',NULL,now(),NULL,73443),
(2429,5003,'Rich Girls',NULL,now(),NULL,73445),
(2430,4934,'Rachel Gunn, R.N.',NULL,now(),NULL,73446),
(2431,4135,'King of the Jungle',NULL,now(),NULL,73447),
(2432,17070,'Eye To Eye',NULL,now(),NULL,73448),
(2433,174,'Out of the Blue',NULL,now(),NULL,73449),
(2434,22,'Dr. Kildare',NULL,now(),NULL,73451),
(2435,16728,'Trauma Center',NULL,now(),NULL,73452),
(2436,8380,'Safe And Sound',NULL,now(),NULL,73454),
(2437,10186,'Partners in Crime',NULL,now(),NULL,73455),
(2438,12570,'Love Story',NULL,now(),NULL,73456),
(2439,10567,'The Ray Bolger Show',NULL,now(),NULL,73457),
(2440,1390,'Pugwall',NULL,now(),NULL,73458),
(2441,1391,'Mad About Alice',NULL,now(),NULL,73459),
(2442,1392,'That Was the Week We Watched',NULL,now(),NULL,73460),
(2443,4124,'Kids Incorporated',NULL,now(),NULL,73461),
(2444,4615,'Name Your Adventure',NULL,now(),NULL,73462),
(2445,3958,'Infamous',NULL,now(),NULL,73463),
(2446,1394,'Blue\'s Room',NULL,now(),NULL,73464),
(2447,14652,'Masters Of The Maze',NULL,now(),NULL,73465),
(2448,3205,'Daddy\'s Girls',NULL,now(),NULL,73466),
(2449,5954,'The Lovers',NULL,now(),NULL,73467),
(2450,1395,'Bloomers',NULL,now(),NULL,73468),
(2451,5641,'The Boys are Back',NULL,now(),NULL,73469),
(2452,6689,'Yo Yogi!',NULL,now(),NULL,73471),
(2453,5428,'Sweet Justice',NULL,now(),NULL,73472),
(2454,5318,'Stand by Your Man',NULL,now(),NULL,73474),
(2455,6976,'Your World with Neil Cavuto',NULL,now(),NULL,73476),
(2456,3230,'Daniel Boone',NULL,now(),NULL,73477),
(2457,17289,'Teachers Only',NULL,now(),NULL,73478),
(2458,10855,'Take Your Pick',NULL,now(),NULL,73480),
(2459,10585,'Reggie',NULL,now(),NULL,73481),
(2460,6486,'Unsolved History',NULL,now(),NULL,73482),
(2461,6671,'Xyber 9: New Dawn',NULL,now(),NULL,73483),
(2462,5249,'Something Wilder',NULL,now(),NULL,73484),
(2463,8362,'Down to Earth (1995)',NULL,now(),NULL,73486),
(2464,17074,'Take Five',NULL,now(),NULL,73488),
(2465,5442,'Tabitha',NULL,now(),NULL,73489),
(2466,7002,'Mutual Of Omaha\'s Wild Kingdom',NULL,now(),NULL,73490),
(2467,175,'The Paul Lynde Show',NULL,now(),NULL,73491),
(2468,4741,'Our Miss Brooks',NULL,now(),NULL,73493),
(2469,176,'It\'s a Business',NULL,now(),NULL,73494),
(2470,1404,'Ping-Pong Club',NULL,now(),NULL,73495),
(2471,4346,'Make Room for Daddy',NULL,now(),NULL,73496),
(2472,5146,'Sell This House',NULL,now(),NULL,73497),
(2473,15658,'Guestward Ho!',NULL,now(),NULL,73498),
(2474,1405,'Blue Submarine No. 6',NULL,now(),NULL,73499),
(2475,5724,'The Deputy',NULL,now(),NULL,73500),
(2476,1406,'Di Gi Charat',NULL,now(),NULL,73501),
(2477,1408,'Major League Wrestling: Underground TV',NULL,now(),NULL,73502),
(2478,6047,'The New WKRP in Cincinnati',NULL,now(),NULL,73503),
(2479,177,'She TV',NULL,now(),NULL,73504),
(2480,4712,'On Air With Ryan Seacrest',NULL,now(),NULL,73505),
(2481,6566,'What\'s Happening!!',NULL,now(),NULL,73506),
(2482,178,'Lidsville',NULL,now(),NULL,73510),
(2483,4294,'Love and Marriage',NULL,now(),NULL,73511),
(2484,1413,'ECW Pay-Per-View',NULL,now(),NULL,73512),
(2485,3771,'H.R. Pufnstuf',NULL,now(),NULL,73513),
(2486,1415,'Joshua Jones',NULL,now(),NULL,73514),
(2487,2932,'The Bugaloos',NULL,now(),NULL,73515),
(2488,17864,'Stranger from Space',NULL,now(),NULL,73516),
(2489,1416,'Fireman Sam',NULL,now(),NULL,73517),
(2490,23,'Empire (1962)',NULL,now(),NULL,73519);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(2491,9400,'Headbangers Ball',NULL,now(),NULL,73520),
(2492,3364,'Dr. Shrinker',NULL,now(),NULL,73521),
(2493,6017,'The New Adventures of Gilligan',NULL,now(),NULL,73523),
(2494,16769,'Phyl & Mikhy',NULL,now(),NULL,73524),
(2495,179,'Gilligan\'s Planet',NULL,now(),NULL,73525),
(2496,1422,'Burn Up! eXcess',NULL,now(),NULL,73526),
(2497,4976,'Renegade Press.Com',NULL,now(),NULL,73527),
(2498,5174,'Sherlock Holmes',NULL,now(),NULL,73529),
(2499,10373,'High Rollers',NULL,now(),NULL,73530),
(2500,4433,'Meego',NULL,now(),NULL,73531),
(2501,6108,'Prince Of Tennis',NULL,now(),NULL,73532),
(2502,3115,'Combat!',NULL,now(),NULL,73534),
(2503,180,'Wonderbug',NULL,now(),NULL,73537),
(2504,5449,'Tales From the Cryptkeeper',NULL,now(),NULL,73540),
(2505,4611,'My Wildest Dreams',NULL,now(),NULL,73541),
(2506,14868,'Parallel 9',NULL,now(),NULL,73542),
(2507,5510,'The 13 Ghosts of Scooby Doo',NULL,now(),NULL,73543),
(2508,1432,'Bedsitcom',NULL,now(),NULL,73544),
(2509,5948,'The Lost Saucer',NULL,now(),NULL,73549),
(2510,3822,'Heat Guy J',NULL,now(),NULL,73550),
(2511,16979,'Brenner',NULL,now(),NULL,73552),
(2512,4209,'LAX',NULL,now(),NULL,73553),
(2513,2702,'Baggy Pants and the Nitwits',NULL,now(),NULL,73554),
(2514,3805,'Hawaii',NULL,now(),NULL,73555),
(2515,6445,'TV Land Moguls',NULL,now(),NULL,73556),
(2516,3766,'Gunsmoke',NULL,now(),NULL,73559),
(2517,5299,'Spider-Man Unlimited',NULL,now(),NULL,73560),
(2518,9111,'Ben Elton, the Man from Auntie',NULL,now(),NULL,73561),
(2519,2741,'Beast Wars',NULL,now(),NULL,73562),
(2520,1437,'The Ghost Hunter (UK)',NULL,now(),NULL,73563),
(2521,1439,'Ugetme',NULL,now(),NULL,73564),
(2522,1442,'Ultraman',NULL,now(),NULL,73567),
(2523,6678,'Yogi\'s Treasure Hunt',NULL,now(),NULL,73568),
(2524,4456,'Method & Red',NULL,now(),NULL,73569),
(2525,10105,'Inventions That Changed the World',NULL,now(),NULL,73570),
(2526,6495,'Vampire High',NULL,now(),NULL,73572),
(2527,1447,'The Fosters',NULL,now(),NULL,73573),
(2528,5461,'Tanner \'88',NULL,now(),NULL,73575),
(2529,8830,'The Old Campaigner',NULL,now(),NULL,73576),
(2530,5496,'Thanks',NULL,now(),NULL,73577),
(2531,1449,'Madballs',NULL,now(),NULL,73578),
(2532,5649,'The Brian Benben Show',NULL,now(),NULL,73579),
(2533,9239,'The Comedy Network',NULL,now(),NULL,73581),
(2534,2475,'Abe Burrows\' Almanac',NULL,now(),NULL,73582),
(2535,5830,'The Gregory Hines Show',NULL,now(),NULL,73583),
(2536,1450,'3-2-1 Contact',NULL,now(),NULL,73584),
(2537,1451,'The Great Space Coaster',NULL,now(),NULL,73588),
(2538,1452,'Mathica\'s Mathshop',NULL,now(),NULL,73589),
(2539,4424,'Me and the Boys',NULL,now(),NULL,73590),
(2540,1453,'Read All About It!',NULL,now(),NULL,73591),
(2541,1454,'Sky Commanders',NULL,now(),NULL,73592),
(2542,4059,'Joe\'s Life',NULL,now(),NULL,73594),
(2543,12033,'Turkey Television',NULL,now(),NULL,73595),
(2544,1457,'NFL Game of the Week',NULL,now(),NULL,73597),
(2545,14913,'Man of the World',NULL,now(),NULL,73598),
(2546,3242,'Dark Shadows',NULL,now(),NULL,73599),
(2547,3264,'Dead Man\'s Gun',NULL,now(),NULL,73600),
(2548,1458,'Human Cargo',NULL,now(),NULL,73602),
(2549,4878,'Powers',NULL,now(),NULL,73603),
(2550,13105,'Churchill',NULL,now(),NULL,73604),
(2551,8401,'Charlie & Co.',NULL,now(),NULL,73606),
(2552,1459,'Ricky\'s Room',NULL,now(),NULL,73610),
(2553,4734,'Opposite Sex',NULL,now(),NULL,73611),
(2554,10543,'Friday Island',NULL,now(),NULL,73612),
(2555,10736,'Albert\'s Place',NULL,now(),NULL,73613),
(2556,2534,'Alfred Hitchcock Presents',NULL,now(),NULL,73614),
(2557,3139,'Corner Gas',NULL,now(),NULL,73615),
(2558,3733,'Gravedale High',NULL,now(),NULL,73617),
(2559,4571,'My Big Fat Obnoxious Fianci',NULL,now(),NULL,73618),
(2560,1465,'The Archie Show',NULL,now(),NULL,73619),
(2561,1466,'The Archie Comedy Hour',NULL,now(),NULL,73620),
(2562,1467,'Sabrina and the Groovie Goolies',NULL,now(),NULL,73621),
(2563,5007,'Ricki Lake',NULL,now(),NULL,73622),
(2564,2641,'Archie\'s TV Funnies',NULL,now(),NULL,73623),
(2565,18792,'Fred and Barney Meet the Thing',NULL,now(),NULL,73624),
(2566,3152,'Courthouse',NULL,now(),NULL,73627),
(2567,1469,'Zoobilee Zoo',NULL,now(),NULL,73628),
(2568,5250,'Sonic the Hedgehog',NULL,now(),NULL,73629),
(2569,5543,'The Adventures of Sonic the Hedgehog',NULL,now(),NULL,73630),
(2570,18134,'To Tell the Truth (1980)',NULL,now(),NULL,73631),
(2571,6371,'Today (Australia)',NULL,now(),NULL,73632),
(2572,5251,'Sonic Underground',NULL,now(),NULL,73634),
(2573,9189,'Camp Jim',NULL,now(),NULL,73635),
(2574,1471,'Squirt',NULL,now(),NULL,73637),
(2575,2795,'Big Guy and Rusty the Boy Robot',NULL,now(),NULL,73642),
(2576,1472,'The Voyage of the Mimi',NULL,now(),NULL,73643),
(2577,1473,'Math Wise',NULL,now(),NULL,73644),
(2578,1474,'In Other Words',NULL,now(),NULL,73645),
(2579,6033,'The New Lassie',NULL,now(),NULL,73646),
(2580,1475,'Funky Phantom',NULL,now(),NULL,73647),
(2581,2867,'Boiling Points',NULL,now(),NULL,73648),
(2582,2763,'Bergerac',NULL,now(),NULL,73650),
(2583,5943,'The Look for Less',NULL,now(),NULL,73651),
(2584,182,'Space Rangers',NULL,now(),NULL,73652),
(2585,13919,'Brainiac: History Abuse',NULL,now(),NULL,73653),
(2586,1479,'The Bozo Super Sunday Show',NULL,now(),NULL,73654),
(2587,1480,'Memphis Wrestling',NULL,now(),NULL,73655),
(2588,1481,'The Animal Show',NULL,now(),NULL,73658),
(2589,5220,'Sledge Hammer!',NULL,now(),NULL,79170),
(2590,3915,'How Do I Look',NULL,now(),NULL,73660),
(2591,16813,'H.E.L.P. (1990)',NULL,now(),NULL,73661),
(2592,6707,'Zorro (1990)',NULL,now(),NULL,73662),
(2593,3466,'Everybody Loves Raymond',NULL,now(),NULL,73663),
(2594,2689,'B.L. Stryker',NULL,now(),NULL,73664),
(2595,5494,'Terry Jones\' Medieval Lives',NULL,now(),NULL,73665),
(2596,5746,'The Electric Company',NULL,now(),NULL,73666),
(2597,11023,'Caribe',NULL,now(),NULL,73667),
(2598,7433,'Who Pays?',NULL,now(),NULL,73668),
(2599,6332,'This Is America Charlie Brown',NULL,now(),NULL,73669),
(2600,1483,'Hana Yori Dango',NULL,now(),NULL,73670),
(2601,11056,'The Sam Levenson Show',NULL,now(),NULL,73671),
(2602,5808,'The Genie From Down Under',NULL,now(),NULL,73672),
(2603,18873,'Trivia Trap',NULL,now(),NULL,73673),
(2604,1485,'Early Doors',NULL,now(),NULL,73674),
(2605,1486,'Home Farm Twins',NULL,now(),NULL,73675),
(2606,1488,'College Hill',NULL,now(),NULL,73676),
(2607,1490,'Chocky',NULL,now(),NULL,73677),
(2608,1491,'Todd TV',NULL,now(),NULL,73678),
(2609,6355,'Til Death Do Us Part: Carmen + Dave',NULL,now(),NULL,73679),
(2610,6021,'The New Adventures of Robin Hood',NULL,now(),NULL,73680),
(2611,11510,'Masquerade Party',NULL,now(),NULL,73682),
(2612,6357,'Timecop',NULL,now(),NULL,73684),
(2613,16821,'Khan',NULL,now(),NULL,73685),
(2614,3507,'Family Law',NULL,now(),NULL,73686),
(2615,4495,'Mister T',NULL,now(),NULL,73687),
(2616,18630,'T. and T.',NULL,now(),NULL,73689),
(2617,2510,'After Henry',NULL,now(),NULL,73691),
(2618,3083,'Clean Sweep',NULL,now(),NULL,73692),
(2619,3971,'Inspector Gadget',NULL,now(),NULL,73693),
(2620,11012,'Living it up with Ali & Jack',NULL,now(),NULL,73695),
(2621,3469,'Ewoks',NULL,now(),NULL,73697),
(2622,3830,'Hello Kitty\'s Furry Tale Theater',NULL,now(),NULL,73699),
(2623,7421,'What Happened?',NULL,now(),NULL,73700),
(2624,5223,'Slimer! And the Real Ghostbusters',NULL,now(),NULL,73701),
(2625,183,'Widget, the World Watcher',NULL,now(),NULL,73702),
(2626,6365,'Tiny Toon Adventures',NULL,now(),NULL,73704),
(2627,4899,'Problem Child',NULL,now(),NULL,73705),
(2628,6261,'The Twisted Adventures of Felix the Cat',NULL,now(),NULL,73707),
(2629,4327,'Mad Mad House',NULL,now(),NULL,73708),
(2630,2709,'Bands Reunited',NULL,now(),NULL,73709),
(2631,1498,'Boohbah',NULL,now(),NULL,73712),
(2632,12834,'Disco',NULL,now(),NULL,73713),
(2633,9483,'Strictly T-T',NULL,now(),NULL,73714),
(2634,184,'Project G.eeK.eR.',NULL,now(),NULL,73715),
(2635,4329,'Mad Mike and Mark',NULL,now(),NULL,73716),
(2636,4085,'Jungle Cubs',NULL,now(),NULL,73717),
(2637,1499,'Hollywood Justice',NULL,now(),NULL,73718),
(2638,4799,'Pepper Ann',NULL,now(),NULL,73720),
(2639,4237,'Life on a Stick',NULL,now(),NULL,73722),
(2640,4830,'Pinky, Elmyra & the Brain',NULL,now(),NULL,73723),
(2641,12814,'Oil Strike North',NULL,now(),NULL,73724),
(2642,8315,'Life of Shakespeare',NULL,now(),NULL,73725),
(2643,10985,'Rotten TV',NULL,now(),NULL,73726),
(2644,13505,'The Lotus Eaters',NULL,now(),NULL,73727),
(2645,185,'Histeria!',NULL,now(),NULL,73728),
(2646,4429,'Medical Investigation',NULL,now(),NULL,73729),
(2647,3125,'Committed',NULL,now(),NULL,73731),
(2648,4932,'Quintuplets',NULL,now(),NULL,73732),
(2649,5475,'Teacher\'s Pet',NULL,now(),NULL,73733),
(2650,3088,'Clifford the Big Red Dog',NULL,now(),NULL,73734),
(2651,3074,'City of Angels',NULL,now(),NULL,73735),
(2652,186,'The Law & Harry McGraw',NULL,now(),NULL,73736),
(2653,3288,'Denver The Last Dinosaur',NULL,now(),NULL,73737),
(2654,1502,'Lazer Tag Academy',NULL,now(),NULL,73738),
(2655,2807,'Billy',NULL,now(),NULL,73740),
(2656,4985,'Rescue Me',NULL,now(),NULL,73741),
(2657,4261,'Live Shot',NULL,now(),NULL,73742),
(2658,5970,'The Marshal',NULL,now(),NULL,73743),
(2659,7373,'Walking Tall',NULL,now(),NULL,73746),
(2660,4314,'Lush Life',NULL,now(),NULL,73747),
(2661,187,'The Thorns',NULL,now(),NULL,73748),
(2662,4076,'Jonny Zero',NULL,now(),NULL,73751),
(2663,1505,'Milton the Monster',NULL,now(),NULL,73753),
(2664,1506,'Gulf Playhouse',NULL,now(),NULL,73754),
(2665,188,'Bruno the Kid',NULL,now(),NULL,73757),
(2666,7334,'Xchange',NULL,now(),NULL,73758),
(2667,6176,'The Secret Files of the Spy Dogs',NULL,now(),NULL,73759),
(2668,3365,'DR. Vegas',NULL,now(),NULL,73760),
(2669,4177,'Land\'s End',NULL,now(),NULL,73761),
(2670,1510,'Ministry of Mayhem',NULL,now(),NULL,73763),
(2671,4149,'Knock First',NULL,now(),NULL,73764),
(2672,5050,'Room 101',NULL,now(),NULL,73765),
(2673,17076,'Trial and Error (1988)',NULL,now(),NULL,73767),
(2674,6392,'Touched by an Angel',NULL,now(),NULL,73768),
(2675,190,'The Paper Chase',NULL,now(),NULL,73769),
(2676,7429,'Private Benjamin',NULL,now(),NULL,73772),
(2677,16737,'Women in Prison',NULL,now(),NULL,73773),
(2678,4036,'Jake in Progress',NULL,now(),NULL,73774),
(2679,10259,'Raising Miranda',NULL,now(),NULL,73775),
(2680,3947,'I Love The 90\'s',NULL,now(),NULL,73776),
(2681,3036,'Charlie Hoover',NULL,now(),NULL,73777),
(2682,1513,'USA Championship Wrestling',NULL,now(),NULL,73778),
(2683,8486,'Kessler',NULL,now(),NULL,73779),
(2684,7486,'Quick Before They Catch Us',NULL,now(),NULL,73780),
(2685,11171,'Beasts',NULL,now(),NULL,73781),
(2686,2471,'Aaron\'s Way',NULL,now(),NULL,73782),
(2687,15712,'The Investigator',NULL,now(),NULL,73783),
(2688,4148,'Knightwatch',NULL,now(),NULL,73784),
(2689,7259,'21 Beacon Street',NULL,now(),NULL,73785),
(2690,4729,'On the Air',NULL,now(),NULL,73786),
(2691,5076,'Ryan Caulfield: Year One',NULL,now(),NULL,73788),
(2692,4653,'Too Something',NULL,now(),NULL,73789),
(2693,10136,'Mama\'s Boy',NULL,now(),NULL,73790),
(2694,3467,'Everything\'s Relative',NULL,now(),NULL,73791),
(2695,5332,'Starting Over',NULL,now(),NULL,73792),
(2696,5993,'The Mountain',NULL,now(),NULL,73795),
(2697,13181,'Paul Temple',NULL,now(),NULL,73797),
(2698,191,'A New Kind of Family',NULL,now(),NULL,73798),
(2699,21575,'The Tavis Smiley Show',NULL,now(),NULL,73799),
(2700,2456,'55 Degrees North',NULL,now(),NULL,73801),
(2701,19091,'Letters to Laugh-In',NULL,now(),NULL,73803),
(2702,5616,'The Big Bow Wow',NULL,now(),NULL,73805),
(2703,6394,'Touching Evil',NULL,now(),NULL,73806),
(2704,18092,'Love Chain',NULL,now(),NULL,73807),
(2705,5198,'Significant Others (2004)',NULL,now(),NULL,73808),
(2706,4970,'Relic Hunter',NULL,now(),NULL,73810),
(2707,3091,'Clubhouse',NULL,now(),NULL,73811),
(2708,2865,'Body Hits',NULL,now(),NULL,73812),
(2709,1518,'Hi-5',NULL,now(),NULL,73813),
(2710,2798,'Big Man on Campus',NULL,now(),NULL,73814),
(2711,2840,'Blue Collar TV',NULL,now(),NULL,73815),
(2712,6168,'The Scooby Doo Show',NULL,now(),NULL,73817),
(2713,8930,'A Life of Bliss',NULL,now(),NULL,73818),
(2714,5673,'The Catherine Tate Show',NULL,now(),NULL,73819),
(2715,192,'The Ray Stevens Show',NULL,now(),NULL,73822),
(2716,1522,'Moondial',NULL,now(),NULL,73823),
(2717,1523,'Portal',NULL,now(),NULL,73824),
(2718,1524,'Jasper Carrott: Back to the Front',NULL,now(),NULL,73825),
(2719,193,'Solid Gold',NULL,now(),NULL,73826),
(2720,13310,'Interpol Calling',NULL,now(),NULL,73828),
(2721,16846,'The Marriage',NULL,now(),NULL,73829),
(2722,1525,'Lost Treasures of NFL Films',NULL,now(),NULL,73831),
(2723,4728,'On Our Own',NULL,now(),NULL,73832),
(2724,1526,'The Littlest Groom',NULL,now(),NULL,73835),
(2725,13531,'Fox',NULL,now(),NULL,73836),
(2726,1527,'The American Frontier',NULL,now(),NULL,73840),
(2727,3073,'City Life',NULL,now(),NULL,73841),
(2728,1528,'Back to Reality',NULL,now(),NULL,73842),
(2729,11304,'Tonight with Bert Newton',NULL,now(),NULL,73843),
(2730,1529,'Dramarama',NULL,now(),NULL,73844),
(2731,2768,'Comic View',NULL,now(),NULL,73845),
(2732,2767,'Best Week Ever',NULL,now(),NULL,73846),
(2733,194,'A Country Practice',NULL,now(),NULL,73847),
(2734,13309,'Silent Evidence',NULL,now(),NULL,73849),
(2735,1531,'The Kinetic Karnival of Jearl Walker',NULL,now(),NULL,73851),
(2736,3715,'Good Guys Bad Guys',NULL,now(),NULL,73852),
(2737,7684,'Countdown (1974)',NULL,now(),NULL,73855),
(2738,1532,'The Science Alliance',NULL,now(),NULL,73856),
(2739,195,'Police Rescue',NULL,now(),NULL,73859),
(2740,196,'We Got It Made',NULL,now(),NULL,73868),
(2741,3899,'Hope Island',NULL,now(),NULL,73869),
(2742,17060,'Free Country',NULL,now(),NULL,73870),
(2743,197,'Morton & Hayes',NULL,now(),NULL,73872),
(2744,3037,'Charlie Jade',NULL,now(),NULL,73873),
(2745,7464,'K-Club',NULL,now(),NULL,73874),
(2746,15021,'Smart Hart',NULL,now(),NULL,73875),
(2747,4459,'Miami Animal Police',NULL,now(),NULL,73876),
(2748,5370,'Street Hawk',NULL,now(),NULL,73877),
(2749,4841,'Playing It Straight',NULL,now(),NULL,73878),
(2750,11194,'Angel Falls',NULL,now(),NULL,73880),
(2751,1536,'I\'m Famous and Frightened',NULL,now(),NULL,73882),
(2752,3934,'Hypernauts',NULL,now(),NULL,73884),
(2753,15902,'These are the Days',NULL,now(),NULL,73886),
(2754,1537,'Science Ninja Team Gatchaman',NULL,now(),NULL,73887),
(2755,3284,'The Dennis Miller Show',NULL,now(),NULL,73889),
(2756,5256,'Sooty & Co',NULL,now(),NULL,73891),
(2757,1539,'Keeping Mum',NULL,now(),NULL,73895),
(2758,3408,'E!\'s 101',NULL,now(),NULL,73896),
(2759,9875,'That Beryl Marston...!',NULL,now(),NULL,73897),
(2760,9105,'Battle of the Sexes',NULL,now(),NULL,73898),
(2761,1540,'The Impressionable Jon Culshaw',NULL,now(),NULL,73901),
(2762,4134,'King of the Hill',NULL,now(),NULL,73903),
(2763,4110,'Kelly Kelly',NULL,now(),NULL,73904),
(2764,13042,'Holding On',NULL,now(),NULL,73906),
(2765,3238,'Dark Justice',NULL,now(),NULL,73907),
(2766,1542,'Busy Buses',NULL,now(),NULL,73908),
(2767,198,'Mickey Spillane\'s Mike Hammer',NULL,now(),NULL,73909),
(2768,6386,'Top of the Pops Saturday',NULL,now(),NULL,73911),
(2769,12035,'Against the Odds',NULL,now(),NULL,73913),
(2770,13397,'Judge Dee',NULL,now(),NULL,73917),
(2771,3623,'Full House (1985)',NULL,now(),NULL,73919),
(2772,8357,'Encyclopedia Brown',NULL,now(),NULL,73920),
(2773,1544,'The Arrows Show',NULL,now(),NULL,73921),
(2774,5596,'The Bad Girl\'s Guide',NULL,now(),NULL,73922),
(2775,5858,'The Honeymooners',NULL,now(),NULL,73923),
(2776,1545,'Quadratics',NULL,now(),NULL,73924),
(2777,1546,'Beast Wars Neo',NULL,now(),NULL,73925),
(2778,4267,'Living with Fran',NULL,now(),NULL,73926),
(2779,1547,'Crashbox',NULL,now(),NULL,73927),
(2780,9448,'Ours is a Nice House',NULL,now(),NULL,73929),
(2781,5454,'Tales of The Wizard of Oz',NULL,now(),NULL,73930),
(2782,8831,'On the House',NULL,now(),NULL,73931),
(2783,3654,'General Hospital (1972)',NULL,now(),NULL,73932),
(2784,1548,'Crash Test',NULL,now(),NULL,73933),
(2785,3990,'Invent This!',NULL,now(),NULL,73936),
(2786,1550,'Nick Arcade',NULL,now(),NULL,73937),
(2787,11593,'Lift Off',NULL,now(),NULL,73938),
(2788,7860,'Within These Walls',NULL,now(),NULL,73939),
(2789,1551,'The Manly Bee',NULL,now(),NULL,73941),
(2790,3354,'Down to Earth (1984)',NULL,now(),NULL,73945),
(2791,1552,'Corneil & Bernie',NULL,now(),NULL,73950),
(2792,15872,'Here We Go Again',NULL,now(),NULL,73951),
(2793,1553,'Dragons, Wagons & Wax',NULL,now(),NULL,73954),
(2794,199,'Hello, Larry',NULL,now(),NULL,73957),
(2795,8903,'Just a Gigolo',NULL,now(),NULL,73958),
(2796,1555,'The Big Read',NULL,now(),NULL,73959),
(2797,2982,'Captain Simian & The Space Monkeys',NULL,now(),NULL,73960),
(2798,7315,'You\'re On Your Own',NULL,now(),NULL,73961),
(2799,14933,'A Man of Our Times',NULL,now(),NULL,73963),
(2800,4765,'Pacific Palisades',NULL,now(),NULL,73966),
(2801,2922,'Brush Strokes',NULL,now(),NULL,73967),
(2802,1558,'Catterick',NULL,now(),NULL,73968),
(2803,7295,'A Year at the Top',NULL,now(),NULL,73969),
(2804,8722,'Have I Got You ... Where You Want Me?',NULL,now(),NULL,73970),
(2805,8728,'Her Majesty\'s Pleasure',NULL,now(),NULL,73971),
(2806,3977,'In the Beginning',NULL,now(),NULL,73972),
(2807,16776,'Dorothy',NULL,now(),NULL,73973),
(2808,8988,'Baby Makes Five',NULL,now(),NULL,73974),
(2809,10587,'Woman with a Past',NULL,now(),NULL,73975),
(2810,2736,'Beakman\'s World',NULL,now(),NULL,73976),
(2811,12544,'Private Screenings',NULL,now(),NULL,73977),
(2812,200,'Heartbreak High',NULL,now(),NULL,73979),
(2813,3536,'Felicity',NULL,now(),NULL,73980),
(2814,2528,'Aladdin',NULL,now(),NULL,73981),
(2815,1559,'The Space Explorers',NULL,now(),NULL,73982),
(2816,5187,'Shortland Street',NULL,now(),NULL,73983),
(2817,6141,'The Road to Stardom with Missy Elliott',NULL,now(),NULL,73984),
(2818,6663,'WWF Wrestling Challenge',NULL,now(),NULL,73985),
(2819,18774,'Secrets of the Dead',NULL,now(),NULL,73986),
(2820,16657,'Tammy',NULL,now(),NULL,73987),
(2821,4218,'Leg Work',NULL,now(),NULL,73991),
(2822,8715,'Rosie',NULL,now(),NULL,73992),
(2823,9445,'Sykes and a...',NULL,now(),NULL,73994),
(2824,6137,'The Richard Pryor Show',NULL,now(),NULL,73995),
(2825,9041,'Sorry, I\'m a Stranger Here Myself',NULL,now(),NULL,73996),
(2826,3058,'Chop Cut Rebuild',NULL,now(),NULL,73998),
(2827,1561,'Mr Don & Mr George',NULL,now(),NULL,73999),
(2828,201,'The Marsha Warfield Show',NULL,now(),NULL,74000),
(2829,1562,'Massive Landmarks Of The Twentieth Century',NULL,now(),NULL,74001),
(2830,1563,'Monday Night Football',NULL,now(),NULL,74002),
(2831,202,'If Not for You',NULL,now(),NULL,74003),
(2832,1564,'The Borderers',NULL,now(),NULL,74005),
(2833,16836,'Boss Lady',NULL,now(),NULL,74006),
(2834,6122,'The Real Gilligan\'s Island',NULL,now(),NULL,74008),
(2835,5375,'Stressed Eric',NULL,now(),NULL,74009),
(2836,8408,'The Art Maker',NULL,now(),NULL,74016),
(2837,4867,'Popular',NULL,now(),NULL,74018),
(2838,1565,'GED on TV',NULL,now(),NULL,74019),
(2839,1566,'Learn to Read',NULL,now(),NULL,74020),
(2840,7369,'Hot Ticket',NULL,now(),NULL,74022),
(2841,22471,'Broadway Spotlight',NULL,now(),NULL,74024),
(2842,22472,'Buzzy Wuzzy',NULL,now(),NULL,74026),
(2843,3459,'Eureka! (1981)',NULL,now(),NULL,74031),
(2844,1568,'It Figures',NULL,now(),NULL,74032),
(2845,2530,'Albert and Victoria',NULL,now(),NULL,74034),
(2846,8629,'Alcock and Gander',NULL,now(),NULL,74035),
(2847,13151,'Cameo Theatre',NULL,now(),NULL,74037),
(2848,9190,'Can Do',NULL,now(),NULL,74038),
(2849,5669,'The Captain and Tennille',NULL,now(),NULL,74039),
(2850,9194,'Captain Billy\'s Mississippi Music Hall',NULL,now(),NULL,74040),
(2851,8479,'Comedy Connections',NULL,now(),NULL,74042),
(2852,2524,'Airport (UK)',NULL,now(),NULL,74043),
(2853,22298,'Cavalcade of Bands',NULL,now(),NULL,74047),
(2854,10027,'Bob\'s Your Uncle',NULL,now(),NULL,74048),
(2855,22486,'Chance of a Lifetime',NULL,now(),NULL,74049),
(2856,3484,'Fabulous Life of',NULL,now(),NULL,74050),
(2857,13513,'The Plane Makers',NULL,now(),NULL,74051),
(2858,6096,'The PJs',NULL,now(),NULL,74052),
(2859,10343,'The John Gary Show',NULL,now(),NULL,74053),
(2860,18944,'China Smith',NULL,now(),NULL,74055),
(2861,14483,'The Best of the Post',NULL,now(),NULL,74056),
(2862,8952,'The Prince of Denmark',NULL,now(),NULL,74057),
(2863,22498,'The Cliff Edwards Show',NULL,now(),NULL,74059),
(2864,18947,'Combat Sergeant',NULL,now(),NULL,74060),
(2865,16833,'Trouble with Father',NULL,now(),NULL,74064),
(2866,1570,'Mr. & Mrs. North',NULL,now(),NULL,74065),
(2867,4791,'Peck\'s Bad Girl',NULL,now(),NULL,74066),
(2868,203,'Hazel',NULL,now(),NULL,74067),
(2869,13411,'King of the River',NULL,now(),NULL,74068),
(2870,13525,'Conversations with Eric Sevareid',NULL,now(),NULL,74069),
(2871,4579,'My Guys',NULL,now(),NULL,74070),
(2872,22402,'Country Style',NULL,now(),NULL,74072),
(2873,22624,'Crash Corrigan\'s Ranch',NULL,now(),NULL,74073),
(2874,19013,'Crime with Father',NULL,now(),NULL,74075),
(2875,8777,'Marriage Lines',NULL,now(),NULL,74078),
(2876,4172,'Lady Blue',NULL,now(),NULL,74079),
(2877,7389,'Waterfront',NULL,now(),NULL,74080),
(2878,5694,'The Contender',NULL,now(),NULL,74081),
(2879,2612,'Angel Street',NULL,now(),NULL,74082),
(2880,7275,'7 Lives X-posed',NULL,now(),NULL,74083),
(2881,5883,'The Jamie Foxx Show',NULL,now(),NULL,74084),
(2882,16589,'My Sister Eileen',NULL,now(),NULL,74085),
(2883,10538,'Concerning Miss Marlowe',NULL,now(),NULL,74087),
(2884,10096,'The Hollywood Squares (1966)',NULL,now(),NULL,74088),
(2885,22509,'Crystal Room',NULL,now(),NULL,74090),
(2886,5712,'The D.A.\'s Man',NULL,now(),NULL,74091),
(2887,3788,'Hardball',NULL,now(),NULL,74093),
(2888,4441,'Megas XLR',NULL,now(),NULL,74095),
(2889,1571,'Skeleton Warriors',NULL,now(),NULL,74097),
(2890,2021,'Creepschool',NULL,now(),NULL,74098),
(2891,3134,'Conspiracies (2001)',NULL,now(),NULL,74099),
(2892,5586,'The Ashlee Simpson Show',NULL,now(),NULL,74101),
(2893,2022,'The Delphi Bureau',NULL,now(),NULL,74102),
(2894,5930,'The Lenny Henry Show (2004)',NULL,now(),NULL,74103),
(2895,6231,'The Swan',NULL,now(),NULL,74104),
(2896,7178,'Rides',NULL,now(),NULL,74105),
(2897,17366,'Casper & the Angels',NULL,now(),NULL,74106),
(2898,5564,'The Amazing Chan and the Chan Clan',NULL,now(),NULL,74107),
(2899,2024,'Roobarb',NULL,now(),NULL,74108),
(2900,5522,'The Addams Family (1973)',NULL,now(),NULL,74109),
(2901,10143,'Paper Moon',NULL,now(),NULL,74110),
(2902,6213,'The Steve Harvey Show',NULL,now(),NULL,74112),
(2903,2913,'Broken Badges',NULL,now(),NULL,74113),
(2904,10080,'Do You Trust Your Wife?',NULL,now(),NULL,74114),
(2905,4566,'Muscle',NULL,now(),NULL,74115),
(2906,12459,'Dollar a Second',NULL,now(),NULL,74116),
(2907,2026,'I am Not an Animal',NULL,now(),NULL,74117),
(2908,2027,'The Keith Barret Show',NULL,now(),NULL,74118),
(2909,2028,'Pulling Moves',NULL,now(),NULL,74119),
(2910,6821,'Timewatch',NULL,now(),NULL,74120),
(2911,4558,'Murder in Suburbia',NULL,now(),NULL,74121),
(2912,2029,'Bachelor Pad',NULL,now(),NULL,74123),
(2913,10076,'Dotto',NULL,now(),NULL,74124),
(2914,4199,'Laurie Hill',NULL,now(),NULL,74126),
(2915,205,'Open House',NULL,now(),NULL,74127),
(2916,22382,'The Eddy Arnold Show',NULL,now(),NULL,74131),
(2917,7440,'The Whoopi Goldberg Show',NULL,now(),NULL,74135),
(2918,7487,'The Dustbinmen',NULL,now(),NULL,74138),
(2919,3082,'Clean House',NULL,now(),NULL,74140),
(2920,8902,'John Browne\'s Body',NULL,now(),NULL,74141),
(2921,2032,'The Supernatural (1977)',NULL,now(),NULL,74143),
(2922,2033,'Final Appeal: From the Files of Unsolved Mysteries',NULL,now(),NULL,74144),
(2923,3586,'For Your Love',NULL,now(),NULL,74145),
(2924,5778,'The Fifth Corner',NULL,now(),NULL,74146),
(2925,10241,'Rap City',NULL,now(),NULL,74147),
(2926,2034,'Pioneer Quest: A Year in the Real West',NULL,now(),NULL,74148),
(2927,2975,'Captain N & the Adventures of Super Mario Bros. 3',NULL,now(),NULL,74149),
(2928,206,'Manchester Prep',NULL,now(),NULL,74151),
(2929,4142,'Kiss Me Kate',NULL,now(),NULL,74155),
(2930,2797,'Big Love',NULL,now(),NULL,74156),
(2931,6542,'Wasteland',NULL,now(),NULL,74157),
(2932,16834,'From a Bird\'s Eye View',NULL,now(),NULL,74159),
(2933,7699,'The Resort',NULL,now(),NULL,74163),
(2934,2035,'Camberwick Green',NULL,now(),NULL,74164),
(2935,2036,'Trumpton',NULL,now(),NULL,74165),
(2936,17756,'Chigley',NULL,now(),NULL,74166),
(2937,8703,'The Gaffer',NULL,now(),NULL,74168),
(2938,8710,'Girls About Town',NULL,now(),NULL,74169),
(2939,2037,'Eagle Riders',NULL,now(),NULL,74170),
(2940,2038,'Murder City',NULL,now(),NULL,74171),
(2941,4270,'Lloyd in Space',NULL,now(),NULL,74174),
(2942,13142,'The Mask of Janus',NULL,now(),NULL,74175),
(2943,8801,'My Good Woman',NULL,now(),NULL,74176),
(2944,8806,'My Son Reuben',NULL,now(),NULL,74177),
(2945,2039,'Big Girl\'s Blouse',NULL,now(),NULL,74178),
(2946,17936,'Newsround',NULL,now(),NULL,74180),
(2947,6326,'The Zeta Project',NULL,now(),NULL,74182),
(2948,8804,'My Old Man',NULL,now(),NULL,74183),
(2949,2041,'Bodies',NULL,now(),NULL,74184),
(2950,3815,'Headland',NULL,now(),NULL,74185),
(2951,3016,'Celebrity Deathmatch',NULL,now(),NULL,74186),
(2952,8891,'For the Love of Ada',NULL,now(),NULL,74187),
(2953,9044,'Spooner\'s Patch',NULL,now(),NULL,74188),
(2954,3323,'House of Mouse',NULL,now(),NULL,74189),
(2955,4205,'Law & Order: Trial by Jury',NULL,now(),NULL,74191),
(2956,15839,'The Chisholms',NULL,now(),NULL,74192),
(2957,13190,'Champion House',NULL,now(),NULL,74193),
(2958,2439,'1st & Ten',NULL,now(),NULL,74194),
(2959,207,'PB&J Otter',NULL,now(),NULL,74195),
(2960,208,'Sapphire and Steel',NULL,now(),NULL,74196),
(2961,5650,'The Brittas Empire',NULL,now(),NULL,74197),
(2962,2834,'Blood of the Samurai: The Series',NULL,now(),NULL,74198),
(2963,2044,'Kikaida-01',NULL,now(),NULL,74199),
(2964,2045,'Inazuman',NULL,now(),NULL,74200),
(2965,3996,'Isidingo',NULL,now(),NULL,74201),
(2966,6366,'Titans',NULL,now(),NULL,74202),
(2967,6460,'UK Top 40',NULL,now(),NULL,74203),
(2968,5188,'Showbiz Moms & Dads',NULL,now(),NULL,74204),
(2969,209,'Man of the People',NULL,now(),NULL,74206),
(2970,2046,'Balamory',NULL,now(),NULL,74207),
(2971,13069,'The First Lady',NULL,now(),NULL,74208),
(2972,4829,'Pimp My Ride',NULL,now(),NULL,74210),
(2973,2941,'Butch Cassidy & The Sundance Kids',NULL,now(),NULL,74211),
(2974,8758,'I Didn\'t Know You Cared',NULL,now(),NULL,74212),
(2975,8181,'Go!',NULL,now(),NULL,74213),
(2976,8759,'I Thought You\'d Gone',NULL,now(),NULL,74214),
(2977,8760,'I Woke Up One Morning',NULL,now(),NULL,74215),
(2978,16659,'The Jean Arthur Show',NULL,now(),NULL,74216),
(2979,20700,'Good Morning Britain',NULL,now(),NULL,74217),
(2980,11220,'Good Morning... with Anne and Nick',NULL,now(),NULL,74219),
(2981,13620,'Look and Read',NULL,now(),NULL,74220),
(2982,2049,'Harry and the Wrinklies',NULL,now(),NULL,74221),
(2983,4689,'Nothing Sacred',NULL,now(),NULL,74222),
(2984,210,'Midnight Caller',NULL,now(),NULL,74223),
(2985,7077,'Hero to Zero',NULL,now(),NULL,74224),
(2986,15943,'Hey Jeannie!',NULL,now(),NULL,74225),
(2987,13393,'Thorndyke',NULL,now(),NULL,74226),
(2988,8761,'If It Moves, File It',NULL,now(),NULL,74229);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(2989,2623,'Anna Lee',NULL,now(),NULL,74230),
(2990,8363,'If You See God, Tell Him',NULL,now(),NULL,74231),
(2991,10680,'The Neighbors',NULL,now(),NULL,74232),
(2992,18895,'Give-n-Take',NULL,now(),NULL,74233),
(2993,9946,'Viva Cabaret',NULL,now(),NULL,74234),
(2994,8683,'Citizen James',NULL,now(),NULL,74235),
(2995,13261,'Castles',NULL,now(),NULL,74237),
(2996,7260,'24 Hour Quiz',NULL,now(),NULL,74238),
(2997,2051,'Roboroach',NULL,now(),NULL,74240),
(2998,2052,'Peep and the Big Wide World',NULL,now(),NULL,74241),
(2999,2054,'The Most Extreme',NULL,now(),NULL,74242),
(3000,2055,'Two in Clover',NULL,now(),NULL,74243),
(3001,6503,'Venom ER',NULL,now(),NULL,74246),
(3002,7070,'CMT Insider',NULL,now(),NULL,74247),
(3003,9574,'Taxi! (1963)',NULL,now(),NULL,74249),
(3004,5424,'Swamp Thing: The Animated Series',NULL,now(),NULL,74250),
(3005,4957,'Recipe for Success',NULL,now(),NULL,74251),
(3006,4017,'I Want a Famous Face',NULL,now(),NULL,74253),
(3007,9012,'Playboy\'s Penthouse',NULL,now(),NULL,74255),
(3008,2058,'Historyonics',NULL,now(),NULL,74264),
(3009,2890,'Brady\'s Beasts',NULL,now(),NULL,74265),
(3010,11951,'That\'s Showbusiness',NULL,now(),NULL,74266),
(3011,3869,'History Detectives',NULL,now(),NULL,74267),
(3012,2731,'Battletoads',NULL,now(),NULL,74268),
(3013,10320,'On the Edge',NULL,now(),NULL,74269),
(3014,2059,'My Life in Film',NULL,now(),NULL,74272),
(3015,16876,'Scientific American Frontiers',NULL,now(),NULL,74273),
(3016,4988,'Return to Eden',NULL,now(),NULL,74274),
(3017,12816,'Top Secret',NULL,now(),NULL,74275),
(3018,2897,'Brat Camp',NULL,now(),NULL,74277),
(3019,2060,'Angels',NULL,now(),NULL,74278),
(3020,13354,'Dial M for Murder',NULL,now(),NULL,74279),
(3021,3097,'Code Lyoko',NULL,now(),NULL,74280),
(3022,3804,'Have I Got News for You',NULL,now(),NULL,74281),
(3023,9739,'Pulp Video',NULL,now(),NULL,74282),
(3024,4903,'Project Runway',NULL,now(),NULL,74285),
(3025,5000,'Richard & Judy',NULL,now(),NULL,74286),
(3026,2895,'Brandy & Mr. Whiskers',NULL,now(),NULL,74289),
(3027,2061,'Spearhead',NULL,now(),NULL,74297),
(3028,211,'Hey Hey! It\'s Saturday',NULL,now(),NULL,74298),
(3029,7375,'Wall $treet Week',NULL,now(),NULL,74301),
(3030,13104,'Light Lunch',NULL,now(),NULL,74303),
(3031,2063,'Jinzo Ningen Kikaida',NULL,now(),NULL,74304),
(3032,2064,'The Timmy Toodles Tribute Show',NULL,now(),NULL,74307),
(3033,8713,'The Gnomes of Dulwich',NULL,now(),NULL,74308),
(3034,3384,'Drew Carey\'s Green Screen Show',NULL,now(),NULL,74311),
(3035,2067,'The Revenue Men',NULL,now(),NULL,74313),
(3036,9594,'Marlene Marlowe Investigates',NULL,now(),NULL,74314),
(3037,2570,'Almost Perfect',NULL,now(),NULL,74316),
(3038,3878,'Hollyoaks: Movin\' On',NULL,now(),NULL,74317),
(3039,2068,'Hamish and Andy',NULL,now(),NULL,74318),
(3040,12194,'Rocked with Gina Gershon',NULL,now(),NULL,74319),
(3041,13369,'Clive Anderson All Talk',NULL,now(),NULL,74320),
(3042,13113,'The Grand (1997)',NULL,now(),NULL,74321),
(3043,8963,'Rings on their Fingers',NULL,now(),NULL,74322),
(3044,6846,'Icons',NULL,now(),NULL,74324),
(3045,3994,'Iron Chef America',NULL,now(),NULL,74325),
(3046,2657,'Ask This Old House',NULL,now(),NULL,74326),
(3047,2070,'The Mysti Show',NULL,now(),NULL,74328),
(3048,2071,'I\'m Still Alive',NULL,now(),NULL,74329),
(3049,12074,'Saturday Superstore',NULL,now(),NULL,74330),
(3050,10028,'Bognor or Bust',NULL,now(),NULL,74331),
(3051,18180,'The New Adventures of Mighty Mouse and Heckle and Jeckle',NULL,now(),NULL,74332),
(3052,17360,'The C.B. Bears',NULL,now(),NULL,74333),
(3053,3117,'Comedy Central Presents',NULL,now(),NULL,74337),
(3054,9198,'Carol Vorderman\'s Better Homes',NULL,now(),NULL,74339),
(3055,7276,'The 8:15 from Manchester',NULL,now(),NULL,74342),
(3056,6579,'Wheelie and the Chopper Bunch',NULL,now(),NULL,74343),
(3057,2595,'American Dragon: Jake Long',NULL,now(),NULL,74344),
(3058,2073,'Ideal',NULL,now(),NULL,74345),
(3059,212,'Rendez-View',NULL,now(),NULL,74349),
(3060,14833,'The Late Late Breakfast Show',NULL,now(),NULL,74350),
(3061,6843,'Cheat',NULL,now(),NULL,74351),
(3062,2076,'Big Bite',NULL,now(),NULL,74352),
(3063,12958,'Rockliffe\'s Babies',NULL,now(),NULL,74353),
(3064,2077,'Omnibus (1967)',NULL,now(),NULL,74354),
(3065,5607,'The Benefactor',NULL,now(),NULL,74356),
(3066,8783,'The Misfit',NULL,now(),NULL,74357),
(3067,5672,'The Casino',NULL,now(),NULL,74358),
(3068,6311,'The WWE Experience',NULL,now(),NULL,74360),
(3069,4915,'Punky Brewster',NULL,now(),NULL,74362),
(3070,3483,'Fabian of the Yard',NULL,now(),NULL,74367),
(3071,8686,'A Class By Himself',NULL,now(),NULL,74368),
(3072,12116,'Life on Earth',NULL,now(),NULL,74371),
(3073,2084,'Life in the Freezer',NULL,now(),NULL,74373),
(3074,2085,'The Brief',NULL,now(),NULL,74374),
(3075,19337,'An Evening at the Improv',NULL,now(),NULL,74375),
(3076,8367,'Every Silver Lining',NULL,now(),NULL,74376),
(3077,8619,'Time of My Life',NULL,now(),NULL,74377),
(3078,5885,'The Jane Pauley Show',NULL,now(),NULL,74378),
(3079,7864,'Horizon',NULL,now(),NULL,74379),
(3080,13254,'O.S.S.',NULL,now(),NULL,74381),
(3081,8339,'ABC Funfit',NULL,now(),NULL,74385),
(3082,3009,'CBS Storybreak',NULL,now(),NULL,74386),
(3083,2507,'A Fine Romance (1981)',NULL,now(),NULL,74388),
(3084,7316,'You\'re Only Young Twice',NULL,now(),NULL,74389),
(3085,8719,'The Happy Apple',NULL,now(),NULL,74390),
(3086,8706,'Happy Ever After',NULL,now(),NULL,74391),
(3087,4112,'Kenan & Kel',NULL,now(),NULL,74392),
(3088,10870,'Trivial Pursuit',NULL,now(),NULL,74393),
(3089,2774,'Between Brothers',NULL,now(),NULL,74394),
(3090,3883,'Homeboys in Outer Space',NULL,now(),NULL,74395),
(3091,2088,'Life\'s a Bitch',NULL,now(),NULL,74396),
(3092,8817,'The Nineteenth Hole',NULL,now(),NULL,74397),
(3093,3935,'I\'ll Fly Away',NULL,now(),NULL,74398),
(3094,8507,'Now Take My Wife',NULL,now(),NULL,74399),
(3095,4736,'Orleans',NULL,now(),NULL,74400),
(3096,3991,'Investigating History',NULL,now(),NULL,74401),
(3097,3454,'Escape from Jupiter',NULL,now(),NULL,74404),
(3098,11818,'The Don Lane Show',NULL,now(),NULL,74405),
(3099,3512,'Family Ties',NULL,now(),NULL,74409),
(3100,17075,'The Popcorn Kid',NULL,now(),NULL,74410),
(3101,2090,'Marvin the Tap-Dancing Horse',NULL,now(),NULL,74411),
(3102,215,'Insight',NULL,now(),NULL,74412),
(3103,5638,'The Boondocks',NULL,now(),NULL,74413),
(3104,6863,'Backyard Blitz',NULL,now(),NULL,74414),
(3105,216,'The Governor & J.J.',NULL,now(),NULL,74415),
(3106,15861,'Stage 13',NULL,now(),NULL,74417),
(3107,2091,'Knightmare',NULL,now(),NULL,74418),
(3108,4018,'I Want To Be a Hilton',NULL,now(),NULL,74419),
(3109,217,'Beacon Hill',NULL,now(),NULL,74420),
(3110,2092,'Rickety Rocket',NULL,now(),NULL,74421),
(3111,9193,'Capital City',NULL,now(),NULL,74422),
(3112,3951,'Impact: Stories of Survival',NULL,now(),NULL,74424),
(3113,218,'Ball Four',NULL,now(),NULL,74426),
(3114,2093,'TV Land Top 10',NULL,now(),NULL,74427),
(3115,11810,'The Emigrants',NULL,now(),NULL,74428),
(3116,9864,'Lady Killers',NULL,now(),NULL,74429),
(3117,15668,'Mr. Smith Goes to Washington',NULL,now(),NULL,74430),
(3118,2934,'Built to Last',NULL,now(),NULL,74431),
(3119,13155,'Brett',NULL,now(),NULL,74434),
(3120,21236,'Perfect Partner',NULL,now(),NULL,74435),
(3121,6178,'The Secret Lives of Men',NULL,now(),NULL,74437),
(3122,4574,'My Family',NULL,now(),NULL,74438),
(3123,8400,'Hockey Night in Canada',NULL,now(),NULL,74441),
(3124,2535,'Alfred Hitchcock Presents (1985)',NULL,now(),NULL,74443),
(3125,8968,'A Roof Over My Head',NULL,now(),NULL,74445),
(3126,8718,'Hallelujah!',NULL,now(),NULL,74446),
(3127,5967,'The Man Show',NULL,now(),NULL,74447),
(3128,3865,'Hill Street Blues',NULL,now(),NULL,74449),
(3129,7067,'The Tom Green Show',NULL,now(),NULL,74450),
(3130,15643,'Honestly, Celeste!',NULL,now(),NULL,74452),
(3131,219,'Karen (1964)',NULL,now(),NULL,74454),
(3132,10137,'Nancy',NULL,now(),NULL,74456),
(3133,220,'Karen (1975)',NULL,now(),NULL,74458),
(3134,2098,'The Sooty Show',NULL,now(),NULL,74461),
(3135,2099,'Kaze No Yojimbo',NULL,now(),NULL,74462),
(3136,4155,'Kolchak: The Night Stalker',NULL,now(),NULL,74463),
(3137,9597,'I, Max',NULL,now(),NULL,74465),
(3138,4828,'Pilot Season',NULL,now(),NULL,74466),
(3139,3369,'Dragnet (1989)',NULL,now(),NULL,74467),
(3140,6897,'CBS Evening News',NULL,now(),NULL,74468),
(3141,221,'Family Passions',NULL,now(),NULL,74469),
(3142,10657,'On the Cover',NULL,now(),NULL,74470),
(3143,12965,'Merrick & Rosso Unplanned',NULL,now(),NULL,74471),
(3144,4283,'Los Luchadores',NULL,now(),NULL,74473),
(3145,3785,'Happy Days',NULL,now(),NULL,74475),
(3146,2803,'Big Wolf on Campus',NULL,now(),NULL,74477),
(3147,9492,'Jossy\'s Giants',NULL,now(),NULL,74478),
(3148,13324,'Call the Gun Expert',NULL,now(),NULL,74479),
(3149,6618,'Wind at My Back',NULL,now(),NULL,74480),
(3150,2685,'Road to Avonlea',NULL,now(),NULL,74481),
(3151,19148,'The Don Rickles Show',NULL,now(),NULL,74482),
(3152,2788,'Big Brother - Up Late',NULL,now(),NULL,74483),
(3153,3272,'Deep Sea Detectives',NULL,now(),NULL,74485),
(3154,2104,'Restoration',NULL,now(),NULL,74487),
(3155,5557,'The Alfred Hitchcock Hour',NULL,now(),NULL,74488),
(3156,7533,'Better Homes and Gardens',NULL,now(),NULL,74490),
(3157,2108,'The Benny Hill Show (1955)',NULL,now(),NULL,74492),
(3158,8381,'Side By Side',NULL,now(),NULL,74498),
(3159,2111,'Fangface',NULL,now(),NULL,74499),
(3160,13224,'No Exit',NULL,now(),NULL,74503),
(3161,2113,'Fairy Princess Minky Momo',NULL,now(),NULL,74504),
(3162,13242,'Maybury',NULL,now(),NULL,74505),
(3163,7340,'Quiller',NULL,now(),NULL,74506),
(3164,12387,'Trisha',NULL,now(),NULL,74507),
(3165,3510,'Family Plots',NULL,now(),NULL,74509),
(3166,14948,'Behind Closed Doors',NULL,now(),NULL,74510),
(3167,6603,'Wife Swap',NULL,now(),NULL,74512),
(3168,3402,'Dumbo\'s Circus',NULL,now(),NULL,74513),
(3169,5511,'The 1900 House',NULL,now(),NULL,74515),
(3170,7252,'The 1940s House',NULL,now(),NULL,74516),
(3171,13075,'Barlow at Large',NULL,now(),NULL,74517),
(3172,9675,'O.T.T.',NULL,now(),NULL,74518),
(3173,14794,'Vanessa',NULL,now(),NULL,74519),
(3174,4223,'Level 9',NULL,now(),NULL,74520),
(3175,14736,'The Jazz Age',NULL,now(),NULL,74524),
(3176,8592,'White Hunter',NULL,now(),NULL,74525),
(3177,2123,'Denjin Zaboga',NULL,now(),NULL,74526),
(3178,2124,'The Secret Life of Machines',NULL,now(),NULL,74528),
(3179,9121,'Black and Blue',NULL,now(),NULL,74529),
(3180,6195,'The Six Wives of Henry VIII (1970)',NULL,now(),NULL,74530),
(3181,8924,'Leave It to Charlie',NULL,now(),NULL,74532),
(3182,7338,'Queenie\'s Castle',NULL,now(),NULL,74533),
(3183,8632,'All Our Saturdays',NULL,now(),NULL,74534),
(3184,9071,'Alan Young',NULL,now(),NULL,74536),
(3185,22412,'Saturday Night Revue',NULL,now(),NULL,74537),
(3186,2126,'Nazca',NULL,now(),NULL,74539),
(3187,6446,'TV Revolution',NULL,now(),NULL,74541),
(3188,8684,'City Lights',NULL,now(),NULL,74542),
(3189,2128,'The Ultimate Love Test',NULL,now(),NULL,74547),
(3190,2129,'Making Waves',NULL,now(),NULL,74548),
(3191,2130,'Hell\'s Kitchen (UK)',NULL,now(),NULL,74549),
(3192,8741,'How\'s Your Father',NULL,now(),NULL,74555),
(3193,10135,'Ladies in Charge',NULL,now(),NULL,74556),
(3194,2131,'Ego from Mars',NULL,now(),NULL,74557),
(3195,2821,'Black Tie Nights',NULL,now(),NULL,74558),
(3196,14737,'Victorian Scandals',NULL,now(),NULL,74560),
(3197,3013,'Celebrity Blackjack',NULL,now(),NULL,74561),
(3198,12162,'Britain Goes Wild with Bill Oddie',NULL,now(),NULL,74562),
(3199,9478,'The Fall of Eagles',NULL,now(),NULL,74563),
(3200,7370,'The Wackers',NULL,now(),NULL,74564),
(3201,9050,'That\'s Your Funeral',NULL,now(),NULL,74565),
(3202,8655,'Bootsie and Snudge',NULL,now(),NULL,74567),
(3203,8635,'The Army Game',NULL,now(),NULL,74568),
(3204,2132,'Maigret',NULL,now(),NULL,74569),
(3205,8828,'Oh Happy Band!',NULL,now(),NULL,74573),
(3206,5488,'Temperatures Rising',NULL,now(),NULL,74576),
(3207,2133,'Rocket Robin Hood',NULL,now(),NULL,74577),
(3208,4506,'Moesha',NULL,now(),NULL,74578),
(3209,8681,'Chance in a Million',NULL,now(),NULL,74580),
(3210,2784,'BIG!',NULL,now(),NULL,74581),
(3211,6226,'The Super Mario Bros. Super Show!',NULL,now(),NULL,74583),
(3212,13429,'The Befrienders',NULL,now(),NULL,74585),
(3213,6585,'Where on Earth is Carmen Sandiego?',NULL,now(),NULL,74586),
(3214,9013,'Up the Garden Path',NULL,now(),NULL,74587),
(3215,3909,'House Doctor',NULL,now(),NULL,74588),
(3216,18135,'To Tell The Truth',NULL,now(),NULL,74589),
(3217,222,'Frontier Doctor',NULL,now(),NULL,74590),
(3218,223,'The Philco Television Playhouse',NULL,now(),NULL,74591),
(3219,6075,'The Parkers',NULL,now(),NULL,74592),
(3220,4807,'Peter Benchley\'s Amazon',NULL,now(),NULL,74593),
(3221,12608,'Private Investigator',NULL,now(),NULL,74595),
(3222,5955,'The Lucy-Desi Comedy Hour',NULL,now(),NULL,74596),
(3223,4457,'Mew Mew Power',NULL,now(),NULL,74598),
(3224,2136,'Monster',NULL,now(),NULL,74599),
(3225,2137,'Dr. Slump',NULL,now(),NULL,74600),
(3226,2138,'The Toy Castle',NULL,now(),NULL,74602),
(3227,4160,'Krypto: the Superdog',NULL,now(),NULL,74603),
(3228,5059,'Rough Science',NULL,now(),NULL,74607),
(3229,6129,'The Redd Foxx Show',NULL,now(),NULL,74610),
(3230,5863,'The Hughleys',NULL,now(),NULL,74613),
(3231,3153,'Cousin Skeeter',NULL,now(),NULL,74614),
(3232,17034,'Click',NULL,now(),NULL,74615),
(3233,5891,'The Jersey',NULL,now(),NULL,74616),
(3234,2838,'Blow Out',NULL,now(),NULL,74617),
(3235,224,'Ripcord',NULL,now(),NULL,74620),
(3236,14939,'Hothouse',NULL,now(),NULL,74622),
(3237,8901,'Jack of Diamonds',NULL,now(),NULL,74623),
(3238,8365,'Oh, Doctor Beeching!',NULL,now(),NULL,74624),
(3239,8372,'An Actor\'s Life for Me',NULL,now(),NULL,74625),
(3240,6313,'The X Factor',NULL,now(),NULL,74626),
(3241,225,'Pacific Drive',NULL,now(),NULL,74627),
(3242,3686,'Girlfriends',NULL,now(),NULL,74631),
(3243,2140,'Peace Maker Kurogane',NULL,now(),NULL,74633),
(3244,7311,'You Write the Songs',NULL,now(),NULL,74634),
(3245,5006,'Richmond Hill',NULL,now(),NULL,74635),
(3246,5577,'The Animals of Farthing Wood',NULL,now(),NULL,74638),
(3247,5640,'The Boy',NULL,now(),NULL,74639),
(3248,14967,'Cluff',NULL,now(),NULL,74642),
(3249,6485,'Unscripted',NULL,now(),NULL,74643),
(3250,2141,'Growing Pains (1992)',NULL,now(),NULL,74644),
(3251,2142,'Beauty & the Beast (1964)',NULL,now(),NULL,74645),
(3252,5479,'Tech Effect',NULL,now(),NULL,74646),
(3253,6700,'Zoey 101',NULL,now(),NULL,74647),
(3254,8966,'Romany Jones',NULL,now(),NULL,74648),
(3255,8513,'Room Service',NULL,now(),NULL,74649),
(3256,8534,'Meet Corliss Archer',NULL,now(),NULL,74651),
(3257,13500,'The Prime Of Miss Jean Brodie',NULL,now(),NULL,74652),
(3258,2143,'Crossballs',NULL,now(),NULL,74655),
(3259,2144,'The Courtroom',NULL,now(),NULL,74656),
(3260,4436,'Meet the Press',NULL,now(),NULL,74657),
(3261,4703,'Odd Job Jack',NULL,now(),NULL,74659),
(3262,9480,'Fame is the Spur',NULL,now(),NULL,74660),
(3263,3925,'Huff',NULL,now(),NULL,74663),
(3264,6384,'Top Cops',NULL,now(),NULL,74664),
(3265,2146,'Extreme Dodgeball',NULL,now(),NULL,74665),
(3266,4929,'Queer Eye for the Straight Guy (UK)',NULL,now(),NULL,74666),
(3267,2147,'Blankety Blanks (US)',NULL,now(),NULL,74667),
(3268,6438,'Tucker',NULL,now(),NULL,74668),
(3269,3859,'High Tide',NULL,now(),NULL,74669),
(3270,5537,'The Adventures of Pete & Pete',NULL,now(),NULL,74670),
(3271,14204,'The Late Late Show (Ireland)',NULL,now(),NULL,74673),
(3272,18182,'This Man Dawson',NULL,now(),NULL,74674),
(3273,6189,'The Simple Life (1998)',NULL,now(),NULL,74676),
(3274,2152,'Club Reps',NULL,now(),NULL,74677),
(3275,2652,'Arthur',NULL,now(),NULL,74678),
(3276,2958,'California Dreams',NULL,now(),NULL,74680),
(3277,2155,'Report to Murphy',NULL,now(),NULL,74682),
(3278,3819,'Heartbeat',NULL,now(),NULL,74684),
(3279,4700,'O\'Grady',NULL,now(),NULL,74685),
(3280,6418,'Travels in Europe with Rick Steves',NULL,now(),NULL,74690),
(3281,226,'Jessie',NULL,now(),NULL,74692),
(3282,2473,'ABC Showcase',NULL,now(),NULL,74693),
(3283,4504,'Models Inc.',NULL,now(),NULL,74694),
(3284,2157,'Burned',NULL,now(),NULL,74695),
(3285,227,'Sin City Spectacular',NULL,now(),NULL,74698),
(3286,5303,'Spiral Zone',NULL,now(),NULL,74699),
(3287,2853,'Bob and Margaret',NULL,now(),NULL,74700),
(3288,2158,'Bounty Hamster',NULL,now(),NULL,74702),
(3289,3974,'Instant Star',NULL,now(),NULL,74703),
(3290,228,'The Porter Wagoner Show',NULL,now(),NULL,74704),
(3291,6478,'Unfabulous',NULL,now(),NULL,74706),
(3292,2161,'In a Fix',NULL,now(),NULL,74707),
(3293,2163,'I Hate My Job',NULL,now(),NULL,74708),
(3294,3138,'Cops',NULL,now(),NULL,74709),
(3295,5054,'Rose and Maloney',NULL,now(),NULL,74710),
(3296,2166,'The Long Firm',NULL,now(),NULL,74713),
(3297,2999,'Casablanca (1955)',NULL,now(),NULL,74714),
(3298,7385,'Washington Week',NULL,now(),NULL,74715),
(3299,7248,'110% Tony Squires',NULL,now(),NULL,74717),
(3300,2167,'Bomberman Jetters',NULL,now(),NULL,74718),
(3301,16835,'Have Faith',NULL,now(),NULL,74719),
(3302,6462,'Ultimate Film Fanatic',NULL,now(),NULL,74720),
(3303,2169,'Stephen Hawking\'s Universe',NULL,now(),NULL,74725),
(3304,18896,'Pass the Buck (US)',NULL,now(),NULL,74726),
(3305,3914,'Howard Stern: The Teenage Years',NULL,now(),NULL,74727),
(3306,6208,'The South Bank Show',NULL,now(),NULL,74728),
(3307,5365,'Strange World',NULL,now(),NULL,74730),
(3308,2171,'Ghost Sweeper Mikami',NULL,now(),NULL,74731),
(3309,5691,'The Commander',NULL,now(),NULL,74734),
(3310,4704,'Odd Man Out',NULL,now(),NULL,74735),
(3311,6471,'Under Cover',NULL,now(),NULL,74736),
(3312,2174,'Legendz: Yomigaeru Ryuuou Densetsu',NULL,now(),NULL,74737),
(3313,229,'Strike Force',NULL,now(),NULL,74738),
(3314,230,'Executive Suite',NULL,now(),NULL,74740),
(3315,16362,'Curb Appeal',NULL,now(),NULL,74741),
(3316,3926,'Hulk',NULL,now(),NULL,74742),
(3317,2970,'Captain America',NULL,now(),NULL,74743),
(3318,231,'Pink Lady and Jeff',NULL,now(),NULL,74744),
(3319,16636,'Barbara Mandrell and the Mandrell Sisters',NULL,now(),NULL,74745),
(3320,13633,'Wonders In Letterland',NULL,now(),NULL,74746),
(3321,12400,'D.C. Follies',NULL,now(),NULL,74747),
(3322,6329,'Things I Hate About You',NULL,now(),NULL,74748),
(3323,18907,'Time Machine',NULL,now(),NULL,74749),
(3324,232,'Mirror Mirror',NULL,now(),NULL,74750),
(3325,4388,'Masterpiece',NULL,now(),NULL,74751),
(3326,233,'Big Sky',NULL,now(),NULL,74752),
(3327,2181,'Bombshell',NULL,now(),NULL,74753),
(3328,4111,'Kelsey Grammer Presents The Sketch Show',NULL,now(),NULL,74754),
(3329,2901,'Breakers',NULL,now(),NULL,74755),
(3330,6380,'Tommy Sledge, P.I.',NULL,now(),NULL,74756),
(3331,2182,'Zero One (1962)',NULL,now(),NULL,74757),
(3332,4930,'Que Pasa, USA?',NULL,now(),NULL,74758),
(3333,2183,'Island at War',NULL,now(),NULL,74759),
(3334,6417,'Travel Sick',NULL,now(),NULL,74761),
(3335,234,'Crash Palace',NULL,now(),NULL,74762),
(3336,9873,'Lanigan\'s Rabbi',NULL,now(),NULL,74763),
(3337,2818,'Black Scorpion',NULL,now(),NULL,74764),
(3338,2632,'Any Day Now',NULL,now(),NULL,74765),
(3339,9899,'Malibu U',NULL,now(),NULL,74766),
(3340,4744,'Outback Jack',NULL,now(),NULL,74767),
(3341,2185,'The Block',NULL,now(),NULL,74768),
(3342,8798,'Mum\'s Boys',NULL,now(),NULL,74769),
(3343,7586,'Before the Game',NULL,now(),NULL,74771),
(3344,15692,'Gibbsville',NULL,now(),NULL,74772),
(3345,3825,'Heaven Help Us',NULL,now(),NULL,74773),
(3346,5874,'The Insider',NULL,now(),NULL,74774),
(3347,2187,'Pfish and Chip',NULL,now(),NULL,74775),
(3348,2775,'Between the Lions',NULL,now(),NULL,74776),
(3349,13415,'The Venturers',NULL,now(),NULL,74777),
(3350,5868,'The Immortal',NULL,now(),NULL,74778),
(3351,23021,'The Big House (1998)',NULL,now(),NULL,74779),
(3352,6406,'Trading Spouses',NULL,now(),NULL,74780),
(3353,6429,'TRL',NULL,now(),NULL,74781),
(3354,7274,'60 Minute Makeover',NULL,now(),NULL,74785),
(3355,18988,'Life With Father',NULL,now(),NULL,74786),
(3356,7019,'Takeshi\'s Castle',NULL,now(),NULL,74787),
(3357,19568,'Party Starters',NULL,now(),NULL,74788),
(3358,6068,'The Others',NULL,now(),NULL,74789),
(3359,3412,'E/R (1984)',NULL,now(),NULL,74790),
(3360,16905,'Room by Room',NULL,now(),NULL,74791),
(3361,5110,'S Club 7',NULL,now(),NULL,74792),
(3362,2465,'90 Days in Hollywood',NULL,now(),NULL,74793),
(3363,13381,'Unnatural Causes',NULL,now(),NULL,74795),
(3364,3353,'Down and Out in Beverly Hills',NULL,now(),NULL,74797),
(3365,4503,'Model Citizens',NULL,now(),NULL,74798),
(3366,6248,'The Tony Danza Show',NULL,now(),NULL,74800),
(3367,236,'Days Like These',NULL,now(),NULL,74801),
(3368,7290,'Yankeeography',NULL,now(),NULL,74802),
(3369,7244,'10 Things Every Guy Should Experience',NULL,now(),NULL,74803),
(3370,3981,'Into Character',NULL,now(),NULL,74804),
(3371,4601,'Mystery Science Theater 3000',NULL,now(),NULL,74806),
(3372,5595,'The Backyardigans',NULL,now(),NULL,74808),
(3373,13102,'TFI Friday',NULL,now(),NULL,74809),
(3374,2923,'Brutally Normal',NULL,now(),NULL,74811),
(3375,2190,'Family Bonds',NULL,now(),NULL,74812),
(3376,3522,'Fat Actress',NULL,now(),NULL,74813),
(3377,3103,'Cold Case Files',NULL,now(),NULL,74814),
(3378,2963,'Camp Lazlo',NULL,now(),NULL,74816),
(3379,5153,'Sesame Unpaved',NULL,now(),NULL,74817),
(3380,2191,'Moody and Pegg',NULL,now(),NULL,74818),
(3381,3748,'Growing Up Gotti',NULL,now(),NULL,74819),
(3382,2192,'Dancin\' On Air',NULL,now(),NULL,74821),
(3383,3358,'Dr. 90210',NULL,now(),NULL,74823),
(3384,6396,'Town Haul',NULL,now(),NULL,74824),
(3385,2193,'Missing You Already',NULL,now(),NULL,74827),
(3386,2712,'Barbershop',NULL,now(),NULL,74828),
(3387,3436,'Emeril',NULL,now(),NULL,74829),
(3388,2605,'Amish in the City',NULL,now(),NULL,74830),
(3389,2866,'Body Work',NULL,now(),NULL,74831),
(3390,13661,'City Beneath the Sea',NULL,now(),NULL,74832),
(3391,4879,'Praise the Lord',NULL,now(),NULL,74833),
(3392,6358,'Time of Your Life',NULL,now(),NULL,74834),
(3393,2801,'Big Sound',NULL,now(),NULL,74835),
(3394,7495,'British Isles: A Natural History',NULL,now(),NULL,74836),
(3395,2194,'Carrie & Barry',NULL,now(),NULL,74837),
(3396,7079,'Hard Spell',NULL,now(),NULL,74839),
(3397,21800,'Take It Off!',NULL,now(),NULL,74840),
(3398,5366,'Strawberry Shortcake',NULL,now(),NULL,74841),
(3399,16578,'James May\'s 20th Century',NULL,now(),NULL,80346),
(3400,2459,'700 Club',NULL,now(),NULL,74846),
(3401,23202,'Hate',NULL,now(),NULL,74847),
(3402,4980,'Renovate My Family',NULL,now(),NULL,74848),
(3403,5409,'Supernanny',NULL,now(),NULL,74850),
(3404,10784,'Locura de amor',NULL,now(),NULL,74851),
(3405,8645,'Big Boy Now!',NULL,now(),NULL,74853),
(3406,4720,'One Night Stand',NULL,now(),NULL,74855),
(3407,2200,'The Red Hand Gang',NULL,now(),NULL,74856),
(3408,5040,'Rock Star: INXS',NULL,now(),NULL,74858),
(3409,6098,'The Player',NULL,now(),NULL,74860),
(3410,4166,'L. A. Doctors',NULL,now(),NULL,74861),
(3411,10933,'Little Robots',NULL,now(),NULL,74862),
(3412,9910,'We Have Ways of Making You Laugh',NULL,now(),NULL,74863),
(3413,2751,'Beetleborgs',NULL,now(),NULL,74864),
(3414,18874,'Bruce Forsyth\'s Hot Streak',NULL,now(),NULL,74870),
(3415,8063,'The Plainclothesman',NULL,now(),NULL,74871),
(3416,6271,'The Vicar of Dibley',NULL,now(),NULL,74873),
(3417,18849,'Child\'s Play',NULL,now(),NULL,74874),
(3418,4796,'Pennies from Heaven',NULL,now(),NULL,74876),
(3419,5693,'The Complex: Malibu',NULL,now(),NULL,74878),
(3420,2206,'The Lisa Maxwell Show',NULL,now(),NULL,74879),
(3421,16353,'Kilroy',NULL,now(),NULL,74880),
(3422,3207,'DAG',NULL,now(),NULL,74881),
(3423,4719,'One Life to Live',NULL,now(),NULL,74882),
(3424,4077,'Josie and the Pussycats',NULL,now(),NULL,74883),
(3425,5129,'Search',NULL,now(),NULL,74884),
(3426,2207,'Living The Dream',NULL,now(),NULL,74885),
(3427,6569,'What\'s New, Mr. Magoo?',NULL,now(),NULL,74886),
(3428,4725,'One West Waikiki',NULL,now(),NULL,74889),
(3429,4501,'Mobile Suit Gundam SEED Destiny',NULL,now(),NULL,74890),
(3430,5434,'Switch',NULL,now(),NULL,74892),
(3431,3513,'The Fantastic Journey',NULL,now(),NULL,74893),
(3432,13192,'Studio Four',NULL,now(),NULL,74894),
(3433,6162,'The San Pedro Beach Bums',NULL,now(),NULL,74896),
(3434,3828,'Hell\'s Kitchen',NULL,now(),NULL,74897),
(3435,2945,'Buzz',NULL,now(),NULL,74899),
(3436,13471,'Worzel Gummidge Down Under',NULL,now(),NULL,74902),
(3437,24,'Fifteen',NULL,now(),NULL,74903),
(3438,4837,'Platypus Man',NULL,now(),NULL,74904),
(3439,12761,'Norby',NULL,now(),NULL,74906),
(3440,3992,'Invincible Iron Man',NULL,now(),NULL,74907),
(3441,3130,'Con',NULL,now(),NULL,74909),
(3442,11237,'Imagine ...',NULL,now(),NULL,74910),
(3443,2211,'Rock Camp',NULL,now(),NULL,74911),
(3444,10565,'Stump the Schwab',NULL,now(),NULL,74912),
(3445,10047,'Catch Phrase',NULL,now(),NULL,74913),
(3446,239,'Galaxy High School',NULL,now(),NULL,74914),
(3447,5563,'The Amanda Show',NULL,now(),NULL,74916),
(3448,4806,'Pet Alien',NULL,now(),NULL,74917),
(3449,2881,'Boston Common',NULL,now(),NULL,74918),
(3450,8583,'Number 10',NULL,now(),NULL,74919),
(3451,5233,'Snapped',NULL,now(),NULL,74921),
(3452,4529,'Mortal Kombat: Conquest',NULL,now(),NULL,74923),
(3453,8605,'Take My Wife',NULL,now(),NULL,74926),
(3454,4211,'LazyTown',NULL,now(),NULL,74928),
(3455,2213,'Surgical Spirit',NULL,now(),NULL,74929),
(3456,2705,'Banacek',NULL,now(),NULL,74930),
(3457,240,'The Iron Horse',NULL,now(),NULL,74931),
(3458,18883,'The Hot Seat',NULL,now(),NULL,74932),
(3459,2214,'Hime Chan\'s Ribbon',NULL,now(),NULL,74933),
(3460,12571,'The Nurses (1962)',NULL,now(),NULL,74934),
(3461,7328,'You\'re On! (1998)',NULL,now(),NULL,74935),
(3462,241,'Way Out',NULL,now(),NULL,74937),
(3463,5231,'Snagglepuss',NULL,now(),NULL,74938),
(3464,2215,'Best Friend\'s Date',NULL,now(),NULL,74939),
(3465,242,'Hot L Baltimore',NULL,now(),NULL,74940),
(3466,14985,'The Danny Baker Show',NULL,now(),NULL,74941),
(3467,2588,'America\'s Most Wanted',NULL,now(),NULL,74943),
(3468,7293,'A Year at Kew',NULL,now(),NULL,74944),
(3469,2950,'C.O.P.S.',NULL,now(),NULL,74946),
(3470,3108,'Cold Turkey',NULL,now(),NULL,74947),
(3471,18226,'The O. Henry Playhouse',NULL,now(),NULL,74951),
(3472,5675,'Centurions',NULL,now(),NULL,74952),
(3473,4411,'McBride',NULL,now(),NULL,74955),
(3474,12117,'Old Grey Whistle Test',NULL,now(),NULL,74956),
(3475,2219,'Requiem From The Darkness',NULL,now(),NULL,74957),
(3476,7811,'e-Talk Daily',NULL,now(),NULL,74958),
(3477,3047,'Cher',NULL,now(),NULL,74960),
(3478,6204,'The Sonny & Cher Comedy Hour',NULL,now(),NULL,74962),
(3479,2220,'A Thing Called Love',NULL,now(),NULL,74963),
(3480,2718,'Baseball Tonight',NULL,now(),NULL,74966),
(3481,5731,'The Don Rickles Show (1972)',NULL,now(),NULL,74967),
(3482,2221,'The Insiders Guide to Happiness',NULL,now(),NULL,74968),
(3483,4823,'Picket Fences',NULL,now(),NULL,74969),
(3484,3191,'Curse of Dracula',NULL,now(),NULL,74970),
(3485,7285,'Zillion',NULL,now(),NULL,74973),
(3486,6356,'Tilt',NULL,now(),NULL,74974);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(3487,7386,'Waste Not',NULL,now(),NULL,74979),
(3488,3965,'Inside Dish with Rachael Ray',NULL,now(),NULL,74980),
(3489,2226,'Miracle\'s Boys',NULL,now(),NULL,74982),
(3490,3705,'Going to Extremes',NULL,now(),NULL,74986),
(3491,19218,'Brains & Brawn',NULL,now(),NULL,74987),
(3492,2228,'Watt On Earth',NULL,now(),NULL,74989),
(3493,243,'The Busy World of Richard Scarry',NULL,now(),NULL,74992),
(3494,5070,'Running The Halls',NULL,now(),NULL,74994),
(3495,2231,'Cosmos',NULL,now(),NULL,74995),
(3496,2232,'The Marvel Superheroes',NULL,now(),NULL,74996),
(3497,4169,'L.A. Heat',NULL,now(),NULL,74998),
(3498,2233,'Melody of Oblivion',NULL,now(),NULL,75000),
(3499,2234,'Madlax',NULL,now(),NULL,75001),
(3500,2235,'Mamotte Shugogetten',NULL,now(),NULL,75002),
(3501,2239,'Maburaho',NULL,now(),NULL,75003),
(3502,5708,'The Cut',NULL,now(),NULL,75007),
(3503,10996,'The RuPaul Show',NULL,now(),NULL,75008),
(3504,13285,'International Detective',NULL,now(),NULL,75009),
(3505,6064,'The Oprah Winfrey Show',NULL,now(),NULL,75010),
(3506,245,'The Midnight Special',NULL,now(),NULL,75011),
(3507,13189,'The View from Daniel Pike',NULL,now(),NULL,75012),
(3508,4437,'Meet the Royals',NULL,now(),NULL,75013),
(3509,3964,'In Search Of The Partridge Family',NULL,now(),NULL,75014),
(3510,246,'Intimate Portrait',NULL,now(),NULL,75015),
(3511,2242,'Mutant League',NULL,now(),NULL,75016),
(3512,8334,'The Dark Island',NULL,now(),NULL,75017),
(3513,247,'Out of the Box',NULL,now(),NULL,75018),
(3514,10991,'Roxie',NULL,now(),NULL,75020),
(3515,3852,'Highlander: The Raven',NULL,now(),NULL,75021),
(3516,2243,'Wish*a*roo Park',NULL,now(),NULL,75022),
(3517,5372,'Street Legal',NULL,now(),NULL,75023),
(3518,2244,'Bertha',NULL,now(),NULL,75024),
(3519,9187,'Calvin and the Colonel',NULL,now(),NULL,75025),
(3520,21661,'Porno Valley',NULL,now(),NULL,75026),
(3521,3145,'Costello',NULL,now(),NULL,75027),
(3522,3685,'Gimme, Gimme, Gimme',NULL,now(),NULL,75031),
(3523,3006,'Cathouse: The Series',NULL,now(),NULL,75032),
(3524,4073,'Jonathan Creek',NULL,now(),NULL,75033),
(3525,2247,'1 Day With...',NULL,now(),NULL,75034),
(3526,3727,'Grace Under Fire',NULL,now(),NULL,75035),
(3527,3337,'Dog the Bounty Hunter',NULL,now(),NULL,75036),
(3528,4777,'Parkinson',NULL,now(),NULL,75037),
(3529,4016,'I Wanna Be a Soap Star',NULL,now(),NULL,75040),
(3530,2248,'Return to River Cottage',NULL,now(),NULL,75041),
(3531,2249,'River Cottage Forever',NULL,now(),NULL,75042),
(3532,7233,'Barely Famous',NULL,now(),NULL,75044),
(3533,8104,'Double Dare (1976)',NULL,now(),NULL,75046),
(3534,2251,'Moccasin Flats',NULL,now(),NULL,75047),
(3535,5033,'Robson Arms',NULL,now(),NULL,75048),
(3536,22180,'Real Sports with Bryant Gumbel',NULL,now(),NULL,75049),
(3537,16656,'Mona McCluskey',NULL,now(),NULL,75051),
(3538,2891,'Brainiac: Science Abuse',NULL,now(),NULL,75052),
(3539,5932,'The Life and Times of Grizzly Adams',NULL,now(),NULL,75054),
(3540,3679,'Gideon\'s Crossing',NULL,now(),NULL,75055),
(3541,5293,'Spicy City',NULL,now(),NULL,75056),
(3542,2253,'Totally Obsessed',NULL,now(),NULL,75060),
(3543,4444,'Men, Women & Dogs',NULL,now(),NULL,75061),
(3544,2255,'Coronation Street Family Album',NULL,now(),NULL,75064),
(3545,5123,'Sea Hunt',NULL,now(),NULL,75065),
(3546,3176,'Crossing Over with John Edward',NULL,now(),NULL,75066),
(3547,6082,'The Paul O\'Grady Show',NULL,now(),NULL,75068),
(3548,9886,'The Mackinnons',NULL,now(),NULL,75069),
(3549,10993,'The Ruby Wax Show',NULL,now(),NULL,75071),
(3550,4856,'Politically Incorrect',NULL,now(),NULL,75073),
(3551,14035,'The New Hollywood Squares',NULL,now(),NULL,75074),
(3552,7061,'Conviction',NULL,now(),NULL,75075),
(3553,2257,'Oh Yeah! Cartoons',NULL,now(),NULL,75076),
(3554,2258,'Popotan',NULL,now(),NULL,75077),
(3555,4525,'Moonlighting',NULL,now(),NULL,75078),
(3556,4733,'Operation Petticoat',NULL,now(),NULL,75079),
(3557,16707,'Struck by Lightning',NULL,now(),NULL,75080),
(3558,2259,'Sniz & Fondue',NULL,now(),NULL,75082),
(3559,2260,'Bro\'Town',NULL,now(),NULL,75084),
(3560,6696,'Zeroman',NULL,now(),NULL,75087),
(3561,4198,'Late Show with David Letterman',NULL,now(),NULL,75088),
(3562,6444,'T.V. Funhouse',NULL,now(),NULL,75089),
(3563,248,'Boot Camp',NULL,now(),NULL,75090),
(3564,4781,'Pasadena',NULL,now(),NULL,75091),
(3565,4773,'Paranoia Agent',NULL,now(),NULL,75092),
(3566,4228,'Life & Style',NULL,now(),NULL,75094),
(3567,4875,'PoweR Girls',NULL,now(),NULL,75095),
(3568,19612,'Showcase \'68',NULL,now(),NULL,75096),
(3569,7017,'In the Moment',NULL,now(),NULL,75098),
(3570,2263,'The Comedy Company',NULL,now(),NULL,75102),
(3571,249,'The Tripods',NULL,now(),NULL,75104),
(3572,2265,'I Dream',NULL,now(),NULL,75105),
(3573,2266,'Leader Dog',NULL,now(),NULL,75106),
(3574,18926,'Wordplay',NULL,now(),NULL,75107),
(3575,19615,'Monopoly',NULL,now(),NULL,75108),
(3576,8825,'Now And Then',NULL,now(),NULL,75110),
(3577,2268,'I\'ll / CKBC',NULL,now(),NULL,75111),
(3578,2269,'Dr. Slump (1981)',NULL,now(),NULL,75112),
(3579,7339,'The Quiz Kids Challenge',NULL,now(),NULL,75115),
(3580,17788,'Antiques Roadshow (US)',NULL,now(),NULL,75117),
(3581,2630,'Your Movie Show',NULL,now(),NULL,75118),
(3582,3543,'Film School',NULL,now(),NULL,75119),
(3583,5284,'Special Branch',NULL,now(),NULL,75123),
(3584,6111,'The Protectors',NULL,now(),NULL,75124),
(3585,14847,'The Robert MacNeil Report',NULL,now(),NULL,75126),
(3586,250,'Jason King',NULL,now(),NULL,75127),
(3587,3123,'Commander In Chief',NULL,now(),NULL,75129),
(3588,2959,'Callan',NULL,now(),NULL,75132),
(3589,6158,'The Saint',NULL,now(),NULL,75134),
(3590,2981,'Captain Scarlet',NULL,now(),NULL,75135),
(3591,3610,'French & Saunders',NULL,now(),NULL,75139),
(3592,2278,'Doraemon',NULL,now(),NULL,75141),
(3593,6902,'The Evidence',NULL,now(),NULL,75142),
(3594,15026,'Rolf on Art',NULL,now(),NULL,75144),
(3595,4822,'Piccadilly Palace',NULL,now(),NULL,75145),
(3596,2279,'Vegetable Soup',NULL,now(),NULL,75146),
(3597,251,'Tenko',NULL,now(),NULL,75147),
(3598,4460,'Miami Slice',NULL,now(),NULL,75149),
(3599,18056,'Button Moon',NULL,now(),NULL,75150),
(3600,13356,'Connie',NULL,now(),NULL,75151),
(3601,252,'Cluedo',NULL,now(),NULL,75153),
(3602,2281,'You\'ve Got A Friend',NULL,now(),NULL,75154),
(3603,3861,'Hi Hi Puffy AmiYumi',NULL,now(),NULL,75159),
(3604,2282,'Killinaskully',NULL,now(),NULL,75163),
(3605,5094,'Samurai Jack',NULL,now(),NULL,75164),
(3606,5618,'The Biggest Loser',NULL,now(),NULL,75166),
(3607,3525,'Father Dowling Mysteries',NULL,now(),NULL,75167),
(3608,10760,'Nana',NULL,now(),NULL,80742),
(3609,5381,'Student Bodies',NULL,now(),NULL,75169),
(3610,5529,'The Adventures of Hyperman',NULL,now(),NULL,75170),
(3611,3297,'Detention',NULL,now(),NULL,75171),
(3612,2283,'Super President',NULL,now(),NULL,75172),
(3613,7443,'Why Don\'t You?',NULL,now(),NULL,75173),
(3614,6092,'The Pink Panther and Sons',NULL,now(),NULL,75174),
(3615,253,'Dangerous Women',NULL,now(),NULL,75175),
(3616,16742,'The Oddball Couple',NULL,now(),NULL,75179),
(3617,2284,'Trucks',NULL,now(),NULL,75180),
(3618,4512,'Mondo Magic',NULL,now(),NULL,75181),
(3619,5917,'The Law Firm',NULL,now(),NULL,75183),
(3620,4906,'Proof Positive',NULL,now(),NULL,75185),
(3621,9038,'Son of Fred',NULL,now(),NULL,75187),
(3622,9028,'A Show Called Fred',NULL,now(),NULL,75188),
(3623,2980,'Captain Power and the Soldiers of the Future',NULL,now(),NULL,75189),
(3624,9634,'My Life as a Popat',NULL,now(),NULL,75191),
(3625,13325,'Catch Hand',NULL,now(),NULL,75193),
(3626,4772,'Parallax',NULL,now(),NULL,75194),
(3627,3813,'He\'s A Lady',NULL,now(),NULL,75196),
(3628,6500,'Vanishing Son',NULL,now(),NULL,75197),
(3629,5573,'The Andy Dick Show',NULL,now(),NULL,75201),
(3630,4570,'My Big Fat Obnoxious Boss',NULL,now(),NULL,75202),
(3631,6896,'Face the Nation',NULL,now(),NULL,75204),
(3632,2753,'Beet the Vandel Buster',NULL,now(),NULL,75205),
(3633,6453,'Two',NULL,now(),NULL,75207),
(3634,2566,'All That',NULL,now(),NULL,75208),
(3635,2290,'Murder Prevention',NULL,now(),NULL,75209),
(3636,2291,'Untold',NULL,now(),NULL,75214),
(3637,4357,'Malibu Shores',NULL,now(),NULL,75215),
(3638,4552,'Muppet Babies',NULL,now(),NULL,75216),
(3639,254,'Out of Control',NULL,now(),NULL,75218),
(3640,12504,'Ride with Funkmaster Flex',NULL,now(),NULL,75220),
(3641,6379,'Tom Goes to the Mayor',NULL,now(),NULL,75221),
(3642,2585,'America\'s Funniest People',NULL,now(),NULL,75222),
(3643,2572,'Alright Already',NULL,now(),NULL,75224),
(3644,2293,'The Cassidys',NULL,now(),NULL,75225),
(3645,5133,'Second Noah',NULL,now(),NULL,75227),
(3646,4363,'Manhunt: The Search for America\'s Most Gorgeous Male Model',NULL,now(),NULL,75228),
(3647,5695,'The Cooks',NULL,now(),NULL,75229),
(3648,4123,'Kids From C.A.P.E.R.',NULL,now(),NULL,75232),
(3649,4278,'Long Way Round',NULL,now(),NULL,75233),
(3650,2295,'Spooks Of Bottle Bay',NULL,now(),NULL,75234),
(3651,6439,'Tucker Carlson: Unfiltered',NULL,now(),NULL,75235),
(3652,2296,'Quest for the Bay',NULL,now(),NULL,75236),
(3653,8317,'The Ken Dodd Show',NULL,now(),NULL,75237),
(3654,9267,'Doddy\'s Music Box',NULL,now(),NULL,75238),
(3655,8318,'Ken Dodd And The Diddymen',NULL,now(),NULL,75239),
(3656,8319,'Ken Dodd In \'Funny You Should Say That\'',NULL,now(),NULL,75240),
(3657,8304,'Ken Dodd\'s World of Laughter',NULL,now(),NULL,75241),
(3658,9266,'Dodd On His Todd',NULL,now(),NULL,75242),
(3659,8320,'Ken Dodd\'s Showbiz',NULL,now(),NULL,75243),
(3660,3052,'Chicago Story',NULL,now(),NULL,75244),
(3661,7008,'Modern Men',NULL,now(),NULL,75246),
(3662,2961,'Call to Glory',NULL,now(),NULL,75247),
(3663,10289,'The Tim Conway Show',NULL,now(),NULL,75248),
(3664,2297,'House of Clues',NULL,now(),NULL,75249),
(3665,3989,'Invasion Iowa',NULL,now(),NULL,75250),
(3666,2298,'Barking',NULL,now(),NULL,75251),
(3667,2726,'Battle for Ozzfest',NULL,now(),NULL,75252),
(3668,4618,'Nanny 911',NULL,now(),NULL,75253),
(3669,12832,'Beat! Beat! Beat!',NULL,now(),NULL,75254),
(3670,4487,'Mission Hill',NULL,now(),NULL,75255),
(3671,4911,'Psychic Detectives',NULL,now(),NULL,75256),
(3672,18862,'The Money Maze',NULL,now(),NULL,75257),
(3673,5797,'The Fugitive (1963)',NULL,now(),NULL,75258),
(3674,5436,'Sword of Justice',NULL,now(),NULL,75259),
(3675,18889,'The Big Showdown',NULL,now(),NULL,75260),
(3676,5838,'The Guns of Will Sonnett',NULL,now(),NULL,75261),
(3677,5362,'Strange Love',NULL,now(),NULL,75262),
(3678,255,'Never Too Young',NULL,now(),NULL,75263),
(3679,6035,'The New Loretta Young Show',NULL,now(),NULL,75264),
(3680,2301,'HELP! I\'m a Teenage Outlaw',NULL,now(),NULL,75265),
(3681,13427,'A Mug\'s Game',NULL,now(),NULL,75266),
(3682,3321,'Dirty Sally',NULL,now(),NULL,75267),
(3683,13208,'Charlie',NULL,now(),NULL,75268),
(3684,4887,'Pride & Joy',NULL,now(),NULL,75269),
(3685,4759,'Owen Marshall, Counselor At Law',NULL,now(),NULL,75270),
(3686,16670,'Man and the City',NULL,now(),NULL,75272),
(3687,257,'Big Shamus, Little Shamus',NULL,now(),NULL,75276),
(3688,4819,'Philip Marlowe, Private Eye',NULL,now(),NULL,75277),
(3689,2303,'Motormouth',NULL,now(),NULL,75278),
(3690,6623,'W.I.O.U.',NULL,now(),NULL,75281),
(3691,2304,'The Mansion',NULL,now(),NULL,75282),
(3692,6645,'Working Girl',NULL,now(),NULL,75283),
(3693,11001,'Ryan\'s Four',NULL,now(),NULL,75285),
(3694,9857,'The Staggering Stories of Ferdinand De Bargos',NULL,now(),NULL,75286),
(3695,9519,'Klinik!',NULL,now(),NULL,75287),
(3696,2569,'Almost Grown',NULL,now(),NULL,75288),
(3697,2306,'Coolman!',NULL,now(),NULL,75290),
(3698,5471,'Tattinger\'s',NULL,now(),NULL,75293),
(3699,5160,'Shaky Ground',NULL,now(),NULL,75295),
(3700,7287,'Zodiac',NULL,now(),NULL,75296),
(3701,7336,'Queen B',NULL,now(),NULL,75298),
(3702,3667,'Get Smart (1995)',NULL,now(),NULL,75302),
(3703,5904,'The Kids in the Hall',NULL,now(),NULL,75303),
(3704,8198,'The NFL Today',NULL,now(),NULL,75305),
(3705,258,'Down the Shore',NULL,now(),NULL,75307),
(3706,15940,'High Finance',NULL,now(),NULL,75308),
(3707,6187,'The Show',NULL,now(),NULL,75309),
(3708,2310,'Conviction (UK)',NULL,now(),NULL,75310),
(3709,4963,'Regency House Party',NULL,now(),NULL,75311),
(3710,19363,'Speak Up, America',NULL,now(),NULL,75315),
(3711,10568,'To Tell The Truth (2000)',NULL,now(),NULL,75317),
(3712,3888,'Home Movies',NULL,now(),NULL,75318),
(3713,259,'Science Court',NULL,now(),NULL,75319),
(3714,6081,'The Paula Poundstone Show',NULL,now(),NULL,75320),
(3715,2312,'Overman King Gainer',NULL,now(),NULL,75321),
(3716,2313,'Hey! Spring of Trivia',NULL,now(),NULL,75322),
(3717,7329,'Your Number\'s Up',NULL,now(),NULL,75326),
(3718,7306,'You Can Do Magic',NULL,now(),NULL,75327),
(3719,2701,'Bad Girls',NULL,now(),NULL,75328),
(3720,5512,'The 1970s Office',NULL,now(),NULL,75330),
(3721,3884,'Home Delivery',NULL,now(),NULL,75331),
(3722,3653,'General Hospital',NULL,now(),NULL,75332),
(3723,260,'Pinwheel',NULL,now(),NULL,75333),
(3724,6072,'The Outsider (1983)',NULL,now(),NULL,75334),
(3725,3193,'Cuts',NULL,now(),NULL,75335),
(3726,2316,'Outlaws (2004)',NULL,now(),NULL,75336),
(3727,6372,'Toddworld',NULL,now(),NULL,75337),
(3728,5751,'The Ellen Show',NULL,now(),NULL,75338),
(3729,4302,'Love My Way',NULL,now(),NULL,75339),
(3730,5364,'Strangers with Candy',NULL,now(),NULL,75341),
(3731,3843,'Hey Dude',NULL,now(),NULL,75342),
(3732,3511,'Family Rules',NULL,now(),NULL,75344),
(3733,9201,'Cilla\'s World of Comedy',NULL,now(),NULL,75347),
(3734,2319,'Max & Paddy\'s Road to Nowhere',NULL,now(),NULL,75351),
(3735,2320,'True Horror',NULL,now(),NULL,75352),
(3736,25,'Swans Crossing',NULL,now(),NULL,75354),
(3737,2321,'ABC TV Micro-Mini Series',NULL,now(),NULL,75356),
(3738,5889,'The Jenny McCarthy Show',NULL,now(),NULL,75357),
(3739,2322,'Mine All Mine',NULL,now(),NULL,75358),
(3740,3247,'Date My Mom',NULL,now(),NULL,75360),
(3741,13589,'The Fourth Arm',NULL,now(),NULL,75362),
(3742,4450,'Mentors',NULL,now(),NULL,75363),
(3743,2326,'Outrageous Fortune',NULL,now(),NULL,75364),
(3744,2327,'Ladies\' Man (1980)',NULL,now(),NULL,75365),
(3745,9940,'Union Castle',NULL,now(),NULL,75368),
(3746,13226,'The Regiment',NULL,now(),NULL,75371),
(3747,2328,'Can\'t Sing Singers',NULL,now(),NULL,75372),
(3748,5074,'R U The Girl With T-Boz & Chilli',NULL,now(),NULL,75373),
(3749,5664,'The Buzz on Maggie',NULL,now(),NULL,75374),
(3750,3041,'Chasing Farrah',NULL,now(),NULL,75376),
(3751,5429,'Sweet Valley High',NULL,now(),NULL,75377),
(3752,4987,'Retrosexual: The \'80s',NULL,now(),NULL,75379),
(3753,9370,'Marty',NULL,now(),NULL,75381),
(3754,6263,'The Ultimate Fighter',NULL,now(),NULL,75382),
(3755,5689,'The Comeback',NULL,now(),NULL,75383),
(3756,3798,'Harts of the West',NULL,now(),NULL,75385),
(3757,2331,'Metropia',NULL,now(),NULL,75387),
(3758,263,'The Mork & Mindy/Laverne & Shirley/Fonz Hour',NULL,now(),NULL,75389),
(3759,3485,'Faerie Tale Theatre',NULL,now(),NULL,75391),
(3760,264,'Gullah, Gullah Island',NULL,now(),NULL,75392),
(3761,6314,'The X Factor (Australia)',NULL,now(),NULL,75393),
(3762,265,'My Brother and Me',NULL,now(),NULL,75394),
(3763,6451,'Twins',NULL,now(),NULL,75395),
(3764,2333,'Earth Report',NULL,now(),NULL,75396),
(3765,5446,'Taina',NULL,now(),NULL,75398),
(3766,4325,'Madigan Men',NULL,now(),NULL,75401),
(3767,6293,'The Wild Thornberrys',NULL,now(),NULL,75402),
(3768,2334,'Pornucopia',NULL,now(),NULL,75403),
(3769,2634,'Apple\'s Way',NULL,now(),NULL,75405),
(3770,3988,'Invasion America',NULL,now(),NULL,75407),
(3771,14835,'The Ron White Show',NULL,now(),NULL,75408),
(3772,2745,'Beauty and the Geek',NULL,now(),NULL,75409),
(3773,3366,'Dracula: The Series',NULL,now(),NULL,75410),
(3774,2335,'Rozen Maiden',NULL,now(),NULL,75411),
(3775,4299,'Love Is in the Heir',NULL,now(),NULL,75412),
(3776,7402,'We Live Next Door',NULL,now(),NULL,75413),
(3777,4116,'Keroro Gunso',NULL,now(),NULL,75414),
(3778,2338,'I Married a Princess',NULL,now(),NULL,75420),
(3779,10132,'I Can\'t Believe You Said That!',NULL,now(),NULL,75421),
(3780,19007,'I Cover Times Square',NULL,now(),NULL,75422),
(3781,19088,'Haggis Baggis',NULL,now(),NULL,75423),
(3782,6683,'Young Blades',NULL,now(),NULL,75425),
(3783,6073,'The Outsiders',NULL,now(),NULL,75426),
(3784,7325,'Your Friend, Andrew W.K.',NULL,now(),NULL,75427),
(3785,4064,'John Mayer Has a TV Show',NULL,now(),NULL,75428),
(3786,7575,'Blokesworld',NULL,now(),NULL,75429),
(3787,6306,'The Worst Witch',NULL,now(),NULL,75431),
(3788,8352,'B4',NULL,now(),NULL,75432),
(3789,6322,'The Young Pioneers',NULL,now(),NULL,75433),
(3790,3651,'Gastineau Girls',NULL,now(),NULL,75435),
(3791,2339,'Tots TV',NULL,now(),NULL,75442),
(3792,2556,'Allegra\'s Window',NULL,now(),NULL,75443),
(3793,2458,'6teen',NULL,now(),NULL,75444),
(3794,3665,'Get Real',NULL,now(),NULL,75446),
(3795,6492,'Urban Angel',NULL,now(),NULL,75447),
(3796,6600,'Wickedly Perfect',NULL,now(),NULL,75449),
(3797,2343,'The Adventures Of Twizzle',NULL,now(),NULL,75451),
(3798,4235,'Life Is Great With Brooke Burke',NULL,now(),NULL,75452),
(3799,2345,'MuzikMafia TV',NULL,now(),NULL,75455),
(3800,11031,'House Broken',NULL,now(),NULL,75456),
(3801,15566,'The Henry Rollins Show',NULL,now(),NULL,75457),
(3802,267,'Catwalk',NULL,now(),NULL,75459),
(3803,15867,'Tubridy Tonight',NULL,now(),NULL,75461),
(3804,268,'Together We Stand',NULL,now(),NULL,75462),
(3805,4992,'Reunion',NULL,now(),NULL,75463),
(3806,3948,'I Love the 90\'s Part Deux',NULL,now(),NULL,75464),
(3807,7307,'You Can\'t Win (1961)',NULL,now(),NULL,75466),
(3808,9475,'Fair Game',NULL,now(),NULL,75468),
(3809,2347,'High Five',NULL,now(),NULL,75469),
(3810,21604,'Loveline',NULL,now(),NULL,75470),
(3811,5896,'The Joan Rivers Show',NULL,now(),NULL,75471),
(3812,2348,'The Last Chancers',NULL,now(),NULL,75474),
(3813,5457,'TaleSpin',NULL,now(),NULL,75476),
(3814,270,'Leo and Me',NULL,now(),NULL,75478),
(3815,10251,'Palmerstown, U.S.A.',NULL,now(),NULL,75479),
(3816,5307,'Sports Illustrated: Swimsuit Model Search',NULL,now(),NULL,75480),
(3817,3561,'Flipper',NULL,now(),NULL,75481),
(3818,5951,'The Louie Show',NULL,now(),NULL,75482),
(3819,271,'A Family for Joe',NULL,now(),NULL,75484),
(3820,12620,'Outbreak of Murder',NULL,now(),NULL,75485),
(3821,11251,'Rainbow City',NULL,now(),NULL,75486),
(3822,6685,'Young Hercules',NULL,now(),NULL,75491),
(3823,272,'Santo Bugito',NULL,now(),NULL,75493),
(3824,3439,'Emily of New Moon',NULL,now(),NULL,75494),
(3825,2621,'Animorphs',NULL,now(),NULL,75495),
(3826,2457,'64 Zoo Lane',NULL,now(),NULL,75496),
(3827,5142,'The Secret World of Alex Mack',NULL,now(),NULL,75497),
(3828,273,'Learning The Ropes',NULL,now(),NULL,75498),
(3829,2351,'Tall Poppies',NULL,now(),NULL,75499),
(3830,4710,'Old Bear Stories',NULL,now(),NULL,75500),
(3831,3477,'Extreme Makeover: Home Edition: How\'d They Do That?',NULL,now(),NULL,75501),
(3832,2353,'High School Project USA',NULL,now(),NULL,75502),
(3833,4594,'My Secret Identity',NULL,now(),NULL,75503),
(3834,2355,'The Beiderbecke Connection',NULL,now(),NULL,75504),
(3835,4927,'Queer Eye for the Straight Girl',NULL,now(),NULL,75505),
(3836,10973,'Room For One More',NULL,now(),NULL,75506),
(3837,16705,'The Lazarus Syndrome',NULL,now(),NULL,75507),
(3838,6205,'The Sonny & Cher Show',NULL,now(),NULL,75509),
(3839,18855,'Name That Tune (1974)',NULL,now(),NULL,75511),
(3840,2359,'My-Hime',NULL,now(),NULL,75512),
(3841,274,'Hee Haw Honeys',NULL,now(),NULL,75514),
(3842,2360,'Augie Doggie and Doggie Daddy',NULL,now(),NULL,75515),
(3843,5754,'The Entertainer',NULL,now(),NULL,75516),
(3844,2917,'Brooklyn South',NULL,now(),NULL,75517),
(3845,3425,'Eerie, Indiana: The Other Dimension',NULL,now(),NULL,75518),
(3846,16929,'Live-In',NULL,now(),NULL,75519),
(3847,4264,'Living Dolls',NULL,now(),NULL,75521),
(3848,7425,'What\'s It All About, World?',NULL,now(),NULL,75522),
(3849,4786,'Paul Sand in Friends and Lovers',NULL,now(),NULL,75523),
(3850,8850,'Dawson\'s Weekly',NULL,now(),NULL,75524),
(3851,4201,'Laverne & Shirley in the Army',NULL,now(),NULL,75525),
(3852,9200,'Cilla\'s Comedy Six',NULL,now(),NULL,75526),
(3853,6870,'Book of Daniel',NULL,now(),NULL,75527),
(3854,8244,'Bill Oddie\'s How to Watch Wildlife',NULL,now(),NULL,75530),
(3855,7666,'Beyond 2000',NULL,now(),NULL,75531),
(3856,275,'Working Stiffs',NULL,now(),NULL,75533),
(3857,5368,'Street Fighter II: V',NULL,now(),NULL,75534),
(3858,276,'A League of Their Own',NULL,now(),NULL,75535),
(3859,8369,'Get Back',NULL,now(),NULL,75536),
(3860,3668,'Get Some In!',NULL,now(),NULL,75538),
(3861,277,'Uptown Comedy Club',NULL,now(),NULL,75540),
(3862,3050,'Chicago Hope',NULL,now(),NULL,75544),
(3863,7147,'Painkiller Jane',NULL,now(),NULL,75546),
(3864,5459,'Talk to Me',NULL,now(),NULL,75547),
(3865,5265,'South of Sunset',NULL,now(),NULL,75549),
(3866,278,'Love Is a Many Splendored Thing',NULL,now(),NULL,75551),
(3867,4141,'Kirsty\'s TV Meltdown',NULL,now(),NULL,75552),
(3868,3814,'Head Cases',NULL,now(),NULL,75554),
(3869,7264,'3 Lbs.',NULL,now(),NULL,75555),
(3870,6686,'Young Maverick',NULL,now(),NULL,75556),
(3871,279,'The Oregon Trail',NULL,now(),NULL,75557),
(3872,2758,'Being Ian',NULL,now(),NULL,75558),
(3873,5994,'The Moxy Show',NULL,now(),NULL,75561),
(3874,2855,'Bobcat\'s Big Ass Show',NULL,now(),NULL,75562),
(3875,3032,'Charles in Charge',NULL,now(),NULL,75563),
(3876,2823,'Blake\'s 7',NULL,now(),NULL,75565),
(3877,3163,'Crash Zone',NULL,now(),NULL,75566),
(3878,12949,'The Xtra Factor',NULL,now(),NULL,75567),
(3879,3090,'Closing Arguments',NULL,now(),NULL,75568),
(3880,281,'Tenafly',NULL,now(),NULL,75569),
(3881,6864,'Vincent',NULL,now(),NULL,75570),
(3882,5606,'The Bedford Diaries',NULL,now(),NULL,75571),
(3883,2366,'Tortellini Western',NULL,now(),NULL,75572),
(3884,17051,'Snoop Sisters',NULL,now(),NULL,75573),
(3885,282,'Cool Million',NULL,now(),NULL,75576),
(3886,283,'Faraday and Company',NULL,now(),NULL,75577),
(3887,3769,'Guys Like Us',NULL,now(),NULL,75581),
(3888,10090,'Full Circle',NULL,now(),NULL,75582),
(3889,2996,'Carter Country',NULL,now(),NULL,75586),
(3890,7179,'The Tournament',NULL,now(),NULL,75591),
(3891,10019,'Battle of the Ages',NULL,now(),NULL,75593),
(3892,7432,'Who Said That?',NULL,now(),NULL,75594),
(3893,2372,'Mr. Romance',NULL,now(),NULL,75597),
(3894,284,'Honeymoon Suite',NULL,now(),NULL,75598),
(3895,4183,'Largo Winch',NULL,now(),NULL,75601),
(3896,2373,'Trisha Goddard',NULL,now(),NULL,75603),
(3897,7087,'Calorie Commando',NULL,now(),NULL,75605),
(3898,5200,'Silk Stalkings',NULL,now(),NULL,75607),
(3899,285,'The Campbells',NULL,now(),NULL,75608),
(3900,5005,'Richie Rich',NULL,now(),NULL,75609),
(3901,2495,'Adderly',NULL,now(),NULL,75610),
(3902,7262,'29 Minutes of Fame',NULL,now(),NULL,75611),
(3903,286,'Figure It Out',NULL,now(),NULL,75612),
(3904,2374,'The Lance Krall Show',NULL,now(),NULL,75614),
(3905,18610,'The Everly Brothers Show',NULL,now(),NULL,75616),
(3906,2375,'Trippin\'',NULL,now(),NULL,75617),
(3907,287,'Christine Cromwell',NULL,now(),NULL,75618),
(3908,2756,'Being Bobby Brown',NULL,now(),NULL,75621),
(3909,5680,'The Chimp Channel',NULL,now(),NULL,75622),
(3910,288,'Tucker\'s Witch',NULL,now(),NULL,75623),
(3911,6154,'The Royle Family',NULL,now(),NULL,75624),
(3912,2376,'World\'s Wildest Police Videos',NULL,now(),NULL,75626),
(3913,3258,'Dead at 21',NULL,now(),NULL,75627),
(3914,4727,'Only Fools and Horses',NULL,now(),NULL,75628),
(3915,11018,'Shortchange',NULL,now(),NULL,75630),
(3916,17362,'Search and Rescue',NULL,now(),NULL,75631),
(3917,6482,'United States',NULL,now(),NULL,75633),
(3918,14615,'Jessica Novak',NULL,now(),NULL,75635),
(3919,4604,'My Super Sweet 16',NULL,now(),NULL,75638),
(3920,289,'Brak Presents the Brak Show Starring Brak',NULL,now(),NULL,75639),
(3921,8498,'Ukool',NULL,now(),NULL,75643),
(3922,2755,'Behind The Music',NULL,now(),NULL,75644),
(3923,3954,'Inconceivable',NULL,now(),NULL,75645),
(3924,6695,'Zatch Bell!',NULL,now(),NULL,75647),
(3925,8991,'Banyon',NULL,now(),NULL,75648),
(3926,4176,'Lancer',NULL,now(),NULL,75649),
(3927,3728,'Grand',NULL,now(),NULL,75654),
(3928,290,'Search Party',NULL,now(),NULL,75655),
(3929,2380,'Timeless',NULL,now(),NULL,75657),
(3930,14580,'Dame Edna\'s Hollywood',NULL,now(),NULL,75658),
(3931,6041,'The New Scooby Doo Movies',NULL,now(),NULL,75661),
(3932,5966,'The Man From U.N.C.L.E.',NULL,now(),NULL,75663),
(3933,291,'The Match Game',NULL,now(),NULL,75664),
(3934,9817,'Jackie Gleason and His American Scene Magazine',NULL,now(),NULL,75666),
(3935,4658,'Nichols',NULL,now(),NULL,75667),
(3936,4191,'Last Man Standing',NULL,now(),NULL,75670),
(3937,292,'Channel Umptee-3',NULL,now(),NULL,75672),
(3938,293,'The Benny Hill Show',NULL,now(),NULL,75673),
(3939,2382,'The Rotters\' Club',NULL,now(),NULL,75674),
(3940,294,'High Flyers',NULL,now(),NULL,75676),
(3941,5060,'Round The Twist',NULL,now(),NULL,75677),
(3942,14879,'A History of Britain',NULL,now(),NULL,75678),
(3943,2842,'Blue Heelers',NULL,now(),NULL,75680),
(3944,7966,'Briar & Graves',NULL,now(),NULL,75681),
(3945,9259,'Dial RIX',NULL,now(),NULL,75683),
(3946,7391,'Waugh on Crime',NULL,now(),NULL,75684),
(3947,6398,'Trace Evidence: The Case Files of Dr. Henry Lee',NULL,now(),NULL,75685),
(3948,6803,'Love Monkey',NULL,now(),NULL,75686),
(3949,16637,'On Our Own (1977)',NULL,now(),NULL,75687),
(3950,7164,'Beckhams Hotshots',NULL,now(),NULL,75689),
(3951,5121,'Seachange',NULL,now(),NULL,75690),
(3952,4968,'Related',NULL,now(),NULL,75693),
(3953,296,'The Danny Kaye Show',NULL,now(),NULL,75694),
(3954,4800,'Pepper Dennis',NULL,now(),NULL,75695),
(3955,9220,'Cooperama',NULL,now(),NULL,75699),
(3956,9756,'The Tommy Cooper Hour',NULL,now(),NULL,75700),
(3957,9217,'Cooper, Just Like That',NULL,now(),NULL,75701),
(3958,3959,'In Justice',NULL,now(),NULL,75702),
(3959,5581,'The Apprentice - Martha Stewart',NULL,now(),NULL,75703),
(3960,2781,'Beyond Belief: Fact or Fiction',NULL,now(),NULL,75706),
(3961,3438,'Emily\'s Reasons Why Not',NULL,now(),NULL,75711),
(3962,3905,'Hot Properties',NULL,now(),NULL,75713),
(3963,4907,'Providence',NULL,now(),NULL,75715),
(3964,5421,'Suspense',NULL,now(),NULL,75716),
(3965,2384,'Petite Princess Yucie',NULL,now(),NULL,75717),
(3966,5358,'Stop Susan Williams',NULL,now(),NULL,75718),
(3967,4749,'Out of Practice',NULL,now(),NULL,75722),
(3968,6539,'Wanted: Dead or Alive',NULL,now(),NULL,75724),
(3969,4383,'Martin',NULL,now(),NULL,75725),
(3970,297,'Trackdown',NULL,now(),NULL,75726),
(3971,22401,'Sing Along',NULL,now(),NULL,75728),
(3972,5242,'So Haunt Me',NULL,now(),NULL,75730),
(3973,16809,'Singer & Sons',NULL,now(),NULL,75731),
(3974,7569,'Today Tonight',NULL,now(),NULL,75732),
(3975,298,'Eugenie Sandler P.I.',NULL,now(),NULL,75733),
(3976,4469,'The Mighty Mouse Playhouse',NULL,now(),NULL,75735),
(3977,5934,'The Life of Riley (1949)',NULL,now(),NULL,75736),
(3978,7911,'Wojeck',NULL,now(),NULL,75740),
(3979,6609,'Wildfire (1986)',NULL,now(),NULL,75741),
(3980,5216,'Skippy',NULL,now(),NULL,75742),
(3981,3181,'Crumbs',NULL,now(),NULL,75743),
(3982,6354,'Thunderstone',NULL,now(),NULL,75746),
(3983,299,'The Wayne Manifesto',NULL,now(),NULL,75747),
(3984,9553,'The Culture Show',NULL,now(),NULL,75748);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(3985,2385,'Miss Popularity',NULL,now(),NULL,75754),
(3986,7775,'Bumper Stumpers',NULL,now(),NULL,75755),
(3987,7037,'Courting Alex',NULL,now(),NULL,75757),
(3988,300,'Half Way Across The Galaxy and Turn Left',NULL,now(),NULL,75761),
(3989,6220,'The Sub-Mariner',NULL,now(),NULL,75763),
(3990,4144,'Kitchen Confidential',NULL,now(),NULL,75768),
(3991,301,'Mirror Mirror 2',NULL,now(),NULL,75770),
(3992,9658,'The Look (2005)',NULL,now(),NULL,75772),
(3993,6240,'The Tim McCarver Show',NULL,now(),NULL,75774),
(3994,302,'Sky Trackers',NULL,now(),NULL,75775),
(3995,18953,'Craig Kennedy, Criminologist',NULL,now(),NULL,75776),
(3996,11004,'Hotel Cosmopolitan',NULL,now(),NULL,75778),
(3997,2387,'The James Whale Radio Show',NULL,now(),NULL,75780),
(3998,4227,'Lie Detector',NULL,now(),NULL,75783),
(3999,4572,'My Brand New Life',NULL,now(),NULL,75786),
(4000,4479,'Misconceptions',NULL,now(),NULL,75787),
(4001,8931,'Life with the Lyons',NULL,now(),NULL,75788),
(4002,2388,'Twisted Tales (UK)',NULL,now(),NULL,75789),
(4003,2390,'Back of the Y',NULL,now(),NULL,75790),
(4004,3898,'Hope and Gloria',NULL,now(),NULL,75792),
(4005,6763,'Tommy Lee Goes to College',NULL,now(),NULL,75793),
(4006,18888,'Rhyme and Reason',NULL,now(),NULL,75794),
(4007,5582,'The Apprentice UK',NULL,now(),NULL,75795),
(4008,6515,'Viewtiful Joe',NULL,now(),NULL,75797),
(4009,2393,'Unnovations',NULL,now(),NULL,75798),
(4010,3987,'Invasion: Earth',NULL,now(),NULL,75803),
(4011,2394,'Making The Band 3',NULL,now(),NULL,75806),
(4012,2395,'Meet the Barkers',NULL,now(),NULL,75807),
(4013,4602,'Mystery Woman',NULL,now(),NULL,75808),
(4014,9197,'Carson\'s Cellar',NULL,now(),NULL,75810),
(4015,10081,'Earn Your Vacation',NULL,now(),NULL,75812),
(4016,3001,'Casanova',NULL,now(),NULL,75817),
(4017,2396,'King Arthur\'s Disasters',NULL,now(),NULL,75818),
(4018,2883,'Boston Public',NULL,now(),NULL,75820),
(4019,2935,'Bull',NULL,now(),NULL,75821),
(4020,6903,'MTV Unplugged',NULL,now(),NULL,75822),
(4021,2860,'Bob Patterson',NULL,now(),NULL,75824),
(4022,5610,'The Bernie Mac Show',NULL,now(),NULL,75825),
(4023,303,'Emerald Cove',NULL,now(),NULL,75826),
(4024,2398,'Pappyland',NULL,now(),NULL,75827),
(4025,5860,'The Hour',NULL,now(),NULL,75828),
(4026,4238,'Lifestories: Families in Crisis',NULL,now(),NULL,75833),
(4027,9334,'The Ferguson Theory',NULL,now(),NULL,75834),
(4028,2997,'Cartoon Alley',NULL,now(),NULL,75835),
(4029,4292,'Love, Inc.',NULL,now(),NULL,75836),
(4030,4216,'Legend',NULL,now(),NULL,75837),
(4031,3531,'Father Ted',NULL,now(),NULL,75842),
(4032,8727,'Help (2005)',NULL,now(),NULL,75843),
(4033,3857,'High Spirits with Shirley Ghostman',NULL,now(),NULL,75844),
(4034,5261,'Soul Man',NULL,now(),NULL,75845),
(4035,6044,'The New Statesman',NULL,now(),NULL,75846),
(4036,15798,'Bloopy\'s Buddies',NULL,now(),NULL,75852),
(4037,7263,'3 for the Money',NULL,now(),NULL,75855),
(4038,4013,'It Takes a Thief (2005)',NULL,now(),NULL,75857),
(4039,2888,'Braceface',NULL,now(),NULL,75858),
(4040,5950,'The Lot',NULL,now(),NULL,75860),
(4041,2400,'The Smoggies',NULL,now(),NULL,75862),
(4042,2746,'Beavis and Butt-head',NULL,now(),NULL,75863),
(4043,2401,'Coach Trip',NULL,now(),NULL,75864),
(4044,6378,'Tom Corbett, Space Cadet',NULL,now(),NULL,75865),
(4045,6525,'VR.5',NULL,now(),NULL,75870),
(4046,9965,'To Tell The Truth (1956)',NULL,now(),NULL,75871),
(4047,4453,'Mermaid Melody: Pichi Pichi Pitch',NULL,now(),NULL,75872),
(4048,12189,'Hanaukyo Maid Team: La Verite',NULL,now(),NULL,75873),
(4049,4849,'Poker Royale',NULL,now(),NULL,75875),
(4050,16685,'On the Rocks',NULL,now(),NULL,75877),
(4051,7405,'We\'ve Got Each Other',NULL,now(),NULL,75878),
(4052,3714,'Good Grief',NULL,now(),NULL,75879),
(4053,10126,'Cannonball Run 2001: Race Across America',NULL,now(),NULL,75883),
(4054,6359,'Time Squad',NULL,now(),NULL,75885),
(4055,2403,'The Hollow Men',NULL,now(),NULL,75887),
(4056,3609,'Free Spirit',NULL,now(),NULL,75890),
(4057,15702,'ESPN25: Who\'s #1?',NULL,now(),NULL,75895),
(4058,3976,'Intervention',NULL,now(),NULL,75896),
(4059,3099,'Code Name: Eternity',NULL,now(),NULL,75898),
(4060,6458,'UC: Undercover',NULL,now(),NULL,75899),
(4061,7937,'Dave Allen at Large',NULL,now(),NULL,75900),
(4062,9922,'Tonight with Dave Allen',NULL,now(),NULL,75902),
(4063,2405,'Make Me a Supermodel UK',NULL,now(),NULL,75904),
(4064,9248,'Dave Allen (1993)',NULL,now(),NULL,75905),
(4065,7474,'My Crazy Life',NULL,now(),NULL,75909),
(4066,5087,'Sailor Moon',NULL,now(),NULL,75910),
(4067,4747,'Outlaw Star',NULL,now(),NULL,75911),
(4068,8981,'Aspel & Company',NULL,now(),NULL,75913),
(4069,4627,'Naturally, Sadie',NULL,now(),NULL,75916),
(4070,3967,'Inside Schwartz',NULL,now(),NULL,75917),
(4071,7428,'When Good Times Go Bad',NULL,now(),NULL,75919),
(4072,2406,'Hollywood Hold \'Em',NULL,now(),NULL,75920),
(4073,2936,'Bullet & Gunn',NULL,now(),NULL,75921),
(4074,8091,'Red and Blue',NULL,now(),NULL,75922),
(4075,2407,'The Crust',NULL,now(),NULL,75923),
(4076,10240,'Bamzooki',NULL,now(),NULL,75924),
(4077,9397,'Kelly',NULL,now(),NULL,75925),
(4078,2716,'Barney Miller',NULL,now(),NULL,75927),
(4079,6328,'Thieves',NULL,now(),NULL,75928),
(4080,12970,'Monkey Trousers',NULL,now(),NULL,75934),
(4081,304,'You Don\'t Know Jack',NULL,now(),NULL,75936),
(4082,18055,'Ski Sunday',NULL,now(),NULL,84859),
(4083,7331,'Xperimental',NULL,now(),NULL,75938),
(4084,8113,'Battle B-Daman',NULL,now(),NULL,75939),
(4085,2408,'Onmyou Taisenki',NULL,now(),NULL,75940),
(4086,2914,'Bromwell High',NULL,now(),NULL,75942),
(4087,2410,'Ultraman: Towards the Future',NULL,now(),NULL,75943),
(4088,4784,'Pat Croce: Moving In',NULL,now(),NULL,75944),
(4089,4476,'Minding The Store',NULL,now(),NULL,75945),
(4090,5977,'The Memphis Trousers Half Hour',NULL,now(),NULL,75946),
(4091,5493,'Tenspeed & Brownshoe',NULL,now(),NULL,75947),
(4092,8050,'Starlight',NULL,now(),NULL,75948),
(4093,4576,'My Favorite Martians',NULL,now(),NULL,75949),
(4094,4496,'MMC',NULL,now(),NULL,75950),
(4095,9069,'Agent \'Z\' and the Penguin From Mars',NULL,now(),NULL,75951),
(4096,305,'Amy Prentiss',NULL,now(),NULL,75952),
(4097,16602,'The Fitzpatricks',NULL,now(),NULL,75954),
(4098,2411,'My Restaurant Rules',NULL,now(),NULL,75955),
(4099,7257,'20th Century Roadshow',NULL,now(),NULL,75956),
(4100,11925,'The Bush Gang',NULL,now(),NULL,75957),
(4101,8182,'Card Sharks',NULL,now(),NULL,75958),
(4102,3192,'Custer',NULL,now(),NULL,75959),
(4103,3042,'Cheap Seats',NULL,now(),NULL,75960),
(4104,5665,'Byrds of Paradise',NULL,now(),NULL,75961),
(4105,16677,'Born Free',NULL,now(),NULL,75962),
(4106,6194,'The Sixth Sense',NULL,now(),NULL,75963),
(4107,2412,'Liquid Television',NULL,now(),NULL,75964),
(4108,6529,'The Wackiest Ship in the Army',NULL,now(),NULL,75966),
(4109,8161,'Chance! Pop Sessions',NULL,now(),NULL,75968),
(4110,16720,'Szysznyk',NULL,now(),NULL,75970),
(4111,306,'Sons and Daughters (1991)',NULL,now(),NULL,75971),
(4112,8413,'The Baseball Bunch (1982)',NULL,now(),NULL,75972),
(4113,11204,'MTV Week In Rock',NULL,now(),NULL,75980),
(4114,4756,'Over My Dead Body',NULL,now(),NULL,75983),
(4115,8735,'Holding the Baby',NULL,now(),NULL,75984),
(4116,3871,'Hit Me Baby One More Time (UK)',NULL,now(),NULL,75985),
(4117,2418,'Movie Life: House of Wax',NULL,now(),NULL,75986),
(4118,2419,'Playing It Straight (UK)',NULL,now(),NULL,75987),
(4119,3636,'Gabriel\'s Fire',NULL,now(),NULL,75990),
(4120,5605,'The Beatles',NULL,now(),NULL,75993),
(4121,6251,'The Tortellis',NULL,now(),NULL,75995),
(4122,6557,'Weird Science',NULL,now(),NULL,75996),
(4123,23885,'The People\'s Court (UK)',NULL,now(),NULL,75998),
(4124,3048,'Chevy Top 20',NULL,now(),NULL,75999),
(4125,19161,'The Jim Stafford Show',NULL,now(),NULL,76001),
(4126,10548,'General Motors Presents',NULL,now(),NULL,76002),
(4127,2912,'Britney and Kevin: Chaotic',NULL,now(),NULL,76003),
(4128,6828,'Rock School',NULL,now(),NULL,76006),
(4129,3461,'Evening Shade',NULL,now(),NULL,76007),
(4130,3107,'Cold Squad',NULL,now(),NULL,76008),
(4131,9793,'The Robinsons',NULL,now(),NULL,76012),
(4132,4343,'Major',NULL,now(),NULL,76013),
(4133,6672,'Yakitate!! Japan',NULL,now(),NULL,76014),
(4134,8995,'A.M. America',NULL,now(),NULL,76016),
(4135,3875,'Hogan Knows Best',NULL,now(),NULL,76017),
(4136,10670,'Teammates',NULL,now(),NULL,76019),
(4137,2422,'Las Vegas Garden of Love',NULL,now(),NULL,76020),
(4138,3096,'Coconut Fred\'s Fruit Salad Island',NULL,now(),NULL,76021),
(4139,2423,'Tsubasa Chronicle',NULL,now(),NULL,76023),
(4140,7401,'We Are One Body',NULL,now(),NULL,76024),
(4141,13134,'Bognor',NULL,now(),NULL,76028),
(4142,17071,'Mr. Sunshine',NULL,now(),NULL,76030),
(4143,19103,'Popularity Contest',NULL,now(),NULL,76032),
(4144,7534,'Celebrity Overhaul',NULL,now(),NULL,76033),
(4145,4861,'Popcultured',NULL,now(),NULL,76034),
(4146,7467,'Kathy\'s So-Called Reality',NULL,now(),NULL,76035),
(4147,5683,'The Chronicle',NULL,now(),NULL,76036),
(4148,12819,'Hawaiian Heat',NULL,now(),NULL,76038),
(4149,4977,'Renegades',NULL,now(),NULL,76040),
(4150,4473,'Mike Hammer, Private Eye',NULL,now(),NULL,76041),
(4151,307,'Danger Theatre',NULL,now(),NULL,76042),
(4152,2849,'Blue Thunder',NULL,now(),NULL,76043),
(4153,5933,'The Life and Times of Juniper Lee',NULL,now(),NULL,76044),
(4154,2427,'Princess Tutu',NULL,now(),NULL,76045),
(4155,12947,'The Morning After Show',NULL,now(),NULL,76046),
(4156,7326,'Your Kids Are in Charge',NULL,now(),NULL,76051),
(4157,2428,'Mr. Magoo',NULL,now(),NULL,76052),
(4158,308,'It\'s a Great Life',NULL,now(),NULL,76054),
(4159,16600,'The Eve Arden Show',NULL,now(),NULL,76055),
(4160,4467,'Mighty Ducks',NULL,now(),NULL,76059),
(4161,5248,'Something So Right',NULL,now(),NULL,76063),
(4162,6003,'The Mystery Files of Shelby Woo',NULL,now(),NULL,76064),
(4163,2837,'Blowing Up: Fatty Koo',NULL,now(),NULL,76065),
(4164,23007,'Celebrity Wrestling',NULL,now(),NULL,76067),
(4165,309,'Profiles in Courage',NULL,now(),NULL,76069),
(4166,14844,'Bronk',NULL,now(),NULL,76076),
(4167,6347,'Three Wishes',NULL,now(),NULL,76078),
(4168,2989,'Care Bears',NULL,now(),NULL,76079),
(4169,19244,'Road Hammers',NULL,now(),NULL,76080),
(4170,3144,'Costas Now',NULL,now(),NULL,76082),
(4171,2735,'Beach Girls',NULL,now(),NULL,76083),
(4172,3015,'Celebrity Charades',NULL,now(),NULL,76084),
(4173,5575,'The Andy Milonakis Show',NULL,now(),NULL,76086),
(4174,2924,'BSTV',NULL,now(),NULL,76087),
(4175,2679,'Automaniac',NULL,now(),NULL,76089),
(4176,9691,'The Gangster Chronicles',NULL,now(),NULL,76090),
(4177,7098,'Huckleberry Hound',NULL,now(),NULL,76091),
(4178,12068,'Ohara',NULL,now(),NULL,76092),
(4179,18252,'Celebrity Circus',NULL,now(),NULL,76094),
(4180,5645,'The Brady Kids',NULL,now(),NULL,76096),
(4181,2727,'Battle of the Gridiron Stars',NULL,now(),NULL,76099),
(4182,310,'Medivac',NULL,now(),NULL,76102),
(4183,5988,'The Mole (Australia)',NULL,now(),NULL,76104),
(4184,311,'The Adventures of A.R.K.',NULL,now(),NULL,76105),
(4185,3411,'E.N.G.',NULL,now(),NULL,76106),
(4186,1622,'Gormenghast',NULL,now(),NULL,79523),
(4187,4665,'Night Heat',NULL,now(),NULL,76108),
(4188,4687,'North of 60',NULL,now(),NULL,76109),
(4189,2850,'Blue Water High',NULL,now(),NULL,76110),
(4190,3282,'Delta House',NULL,now(),NULL,76112),
(4191,10125,'B.A.D. Cats',NULL,now(),NULL,76114),
(4192,6884,'Eleventh Hour (UK)',NULL,now(),NULL,76116),
(4193,4351,'Making The Most Of The Micro',NULL,now(),NULL,76117),
(4194,4114,'Kept',NULL,now(),NULL,76118),
(4195,6771,'NOVA',NULL,now(),NULL,76119),
(4196,4234,'Life Goes On',NULL,now(),NULL,76120),
(4197,312,'Park Place',NULL,now(),NULL,76121),
(4198,313,'Medical Story',NULL,now(),NULL,76122),
(4199,7573,'Temptation',NULL,now(),NULL,76123),
(4200,6244,'The Tom Show',NULL,now(),NULL,76124),
(4201,2619,'Animal X',NULL,now(),NULL,76125),
(4202,314,'Townsend Television',NULL,now(),NULL,76126),
(4203,3019,'Celebrity Love Island',NULL,now(),NULL,76127),
(4204,315,'Salute Your Shorts',NULL,now(),NULL,76128),
(4205,5759,'The Facts of Life',NULL,now(),NULL,76130),
(4206,4168,'L.A. Firefighters',NULL,now(),NULL,76131),
(4207,316,'Wishbone',NULL,now(),NULL,76132),
(4208,6817,'The Comic Side of 7 Days',NULL,now(),NULL,76134),
(4209,5793,'The Frank Sinatra Show',NULL,now(),NULL,76135),
(4210,5783,'The Flintstone Comedy Show',NULL,now(),NULL,76138),
(4211,3222,'Danger Bay',NULL,now(),NULL,76139),
(4212,7491,'Three Moons Over Milford',NULL,now(),NULL,76142),
(4213,5859,'The Hoop Life',NULL,now(),NULL,76145),
(4214,3614,'Fridays',NULL,now(),NULL,76147),
(4215,10835,'Singled Out',NULL,now(),NULL,76150),
(4216,317,'G.P.',NULL,now(),NULL,76151),
(4217,11856,'Close-Up',NULL,now(),NULL,76152),
(4218,3906,'Hot Shots',NULL,now(),NULL,76153),
(4219,318,'Hammerman',NULL,now(),NULL,76154),
(4220,4483,'Missing Persons',NULL,now(),NULL,76155),
(4221,3555,'Flamingo Road',NULL,now(),NULL,76157),
(4222,14873,'Stone',NULL,now(),NULL,76158),
(4223,320,'Eischied',NULL,now(),NULL,76159),
(4224,13171,'Husbands, Wives & Lovers',NULL,now(),NULL,76161),
(4225,322,'Up Pompeii',NULL,now(),NULL,76162),
(4226,3708,'Good & Evil',NULL,now(),NULL,76163),
(4227,3225,'Danger Man',NULL,now(),NULL,76164),
(4228,5273,'Space Cases',NULL,now(),NULL,76165),
(4229,4323,'Made in Canada',NULL,now(),NULL,76166),
(4230,6433,'True Blue',NULL,now(),NULL,76167),
(4231,2721,'Batman: The Animated Series',NULL,now(),NULL,76168),
(4232,5980,'The Mickey Mouse Club',NULL,now(),NULL,76169),
(4233,323,'Crusader Rabbit',NULL,now(),NULL,76170),
(4234,6383,'Top Cat',NULL,now(),NULL,76172),
(4235,6612,'Wildside',NULL,now(),NULL,76173),
(4236,324,'Wild Palms',NULL,now(),NULL,76174),
(4237,325,'The Adams Chronicles',NULL,now(),NULL,76175),
(4238,326,'The Hathaways',NULL,now(),NULL,76176),
(4239,5098,'Saturday Night Live',NULL,now(),NULL,76177),
(4240,5089,'Salvage 1',NULL,now(),NULL,76178),
(4241,4690,'Nothing Too Good for a Cowboy',NULL,now(),NULL,76179),
(4242,5857,'The Home Court',NULL,now(),NULL,76181),
(4243,4971,'Remember WENN',NULL,now(),NULL,76182),
(4244,327,'The Human Jungle',NULL,now(),NULL,76185),
(4245,3853,'High Mountain Rangers',NULL,now(),NULL,76186),
(4246,3213,'Dalziel and Pascoe',NULL,now(),NULL,76187),
(4247,4371,'Marblehead Manor',NULL,now(),NULL,76188),
(4248,4581,'My Hero',NULL,now(),NULL,76189),
(4249,9929,'Gideon Oliver',NULL,now(),NULL,76191),
(4250,2928,'Buddy Faro',NULL,now(),NULL,76192),
(4251,6011,'Then Came You',NULL,now(),NULL,76193),
(4252,6619,'Wind on Water',NULL,now(),NULL,76195),
(4253,3156,'Cow and Chicken',NULL,now(),NULL,76196),
(4254,5997,'The Mummy: The Animated Series',NULL,now(),NULL,76197),
(4255,4754,'Out There (1951)',NULL,now(),NULL,76198),
(4256,4066,'Johnny Bravo',NULL,now(),NULL,76201),
(4257,328,'Random Play',NULL,now(),NULL,76202),
(4258,3894,'Hong Kong Phooey',NULL,now(),NULL,76205),
(4259,6046,'The New Tom & Jerry Show',NULL,now(),NULL,76206),
(4260,3606,'Fred Flintstone & Friends',NULL,now(),NULL,76207),
(4261,5589,'The Atom Ant/Secret Squirrel Show',NULL,now(),NULL,76208),
(4262,4337,'Magilla Gorilla',NULL,now(),NULL,76209),
(4263,5590,'The Atom Ant Show',NULL,now(),NULL,76210),
(4264,329,'Inch High, Private Eye',NULL,now(),NULL,76211),
(4265,14779,'Where\'s Huddles',NULL,now(),NULL,76212),
(4266,4445,'Men Behaving Badly',NULL,now(),NULL,76213),
(4267,5592,'The Babysitters Club',NULL,now(),NULL,76214),
(4268,5423,'Swamp Thing',NULL,now(),NULL,76215),
(4269,330,'Where the Heart Is (1969)',NULL,now(),NULL,76216),
(4270,5837,'The Gumby Show',NULL,now(),NULL,76217),
(4271,3756,'Gumby Adventures',NULL,now(),NULL,76218),
(4272,6224,'The Super Globetrotters',NULL,now(),NULL,76219),
(4273,17345,'The Harlem Globetrotters',NULL,now(),NULL,76220),
(4274,331,'Toonsylvania',NULL,now(),NULL,76221),
(4275,4763,'Pac-Man',NULL,now(),NULL,76222),
(4276,332,'Space-Stars',NULL,now(),NULL,76223),
(4277,4074,'Jonny Quest',NULL,now(),NULL,76224),
(4278,5063,'Rove Live',NULL,now(),NULL,76225),
(4279,6114,'The Queen Latifah Show',NULL,now(),NULL,76227),
(4280,5196,'Sightings',NULL,now(),NULL,76228),
(4281,4764,'Pacific Blue',NULL,now(),NULL,76229),
(4282,8301,'Dreams',NULL,now(),NULL,76230),
(4283,333,'You Again?',NULL,now(),NULL,76231),
(4284,334,'Little Rosie',NULL,now(),NULL,76232),
(4285,4655,'New York Undercover',NULL,now(),NULL,76233),
(4286,2584,'America\'s Funniest Home Videos',NULL,now(),NULL,76235),
(4287,4940,'Raising Dad',NULL,now(),NULL,76236),
(4288,2790,'Big Brother (Australia)',NULL,now(),NULL,76237),
(4289,5237,'Snowy River: The McGregor Saga',NULL,now(),NULL,76239),
(4290,17363,'Spencer',NULL,now(),NULL,76240),
(4291,4520,'Monsters',NULL,now(),NULL,76241),
(4292,4637,'Neon Rider',NULL,now(),NULL,76242),
(4293,10709,'Ben Jerrod',NULL,now(),NULL,76243),
(4294,335,'Mr. Novak',NULL,now(),NULL,76244),
(4295,3895,'Hooperman',NULL,now(),NULL,76246),
(4296,2667,'A Touch of Frost',NULL,now(),NULL,76247),
(4297,336,'Camp Runamuck',NULL,now(),NULL,76248),
(4298,337,'Westside Medical',NULL,now(),NULL,76249),
(4299,338,'Wish Kid',NULL,now(),NULL,76250),
(4300,339,'Duggan',NULL,now(),NULL,76252),
(4301,5924,'The Legend of William Tell',NULL,now(),NULL,76253),
(4302,6031,'The New Ghostwriter Mysteries',NULL,now(),NULL,76254),
(4303,6058,'The Odyssey',NULL,now(),NULL,76255),
(4304,4667,'Nightingales',NULL,now(),NULL,76256),
(4305,4564,'Murphy\'s Law (1988)',NULL,now(),NULL,76257),
(4306,4393,'Matrix',NULL,now(),NULL,76258),
(4307,6119,'The Ray Bradbury Theatre',NULL,now(),NULL,76259),
(4308,340,'The Banana Splits Adventure Hour',NULL,now(),NULL,76260),
(4309,23664,'Dancing With the Stars',NULL,now(),NULL,79590),
(4310,6530,'Wacky Races',NULL,now(),NULL,76261),
(4311,341,'Perils of Penelope Pitstop',NULL,now(),NULL,76262),
(4312,3246,'Dastardly and Muttley in Their Flying Machines',NULL,now(),NULL,76263),
(4313,342,'Help! It\'s the Hair Bear Bunch',NULL,now(),NULL,76264),
(4314,5069,'Runaround',NULL,now(),NULL,76265),
(4315,2960,'Call of the Wild',NULL,now(),NULL,76266),
(4316,343,'Goober and the Ghost-Chasers',NULL,now(),NULL,76267),
(4317,3075,'City of Angels (1976)',NULL,now(),NULL,76268),
(4318,6237,'The Tick (2001)',NULL,now(),NULL,76269),
(4319,4507,'Moloney',NULL,now(),NULL,76271),
(4320,4083,'Jumanji',NULL,now(),NULL,76272),
(4321,6625,'Wiseguy',NULL,now(),NULL,76273),
(4322,3927,'Hulk Hogan\'s Rock \'N\' Wrestling',NULL,now(),NULL,76274),
(4323,4941,'Rambo',NULL,now(),NULL,76275),
(4324,10223,'Chuck Norris: Karate Kommandos',NULL,now(),NULL,76276),
(4325,7117,'FM',NULL,now(),NULL,76277),
(4326,7997,'Match Game (1998)',NULL,now(),NULL,76279),
(4327,4890,'Primetime Glick',NULL,now(),NULL,76280),
(4328,345,'It\'s Punky Brewster',NULL,now(),NULL,76281),
(4329,15945,'The Hero',NULL,now(),NULL,76285),
(4330,16675,'Diana',NULL,now(),NULL,76286),
(4331,346,'Runaway Island',NULL,now(),NULL,76287),
(4332,3592,'Frank\'s Place',NULL,now(),NULL,76289),
(4333,4868,'Popular Mechanics for Kids',NULL,now(),NULL,76291),
(4334,4999,'Rhythm & Blues',NULL,now(),NULL,76293),
(4335,12531,'Tall Hopes',NULL,now(),NULL,76294),
(4336,18458,'First Impressions',NULL,now(),NULL,76295),
(4337,348,'Eek! the Cat',NULL,now(),NULL,76296),
(4338,2450,'2 Stupid Dogs',NULL,now(),NULL,76297),
(4339,2804,'Biker Mice from Mars',NULL,now(),NULL,76298),
(4340,5771,'The Fantastic Four',NULL,now(),NULL,76299),
(4341,5107,'Schnookums and Meat Funny Cartoon Show',NULL,now(),NULL,76300),
(4342,6113,'The Pursuit of Happiness',NULL,now(),NULL,76301),
(4343,4521,'Monty',NULL,now(),NULL,76303),
(4344,3551,'First Time Out',NULL,now(),NULL,76304),
(4345,5716,'The Darling Buds of May',NULL,now(),NULL,76306),
(4346,3051,'Chicago Sons',NULL,now(),NULL,76309),
(4347,349,'The Uninvited',NULL,now(),NULL,76310),
(4348,6337,'This Life',NULL,now(),NULL,76311),
(4349,2786,'Big Bad World',NULL,now(),NULL,76312),
(4350,3489,'Faith in the Future',NULL,now(),NULL,76313),
(4351,350,'Life Support',NULL,now(),NULL,76314),
(4352,3393,'Duck Patrol',NULL,now(),NULL,76315),
(4353,5686,'The Closer (1998)',NULL,now(),NULL,76317),
(4354,6320,'The Young Ones',NULL,now(),NULL,76318),
(4355,351,'Open All Hours',NULL,now(),NULL,76319),
(4356,352,'Oh Madeline',NULL,now(),NULL,76322),
(4357,16755,'Chicken Soup',NULL,now(),NULL,76323),
(4358,3596,'Frannie\'s Turn',NULL,now(),NULL,76324),
(4359,7305,'You Bet Your Life (1992)',NULL,now(),NULL,76325),
(4360,3216,'Damon',NULL,now(),NULL,76326),
(4361,19228,'The Man in the Family',NULL,now(),NULL,76327),
(4362,4889,'Prime Suspect',NULL,now(),NULL,76329),
(4363,8832,'One for the Road',NULL,now(),NULL,76330),
(4364,353,'A Many Splintered Thing',NULL,now(),NULL,76331),
(4365,3252,'David Copperfield',NULL,now(),NULL,76332),
(4366,3722,'Goodnight Sweetheart',NULL,now(),NULL,76333),
(4367,2944,'Butterflies',NULL,now(),NULL,76334),
(4368,354,'Going Straight',NULL,now(),NULL,76335),
(4369,355,'Between The Lines',NULL,now(),NULL,76336),
(4370,3218,'Dance Party USA',NULL,now(),NULL,76338),
(4371,357,'Happiness',NULL,now(),NULL,76340),
(4372,6266,'The Upper Hand',NULL,now(),NULL,76341),
(4373,3203,'Dad\'s Army',NULL,now(),NULL,76342),
(4374,3840,'Hetty Wainthropp Investigates',NULL,now(),NULL,76343),
(4375,26,'Santa Barbara',NULL,now(),NULL,76344),
(4376,358,'Harry Enfield and Chums',NULL,now(),NULL,76345),
(4377,19177,'Johnny Cash and Friends',NULL,now(),NULL,76346),
(4378,16661,'Off to See the Wizard',NULL,now(),NULL,76347),
(4379,3350,'Double Rush',NULL,now(),NULL,76348),
(4380,5822,'The Gong Show',NULL,now(),NULL,76349),
(4381,8377,'Mrs. Merton and Malcolm',NULL,now(),NULL,76350),
(4382,9652,'The Mrs. Merton Show',NULL,now(),NULL,76351),
(4383,4222,'Let Them Eat Cake',NULL,now(),NULL,76352),
(4384,4560,'Murder Most Horrid',NULL,now(),NULL,76353),
(4385,5199,'Silent Witness',NULL,now(),NULL,76355),
(4386,4917,'Push',NULL,now(),NULL,76356),
(4387,5178,'Sherman Oaks',NULL,now(),NULL,76357),
(4388,16933,'Doc Elliot',NULL,now(),NULL,76358),
(4389,3159,'Cracker (US)',NULL,now(),NULL,76359),
(4390,6026,'The New Avengers',NULL,now(),NULL,76360),
(4391,5601,'The Baron',NULL,now(),NULL,76361),
(4392,4046,'Jenny',NULL,now(),NULL,76363),
(4393,3390,'Dr Willoughby',NULL,now(),NULL,76364),
(4394,6136,'Return of the Saint',NULL,now(),NULL,76365),
(4395,359,'Boney',NULL,now(),NULL,76367),
(4396,4446,'Men Behaving Badly (US)',NULL,now(),NULL,76369),
(4397,4348,'Making the Band',NULL,now(),NULL,76370),
(4398,360,'Pajama Party',NULL,now(),NULL,76371),
(4399,5761,'The Fairly OddParents',NULL,now(),NULL,76372),
(4400,361,'Live and Kicking',NULL,now(),NULL,76373),
(4401,5066,'Rubik, the Amazing Cube',NULL,now(),NULL,76374),
(4402,6063,'The Omega Factor',NULL,now(),NULL,76375),
(4403,362,'Play for Today',NULL,now(),NULL,76376),
(4404,363,'Z Cars',NULL,now(),NULL,76377),
(4405,4242,'Life with Roger',NULL,now(),NULL,76378),
(4406,5809,'The Gentle Touch',NULL,now(),NULL,76379),
(4407,6567,'What\'s Happening Now!',NULL,now(),NULL,76380),
(4408,16773,'Good Time Harry',NULL,now(),NULL,76381),
(4409,364,'Day of the Triffids',NULL,now(),NULL,76382),
(4410,4224,'Lewis & Clark',NULL,now(),NULL,76383),
(4411,365,'Close and True',NULL,now(),NULL,76384),
(4412,5690,'The Comic Strip Presents',NULL,now(),NULL,76387),
(4413,6768,'The Triangle',NULL,now(),NULL,75599),
(4414,368,'Howard\'s Way',NULL,now(),NULL,76389),
(4415,369,'Nothing In Common',NULL,now(),NULL,76390),
(4416,370,'Crow\'s Nest',NULL,now(),NULL,76391),
(4417,371,'Life with Louie',NULL,now(),NULL,76392),
(4418,22459,'The Blue Angel',NULL,now(),NULL,76393),
(4419,18128,'To Tell the Truth (1990)',NULL,now(),NULL,76394),
(4420,3131,'Conan',NULL,now(),NULL,76396),
(4421,372,'Baby I\'m Back',NULL,now(),NULL,76397),
(4422,4012,'It Takes a Thief',NULL,now(),NULL,76398),
(4423,6043,'The Newsroom',NULL,now(),NULL,76399),
(4424,5262,'South Beach (1993)',NULL,now(),NULL,76400),
(4425,22400,'The Dean Martin Summer Show',NULL,now(),NULL,76401),
(4426,16815,'Tough Cookies',NULL,now(),NULL,76402),
(4427,2582,'Amazing Live Sea-Monkeys',NULL,now(),NULL,76403),
(4428,374,'Tales of the South Seas',NULL,now(),NULL,76404),
(4429,5844,'The Hat Squad',NULL,now(),NULL,76405),
(4430,375,'Alcoa Hour',NULL,now(),NULL,76407),
(4431,27,'NYPD',NULL,now(),NULL,76408),
(4432,376,'Goodyear Television Playhouse',NULL,now(),NULL,76409),
(4433,5190,'Showtime at the Apollo',NULL,now(),NULL,76412),
(4434,4468,'Mighty Max',NULL,now(),NULL,76413),
(4435,3849,'High Incident',NULL,now(),NULL,76414),
(4436,4876,'Power Play',NULL,now(),NULL,76415),
(4437,4714,'Once a Thief',NULL,now(),NULL,76416),
(4438,8203,'Side Effects',NULL,now(),NULL,76417),
(4439,2782,'Beyond Reality',NULL,now(),NULL,76419),
(4440,4561,'Murder One',NULL,now(),NULL,76421),
(4441,10332,'Our Time',NULL,now(),NULL,76422),
(4442,377,'Fantastic Max',NULL,now(),NULL,76424),
(4443,5541,'The Adventures of Sinbad',NULL,now(),NULL,76425),
(4444,378,'Bangkok Hilton',NULL,now(),NULL,76426),
(4445,2845,'Blue Peter',NULL,now(),NULL,76427),
(4446,7124,'Studio 57',NULL,now(),NULL,76429),
(4447,379,'Schlitz Playhouse of Stars',NULL,now(),NULL,76430),
(4448,380,'The DuPont Show Of The Month',NULL,now(),NULL,76431),
(4449,5184,'Shirley Temple\'s Storybook',NULL,now(),NULL,76432),
(4450,6127,'The Rebel',NULL,now(),NULL,76434),
(4451,5692,'The Commish',NULL,now(),NULL,76435),
(4452,3577,'Ford Startime',NULL,now(),NULL,76436),
(4453,15778,'The Chevy Mystery Show',NULL,now(),NULL,76437),
(4454,16591,'Harrigan and Son',NULL,now(),NULL,76439),
(4455,9188,'Camera Three',NULL,now(),NULL,76440),
(4456,381,'Channing',NULL,now(),NULL,76441),
(4457,5828,'The Greatest Show on Earth',NULL,now(),NULL,76442),
(4458,382,'Barefoot In The Park',NULL,now(),NULL,76443),
(4459,16669,'The Smith Family',NULL,now(),NULL,76444),
(4460,16890,'The Bigelow Theatre',NULL,now(),NULL,76445),
(4461,383,'Cavalcade of America',NULL,now(),NULL,76446),
(4462,3777,'Hallmark Hall Of Fame',NULL,now(),NULL,76447),
(4463,14908,'Omnibus',NULL,now(),NULL,76448),
(4464,5311,'Spy Game',NULL,now(),NULL,76450),
(4465,384,'The Overland Trail',NULL,now(),NULL,76451),
(4466,386,'The Play of the Week',NULL,now(),NULL,76453),
(4467,387,'The Mothers-In-Law',NULL,now(),NULL,76454),
(4468,5895,'The Jimmy Stewart Show',NULL,now(),NULL,76455),
(4469,12270,'Me and the Chimp',NULL,now(),NULL,76456),
(4470,18515,'The ABC Afternoon Playbreak',NULL,now(),NULL,76457),
(4471,388,'Chopper One',NULL,now(),NULL,76458),
(4472,9208,'Comedy Tonight',NULL,now(),NULL,76462),
(4473,389,'Microsoap',NULL,now(),NULL,76465),
(4474,390,'Byker Grove',NULL,now(),NULL,76466),
(4475,6682,'Young Americans',NULL,now(),NULL,76467),
(4476,5021,'Robert Montgomery Presents Your Lucky Strike Theatre',NULL,now(),NULL,76468),
(4477,3008,'CBS Playhouse',NULL,now(),NULL,76470),
(4478,28,'Social Studies',NULL,now(),NULL,76471),
(4479,8336,'Journey to the Unknown',NULL,now(),NULL,76472),
(4480,5289,'Spencer\'s Pilots',NULL,now(),NULL,76473),
(4481,12678,'Rafferty',NULL,now(),NULL,76474),
(4482,17999,'The Public Defender',NULL,now(),NULL,76475);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(4483,4539,'Mr. Lucky',NULL,now(),NULL,76476),
(4484,15617,'Going My Way',NULL,now(),NULL,76477),
(4485,391,'The Lux Video Theatre',NULL,now(),NULL,76478),
(4486,4723,'One Step Beyond',NULL,now(),NULL,76479),
(4487,3932,'Hype',NULL,now(),NULL,76480),
(4488,2547,'All-American Girl',NULL,now(),NULL,76481),
(4489,392,'The Felony Squad',NULL,now(),NULL,76482),
(4490,393,'Producers\' Showcase',NULL,now(),NULL,76483),
(4491,394,'Movin\' On',NULL,now(),NULL,76484),
(4492,7422,'What Really Happened To the Class Of \'65?',NULL,now(),NULL,76485),
(4493,10995,'Operation: Runaway',NULL,now(),NULL,76486),
(4494,15894,'The Eddie Capra Mysteries',NULL,now(),NULL,76487),
(4495,2578,'A Man Called Shenandoah',NULL,now(),NULL,76489),
(4496,2856,'Bob Hope Presents The Chrysler Theatre',NULL,now(),NULL,76490),
(4497,397,'Noddy',NULL,now(),NULL,76491),
(4498,4949,'Real Kids, Real Adventures',NULL,now(),NULL,76492),
(4499,398,'The Best Of Broadway',NULL,now(),NULL,76493),
(4500,399,'The Jane Wyman Show',NULL,now(),NULL,76494),
(4501,6171,'The Second Hundred Years',NULL,now(),NULL,76495),
(4502,400,'Funny Face',NULL,now(),NULL,76496),
(4503,16676,'Needles and Pins',NULL,now(),NULL,76497),
(4504,17053,'McCoy',NULL,now(),NULL,76498),
(4505,8823,'Nobody\'s Perfect',NULL,now(),NULL,76499),
(4506,3661,'Get a Life',NULL,now(),NULL,76500),
(4507,6474,'Under One Roof',NULL,now(),NULL,76501),
(4508,4282,'Los Dos Bros',NULL,now(),NULL,76503),
(4509,402,'Smack the Pony',NULL,now(),NULL,76504),
(4510,5383,'Studio One',NULL,now(),NULL,76505),
(4511,4808,'Peter Gunn',NULL,now(),NULL,76506),
(4512,4260,'Little Women',NULL,now(),NULL,76507),
(4513,3336,'Dog House',NULL,now(),NULL,76508),
(4514,4364,'Maniac Mansion',NULL,now(),NULL,76510),
(4515,4396,'Matt Waters',NULL,now(),NULL,76511),
(4516,17290,'No Soap, Radio',NULL,now(),NULL,76512),
(4517,22500,'Face The Music',NULL,now(),NULL,76513),
(4518,403,'Liberty Street',NULL,now(),NULL,76515),
(4519,4852,'Police Academy: The Series',NULL,now(),NULL,76516),
(4520,4316,'M.A.N.T.I.S.',NULL,now(),NULL,76517),
(4521,3904,'Hot Line',NULL,now(),NULL,76518),
(4522,15715,'Follow The Sun',NULL,now(),NULL,76519),
(4523,17875,'Leo And Liz In Beverly Hills',NULL,now(),NULL,76520),
(4524,405,'Starting From Scratch',NULL,now(),NULL,76521),
(4525,6125,'The Real World',NULL,now(),NULL,76522),
(4526,6146,'The Rogues',NULL,now(),NULL,76523),
(4527,5401,'Super Force',NULL,now(),NULL,76524),
(4528,13010,'The New Breed',NULL,now(),NULL,76525),
(4529,406,'Sam Benedict',NULL,now(),NULL,76526),
(4530,407,'Grindl',NULL,now(),NULL,76527),
(4531,6010,'Then Came Bronson',NULL,now(),NULL,76528),
(4532,5538,'The Adventures of Rin Tin Tin',NULL,now(),NULL,76529),
(4533,408,'Westinghouse Desilu Playhouse',NULL,now(),NULL,76530),
(4534,3862,'Hi Honey, I\'m Home!',NULL,now(),NULL,76534),
(4535,4407,'Maybe It\'s Me',NULL,now(),NULL,76535),
(4536,2839,'Blue\'s Clues',NULL,now(),NULL,76536),
(4537,409,'These Arms of Mine',NULL,now(),NULL,76537),
(4538,9962,'Bubblegum Crisis Tokyo 2040',NULL,now(),NULL,76538),
(4539,411,'Alcoa Premiere',NULL,now(),NULL,76539),
(4540,412,'Death Valley Days',NULL,now(),NULL,76540),
(4541,413,'Sky King',NULL,now(),NULL,76541),
(4542,414,'Tombstone Territory',NULL,now(),NULL,76542),
(4543,3049,'Cheyenne',NULL,now(),NULL,76543),
(4544,4788,'Payne',NULL,now(),NULL,76544),
(4545,5456,'Tales of Wells Fargo',NULL,now(),NULL,76545),
(4546,3652,'General Electric Theater',NULL,now(),NULL,76546),
(4547,10777,'Great Performances',NULL,now(),NULL,76548),
(4548,5701,'The Court',NULL,now(),NULL,76549),
(4549,4401,'Maximum Bob',NULL,now(),NULL,76550),
(4550,415,'New Attitude',NULL,now(),NULL,76553),
(4551,3783,'Hang Time',NULL,now(),NULL,76554),
(4552,4462,'Michael Hayes',NULL,now(),NULL,76555),
(4553,5388,'Sugarfoot',NULL,now(),NULL,76556),
(4554,416,'Lucas Tanner',NULL,now(),NULL,76558),
(4555,417,'Big John, Little John',NULL,now(),NULL,76559),
(4556,4120,'Kidd Video',NULL,now(),NULL,76560),
(4557,4281,'Loretta Young Show',NULL,now(),NULL,76561),
(4558,418,'Hawk',NULL,now(),NULL,76562),
(4559,6192,'The Single Guy',NULL,now(),NULL,76563),
(4560,16639,'Ichabod and Me',NULL,now(),NULL,76564),
(4561,16587,'Bringing Up Buddy',NULL,now(),NULL,76565),
(4562,419,'Straightaway',NULL,now(),NULL,76566),
(4563,420,'Arrest and Trial',NULL,now(),NULL,76567),
(4564,3497,'Fame L.A.',NULL,now(),NULL,76569),
(4565,2656,'Ask Harriet',NULL,now(),NULL,76572),
(4566,7001,'Wheel of Fortune',NULL,now(),NULL,76573),
(4567,2710,'Barbara',NULL,now(),NULL,76574),
(4568,421,'On The Up',NULL,now(),NULL,76575),
(4569,8717,'Haggard',NULL,now(),NULL,76576),
(4570,4443,'Melrose Place',NULL,now(),NULL,76577),
(4571,422,'Porridge',NULL,now(),NULL,76578),
(4572,423,'Peak Practice',NULL,now(),NULL,76579),
(4573,424,'Paul Merton in Galton & Simpson\'s...',NULL,now(),NULL,76580),
(4574,6518,'Virtual Murder',NULL,now(),NULL,76581),
(4575,3973,'Inspector Morse',NULL,now(),NULL,76582),
(4576,425,'Stay Lucky',NULL,now(),NULL,76583),
(4577,5639,'The Bounder',NULL,now(),NULL,76584),
(4578,8771,'Maggie & Her',NULL,now(),NULL,76585),
(4579,9898,'Making Out',NULL,now(),NULL,76586),
(4580,9024,'Seven of One',NULL,now(),NULL,76588),
(4581,3179,'Crown Court',NULL,now(),NULL,76589),
(4582,426,'The Two Ronnies',NULL,now(),NULL,76590),
(4583,9278,'The Dave Allen Show',NULL,now(),NULL,76591),
(4584,427,'The Liver Birds',NULL,now(),NULL,76592),
(4585,12086,'Emergency-Ward 10',NULL,now(),NULL,76593),
(4586,5922,'The Legend of Robin Hood',NULL,now(),NULL,76595),
(4587,5182,'Ship to Shore',NULL,now(),NULL,76596),
(4588,3752,'Guiding Light',NULL,now(),NULL,76597),
(4589,2499,'Adventures from the Book of Virtues',NULL,now(),NULL,76598),
(4590,428,'Quack Pack',NULL,now(),NULL,76599),
(4591,16975,'The Further Adventures Of Ellery Queen',NULL,now(),NULL,76601),
(4592,429,'The Ann Sothern Show',NULL,now(),NULL,76602),
(4593,29,'Winnetka Road',NULL,now(),NULL,76603),
(4594,8428,'Don\'t Tell Father',NULL,now(),NULL,76604),
(4595,3937,'I\'m Alan Partridge',NULL,now(),NULL,76605),
(4596,430,'The Likely Lads',NULL,now(),NULL,76606),
(4597,431,'Steptoe and Son',NULL,now(),NULL,76607),
(4598,17044,'The Jim Nabors Hour',NULL,now(),NULL,76608),
(4599,3877,'Holding the Baby (US)',NULL,now(),NULL,76609),
(4600,432,'Where The Action Is',NULL,now(),NULL,76610),
(4601,19740,'Max Glick',NULL,now(),NULL,76611),
(4602,30,'Cupid',NULL,now(),NULL,76612),
(4603,2948,'C-16: FBI',NULL,now(),NULL,76613),
(4604,5396,'Super Adventure Team',NULL,now(),NULL,76614),
(4605,5508,'The 100 Lives of Black Jack Savage',NULL,now(),NULL,76615),
(4606,4905,'Promised Land',NULL,now(),NULL,76617),
(4607,5803,'The Gary Coleman Show',NULL,now(),NULL,76618),
(4608,5167,'Shasta McNasty',NULL,now(),NULL,76619),
(4609,433,'Outriders',NULL,now(),NULL,76620),
(4610,4386,'Mary-Kate and Ashley in Action',NULL,now(),NULL,76621),
(4611,4047,'Jenny Jones',NULL,now(),NULL,76622),
(4612,4866,'Popstars',NULL,now(),NULL,76623),
(4613,6584,'Where in Time is Carmen Sandiego?',NULL,now(),NULL,76624),
(4614,435,'Waynehead',NULL,now(),NULL,76625),
(4615,436,'Kraft Television Theatre',NULL,now(),NULL,76626),
(4616,18761,'Eye Witness',NULL,now(),NULL,76628),
(4617,437,'Man Against Crime',NULL,now(),NULL,76629),
(4618,15708,'The Inner Sanctum',NULL,now(),NULL,76630),
(4619,7368,'The Elgin TV Hour',NULL,now(),NULL,76631),
(4620,438,'The Armstrong Circle Theatre',NULL,now(),NULL,76632),
(4621,7212,'The Lawless Years',NULL,now(),NULL,76633),
(4622,5788,'The Flying Doctors',NULL,now(),NULL,76634),
(4623,16598,'Markham',NULL,now(),NULL,76635),
(4624,8303,'The Lineup',NULL,now(),NULL,76636),
(4625,5965,'The Man From Blackhawk',NULL,now(),NULL,76637),
(4626,10146,'The Tab Hunter Show',NULL,now(),NULL,76638),
(4627,16642,'Ensign O\'Toole',NULL,now(),NULL,76640),
(4628,439,'The Lieutenant',NULL,now(),NULL,76641),
(4629,8407,'Arnie',NULL,now(),NULL,76642),
(4630,6487,'Unsolved Mysteries',NULL,now(),NULL,76644),
(4631,440,'Just Good Friends',NULL,now(),NULL,76645),
(4632,2571,'Aloha Paradise',NULL,now(),NULL,76646),
(4633,441,'Small & Frye',NULL,now(),NULL,76647),
(4634,5194,'Sidewalks Entertainment',NULL,now(),NULL,76649),
(4635,5940,'The Loner',NULL,now(),NULL,76650),
(4636,442,'The Road West',NULL,now(),NULL,76651),
(4637,4182,'Laredo',NULL,now(),NULL,76652),
(4638,5518,'The ABC Afterschool Special',NULL,now(),NULL,76653),
(4639,5440,'T.H.E. Cat',NULL,now(),NULL,76655),
(4640,6254,'The Tribe',NULL,now(),NULL,76656),
(4641,15644,'Hometown',NULL,now(),NULL,76657),
(4642,444,'Big Wave Dave\'s',NULL,now(),NULL,76658),
(4643,3657,'George & Leo',NULL,now(),NULL,76660),
(4644,445,'The Pulitzer Prize Playhouse',NULL,now(),NULL,76661),
(4645,5015,'Riverboat',NULL,now(),NULL,76663),
(4646,2841,'Blue Gender',NULL,now(),NULL,76664),
(4647,15772,'The Clock',NULL,now(),NULL,76667),
(4648,6104,'The Powers That Be',NULL,now(),NULL,76668),
(4649,7408,'The Web (1950)',NULL,now(),NULL,76669),
(4650,16892,'Somerset Maugham Theatre',NULL,now(),NULL,76670),
(4651,6085,'The Pepsi Cola Playhouse',NULL,now(),NULL,76671),
(4652,8074,'Glitter',NULL,now(),NULL,76672),
(4653,446,'The Littlest Hobo',NULL,now(),NULL,76674),
(4654,4133,'King of Kensington',NULL,now(),NULL,76675),
(4655,5715,'The Dana Carvey Show',NULL,now(),NULL,76676),
(4656,16752,'The Guys Next Door',NULL,now(),NULL,76677),
(4657,447,'Jackson\'s Wharf',NULL,now(),NULL,76678),
(4658,2835,'Blossom',NULL,now(),NULL,76679),
(4659,5569,'The American Embassy',NULL,now(),NULL,76680),
(4660,6416,'Traps',NULL,now(),NULL,76681),
(4661,4005,'It\'s a Man\'s World',NULL,now(),NULL,76682),
(4662,5550,'Adventures of Wild Bill Hickok',NULL,now(),NULL,76683),
(4663,4081,'Judge Roy Bean',NULL,now(),NULL,76684),
(4664,16796,'The Corner Bar',NULL,now(),NULL,76685),
(4665,4613,'Naked City',NULL,now(),NULL,76686),
(4666,8980,'Appointment with Adventure',NULL,now(),NULL,76687),
(4667,6087,'The Phil Silvers Show',NULL,now(),NULL,76688),
(4668,448,'Kraft Suspense Theatre',NULL,now(),NULL,76689),
(4669,449,'Mr. Broadway',NULL,now(),NULL,76690),
(4670,5043,'Rocky Road',NULL,now(),NULL,76691),
(4671,2490,'Actors Studio',NULL,now(),NULL,76692),
(4672,3585,'Fortune Hunter',NULL,now(),NULL,76693),
(4673,2579,'A Man Called Sloane',NULL,now(),NULL,76694),
(4674,2846,'Blue Seed',NULL,now(),NULL,76695),
(4675,4439,'Mega Man',NULL,now(),NULL,76696),
(4676,2939,'Burke\'s Law',NULL,now(),NULL,76697),
(4677,1572,'Joyce Meyer',NULL,now(),NULL,76699),
(4678,4359,'Man About the House',NULL,now(),NULL,76700),
(4679,1573,'Hancock\'s Half Hour',NULL,now(),NULL,76701),
(4680,1574,'What Ever Happened to the Likely Lads',NULL,now(),NULL,76702),
(4681,31,'PokÃƒÂ©mon',NULL,now(),NULL,76703),
(4682,1575,'Lifegame',NULL,now(),NULL,76704),
(4683,2787,'Big Brother',NULL,now(),NULL,76706),
(4684,3533,'Fear Factor',NULL,now(),NULL,76707),
(4685,2734,'Baywatch Nights',NULL,now(),NULL,76708),
(4686,1576,'Harmony Street',NULL,now(),NULL,76709),
(4687,4981,'Rescue 77',NULL,now(),NULL,76710),
(4688,4028,'Jackie Chan Adventures',NULL,now(),NULL,76711),
(4689,4578,'My Guide to Becoming a Rock Star',NULL,now(),NULL,76712),
(4690,3660,'George Lopez',NULL,now(),NULL,76713),
(4691,6402,'Traders',NULL,now(),NULL,76714),
(4692,3269,'Dear John',NULL,now(),NULL,76715),
(4693,4150,'Knots Landing',NULL,now(),NULL,76716),
(4694,1577,'Little Bear',NULL,now(),NULL,76717),
(4695,4538,'Mr. Belvedere',NULL,now(),NULL,76718),
(4696,4634,'Neighbours',NULL,now(),NULL,76719),
(4697,1580,'The Rez',NULL,now(),NULL,76720),
(4698,5975,'The Master',NULL,now(),NULL,76721),
(4699,18680,'DEA',NULL,now(),NULL,76722),
(4700,1582,'Number 96',NULL,now(),NULL,76723),
(4701,4896,'Prisoner: Cell Block H',NULL,now(),NULL,76724),
(4702,3072,'CityKids',NULL,now(),NULL,76726),
(4703,11782,'Father Dear Father in Australia',NULL,now(),NULL,76727),
(4704,4266,'Living Single',NULL,now(),NULL,76728),
(4705,1583,'Homicide',NULL,now(),NULL,76729),
(4706,1584,'Pizza',NULL,now(),NULL,76731),
(4707,4557,'Murder in Small Town X',NULL,now(),NULL,76734),
(4708,2704,'Ballykissangel',NULL,now(),NULL,76735),
(4709,3307,'Dick Powell Show',NULL,now(),NULL,76739),
(4710,1587,'Four Star Playhouse',NULL,now(),NULL,76740),
(4711,8306,'Long Hot Summer',NULL,now(),NULL,76741),
(4712,14852,'Court-Martial',NULL,now(),NULL,76742),
(4713,18150,'Favorite Story',NULL,now(),NULL,76743),
(4714,2711,'Barbara Stanwyck Show',NULL,now(),NULL,76744),
(4715,14863,'David Niven Show',NULL,now(),NULL,76745),
(4716,1588,'The Detectives',NULL,now(),NULL,76746),
(4717,12225,'Mr. President',NULL,now(),NULL,76747),
(4718,2493,'Adam\'s Rib',NULL,now(),NULL,76748),
(4719,4164,'Kurt Vonnegut\'s Monkey House',NULL,now(),NULL,76749),
(4720,16838,'My Friend Irma',NULL,now(),NULL,76750),
(4721,15558,'The Ed Wynn Show',NULL,now(),NULL,76751),
(4722,22388,'The Eddie Fisher Show',NULL,now(),NULL,76752),
(4723,8312,'The Lux Show With Rosemary Clooney',NULL,now(),NULL,76754),
(4724,15959,'The Garry Moore Show',NULL,now(),NULL,76755),
(4725,4215,'Legacy',NULL,now(),NULL,76756),
(4726,2964,'Camp Wilder',NULL,now(),NULL,76757),
(4727,6373,'To Have & To Hold',NULL,now(),NULL,76758),
(4728,6611,'Wild Oats',NULL,now(),NULL,76759),
(4729,2449,'2point4 Children',NULL,now(),NULL,76760),
(4730,1590,'Destry',NULL,now(),NULL,76762),
(4731,3619,'Frontline',NULL,now(),NULL,76763),
(4732,4107,'Katie Joplin',NULL,now(),NULL,76764),
(4733,1592,'Till Death Us Do Part',NULL,now(),NULL,76766),
(4734,1593,'Eisenhower & Lutz',NULL,now(),NULL,76767),
(4735,17078,'Detective in the House',NULL,now(),NULL,76768),
(4736,3228,'Dangerous Minds',NULL,now(),NULL,76769),
(4737,3658,'George and Mildred',NULL,now(),NULL,76770),
(4738,1594,'The Dave Thomas Comedy Show',NULL,now(),NULL,76771),
(4739,5698,'The Cosby Show',NULL,now(),NULL,76772),
(4740,23883,'Debt',NULL,now(),NULL,76773),
(4741,6656,'WWE Tough Enough',NULL,now(),NULL,76775),
(4742,6473,'Undergrads',NULL,now(),NULL,76776),
(4743,1596,'Angela Anaconda',NULL,now(),NULL,76777),
(4744,15518,'Talk Soup',NULL,now(),NULL,76778),
(4745,6659,'WWE RAW',NULL,now(),NULL,76779),
(4746,1597,'ECW on TNN',NULL,now(),NULL,76780),
(4747,1598,'ECW Hardcore TV',NULL,now(),NULL,76781),
(4748,6550,'WCW Thunder',NULL,now(),NULL,76782),
(4749,2757,'Being Eve',NULL,now(),NULL,76783),
(4750,9818,'The Jackie Gleason Show',NULL,now(),NULL,76784),
(4751,3949,'Imagine That',NULL,now(),NULL,76785),
(4752,11002,'Berrenger\'s',NULL,now(),NULL,76787),
(4753,1599,'Bucky O\'Hare and the Toad Wars',NULL,now(),NULL,76788),
(4754,4588,'My Little Pony Tales',NULL,now(),NULL,76790),
(4755,1600,'Mom, P.I.',NULL,now(),NULL,76791),
(4756,5879,'The Invisible Man (1975)',NULL,now(),NULL,76793),
(4757,1601,'The Thin Man',NULL,now(),NULL,76794),
(4758,21804,'Courage',NULL,now(),NULL,76795),
(4759,1602,'Love in a Cold Climate',NULL,now(),NULL,76798),
(4760,4935,'Radio Active',NULL,now(),NULL,76800),
(4761,16810,'Molloy',NULL,now(),NULL,76801),
(4762,6211,'The Starlost',NULL,now(),NULL,76802),
(4763,5743,'The Edge',NULL,now(),NULL,76803),
(4764,3839,'Herman\'s Head',NULL,now(),NULL,76804),
(4765,3791,'Hard Time on Planet Earth',NULL,now(),NULL,76805),
(4766,16375,'Haywire',NULL,now(),NULL,76806),
(4767,10319,'Totally Hidden Video',NULL,now(),NULL,76807),
(4768,22558,'A Current Affair (1986)',NULL,now(),NULL,76809),
(4769,5655,'The Brothers Garcia',NULL,now(),NULL,76810),
(4770,1603,'Oxbridge Blues',NULL,now(),NULL,76811),
(4771,12064,'Delvecchio',NULL,now(),NULL,76812),
(4772,6180,'Ellen',NULL,now(),NULL,76813),
(4773,6112,'The Proud Family',NULL,now(),NULL,76814),
(4774,4858,'Ponderosa',NULL,now(),NULL,76817),
(4775,1604,'Hearts and Bones',NULL,now(),NULL,76818),
(4776,6272,'The Vice',NULL,now(),NULL,76819),
(4777,1605,'Babes in the Wood',NULL,now(),NULL,76820),
(4778,1606,'City Central',NULL,now(),NULL,76821),
(4779,4607,'MythQuest',NULL,now(),NULL,76822),
(4780,1607,'Upright Citizens Brigade',NULL,now(),NULL,76823),
(4781,1608,'SCTV Network 90',NULL,now(),NULL,76824),
(4782,3808,'Hawkeye',NULL,now(),NULL,76827),
(4783,5028,'Robotech',NULL,now(),NULL,76828),
(4784,5275,'Space Ghost and Dino Boy',NULL,now(),NULL,76830),
(4785,3593,'Frankenstein Jr. and the Impossibles',NULL,now(),NULL,76831),
(4786,5848,'The Herculoids',NULL,now(),NULL,76832),
(4787,1609,'Birdman and the Galaxy Trio',NULL,now(),NULL,76833),
(4788,6019,'The New Adventures of Huck Finn',NULL,now(),NULL,76834),
(4789,6030,'The New Fantastic Four',NULL,now(),NULL,76835),
(4790,5818,'The Godzilla Power Hour',NULL,now(),NULL,76836),
(4791,3027,'Challenge of the SuperFriends',NULL,now(),NULL,76837),
(4792,4549,'Muddling Through',NULL,now(),NULL,76838),
(4793,6582,'Where I Live',NULL,now(),NULL,76839),
(4794,13321,'Martin Kane, Private Eye',NULL,now(),NULL,76840),
(4795,13404,'Villains',NULL,now(),NULL,76842),
(4796,7463,'A Family At War',NULL,now(),NULL,76843),
(4797,6915,'The Big Breakfast',NULL,now(),NULL,76844),
(4798,1610,'Ultraviolet',NULL,now(),NULL,76845),
(4799,5201,'Silver Spoons',NULL,now(),NULL,76848),
(4800,5674,'The Cavanaughs',NULL,now(),NULL,76849),
(4801,1612,'Curiosity Shop',NULL,now(),NULL,76851),
(4802,6252,'The Tracey Ullman Show',NULL,now(),NULL,76852),
(4803,5014,'Rising Damp',NULL,now(),NULL,76853),
(4804,1613,'Leap Years',NULL,now(),NULL,76854),
(4805,4096,'Just the Ten of Us',NULL,now(),NULL,76855),
(4806,4486,'Mission: Magic',NULL,now(),NULL,76856),
(4807,3929,'Human Target',NULL,now(),NULL,76857),
(4808,1614,'Parenthood',NULL,now(),NULL,76858),
(4809,6139,'The Ripping Friends',NULL,now(),NULL,76859),
(4810,4180,'Land of the Lost (1991)',NULL,now(),NULL,76861),
(4811,3860,'Highway to Heaven',NULL,now(),NULL,76862),
(4812,6130,'The Red Green Show',NULL,now(),NULL,76863),
(4813,4825,'Pigs Breakfast',NULL,now(),NULL,76865),
(4814,1616,'raw fm',NULL,now(),NULL,76866),
(4815,3442,'Empty Nest',NULL,now(),NULL,76867),
(4816,5985,'The Misadventures of Sheriff Lobo',NULL,now(),NULL,76869),
(4817,5168,'Shazam!',NULL,now(),NULL,76871),
(4818,3997,'Isis',NULL,now(),NULL,76872),
(4819,10173,'A Peaceable Kingdom',NULL,now(),NULL,76873),
(4820,3029,'Champs',NULL,now(),NULL,76874),
(4821,5144,'Seeing Things',NULL,now(),NULL,76875),
(4822,2496,'A Different World',NULL,now(),NULL,76877),
(4823,5549,'The Adventures of the Black Stallion',NULL,now(),NULL,76878),
(4824,1618,'The Bots Master',NULL,now(),NULL,76879),
(4825,32,'Second Chance',NULL,now(),NULL,76880),
(4826,5738,'The Duck Factory',NULL,now(),NULL,76881),
(4827,6590,'White Fang',NULL,now(),NULL,76882),
(4828,7748,'Airwaves',NULL,now(),NULL,76883),
(4829,5125,'Sealab 2021',NULL,now(),NULL,76884),
(4830,5437,'Sydney',NULL,now(),NULL,76887),
(4831,16797,'A Touch of Grace',NULL,now(),NULL,76888),
(4832,4430,'Medicine Ball',NULL,now(),NULL,76889),
(4833,1620,'The Cutting Edge',NULL,now(),NULL,76890),
(4834,5287,'Speed Buggy',NULL,now(),NULL,76891),
(4835,2752,'Beetlejuice',NULL,now(),NULL,76892),
(4836,33,'Home Free',NULL,now(),NULL,76893),
(4837,6690,'Yu-Gi-Oh!',NULL,now(),NULL,76894),
(4838,5899,'The John Larroquette Show',NULL,now(),NULL,76895),
(4839,4645,'Newhart',NULL,now(),NULL,76896),
(4840,5696,'The Corner',NULL,now(),NULL,76897),
(4841,19179,'The Marilyn McCoo and Billy Davis, Jr. Show',NULL,now(),NULL,76898),
(4842,3314,'Dinnerladies',NULL,now(),NULL,76899),
(4843,3565,'Flying Blind',NULL,now(),NULL,76900),
(4844,2700,'Baddiel\'s Syndrome',NULL,now(),NULL,76901),
(4845,2871,'Bonjour la classe',NULL,now(),NULL,76902),
(4846,3792,'Harry',NULL,now(),NULL,76903),
(4847,4762,'P.S.I. Luv U',NULL,now(),NULL,76904),
(4848,4425,'Medabots',NULL,now(),NULL,76906),
(4849,2847,'Blue Skies',NULL,now(),NULL,76907),
(4850,19259,'Ozzie\'s Girls',NULL,now(),NULL,76909),
(4851,5710,'The D.A. (1971)',NULL,now(),NULL,76910),
(4852,1623,'Bridget Loves Bernie',NULL,now(),NULL,76911),
(4853,10138,'O.K. Crackerby!',NULL,now(),NULL,76912),
(4854,2723,'Battery Park',NULL,now(),NULL,76913),
(4855,1624,'Bedtime',NULL,now(),NULL,76914),
(4856,9084,'Armstrong and Miller',NULL,now(),NULL,76916),
(4857,2739,'Beast',NULL,now(),NULL,76917),
(4858,1625,'The Cattanooga Cats',NULL,now(),NULL,76918),
(4859,1,'Fernwood 2Night',NULL,now(),NULL,76920),
(4860,1626,'Bang! Bang! It\'s Reeves and Mortimer',NULL,now(),NULL,76921),
(4861,7418,'Barking! (1998)',NULL,now(),NULL,76922),
(4862,1627,'Dark Shadows (1991)',NULL,now(),NULL,76925),
(4863,2851,'Bob',NULL,now(),NULL,76926),
(4864,2659,'As Time Goes By',NULL,now(),NULL,76927),
(4865,6443,'TV 101',NULL,now(),NULL,76928),
(4866,1629,'For the People',NULL,now(),NULL,76929),
(4867,4020,'Jabberjaw',NULL,now(),NULL,76930),
(4868,16740,'Jeannie',NULL,now(),NULL,76931),
(4869,6385,'Top of the Heap',NULL,now(),NULL,76933),
(4870,1633,'Black Harbour',NULL,now(),NULL,76935),
(4871,1634,'Taking the Falls',NULL,now(),NULL,76936),
(4872,1635,'Quads',NULL,now(),NULL,76937),
(4873,3567,'Flying Rhino Junior High',NULL,now(),NULL,76938),
(4874,1636,'Princess Sissi',NULL,now(),NULL,76939),
(4875,1637,'Blazing Dragons',NULL,now(),NULL,76940),
(4876,1638,'Beverly Hills Teens',NULL,now(),NULL,76941),
(4877,1639,'Today\'s Special',NULL,now(),NULL,76942),
(4878,6516,'Vinnie & Bobby',NULL,now(),NULL,76943),
(4879,8335,'Bus Stop',NULL,now(),NULL,76944),
(4880,5748,'The Eleventh Hour',NULL,now(),NULL,76945),
(4881,3890,'Hondo',NULL,now(),NULL,76946),
(4882,16775,'Time Express',NULL,now(),NULL,76947),
(4883,5545,'The Adventures of Superboy',NULL,now(),NULL,76948),
(4884,4961,'Red Shoe Diaries',NULL,now(),NULL,76949),
(4885,4256,'Little Dracula',NULL,now(),NULL,76952),
(4886,1640,'The Completely Mental Misadventures of Ed Grimley',NULL,now(),NULL,76953),
(4887,1641,'Second City TV',NULL,now(),NULL,76955),
(4888,1642,'The David Steinberg Show',NULL,now(),NULL,76956),
(4889,2998,'Cartoon Planet',NULL,now(),NULL,76957),
(4890,2697,'Back to Sherwood',NULL,now(),NULL,76958),
(4891,6118,'The Rat Patrol',NULL,now(),NULL,76959),
(4892,1643,'The Complete and Utter History of Britain',NULL,now(),NULL,76960),
(4893,1644,'The Goodies',NULL,now(),NULL,76961),
(4894,6547,'WCW Monday Nitro',NULL,now(),NULL,76962),
(4895,6303,'The Woody Woodpecker Show',NULL,now(),NULL,76964),
(4896,6145,'The Bullwinkle Show',NULL,now(),NULL,76965),
(4897,34,'The Trouble with Larry',NULL,now(),NULL,76966),
(4898,3349,'Double Dragon',NULL,now(),NULL,76967),
(4899,1646,'Please Save My Earth',NULL,now(),NULL,76968),
(4900,2,'America 2Night',NULL,now(),NULL,76969),
(4901,4583,'My Life as a Dog',NULL,now(),NULL,76970),
(4902,4544,'Mr. T and Tina',NULL,now(),NULL,76971),
(4903,1647,'Monster Squad',NULL,now(),NULL,76972),
(4904,12858,'Friends (1979)',NULL,now(),NULL,76973),
(4905,6531,'Wagon Train',NULL,now(),NULL,76974),
(4906,5041,'Rocky and His Friends',NULL,now(),NULL,76975),
(4907,12274,'Homeroom',NULL,now(),NULL,76976),
(4908,3529,'Father of the Bride',NULL,now(),NULL,76977),
(4909,4809,'Peter Pan and the Pirates',NULL,now(),NULL,76978),
(4910,1648,'Backberner',NULL,now(),NULL,76979),
(4911,5758,'The F.B.I.',NULL,now(),NULL,76980),
(4912,11581,'Law of the Land',NULL,now(),NULL,76981),
(4913,1649,'Paradise Beach',NULL,now(),NULL,76982),
(4914,1650,'Fire',NULL,now(),NULL,76983),
(4915,6135,'The Restless Gun',NULL,now(),NULL,76984),
(4916,5984,'The Mind of the Married Man',NULL,now(),NULL,76985),
(4917,1651,'Many Happy Returns',NULL,now(),NULL,76986),
(4918,2691,'Baby Blues',NULL,now(),NULL,76987),
(4919,3300,'Diagnosis: Unknown',NULL,now(),NULL,76988),
(4920,1652,'Taz - Mania',NULL,now(),NULL,76989),
(4921,5191,'Sibs',NULL,now(),NULL,76990),
(4922,3540,'Fievel\'s American Tails',NULL,now(),NULL,76991),
(4923,6238,'The Tick',NULL,now(),NULL,76992),
(4924,3004,'Casper',NULL,now(),NULL,76994),
(4925,5205,'Sins of the City',NULL,now(),NULL,76995),
(4926,6212,'The State',NULL,now(),NULL,76996),
(4927,4151,'Knowing Me, Knowing You with Alan Partridge',NULL,now(),NULL,76997),
(4928,1653,'The Sweeney',NULL,now(),NULL,76998),
(4929,3291,'Designing Women',NULL,now(),NULL,76999),
(4930,5406,'Superhuman Samurai Syber-Squad',NULL,now(),NULL,77000),
(4931,6308,'The Wright Verdicts',NULL,now(),NULL,77001),
(4932,1658,'Kwik Witz',NULL,now(),NULL,77002),
(4933,3281,'Delta',NULL,now(),NULL,77004),
(4934,4108,'Kavanagh Q.C.',NULL,now(),NULL,77006),
(4935,6013,'The Neverending Story',NULL,now(),NULL,77007),
(4936,6636,'Women of the House',NULL,now(),NULL,77008),
(4937,1660,'Not the Nine O\'Clock News',NULL,now(),NULL,77009),
(4938,5643,'The Brady Bunch',NULL,now(),NULL,77010),
(4939,15887,'Good Morning, World',NULL,now(),NULL,77011),
(4940,1661,'Parkinson (1998)',NULL,now(),NULL,77012),
(4941,6505,'Veronica\'s Closet',NULL,now(),NULL,77013),
(4942,6078,'The Partridge Family, 2200 A.D.',NULL,now(),NULL,77014),
(4943,16603,'Mulligan\'s Stew',NULL,now(),NULL,77015),
(4944,3,'Mary Hartman, Mary Hartman',NULL,now(),NULL,77016),
(4945,3143,'Cosby',NULL,now(),NULL,77017),
(4946,5357,'Stockard Channing: In Just Friends',NULL,now(),NULL,77018),
(4947,4730,'On The Buses',NULL,now(),NULL,77019),
(4948,3105,'Colditz',NULL,now(),NULL,77020),
(4949,16774,'Goodtime Girls',NULL,now(),NULL,77021),
(4950,10607,'The Explorers',NULL,now(),NULL,77023),
(4951,1662,'Ned\'s Newt',NULL,now(),NULL,77024),
(4952,15804,'Hold It Please',NULL,now(),NULL,77025),
(4953,18943,'Charlie Wild, Private Detective',NULL,now(),NULL,77026),
(4954,22465,'Bob and Ray',NULL,now(),NULL,77027),
(4955,5815,'The Girl from U.N.C.L.E.',NULL,now(),NULL,77028),
(4956,4290,'Love & War',NULL,now(),NULL,77029),
(4957,4996,'Rex the Runt',NULL,now(),NULL,77030),
(4958,4758,'Over the Top',NULL,now(),NULL,77032),
(4959,1663,'C Bear and Jamal',NULL,now(),NULL,77033),
(4960,2852,'Bob and Rose',NULL,now(),NULL,77034),
(4961,5490,'Temptation Island',NULL,now(),NULL,77036),
(4962,6422,'Trials of O\'Brien',NULL,now(),NULL,77037),
(4963,5068,'Rugrats',NULL,now(),NULL,77038),
(4964,1664,'Northwood',NULL,now(),NULL,77039),
(4965,3389,'Drop the Dead Donkey',NULL,now(),NULL,77040),
(4966,5725,'The Dick Van Dyke Show',NULL,now(),NULL,77041),
(4967,5140,'Secret Army',NULL,now(),NULL,77042),
(4968,1665,'Some Mothers Do \'Ave \'Em',NULL,now(),NULL,77043),
(4969,4397,'Maude',NULL,now(),NULL,77044),
(4970,10871,'Truth or Consequences',NULL,now(),NULL,77045),
(4971,5141,'Secret Squirrel Show',NULL,now(),NULL,77046),
(4972,12463,'The New Treasure Hunt',NULL,now(),NULL,77047),
(4973,5671,'The Caroline Rhea Show',NULL,now(),NULL,77048),
(4974,12603,'Here Come the Double Deckers',NULL,now(),NULL,77049),
(4975,13043,'And the Beat Goes On',NULL,now(),NULL,77050),
(4976,5973,'The Mary Tyler Moore Show',NULL,now(),NULL,77051),
(4977,5652,'The Broker\'s Man',NULL,now(),NULL,77052),
(4978,2653,'Arthur of the Britons',NULL,now(),NULL,77053),
(4979,1667,'Sabrina (1970)',NULL,now(),NULL,77056),
(4980,6179,'The Secret Lives of Waldo Kitty',NULL,now(),NULL,77057);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(4981,4358,'Mama\'s Family',NULL,now(),NULL,77059),
(4982,14591,'Classic Concentration',NULL,now(),NULL,77060),
(4983,7448,'Win, Lose or Draw',NULL,now(),NULL,77061),
(4984,1668,'Battle of The Planets',NULL,now(),NULL,77062),
(4985,1669,'Cardiac Arrest',NULL,now(),NULL,77063),
(4986,2631,'An Unsuitable Job for a Woman',NULL,now(),NULL,77064),
(4987,7299,'You Are There',NULL,now(),NULL,77065),
(4988,5821,'The Golden Palace',NULL,now(),NULL,77069),
(4989,4646,'New Kids On The Block',NULL,now(),NULL,77071),
(4990,20576,'The Price Is Right',NULL,now(),NULL,77072),
(4991,1670,'A Fine Romance (1989)',NULL,now(),NULL,77073),
(4992,18408,'Cutters',NULL,now(),NULL,77074),
(4993,7879,'Jeopardy!',NULL,now(),NULL,77075),
(4994,5441,'T.J. Hooker',NULL,now(),NULL,77076),
(4995,11045,'The Dating Game',NULL,now(),NULL,77077),
(4996,11115,'The Newlywed Game',NULL,now(),NULL,77078),
(4997,3559,'Flesh \'n\' Blood',NULL,now(),NULL,77079),
(4998,1671,'Shining Time Station',NULL,now(),NULL,77080),
(4999,18494,'Just in Time',NULL,now(),NULL,77081),
(5000,5920,'The Legend of Calamity Jane',NULL,now(),NULL,77082),
(5001,35,'Day by Day',NULL,now(),NULL,77083),
(5002,12755,'Jigsaw John',NULL,now(),NULL,77085),
(5003,5959,'The Magician',NULL,now(),NULL,77086),
(5004,6411,'Transformers Robots In Disguise',NULL,now(),NULL,77087),
(5005,5936,'The Little Mermaid',NULL,now(),NULL,77088),
(5006,5095,'Samurai Pizza Cats',NULL,now(),NULL,77089),
(5007,1672,'Who\'s Watching the Kids',NULL,now(),NULL,77090),
(5008,5736,'Dream Team',NULL,now(),NULL,77093),
(5009,15632,'House of Buggin\'',NULL,now(),NULL,77095),
(5010,15677,'Just Men!',NULL,now(),NULL,77096),
(5011,18921,'Caesar\'s Challenge',NULL,now(),NULL,77097),
(5012,15172,'Concentration',NULL,now(),NULL,77099),
(5013,18917,'Hit Man',NULL,now(),NULL,77100),
(5014,1674,'Press Your Luck',NULL,now(),NULL,77101),
(5015,1675,'Moolah Beach',NULL,now(),NULL,77102),
(5016,36,'It\'s a Living',NULL,now(),NULL,77103),
(5017,5562,'The Alvin Show',NULL,now(),NULL,77104),
(5018,16638,'Window on Main Street',NULL,now(),NULL,77105),
(5019,1676,'Let\'s Make A Deal',NULL,now(),NULL,77106),
(5020,14424,'Super Password',NULL,now(),NULL,77108),
(5021,10891,'The Wizard of Odds',NULL,now(),NULL,77111),
(5022,18898,'Whew!',NULL,now(),NULL,77113),
(5023,5399,'Supercarrier',NULL,now(),NULL,77115),
(5024,5646,'The Bradys',NULL,now(),NULL,77116),
(5025,1677,'Love Cruise: The Maiden Voyage',NULL,now(),NULL,77117),
(5026,5647,'The Brak Show',NULL,now(),NULL,77118),
(5027,3800,'Harvey Birdman, Attorney at Law',NULL,now(),NULL,77119),
(5028,2638,'Aqua Teen Hunger Force',NULL,now(),NULL,77120),
(5029,17968,'Foul Play',NULL,now(),NULL,77121),
(5030,3035,'Charlie Grace',NULL,now(),NULL,77122),
(5031,5644,'The Brady Bunch Hour',NULL,now(),NULL,77123),
(5032,5823,'The Good Life (1975)',NULL,now(),NULL,77124),
(5033,4265,'Living in Captivity',NULL,now(),NULL,77125),
(5034,3820,'Heart of the City',NULL,now(),NULL,77126),
(5035,4644,'The New Adventures of Beans Baxter',NULL,now(),NULL,77127),
(5036,7404,'We\'ll Get By',NULL,now(),NULL,77128),
(5037,3339,'Domestic Life',NULL,now(),NULL,77130),
(5038,3499,'Family Affair (1966)',NULL,now(),NULL,77131),
(5039,6568,'What\'s My Line?',NULL,now(),NULL,77132),
(5040,6965,'Good Morning Australia',NULL,now(),NULL,77133),
(5041,12013,'The Lawrence Welk Show',NULL,now(),NULL,77134),
(5042,8412,'Baffle',NULL,now(),NULL,77135),
(5043,18848,'Sale of The Century',NULL,now(),NULL,77136),
(5044,5521,'The Addams Family',NULL,now(),NULL,77137),
(5045,7251,'13 Queens Boulevard',NULL,now(),NULL,77138),
(5046,17077,'Off the Rack',NULL,now(),NULL,77139),
(5047,1679,'Turn-On',NULL,now(),NULL,77140),
(5048,12293,'Get Christie Love',NULL,now(),NULL,77141),
(5049,4246,'Likely Suspects',NULL,now(),NULL,77142),
(5050,19050,'Tic Tac Dough',NULL,now(),NULL,77143),
(5051,8163,'The Joker\'s Wild',NULL,now(),NULL,77144),
(5052,10330,'Second Chance (1977)',NULL,now(),NULL,77145),
(5053,1680,'Stickin\' Around',NULL,now(),NULL,77146),
(5054,1681,'Rolie Polie Olie',NULL,now(),NULL,77147),
(5055,3147,'Counterstrike',NULL,now(),NULL,77148),
(5056,1682,'Anatole',NULL,now(),NULL,77149),
(5057,4962,'Brian Jacques\' Redwall',NULL,now(),NULL,77150),
(5058,1683,'Rich Man, Poor Man',NULL,now(),NULL,77151),
(5059,1684,'Cold Feet',NULL,now(),NULL,77152),
(5060,4901,'Profit',NULL,now(),NULL,77154),
(5061,18919,'The Fun Factory',NULL,now(),NULL,77156),
(5062,4361,'Mancuso, FBI',NULL,now(),NULL,77157),
(5063,16684,'Kate McShane',NULL,now(),NULL,77158),
(5064,1686,'Mercury',NULL,now(),NULL,77159),
(5065,10314,'Skag',NULL,now(),NULL,77160),
(5066,4632,'Ned and Stacey',NULL,now(),NULL,77161),
(5067,9879,'True Adventures of Christopher Columbus',NULL,now(),NULL,77162),
(5068,2517,'Agony',NULL,now(),NULL,77163),
(5069,6403,'Trading Spaces',NULL,now(),NULL,77164),
(5070,16762,'E.A.R.T.H. Force',NULL,now(),NULL,77165),
(5071,8992,'Barney Boomer',NULL,now(),NULL,77166),
(5072,3059,'Chopper Squad',NULL,now(),NULL,77167),
(5073,1687,'E Street',NULL,now(),NULL,77168),
(5074,7716,'Echo Point',NULL,now(),NULL,77169),
(5075,3034,'Charlie\'s Angels',NULL,now(),NULL,77170),
(5076,1690,'You\'re the One',NULL,now(),NULL,77172),
(5077,1691,'Changi',NULL,now(),NULL,77173),
(5078,16682,'Joe Forrester',NULL,now(),NULL,77174),
(5079,5186,'Shoestring',NULL,now(),NULL,77175),
(5080,4,'Sara',NULL,now(),NULL,77176),
(5081,6321,'The Young Person\'s Guide to Becoming a Rock Star',NULL,now(),NULL,77177),
(5082,1692,'Who Wants to Be a Millionaire (UK)',NULL,now(),NULL,77178),
(5083,3776,'Halifax FP',NULL,now(),NULL,77179),
(5084,16819,'Bonino',NULL,now(),NULL,77183),
(5085,1693,'The Halls of Ivy',NULL,now(),NULL,77184),
(5086,15757,'The Gertrude Berg Show',NULL,now(),NULL,77185),
(5087,1694,'Temple Houston',NULL,now(),NULL,77186),
(5088,12572,'Doc (1975)',NULL,now(),NULL,77187),
(5089,6363,'Timothy Goes to School',NULL,now(),NULL,77188),
(5090,3564,'Fly by Night',NULL,now(),NULL,77189),
(5091,4853,'Police Squad!',NULL,now(),NULL,77190),
(5092,4098,'Kablam!',NULL,now(),NULL,77191),
(5093,1695,'The Charlie Horse Music Pizza',NULL,now(),NULL,77192),
(5094,3195,'Cybersix',NULL,now(),NULL,77193),
(5095,3377,'Dragon Tales',NULL,now(),NULL,77194),
(5096,1696,'The Pruitts of Southampton',NULL,now(),NULL,77195),
(5097,15821,'The Beautiful Phyllis Diller Show',NULL,now(),NULL,77196),
(5098,3629,'Future Cop',NULL,now(),NULL,77197),
(5099,5806,'Gemini Man',NULL,now(),NULL,77198),
(5100,3172,'Crisis Center',NULL,now(),NULL,77199),
(5101,1697,'My Little Margie',NULL,now(),NULL,77200),
(5102,3858,'High Stakes',NULL,now(),NULL,77201),
(5103,4669,'Nightmare Cafe',NULL,now(),NULL,77202),
(5104,6315,'They Came From Outer Space',NULL,now(),NULL,77205),
(5105,6643,'Woops!',NULL,now(),NULL,77206),
(5106,9896,'Make Room for Granddaddy',NULL,now(),NULL,77207),
(5107,6370,'Toast of the Town',NULL,now(),NULL,77208),
(5108,1698,'Pistols \'n\' Petticoats',NULL,now(),NULL,77209),
(5109,5,'Buffalo Bill',NULL,now(),NULL,77210),
(5110,3892,'Honey West',NULL,now(),NULL,77211),
(5111,1699,'SportsCentury',NULL,now(),NULL,77212),
(5112,4000,'I Spy',NULL,now(),NULL,77213),
(5113,1700,'When the Whistle Blows',NULL,now(),NULL,77214),
(5114,6457,'Two Pints of Lager and a Packet of Crisps',NULL,now(),NULL,77215),
(5115,6679,'You Can\'t Do That on Television',NULL,now(),NULL,77217),
(5116,4178,'Land of the Giants',NULL,now(),NULL,77218),
(5117,8631,'Alfresco',NULL,now(),NULL,77220),
(5118,8983,'Assaulted Nuts',NULL,now(),NULL,77221),
(5119,1702,'Tutti Fruiti',NULL,now(),NULL,77222),
(5120,6341,'Thompson',NULL,now(),NULL,77223),
(5121,3313,'Dinner for Five',NULL,now(),NULL,77224),
(5122,1704,'The Monday Dump',NULL,now(),NULL,77225),
(5123,1705,'Club Buggery',NULL,now(),NULL,77226),
(5124,4589,'My Mother the Car',NULL,now(),NULL,77227),
(5125,2650,'Arresting Behavior',NULL,now(),NULL,77228),
(5126,1706,'Exit 57',NULL,now(),NULL,77229),
(5127,5826,'The Great Defender',NULL,now(),NULL,77230),
(5128,1707,'Sunburn',NULL,now(),NULL,77232),
(5129,3223,'Dangerfield',NULL,now(),NULL,77233),
(5130,1708,'Urban Gothic',NULL,now(),NULL,77234),
(5131,3876,'Holby City',NULL,now(),NULL,77235),
(5132,4931,'Quincy, M.E.',NULL,now(),NULL,77236),
(5133,12375,'The Family Tree',NULL,now(),NULL,77237),
(5134,4598,'Mysterious Island',NULL,now(),NULL,77238),
(5135,3648,'Garrison\'s Gorillas',NULL,now(),NULL,77239),
(5136,5800,'The Gale Storm Show',NULL,now(),NULL,77240),
(5137,19903,'Ramar of the Jungle',NULL,now(),NULL,77241),
(5138,5530,'The Adventures of Jim Bowie',NULL,now(),NULL,77242),
(5139,3455,'Escape of the Artful Dodger',NULL,now(),NULL,77243),
(5140,5203,'Simon and Simon',NULL,now(),NULL,77244),
(5141,9957,'The Brighter Day',NULL,now(),NULL,77245),
(5142,1709,'Kitty Foyle',NULL,now(),NULL,77246),
(5143,18931,'Captains and the Kings',NULL,now(),NULL,77247),
(5144,1710,'Not Necessarily the News',NULL,now(),NULL,77249),
(5145,6543,'Watching Ellie',NULL,now(),NULL,77250),
(5146,12292,'Password Plus',NULL,now(),NULL,77251),
(5147,18880,'Bargain Hunters',NULL,now(),NULL,77252),
(5148,4035,'Jake and The Kid',NULL,now(),NULL,77255),
(5149,10243,'Valiant Lady',NULL,now(),NULL,77256),
(5150,8666,'Star Tonight',NULL,now(),NULL,77257),
(5151,1711,'The Dakotas',NULL,now(),NULL,77258),
(5152,5350,'Steve Canyon',NULL,now(),NULL,77261),
(5153,4068,'Johnny Staccato',NULL,now(),NULL,77262),
(5154,7903,'The Aquanauts',NULL,now(),NULL,77263),
(5155,3893,'Hong Kong',NULL,now(),NULL,77264),
(5156,6150,'The Rough Riders',NULL,now(),NULL,77265),
(5157,5280,'Space Precinct',NULL,now(),NULL,77269),
(5158,1712,'Mr. Adams and Eve',NULL,now(),NULL,77270),
(5159,5829,'The Green Hornet',NULL,now(),NULL,77271),
(5160,4541,'Mr. Merlin',NULL,now(),NULL,77272),
(5161,17848,'I Led Three Lives',NULL,now(),NULL,77273),
(5162,1713,'Operation Good Guys',NULL,now(),NULL,77274),
(5163,16608,'At Ease',NULL,now(),NULL,77275),
(5164,6562,'Werewolf',NULL,now(),NULL,77276),
(5165,2485,'Ace Ventura: Pet Detective',NULL,now(),NULL,77277),
(5166,2690,'Babar',NULL,now(),NULL,77278),
(5167,1714,'Gadget Boy\'s Adventures in History',NULL,now(),NULL,77279),
(5168,5998,'The Munsters',NULL,now(),NULL,77280),
(5169,1715,'Here\'s How',NULL,now(),NULL,77281),
(5170,1717,'Maxie\'s World',NULL,now(),NULL,77282),
(5171,4593,'My Pet Monster',NULL,now(),NULL,77283),
(5172,14144,'ProStars',NULL,now(),NULL,77284),
(5173,6,'The Rousters',NULL,now(),NULL,77286),
(5174,1718,'Wish Kids',NULL,now(),NULL,77289),
(5175,4794,'Pelswick',NULL,now(),NULL,77291),
(5176,3065,'Cimarron City',NULL,now(),NULL,77292),
(5177,1719,'Cover Up',NULL,now(),NULL,77293),
(5178,3896,'Hopalong Cassidy',NULL,now(),NULL,77295),
(5179,6156,'The Roy Rogers Show',NULL,now(),NULL,77296),
(5180,1720,'Time Gentlemen Please',NULL,now(),NULL,77297),
(5181,13214,'Noah\'s Ark',NULL,now(),NULL,77298),
(5182,4431,'Medics',NULL,now(),NULL,77299),
(5183,1721,'Soldier Soldier',NULL,now(),NULL,77300),
(5184,6524,'Voyage to the Bottom of the Sea',NULL,now(),NULL,77301),
(5185,4244,'Lights Out',NULL,now(),NULL,77303),
(5186,14846,'Assignment Vienna',NULL,now(),NULL,77304),
(5187,4737,'Oswald',NULL,now(),NULL,77305),
(5188,12107,'Morningstar/Eveningstar',NULL,now(),NULL,77306),
(5189,16196,'Mary (1978)',NULL,now(),NULL,77307),
(5190,3735,'Great Scott!',NULL,now(),NULL,77308),
(5191,3419,'East Side/West Side',NULL,now(),NULL,77309),
(5192,1722,'Run for Your Life',NULL,now(),NULL,77310),
(5193,3844,'Hey Vern, It\'s Ernest!',NULL,now(),NULL,77311),
(5194,2854,'Bobby\'s World',NULL,now(),NULL,77312),
(5195,5523,'The Addams Family (1992)',NULL,now(),NULL,77313),
(5196,1723,'Private Secretary',NULL,now(),NULL,77314),
(5197,3206,'Daddy Dearest',NULL,now(),NULL,77315),
(5198,5801,'The Gallant Men',NULL,now(),NULL,77316),
(5199,15559,'The Milton Berle Show',NULL,now(),NULL,77318),
(5200,7383,'Washington Square',NULL,now(),NULL,77319),
(5201,9061,'The Perry Como Show',NULL,now(),NULL,77320),
(5202,5921,'The Legend of Jesse James',NULL,now(),NULL,77321),
(5203,16671,'Chicago Teddy Bears',NULL,now(),NULL,77322),
(5204,3566,'Flying High',NULL,now(),NULL,77323),
(5205,6646,'Work With Me',NULL,now(),NULL,77324),
(5206,1724,'Our Private World',NULL,now(),NULL,77325),
(5207,1725,'I Married Joan',NULL,now(),NULL,77326),
(5208,18980,'The Jim Backus Show',NULL,now(),NULL,77327),
(5209,15822,'Talent Scouts',NULL,now(),NULL,77328),
(5210,5765,'The Famous Adventures of Mr. Magoo',NULL,now(),NULL,77329),
(5211,11144,'Blondie (1968)',NULL,now(),NULL,77330),
(5212,6283,'The Wayne Brady Show',NULL,now(),NULL,77331),
(5213,2833,'Blondie',NULL,now(),NULL,77332),
(5214,15646,'Professional Father',NULL,now(),NULL,77334),
(5215,15645,'The Brothers (1956)',NULL,now(),NULL,77336),
(5216,5999,'The Munsters Today',NULL,now(),NULL,77337),
(5217,13147,'Panic',NULL,now(),NULL,77338),
(5218,2880,'Boston Blackie',NULL,now(),NULL,77339),
(5219,5081,'Sable',NULL,now(),NULL,77340),
(5220,16736,'Everything\'s Relative (1987)',NULL,now(),NULL,77341),
(5221,21960,'Saved by the Bell: The College Years',NULL,now(),NULL,77342),
(5222,7016,'Most Wanted',NULL,now(),NULL,77343),
(5223,4792,'Pecola',NULL,now(),NULL,77344),
(5224,1727,'Topper',NULL,now(),NULL,77346),
(5225,15939,'High Performance',NULL,now(),NULL,77347),
(5226,2440,'2000 Malibu Road',NULL,now(),NULL,77348),
(5227,6222,'The Sullivans',NULL,now(),NULL,77350),
(5228,7,'Paper Dolls',NULL,now(),NULL,77351),
(5229,5964,'Man from Atlantis',NULL,now(),NULL,77352),
(5230,1729,'Skyways',NULL,now(),NULL,77353),
(5231,1730,'Henderson Kids',NULL,now(),NULL,77354),
(5232,1731,'Frontier Circus',NULL,now(),NULL,77355),
(5233,3309,'Diff\'rent Strokes',NULL,now(),NULL,77356),
(5234,3723,'Good Times',NULL,now(),NULL,77357),
(5235,13983,'Fireside Theatre',NULL,now(),NULL,77358),
(5236,12820,'The Lloyd Bridges Show',NULL,now(),NULL,77359),
(5237,12624,'The Danny Thomas Hour',NULL,now(),NULL,77361),
(5238,16767,'Rango',NULL,now(),NULL,77362),
(5239,5931,'The Life and Legend of Wyatt Earp',NULL,now(),NULL,77363),
(5240,2580,'Amanda\'s',NULL,now(),NULL,77364),
(5241,2625,'Annie Oakley',NULL,now(),NULL,77365),
(5242,4409,'Maybe This Time',NULL,now(),NULL,77366),
(5243,2814,'Blackhearts in Battersea',NULL,now(),NULL,77367),
(5244,11042,'Behind the Screen',NULL,now(),NULL,77368),
(5245,2904,'Breaking Point',NULL,now(),NULL,77369),
(5246,5238,'Soap',NULL,now(),NULL,77370),
(5247,16667,'Storefront Lawyers',NULL,now(),NULL,77371),
(5248,5561,'Alleyn Mysteries',NULL,now(),NULL,77372),
(5249,4882,'Press Gang',NULL,now(),NULL,77373),
(5250,4448,'Men into Space',NULL,now(),NULL,77374),
(5251,5109,'Science Fiction Theatre',NULL,now(),NULL,77375),
(5252,3113,'Colt .45',NULL,now(),NULL,77376),
(5253,37,'Phyllis',NULL,now(),NULL,77377),
(5254,2817,'Black Saddle',NULL,now(),NULL,77378),
(5255,1732,'State Coroner',NULL,now(),NULL,77379),
(5256,13749,'Hank',NULL,now(),NULL,77380),
(5257,7378,'Walter and Emily',NULL,now(),NULL,77382),
(5258,1733,'Love On A Rooftop',NULL,now(),NULL,77383),
(5259,14836,'Manhunter',NULL,now(),NULL,77384),
(5260,21149,'Dean Martin Presents the Golddiggers',NULL,now(),NULL,77385),
(5261,1734,'The Cindy Margolis Show',NULL,now(),NULL,77386),
(5262,1735,'Return to Peyton Place',NULL,now(),NULL,77387),
(5263,16947,'King\'s Crossing',NULL,now(),NULL,77389),
(5264,19140,'The Sammy Davis, Jr. Show',NULL,now(),NULL,77390),
(5265,16664,'To Rome with Love',NULL,now(),NULL,77391),
(5266,4416,'McKenna',NULL,now(),NULL,77392),
(5267,16607,'Annie McGuire',NULL,now(),NULL,77393),
(5268,1736,'Jason of Star Command',NULL,now(),NULL,77394),
(5269,5722,'The Dean Martin Show',NULL,now(),NULL,77395),
(5270,5130,'Search For Treasure Island',NULL,now(),NULL,77397),
(5271,4385,'Mary',NULL,now(),NULL,77399),
(5272,1737,'The Girl from Tomorrow',NULL,now(),NULL,77400),
(5273,1738,'Captain Gallant of the Foreign Legion',NULL,now(),NULL,77401),
(5274,4654,'New York News',NULL,now(),NULL,77402),
(5275,2504,'Adventures of Superman',NULL,now(),NULL,77403),
(5276,5684,'The Cisco Kid',NULL,now(),NULL,77404),
(5277,1740,'Masquerade',NULL,now(),NULL,77405),
(5278,2539,'Alice',NULL,now(),NULL,77406),
(5279,2877,'Boots and Saddles',NULL,now(),NULL,77407),
(5280,1741,'Brave Eagle',NULL,now(),NULL,77409),
(5281,1742,'Buckskin',NULL,now(),NULL,77410),
(5282,1743,'Buffalo Bill Jr.',NULL,now(),NULL,77411),
(5283,1744,'The Californians',NULL,now(),NULL,77412),
(5284,1745,'Casey Jones',NULL,now(),NULL,77413),
(5285,7504,'The Cheyenne Show',NULL,now(),NULL,77414),
(5286,3066,'Cimarron Strip',NULL,now(),NULL,77415),
(5287,1747,'Circus Boy',NULL,now(),NULL,77416),
(5288,4609,'My Two Dads',NULL,now(),NULL,77417),
(5289,1748,'Cowboy G-Men',NULL,now(),NULL,77418),
(5290,4697,'Nurses',NULL,now(),NULL,77419),
(5291,1749,'The Sandbaggers',NULL,now(),NULL,77420),
(5292,1750,'Cribb',NULL,now(),NULL,77421),
(5293,1751,'Tales of the City',NULL,now(),NULL,77422),
(5294,5135,'Second Thoughts',NULL,now(),NULL,77423),
(5295,17547,'Man with a Camera',NULL,now(),NULL,77424),
(5296,15741,'The Investigators (1961)',NULL,now(),NULL,77425),
(5297,4523,'Moonacre',NULL,now(),NULL,77426),
(5298,7415,'Wendy and Me',NULL,now(),NULL,77427),
(5299,5386,'Suddenly Susan',NULL,now(),NULL,77428),
(5300,15647,'Pride of the Family',NULL,now(),NULL,77429),
(5301,1752,'The People\'s Choice',NULL,now(),NULL,77430),
(5302,1753,'The Tom Ewell Show',NULL,now(),NULL,77431),
(5303,3456,'Espionage',NULL,now(),NULL,77432),
(5304,15667,'McKeever and the Colonel',NULL,now(),NULL,77433),
(5305,15679,'Hollywood House',NULL,now(),NULL,77434),
(5306,3151,'Courage the Cowardly Dog',NULL,now(),NULL,77435),
(5307,4104,'Kate & Allie',NULL,now(),NULL,77436),
(5308,1755,'Frontier',NULL,now(),NULL,77438),
(5309,5147,'Sergeant Preston of the Yukon',NULL,now(),NULL,77439),
(5310,4277,'Longstreet',NULL,now(),NULL,77440),
(5311,4814,'Peyton Place',NULL,now(),NULL,77441),
(5312,5642,'The Brady Brides',NULL,now(),NULL,77442),
(5313,1756,'The Tall Man',NULL,now(),NULL,77443),
(5314,6338,'This Old House',NULL,now(),NULL,77444),
(5315,1757,'The Ferals',NULL,now(),NULL,77445),
(5316,6232,'The Texan',NULL,now(),NULL,77446),
(5317,3528,'Father Murphy',NULL,now(),NULL,77447),
(5318,15688,'Coming of Age',NULL,now(),NULL,77448),
(5319,16812,'Down Home',NULL,now(),NULL,77449),
(5320,10994,'The Ruggles',NULL,now(),NULL,77450),
(5321,38,'Julia',NULL,now(),NULL,77451),
(5322,2561,'All My Children',NULL,now(),NULL,77452),
(5323,5052,'Room for Two',NULL,now(),NULL,77453),
(5324,1758,'Hec Ramsey',NULL,now(),NULL,77454),
(5325,16724,'The Devlin Connection',NULL,now(),NULL,77455),
(5326,1759,'Harris Against the World',NULL,now(),NULL,77456),
(5327,4960,'Redigo',NULL,now(),NULL,77457),
(5328,6351,'Thunder Alley',NULL,now(),NULL,77458),
(5329,1760,'Hotel de Paree',NULL,now(),NULL,77459),
(5330,1761,'Screen Director\'s Playhouse',NULL,now(),NULL,77460),
(5331,16898,'TV Reader\'s Digest',NULL,now(),NULL,77461),
(5332,10617,'The Reporter',NULL,now(),NULL,77462),
(5333,1762,'The Range Rider',NULL,now(),NULL,77463),
(5334,8813,'Front Row Center',NULL,now(),NULL,77464),
(5335,5855,'The Hogan Family',NULL,now(),NULL,77465),
(5336,3421,'Ed, Edd n\' Eddy',NULL,now(),NULL,77466),
(5337,1763,'Happy Families',NULL,now(),NULL,77467),
(5338,6544,'Watership Down',NULL,now(),NULL,77469),
(5339,6059,'The Office (1995)',NULL,now(),NULL,77470),
(5340,1764,'Ted and Alice',NULL,now(),NULL,77471),
(5341,1765,'Birds of a Feather',NULL,now(),NULL,77472),
(5342,5555,'The Alaskans',NULL,now(),NULL,77473),
(5343,1766,'Michael Shayne',NULL,now(),NULL,77474),
(5344,1767,'The Travels of Jaimie McPheeters',NULL,now(),NULL,77475),
(5345,2910,'Bring \'Em Back Alive',NULL,now(),NULL,77476),
(5346,15840,'Family',NULL,now(),NULL,77477),
(5347,15651,'Toma',NULL,now(),NULL,77478),
(5348,14884,'San Francisco International Airport',NULL,now(),NULL,77479),
(5349,6300,'The Wonderful World of Disney',NULL,now(),NULL,77480),
(5350,1768,'Judd for the Defense',NULL,now(),NULL,77481),
(5351,3221,'Danger',NULL,now(),NULL,77483),
(5352,39,'Bachelor Father',NULL,now(),NULL,77484),
(5353,16844,'My Favorite Husband',NULL,now(),NULL,77485),
(5354,1769,'Klondike',NULL,now(),NULL,77486),
(5355,2478,'Acapulco',NULL,now(),NULL,77487),
(5356,16640,'Margie',NULL,now(),NULL,77488),
(5357,9902,'Mama Rosa',NULL,now(),NULL,77489),
(5358,11105,'The Bing Crosby Show',NULL,now(),NULL,77490),
(5359,6077,'The Partridge Family',NULL,now(),NULL,77491),
(5360,6142,'The Roaring 20\'s',NULL,now(),NULL,77492),
(5361,16734,'Kay O\'Brien',NULL,now(),NULL,77493),
(5362,18307,'Dream Street',NULL,now(),NULL,77494),
(5363,3274,'Def Comedy Jam',NULL,now(),NULL,77495),
(5364,6004,'The Mystery Science Theater Hour',NULL,now(),NULL,77496),
(5365,4817,'Phenom',NULL,now(),NULL,77498),
(5366,4262,'Live Through This',NULL,now(),NULL,77499),
(5367,4015,'Ivanhoe',NULL,now(),NULL,77500),
(5368,2955,'Cagney & Lacey',NULL,now(),NULL,77501),
(5369,1770,'The Great American Quiz Show',NULL,now(),NULL,77502),
(5370,1771,'Grand Slam',NULL,now(),NULL,77503),
(5371,1772,'Two Girls Named Smith',NULL,now(),NULL,77504),
(5372,6345,'Three\'s Company',NULL,now(),NULL,77505),
(5373,1773,'Tate',NULL,now(),NULL,77506),
(5374,15780,'Sunday Showcase',NULL,now(),NULL,77507),
(5375,3196,'Cybill',NULL,now(),NULL,77509),
(5376,5170,'She\'s the Sheriff',NULL,now(),NULL,77510),
(5377,5525,'The Adventures of Champion',NULL,now(),NULL,77511),
(5378,5532,'The Adventures of Kit Carson',NULL,now(),NULL,77512),
(5379,5571,'The Americans',NULL,now(),NULL,77513),
(5380,5915,'The Late Late Show with Craig Kilborn',NULL,now(),NULL,77515),
(5381,19021,'Caryl & Marilyn: Real Friends',NULL,now(),NULL,77516),
(5382,4855,'Police Woman',NULL,now(),NULL,77517),
(5383,14945,'Just Our Luck',NULL,now(),NULL,77518),
(5384,1775,'26 Men',NULL,now(),NULL,77519),
(5385,6140,'The Road Home',NULL,now(),NULL,77520),
(5386,3885,'Home Fires',NULL,now(),NULL,77521),
(5387,3782,'Hangin\' with Mr. Cooper',NULL,now(),NULL,77522),
(5388,1776,'The Dumplings',NULL,now(),NULL,77523),
(5389,16725,'The Four Seasons',NULL,now(),NULL,77524),
(5390,1777,'Teachers',NULL,now(),NULL,77525),
(5391,1778,'Mercy Peak',NULL,now(),NULL,77527),
(5392,1779,'The Starland Vocal Band Show',NULL,now(),NULL,77528),
(5393,5717,'The David Letterman Show',NULL,now(),NULL,77529),
(5394,4951,'Reasonable Doubts',NULL,now(),NULL,77530),
(5395,22503,'Coke Time with Eddie Fisher',NULL,now(),NULL,77531),
(5396,17147,'Co-ed Fever',NULL,now(),NULL,77532),
(5397,1780,'The Kumars at No 42',NULL,now(),NULL,77534),
(5398,1782,'Mr Charity',NULL,now(),NULL,77535),
(5399,2513,'Against the Grain',NULL,now(),NULL,77536),
(5400,12865,'Our Family Honor',NULL,now(),NULL,77538),
(5401,4616,'Nancy Drew',NULL,now(),NULL,77539),
(5402,4001,'It\'s About Time',NULL,now(),NULL,77540),
(5403,40,'Nasty Boys',NULL,now(),NULL,77541),
(5404,4333,'Maggie and the Ferocious Beast',NULL,now(),NULL,77543),
(5405,5747,'The Electric Playground',NULL,now(),NULL,77545),
(5406,6006,'The Name of the Game',NULL,now(),NULL,77546),
(5407,12444,'Reading Rainbow',NULL,now(),NULL,77547),
(5408,4908,'Prudential Family Playhouse',NULL,now(),NULL,77548),
(5409,6175,'The Secret Empire',NULL,now(),NULL,77549),
(5410,7409,'A Wedding Story',NULL,now(),NULL,77550),
(5411,6170,'The Second Half',NULL,now(),NULL,77551),
(5412,1784,'Robotica',NULL,now(),NULL,77552),
(5413,4088,'Junkyard Wars',NULL,now(),NULL,77553),
(5414,2576,'A Makeover Story',NULL,now(),NULL,77554),
(5415,4006,'It\'s Christopher Lowell',NULL,now(),NULL,77555),
(5416,1785,'Chris Cross',NULL,now(),NULL,77557),
(5417,3505,'Family Dog',NULL,now(),NULL,77558),
(5418,41,'Working it Out',NULL,now(),NULL,77559),
(5419,13924,'FBI: The Untold Stories',NULL,now(),NULL,77560),
(5420,1786,'Bridges to Cross',NULL,now(),NULL,77561),
(5421,1787,'The Best Times',NULL,now(),NULL,77562),
(5422,7294,'A Year in the Life',NULL,now(),NULL,77563),
(5423,42,'I\'m Dickens--He\'s Fenster',NULL,now(),NULL,77564),
(5424,43,'December Bride',NULL,now(),NULL,77565),
(5425,14845,'Jigsaw',NULL,now(),NULL,77566),
(5426,44,'Pete and Gladys',NULL,now(),NULL,77567),
(5427,1788,'Acropolis Now',NULL,now(),NULL,77568),
(5428,3367,'Dragnet',NULL,now(),NULL,77570),
(5429,4770,'Paradise Falls',NULL,now(),NULL,77571),
(5430,45,'House Calls',NULL,now(),NULL,77572),
(5431,4140,'Kirk',NULL,now(),NULL,77573),
(5432,8235,'Police Surgeon',NULL,now(),NULL,77574),
(5433,10176,'Kodiak',NULL,now(),NULL,77575),
(5434,1790,'Three for the Road',NULL,now(),NULL,77576),
(5435,16655,'The John Forsythe Show',NULL,now(),NULL,77578),
(5436,1791,'Game, Set, and Match',NULL,now(),NULL,77579),
(5437,2857,'Bob Martin',NULL,now(),NULL,77580),
(5438,5622,'The Big Valley',NULL,now(),NULL,77581),
(5439,1792,'The New Land',NULL,now(),NULL,77582),
(5440,5624,'The Bill Cosby Show',NULL,now(),NULL,77583),
(5441,15871,'His & Hers',NULL,now(),NULL,77584),
(5442,2443,'21 Jump Street',NULL,now(),NULL,77585),
(5443,4511,'Mon Colle Knights',NULL,now(),NULL,77586),
(5444,2488,'Action Man (1995)',NULL,now(),NULL,77587),
(5445,6191,'The Sinbad Show',NULL,now(),NULL,77588),
(5446,4508,'Monarch of the Glen',NULL,now(),NULL,77589),
(5447,16820,'The Boys (1993)',NULL,now(),NULL,77590),
(5448,5210,'Sister Kate',NULL,now(),NULL,77591),
(5449,6650,'Worzel Gummidge',NULL,now(),NULL,77592),
(5450,8874,'Educating Marmalade',NULL,now(),NULL,77593),
(5451,1793,'Freddie and Max',NULL,now(),NULL,77594),
(5452,3916,'How Do You Want Me?',NULL,now(),NULL,77595),
(5453,4328,'Madman of the People',NULL,now(),NULL,77596),
(5454,5811,'The George Burns Show',NULL,now(),NULL,77597),
(5455,5661,'The Bugs Bunny Show',NULL,now(),NULL,77598),
(5456,10166,'Peter Loves Mary',NULL,now(),NULL,77599),
(5457,5659,'The Bugs Bunny/Road Runner Hour',NULL,now(),NULL,77601),
(5458,2991,'Carol & Company',NULL,now(),NULL,77602),
(5459,6123,'The Real McCoys',NULL,now(),NULL,77603),
(5460,15327,'Higher Education',NULL,now(),NULL,77605),
(5461,6309,'The Wubbulous World of Dr. Seuss',NULL,now(),NULL,77606),
(5462,3978,'In the Heat of the Night',NULL,now(),NULL,77607),
(5463,4119,'Key West',NULL,now(),NULL,77608),
(5464,10987,'The Rounders',NULL,now(),NULL,77610),
(5465,17763,'From Here to Eternity',NULL,now(),NULL,77611),
(5466,5531,'The Adventures of Jimmy Neutron: Boy Genius',NULL,now(),NULL,77612),
(5467,5584,'The Army Show',NULL,now(),NULL,77613),
(5468,3694,'Gloria',NULL,now(),NULL,77614),
(5469,1796,'Love Bites',NULL,now(),NULL,77615),
(5470,16686,'The Montefuscos',NULL,now(),NULL,77616),
(5471,16732,'Easy Street',NULL,now(),NULL,77617),
(5472,5780,'The Five Mrs. Buchanans',NULL,now(),NULL,77618),
(5473,4226,'Liberty\'s Kids',NULL,now(),NULL,77619),
(5474,1797,'5th Quadrant',NULL,now(),NULL,77620),
(5475,3527,'Father Knows Best',NULL,now(),NULL,77621),
(5476,3795,'Harry and the Hendersons',NULL,now(),NULL,77624),
(5477,6388,'Totally Spies',NULL,now(),NULL,77625),
(5478,5989,'The Mommies',NULL,now(),NULL,77626);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(5479,19594,'Motown Revue',NULL,now(),NULL,77627),
(5480,5202,'Simon',NULL,now(),NULL,77628),
(5481,11035,'The Baileys of Balboa',NULL,now(),NULL,77629),
(5482,1799,'Gentle Ben',NULL,now(),NULL,77630),
(5483,17954,'Salty',NULL,now(),NULL,77631),
(5484,17073,'Gung Ho',NULL,now(),NULL,77632),
(5485,46,'Apple Pie',NULL,now(),NULL,77633),
(5486,7395,'Wax on Wheels',NULL,now(),NULL,77635),
(5487,47,'Blansky\'s Beauties',NULL,now(),NULL,77636),
(5488,7394,'Wax Cracks Hollywood',NULL,now(),NULL,77639),
(5489,1800,'Ruby',NULL,now(),NULL,77642),
(5490,9196,'The Carol Burnett Show (1991)',NULL,now(),NULL,77646),
(5491,48,'True Colors',NULL,now(),NULL,77647),
(5492,12208,'Fox After Breakfast',NULL,now(),NULL,77648),
(5493,11072,'A World Apart',NULL,now(),NULL,77649),
(5494,1801,'The Nancy Walker Show',NULL,now(),NULL,77650),
(5495,2832,'Bliss',NULL,now(),NULL,77651),
(5496,8625,'The Paul Hogan Show',NULL,now(),NULL,77652),
(5497,1802,'The Tide of Life',NULL,now(),NULL,77653),
(5498,3043,'Checkmate',NULL,now(),NULL,77654),
(5499,1803,'Clocking Off',NULL,now(),NULL,77655),
(5500,7390,'Waterfront Beat',NULL,now(),NULL,77656),
(5501,1804,'Liverpool 1',NULL,now(),NULL,77657),
(5502,1805,'Hope and Glory',NULL,now(),NULL,77658),
(5503,4449,'Men of the World',NULL,now(),NULL,77659),
(5504,1806,'Sex, Chips & Rock n\' Roll',NULL,now(),NULL,77660),
(5505,2919,'Brotherly Love',NULL,now(),NULL,77662),
(5506,4415,'McHale\'s Navy',NULL,now(),NULL,77664),
(5507,5767,'The Famous Five',NULL,now(),NULL,77665),
(5508,5566,'The Amazing Race',NULL,now(),NULL,77666),
(5509,5017,'Road Rules',NULL,now(),NULL,77667),
(5510,2943,'Butt Ugly Martians',NULL,now(),NULL,77668),
(5511,1809,'Bread',NULL,now(),NULL,77669),
(5512,49,'Fish',NULL,now(),NULL,77670),
(5513,8599,'Around the World with Orson Welles',NULL,now(),NULL,77671),
(5514,5106,'Scene of the Crime',NULL,now(),NULL,77674),
(5515,4790,'Pearl',NULL,now(),NULL,77675),
(5516,3821,'Hearts Afire',NULL,now(),NULL,77676),
(5517,5517,'The Abbott and Costello Show',NULL,now(),NULL,77677),
(5518,16729,'It\'s Not Easy',NULL,now(),NULL,77678),
(5519,1810,'Gavilan',NULL,now(),NULL,77679),
(5520,1812,'Grafters',NULL,now(),NULL,77682),
(5521,2879,'Bosom Buddies',NULL,now(),NULL,77683),
(5522,10948,'The Robert Guillaume Show',NULL,now(),NULL,77684),
(5523,12176,'Breakfast Time',NULL,now(),NULL,77685),
(5524,1813,'Paris (1994)',NULL,now(),NULL,77687),
(5525,4025,'Jack and Mike',NULL,now(),NULL,77688),
(5526,16715,'Freebie and the Bean',NULL,now(),NULL,77689),
(5527,1814,'Grandpa Goes to Washington',NULL,now(),NULL,77690),
(5528,3280,'Dellaventura',NULL,now(),NULL,77691),
(5529,50,'Burke\'s Law (1994)',NULL,now(),NULL,77692),
(5530,4067,'Johnny Ringo',NULL,now(),NULL,77693),
(5531,8488,'Firehouse',NULL,now(),NULL,77694),
(5532,1815,'Robin\'s Hoods',NULL,now(),NULL,77695),
(5533,6264,'The Untouchables',NULL,now(),NULL,77696),
(5534,3913,'Houston Knights',NULL,now(),NULL,77697),
(5535,2933,'Bugs',NULL,now(),NULL,77698),
(5536,5621,'The Big One',NULL,now(),NULL,77700),
(5537,16790,'Popi',NULL,now(),NULL,77701),
(5538,3000,'Casablanca (1983)',NULL,now(),NULL,77702),
(5539,6094,'The Pirates of Dark Water',NULL,now(),NULL,77704),
(5540,2552,'All Along the Watchtower',NULL,now(),NULL,77705),
(5541,21163,'The X Show',NULL,now(),NULL,77706),
(5542,4942,'Ramona',NULL,now(),NULL,77707),
(5543,1816,'Eric\'s World',NULL,now(),NULL,77708),
(5544,5986,'The Mod Squad',NULL,now(),NULL,77709),
(5545,6465,'Ultra 7',NULL,now(),NULL,77711),
(5546,4947,'Raven',NULL,now(),NULL,77714),
(5547,3440,'Emmerdale',NULL,now(),NULL,77715),
(5548,2875,'Boon',NULL,now(),NULL,77716),
(5549,1818,'Juliet Bravo',NULL,now(),NULL,77717),
(5550,9482,'Families',NULL,now(),NULL,77718),
(5551,13399,'Squadron',NULL,now(),NULL,77719),
(5552,1819,'Fast Forward',NULL,now(),NULL,77720),
(5553,2864,'Bodyguards',NULL,now(),NULL,77721),
(5554,2536,'ALF Tales',NULL,now(),NULL,77723),
(5555,5143,'See How They Run',NULL,now(),NULL,77724),
(5556,3684,'Gimme a Break!',NULL,now(),NULL,77726),
(5557,1822,'Goodness Gracious Me',NULL,now(),NULL,77727),
(5558,1823,'Cold Lazarus',NULL,now(),NULL,77729),
(5559,1824,'Coogan\'s Run',NULL,now(),NULL,77730),
(5560,1825,'London\'s Burning',NULL,now(),NULL,77731),
(5561,51,'Three\'s a Crowd',NULL,now(),NULL,77732),
(5562,3275,'Degrassi: The Next Generation',NULL,now(),NULL,77733),
(5563,1826,'Makin\' It',NULL,now(),NULL,77734),
(5564,1827,'The Imogene Coca Show',NULL,now(),NULL,77735),
(5565,3399,'Duet',NULL,now(),NULL,77736),
(5566,1828,'Your Show of Shows',NULL,now(),NULL,77737),
(5567,19084,'The Sid Caesar Show',NULL,now(),NULL,77738),
(5568,1829,'The Julie Andrews Hour',NULL,now(),NULL,77739),
(5569,52,'Babes',NULL,now(),NULL,77740),
(5570,1831,'The Cara Williams Show',NULL,now(),NULL,77741),
(5571,9479,'Sid Caesar Invites You',NULL,now(),NULL,77742),
(5572,2688,'B.J. and the Bear',NULL,now(),NULL,77744),
(5573,10965,'Roger Miller Show',NULL,now(),NULL,77749),
(5574,4493,'Mister Rogers\' Neighborhood',NULL,now(),NULL,77750),
(5575,8653,'Blue Heaven',NULL,now(),NULL,77751),
(5576,8207,'Private Eye',NULL,now(),NULL,77752),
(5577,1833,'The Directors',NULL,now(),NULL,77753),
(5578,5574,'The Andy Griffith Show',NULL,now(),NULL,77754),
(5579,3109,'Colin\'s Sandwich',NULL,now(),NULL,77755),
(5580,6148,'The Ropers',NULL,now(),NULL,77757),
(5581,11830,'The Nanette Fabray Show',NULL,now(),NULL,77759),
(5582,6214,'The Stockard Channing Show',NULL,now(),NULL,77760),
(5583,6630,'WKRP in Cincinnati',NULL,now(),NULL,77761),
(5584,1834,'Kissyfur',NULL,now(),NULL,77762),
(5585,6472,'Underdog',NULL,now(),NULL,77763),
(5586,5099,'Saturday Night Special',NULL,now(),NULL,77765),
(5587,4740,'Our House',NULL,now(),NULL,77766),
(5588,5057,'The Roswell Conspiracies',NULL,now(),NULL,77767),
(5589,1836,'Whatever Turns You On',NULL,now(),NULL,77768),
(5590,3960,'Ink',NULL,now(),NULL,77769),
(5591,2651,'Arsenio',NULL,now(),NULL,77770),
(5592,3426,'Eight is Enough',NULL,now(),NULL,77771),
(5593,1838,'Star Blazers',NULL,now(),NULL,77773),
(5594,6467,'Ultraman Tiga',NULL,now(),NULL,77774),
(5595,4372,'Marine Boy',NULL,now(),NULL,77775),
(5596,2613,'Angie',NULL,now(),NULL,77776),
(5597,5288,'Speed Racer',NULL,now(),NULL,77777),
(5598,1839,'The Detectives Starring Robert Taylor',NULL,now(),NULL,77778),
(5599,5730,'The Donna Reed Show',NULL,now(),NULL,77779),
(5600,3632,'G.I. Joe',NULL,now(),NULL,77780),
(5601,6552,'Webster',NULL,now(),NULL,77783),
(5602,5498,'That \'80s Show',NULL,now(),NULL,77784),
(5603,16770,'Semi-Tough',NULL,now(),NULL,77785),
(5604,6644,'Working',NULL,now(),NULL,77786),
(5605,17079,'Me and Mrs. C',NULL,now(),NULL,77787),
(5606,2971,'Captain Butler',NULL,now(),NULL,77788),
(5607,3046,'Chelmsford 123',NULL,now(),NULL,77789),
(5608,3563,'Flo',NULL,now(),NULL,77790),
(5609,6461,'The Ultimate Book of Spells',NULL,now(),NULL,77793),
(5610,4157,'Kong: The Animated Series',NULL,now(),NULL,77794),
(5611,16648,'Valentine\'s Day',NULL,now(),NULL,77795),
(5612,16688,'The Practice (1976)',NULL,now(),NULL,77796),
(5613,16714,'I\'m a Big Girl Now',NULL,now(),NULL,77797),
(5614,1842,'The Bill Dana Show',NULL,now(),NULL,77798),
(5615,1843,'Total Access 24/7',NULL,now(),NULL,77801),
(5616,2951,'C.P.O. Sharkey',NULL,now(),NULL,77802),
(5617,16592,'Dan Raven',NULL,now(),NULL,77803),
(5618,1844,'Matt Lincoln',NULL,now(),NULL,77804),
(5619,1845,'The Richard Boone Show',NULL,now(),NULL,77805),
(5620,3709,'Good Advice',NULL,now(),NULL,77806),
(5621,10427,'On Camera',NULL,now(),NULL,77807),
(5622,18961,'The Doctor',NULL,now(),NULL,77808),
(5623,5031,'Robot Wars: Extreme Warriors',NULL,now(),NULL,77809),
(5624,3597,'Frasier',NULL,now(),NULL,77811),
(5625,6596,'Whoops Apocalypse',NULL,now(),NULL,77812),
(5626,3633,'G.I. Joe Extreme',NULL,now(),NULL,77814),
(5627,3244,'Dark Stalkers',NULL,now(),NULL,77815),
(5628,2808,'Bionic Six',NULL,now(),NULL,77816),
(5629,6316,'The Yellow Rose',NULL,now(),NULL,77817),
(5630,2516,'Agatha Christie\'s Partners in Crime',NULL,now(),NULL,77818),
(5631,1846,'Inspector Maigret',NULL,now(),NULL,77819),
(5632,4366,'Man in a Suitcase',NULL,now(),NULL,77820),
(5633,16689,'Busting Loose',NULL,now(),NULL,77821),
(5634,10576,'The Red Buttons Show',NULL,now(),NULL,77822),
(5635,8390,'The Creatives',NULL,now(),NULL,77823),
(5636,17288,'Cassie & Co.',NULL,now(),NULL,77824),
(5637,1847,'Sitting Pretty',NULL,now(),NULL,77825),
(5638,1848,'Movie Surfers',NULL,now(),NULL,77827),
(5639,4768,'Palace Guard',NULL,now(),NULL,77828),
(5640,4874,'Pound Puppies',NULL,now(),NULL,77829),
(5641,1850,'Belfry Witches',NULL,now(),NULL,77830),
(5642,12218,'Fast Times',NULL,now(),NULL,77832),
(5643,5241,'So Graham Norton',NULL,now(),NULL,77833),
(5644,1852,'The Ruth Rendell Mysteries',NULL,now(),NULL,77834),
(5645,1853,'CyberGirl',NULL,now(),NULL,77835),
(5646,53,'Going Places',NULL,now(),NULL,77836),
(5647,4650,'New Scotland Yard',NULL,now(),NULL,77837),
(5648,1855,'Home to Roost',NULL,now(),NULL,77838),
(5649,13371,'Perfect Scoundrels',NULL,now(),NULL,77839),
(5650,1856,'The High Life',NULL,now(),NULL,77840),
(5651,13141,'Blackmail',NULL,now(),NULL,77841),
(5652,5300,'Spider-Woman',NULL,now(),NULL,77843),
(5653,12048,'The Tony Danza Show (1997)',NULL,now(),NULL,77844),
(5654,3285,'Dennis Miller Live',NULL,now(),NULL,77846),
(5655,4320,'MacGyver',NULL,now(),NULL,77847),
(5656,54,'Amen',NULL,now(),NULL,77850),
(5657,16645,'Tom, Dick And Mary',NULL,now(),NULL,77851),
(5658,55,'That\'s My Mama',NULL,now(),NULL,77854),
(5659,14638,'Married People',NULL,now(),NULL,77855),
(5660,16643,'Don\'t Call Me Charlie',NULL,now(),NULL,77856),
(5661,10967,'Roll Out',NULL,now(),NULL,77857),
(5662,5455,'Tales of Tomorrow',NULL,now(),NULL,77858),
(5663,5852,'The Highwayman',NULL,now(),NULL,77859),
(5664,3720,'Good News',NULL,now(),NULL,77860),
(5665,4944,'Randall & Hopkirk (Deceased) (2000)',NULL,now(),NULL,77861),
(5666,22890,'Sha Na Na',NULL,now(),NULL,77862),
(5667,13221,'Strangers',NULL,now(),NULL,77863),
(5668,2444,'227',NULL,now(),NULL,77864),
(5669,3641,'Game On!',NULL,now(),NULL,77865),
(5670,7153,'Wogan',NULL,now(),NULL,77867),
(5671,2987,'Car 54, Where are You?',NULL,now(),NULL,77869),
(5672,3227,'Dangerous Curves',NULL,now(),NULL,77872),
(5673,10469,'Mother\'s Day',NULL,now(),NULL,77873),
(5674,4839,'Players',NULL,now(),NULL,77874),
(5675,6669,'XFL',NULL,now(),NULL,77875),
(5676,1859,'Dark Ages',NULL,now(),NULL,77876),
(5677,3595,'Frank Stubbs Promotes',NULL,now(),NULL,77877),
(5678,6929,'Wild On!',NULL,now(),NULL,77878),
(5679,2645,'Are You Being Served',NULL,now(),NULL,77879),
(5680,15836,'Shaping Up',NULL,now(),NULL,77880),
(5681,1860,'J.J. Starbuck',NULL,now(),NULL,77881),
(5682,1861,'The Restless Years',NULL,now(),NULL,77882),
(5683,9624,'The Marty Feldman Comedy Machine',NULL,now(),NULL,77883),
(5684,18605,'Marie',NULL,now(),NULL,77884),
(5685,3801,'Haunted',NULL,now(),NULL,77885),
(5686,3023,'Central Park West',NULL,now(),NULL,77886),
(5687,3403,'Dundee and the Culhane',NULL,now(),NULL,77887),
(5688,4052,'Jesse',NULL,now(),NULL,77888),
(5689,1862,'Viva Variety',NULL,now(),NULL,77889),
(5690,5183,'Shirley',NULL,now(),NULL,77890),
(5691,3286,'Dennis the Menace',NULL,now(),NULL,77891),
(5692,22419,'Easy Does It Starring Frankie Avalon',NULL,now(),NULL,77892),
(5693,1863,'This is David Harper',NULL,now(),NULL,77893),
(5694,19902,'Sheena, Queen of the Jungle',NULL,now(),NULL,77895),
(5695,10572,'Real People',NULL,now(),NULL,77896),
(5696,4330,'MADtv',NULL,now(),NULL,77897),
(5697,1864,'The Ghost Busters',NULL,now(),NULL,77898),
(5698,3732,'Grapevine (1992)',NULL,now(),NULL,77899),
(5699,13256,'Street Gear',NULL,now(),NULL,77901),
(5700,13164,'Wessex Tales',NULL,now(),NULL,77902),
(5701,3856,'High Society',NULL,now(),NULL,77903),
(5702,2643,'Archie Bunker\'s Place',NULL,now(),NULL,77906),
(5703,6701,'Zoids',NULL,now(),NULL,77907),
(5704,3930,'Hunter',NULL,now(),NULL,77909),
(5705,5065,'Royal Canadian Air Farce',NULL,now(),NULL,77910),
(5706,1865,'Beauty and the Beast (1996)',NULL,now(),NULL,77911),
(5707,5267,'So Weird',NULL,now(),NULL,77913),
(5708,3136,'Convoy',NULL,now(),NULL,77914),
(5709,19548,'The New Show',NULL,now(),NULL,77915),
(5710,1866,'Rideau Hall',NULL,now(),NULL,77917),
(5711,6281,'The Watcher',NULL,now(),NULL,77919),
(5712,1868,'Little Shop',NULL,now(),NULL,77920),
(5713,8337,'Dan Danger',NULL,now(),NULL,77922),
(5714,5742,'The Early Show',NULL,now(),NULL,77924),
(5715,1871,'The Bobby Vinton Show',NULL,now(),NULL,77925),
(5716,56,'Undressed',NULL,now(),NULL,77928),
(5717,8393,'TV to Go',NULL,now(),NULL,77929),
(5718,1872,'Occasional Wife',NULL,now(),NULL,77930),
(5719,6067,'The Other Half',NULL,now(),NULL,77931),
(5720,4186,'Lassie (1997)',NULL,now(),NULL,77932),
(5721,3664,'Get Real (1998)',NULL,now(),NULL,77933),
(5722,1874,'2DTV',NULL,now(),NULL,77935),
(5723,4894,'Prince Street',NULL,now(),NULL,77936),
(5724,4455,'Mersey Beat',NULL,now(),NULL,77937),
(5725,16251,'Breakfast',NULL,now(),NULL,77938),
(5726,2684,'A Very Peculiar Practice',NULL,now(),NULL,77939),
(5727,7052,'GMTV',NULL,now(),NULL,77940),
(5728,57,'Ready or Not',NULL,now(),NULL,77941),
(5729,1876,'Dear Phoebe',NULL,now(),NULL,77942),
(5730,2486,'Action',NULL,now(),NULL,77943),
(5731,4212,'Leap of Faith',NULL,now(),NULL,77944),
(5732,1877,'Last of the Summer Wine',NULL,now(),NULL,77945),
(5733,7272,'6 Dates with Barker',NULL,now(),NULL,77946),
(5734,6040,'The New People',NULL,now(),NULL,77947),
(5735,4993,'Reunited',NULL,now(),NULL,77948),
(5736,3673,'Ghost Hunters',NULL,now(),NULL,77949),
(5737,16723,'George Burns Comedy Week',NULL,now(),NULL,77950),
(5738,16814,'Rich Man, Poor Man Book II',NULL,now(),NULL,77951),
(5739,4857,'Poltergeist: The Legacy',NULL,now(),NULL,77952),
(5740,2956,'Caillou',NULL,now(),NULL,77953),
(5741,8512,'Partners (1999)',NULL,now(),NULL,77956),
(5742,18295,'Spies',NULL,now(),NULL,77957),
(5743,8349,'Say What Karaoke',NULL,now(),NULL,77958),
(5744,5982,'The Mighty Jungle',NULL,now(),NULL,77959),
(5745,2644,'Are You Afraid of the Dark?',NULL,now(),NULL,77960),
(5746,5119,'SCTV Channel',NULL,now(),NULL,77961),
(5747,6117,'The Random Years',NULL,now(),NULL,77962),
(5748,7009,'Finders Keepers',NULL,now(),NULL,77965),
(5749,1879,'It\'s Alive!',NULL,now(),NULL,77966),
(5750,5306,'SportsCenter',NULL,now(),NULL,77967),
(5751,1880,'Border Cafe',NULL,now(),NULL,77968),
(5752,4739,'Our Hero',NULL,now(),NULL,77969),
(5753,58,'Katts and Dog',NULL,now(),NULL,77970),
(5754,4587,'My Little Pony and Friends',NULL,now(),NULL,77971),
(5755,5903,'The Karate Kid',NULL,now(),NULL,77972),
(5756,6377,'Tom and Jerry Kids Show',NULL,now(),NULL,77973),
(5757,1881,'The Further Adventures of SuperTed',NULL,now(),NULL,77975),
(5758,1882,'Droopy: Master Detective',NULL,now(),NULL,77976),
(5759,59,'Zoe, Duncan, Jack & Jane',NULL,now(),NULL,77977),
(5760,4556,'Murder in Mind',NULL,now(),NULL,77978),
(5761,3501,'Family Affairs',NULL,now(),NULL,77979),
(5762,10404,'Misguided Angels',NULL,now(),NULL,77980),
(5763,5379,'Strong Medicine',NULL,now(),NULL,77981),
(5764,16756,'Island Son',NULL,now(),NULL,77982),
(5765,1884,'Timeslip',NULL,now(),NULL,77983),
(5766,10758,'For Richer for Poorer',NULL,now(),NULL,77984),
(5767,1885,'Soul Train',NULL,now(),NULL,77985),
(5768,1886,'Inquiring Minds',NULL,now(),NULL,77987),
(5769,3691,'Glam Metal Detectives',NULL,now(),NULL,77988),
(5770,2796,'Big Kids',NULL,now(),NULL,77989),
(5771,2783,'Beyond Westworld',NULL,now(),NULL,77993),
(5772,60,'Throb',NULL,now(),NULL,77994),
(5773,61,'CatDog',NULL,now(),NULL,77995),
(5774,19274,'Sweet Surrender',NULL,now(),NULL,77997),
(5775,6532,'Waiting for God',NULL,now(),NULL,77998),
(5776,3054,'Chico and the Man',NULL,now(),NULL,78001),
(5777,10950,'The Robert Q. Lewis Show',NULL,now(),NULL,78003),
(5778,10677,'Name That Tune',NULL,now(),NULL,78004),
(5779,3092,'Clue Club',NULL,now(),NULL,78005),
(5780,1888,'Hollyoaks',NULL,now(),NULL,78006),
(5781,17030,'The MacKenzies of Paradise Cove',NULL,now(),NULL,78007),
(5782,1889,'Gun Shy',NULL,now(),NULL,78008),
(5783,18878,'Missing Links',NULL,now(),NULL,78010),
(5784,18904,'Snap Judgment',NULL,now(),NULL,78011),
(5785,18541,'Whodunnit?',NULL,now(),NULL,78012),
(5786,2828,'Blind Date',NULL,now(),NULL,78013),
(5787,15842,'The Jimmy Durante Show',NULL,now(),NULL,78014),
(5788,4219,'Lenny Goes to Town',NULL,now(),NULL,78015),
(5789,5230,'Smart Guy',NULL,now(),NULL,78016),
(5790,4200,'Laverne & Shirley',NULL,now(),NULL,78017),
(5791,1890,'Turnabout',NULL,now(),NULL,78018),
(5792,15733,'The Jon Stewart Show',NULL,now(),NULL,78019),
(5793,2869,'Bone Chillers',NULL,now(),NULL,78021),
(5794,4597,'E! Mysteries & Scandals',NULL,now(),NULL,78024),
(5795,3470,'Exosquad',NULL,now(),NULL,78026),
(5796,1891,'Reach for the Stars',NULL,now(),NULL,78027),
(5797,9686,'Games People Play',NULL,now(),NULL,78028),
(5798,3554,'Five Mile Creek',NULL,now(),NULL,78029),
(5799,11295,'Vietnam',NULL,now(),NULL,78030),
(5800,13637,'Eureka Street',NULL,now(),NULL,78031),
(5801,4984,'Rescue Me (2002)',NULL,now(),NULL,78032),
(5802,5480,'Teech',NULL,now(),NULL,78033),
(5803,1892,'Stunt Dawgs',NULL,now(),NULL,78034),
(5804,23874,'American Journal',NULL,now(),NULL,78035),
(5805,17960,'Puttin on the Hits',NULL,now(),NULL,78037),
(5806,2876,'Boone',NULL,now(),NULL,78038),
(5807,4334,'Maggie Winters',NULL,now(),NULL,78039),
(5808,4145,'Knight & Daye',NULL,now(),NULL,78040),
(5809,8356,'Dads',NULL,now(),NULL,78041),
(5810,13426,'In a Land of Plenty',NULL,now(),NULL,78042),
(5811,5451,'Tales from the Neverending Story',NULL,now(),NULL,78043),
(5812,15565,'Rock & Roll Jeopardy!',NULL,now(),NULL,78047),
(5813,2884,'Bottom',NULL,now(),NULL,78050),
(5814,4641,'Never Mind the Buzzcocks',NULL,now(),NULL,78051),
(5815,5439,'System Crash',NULL,now(),NULL,78052),
(5816,18457,'Fathers and Sons',NULL,now(),NULL,78053),
(5817,4370,'Mapp and Lucia',NULL,now(),NULL,78054),
(5818,1894,'Extreme Gong',NULL,now(),NULL,78055),
(5819,5784,'The Flintstone Kids',NULL,now(),NULL,78056),
(5820,4188,'Last Call with Carson Daly',NULL,now(),NULL,78057),
(5821,62,'Head Start',NULL,now(),NULL,78058),
(5822,4648,'Newlyweds: Nick and Jessica',NULL,now(),NULL,78059),
(5823,1895,'Viva Valdez',NULL,now(),NULL,78061),
(5824,63,'Grace & Favour',NULL,now(),NULL,78062),
(5825,4676,'Ninja Turtles: The Next Mutation',NULL,now(),NULL,78064),
(5826,5880,'The It Factor',NULL,now(),NULL,78065),
(5827,4248,'Lime Street',NULL,now(),NULL,78069),
(5828,4287,'Lottery!',NULL,now(),NULL,78070),
(5829,5757,'The Exile',NULL,now(),NULL,78071),
(5830,3957,'In Deep',NULL,now(),NULL,78073),
(5831,1896,'Beast Machines',NULL,now(),NULL,78075),
(5832,8815,'Nice Work',NULL,now(),NULL,78082),
(5833,1898,'Call Me Mister',NULL,now(),NULL,78083),
(5834,3071,'City Guys',NULL,now(),NULL,78084),
(5835,10052,'The Chair',NULL,now(),NULL,78085),
(5836,3956,'Incredible Story Studio',NULL,now(),NULL,78086),
(5837,6632,'Wolf',NULL,now(),NULL,78087),
(5838,1899,'Devlin',NULL,now(),NULL,78088),
(5839,5630,'The Bob Newhart Show',NULL,now(),NULL,78089),
(5840,5570,'The American Girls',NULL,now(),NULL,78091),
(5841,4969,'Relativity',NULL,now(),NULL,78092),
(5842,3612,'Freshman Dorm',NULL,now(),NULL,78093),
(5843,5884,'The Jamie Kennedy Experiment',NULL,now(),NULL,78096),
(5844,16850,'Those Whiting Girls',NULL,now(),NULL,78097),
(5845,3787,'Harbour Lights',NULL,now(),NULL,78098),
(5846,3427,'Eldorado',NULL,now(),NULL,78099),
(5847,9184,'California Fever',NULL,now(),NULL,78100),
(5848,3736,'Green Acres',NULL,now(),NULL,78101),
(5849,16716,'Secrets of Midland Heights',NULL,now(),NULL,78102),
(5850,65,'Bagdad Cafe',NULL,now(),NULL,78105),
(5851,2671,'A Twist in the Tale',NULL,now(),NULL,78106),
(5852,1900,'Stories of the Century',NULL,now(),NULL,78108),
(5853,1902,'Beat the Geeks',NULL,now(),NULL,78110),
(5854,4026,'Jackass',NULL,now(),NULL,78111),
(5855,4595,'My Sister Sam',NULL,now(),NULL,78112),
(5856,16658,'The Tammy Grimes Show',NULL,now(),NULL,78113),
(5857,3444,'Encounters with the Unexplained',NULL,now(),NULL,78115),
(5858,4078,'Josie and the Pussycats in Outer Space',NULL,now(),NULL,78116),
(5859,2441,'2030 CE',NULL,now(),NULL,78117),
(5860,14726,'Super Jeopardy!',NULL,now(),NULL,78118),
(5861,18916,'Battlestars',NULL,now(),NULL,78120),
(5862,6919,'Double Dare',NULL,now(),NULL,78121),
(5863,3886,'Homefront',NULL,now(),NULL,78123),
(5864,1903,'This Hour Has 22 Minutes',NULL,now(),NULL,78125),
(5865,1904,'Front Page Challenge',NULL,now(),NULL,78126),
(5866,1905,'Maggie Briggs',NULL,now(),NULL,78127),
(5867,1906,'Pay & Display',NULL,now(),NULL,78128),
(5868,12207,'The Hollywood Squares',NULL,now(),NULL,78129),
(5869,3999,'Is It Legal?',NULL,now(),NULL,78130),
(5870,5450,'Tales from the Darkside',NULL,now(),NULL,78132),
(5871,5969,'The Many Loves of Dobie Gillis',NULL,now(),NULL,78133),
(5872,5911,'The Last Frontier',NULL,now(),NULL,78134),
(5873,5744,'The Edison Twins',NULL,now(),NULL,78136),
(5874,1907,'The New Archies',NULL,now(),NULL,78137),
(5875,1908,'The Raccoons',NULL,now(),NULL,78138),
(5876,3817,'Head Over Heels',NULL,now(),NULL,78139),
(5877,16672,'Getting Together',NULL,now(),NULL,78140),
(5878,1909,'Playing the Field',NULL,now(),NULL,78141),
(5879,1910,'Nightmare Ned',NULL,now(),NULL,78143),
(5880,18259,'Tequila and Bonetti',NULL,now(),NULL,78144),
(5881,67,'Black Tie Affair',NULL,now(),NULL,78145),
(5882,4902,'Project Greenlight',NULL,now(),NULL,78146),
(5883,6375,'Tom',NULL,now(),NULL,78147),
(5884,5864,'The Human Factor',NULL,now(),NULL,78148),
(5885,1911,'Riverdale',NULL,now(),NULL,78149),
(5886,3594,'Franklin',NULL,now(),NULL,78150),
(5887,1912,'Once a Hero',NULL,now(),NULL,78151),
(5888,6559,'Welcome Freshmen',NULL,now(),NULL,78152),
(5889,16134,'Hell Town',NULL,now(),NULL,78153),
(5890,6440,'Turks',NULL,now(),NULL,78154),
(5891,2737,'Bearcats',NULL,now(),NULL,78155),
(5892,1913,'Shooting Stars',NULL,now(),NULL,78157),
(5893,4537,'Mr. Bean: The Animated Series',NULL,now(),NULL,78158),
(5894,4254,'Little Bill',NULL,now(),NULL,78159),
(5895,5987,'The Mole',NULL,now(),NULL,78160),
(5896,3178,'Crossroads (1955)',NULL,now(),NULL,78161),
(5897,3749,'Grown Ups',NULL,now(),NULL,78162),
(5898,13250,'Close Relations',NULL,now(),NULL,78163),
(5899,6346,'Three Sisters',NULL,now(),NULL,78164),
(5900,5794,'The Frank Skinner Show',NULL,now(),NULL,78165),
(5901,4565,'Murphy Brown',NULL,now(),NULL,78166),
(5902,13108,'Brothers and Sisters',NULL,now(),NULL,78167),
(5903,3970,'Insomniac with Dave Attell',NULL,now(),NULL,78168),
(5904,8,'Marcus Welby, M.D.',NULL,now(),NULL,78170),
(5905,6662,'WWF Superstars',NULL,now(),NULL,78171),
(5906,6658,'WWF Jakked',NULL,now(),NULL,78172),
(5907,1914,'WWF Livewire',NULL,now(),NULL,78173),
(5908,1916,'Our Friends in the North',NULL,now(),NULL,78174),
(5909,6391,'To the Manor Born',NULL,now(),NULL,78176),
(5910,1917,'Sweet Sixteen',NULL,now(),NULL,78177),
(5911,8368,'Executive Stress',NULL,now(),NULL,78178),
(5912,8793,'Moving',NULL,now(),NULL,78179),
(5913,4657,'Next of Kin',NULL,now(),NULL,78180),
(5914,3773,'Hadleigh',NULL,now(),NULL,78181),
(5915,8305,'Kate',NULL,now(),NULL,78182),
(5916,8351,'Adventures with Kanga Roddy',NULL,now(),NULL,78183),
(5917,2442,'20/20 Downtown',NULL,now(),NULL,78184),
(5918,16768,'Me and Maxx',NULL,now(),NULL,78185),
(5919,1918,'Crossroads (2001)',NULL,now(),NULL,78186),
(5920,1919,'Crossroads (1964)',NULL,now(),NULL,78187),
(5921,10145,'Sanchez of Bel Air',NULL,now(),NULL,78188),
(5922,3433,'Ellery Queen',NULL,now(),NULL,78192),
(5923,5330,'Starsky & Hutch',NULL,now(),NULL,78194),
(5924,1923,'S.W.A.L.K.',NULL,now(),NULL,78195),
(5925,3582,'Forever Knight',NULL,now(),NULL,78196),
(5926,4163,'Kung Fu: The Legend Continues',NULL,now(),NULL,78197),
(5927,15654,'Star Stage',NULL,now(),NULL,78198),
(5928,1924,'Human Remains',NULL,now(),NULL,78199),
(5929,15881,'Highcliffe Manor',NULL,now(),NULL,78201),
(5930,16683,'Doctor\'s Hospital',NULL,now(),NULL,78202),
(5931,1925,'Frontier Justice',NULL,now(),NULL,78204),
(5932,1926,'Western Theater',NULL,now(),NULL,78205),
(5933,1927,'It\'s A Miracle',NULL,now(),NULL,78206),
(5934,5579,'The Antagonists',NULL,now(),NULL,78207),
(5935,1928,'Jonovision',NULL,now(),NULL,78208),
(5936,13196,'Star Maidens',NULL,now(),NULL,78209),
(5937,5676,'The Champions',NULL,now(),NULL,78210),
(5938,68,'My World and Welcome to It',NULL,now(),NULL,78211),
(5939,15334,'Todays FBI',NULL,now(),NULL,78213),
(5940,17067,'The Mississippi',NULL,now(),NULL,78214),
(5941,3796,'Harry O',NULL,now(),NULL,78215),
(5942,4379,'Marshall Law',NULL,now(),NULL,78216),
(5943,2969,'Cannon',NULL,now(),NULL,78217),
(5944,4603,'Mystic Knights of Tir Na Nog',NULL,now(),NULL,78218),
(5945,16717,'The Two of Us',NULL,now(),NULL,78219),
(5946,1929,'Roomies',NULL,now(),NULL,78220),
(5947,6090,'The Piglet Files',NULL,now(),NULL,78221),
(5948,4967,'Reilly: Ace of Spies',NULL,now(),NULL,78222),
(5949,1930,'Monstervision',NULL,now(),NULL,78223),
(5950,5678,'The Charlie Brown and Snoopy Show',NULL,now(),NULL,78225),
(5951,5677,'The Changes',NULL,now(),NULL,78226),
(5952,4324,'Madigan',NULL,now(),NULL,78227),
(5953,4125,'Kids Say the Darndest Things',NULL,now(),NULL,78228),
(5954,16759,'Lenny',NULL,now(),NULL,78229),
(5955,1931,'The Sunday Comics',NULL,now(),NULL,78230),
(5956,5341,'Stat',NULL,now(),NULL,78231),
(5957,1933,'Project A-ko',NULL,now(),NULL,78232),
(5958,1934,'Celebrity Adventures',NULL,now(),NULL,78233),
(5959,1935,'The Mrs Bradley Mysteries',NULL,now(),NULL,78235),
(5960,5477,'Teamo Supremo',NULL,now(),NULL,78237),
(5961,4577,'My Friend Flicka',NULL,now(),NULL,78238),
(5962,3127,'Common As Muck',NULL,now(),NULL,78240),
(5963,7238,'48 Hours',NULL,now(),NULL,78241),
(5964,12627,'The Dinah Shore Chevy Show',NULL,now(),NULL,78242),
(5965,16816,'E! Celebrity Profile',NULL,now(),NULL,78243),
(5966,3360,'Dr. Katz, Professional Therapist',NULL,now(),NULL,78244),
(5967,1937,'Tinker, Tailor, Soldier, Spy',NULL,now(),NULL,78245),
(5968,1938,'Smiley\'s People',NULL,now(),NULL,78246),
(5969,3302,'Diamonds',NULL,now(),NULL,78247),
(5970,4527,'More Tears',NULL,now(),NULL,78248),
(5971,3571,'Foolish Heart',NULL,now(),NULL,78249),
(5972,6020,'The New Adventures of Mighty Mouse',NULL,now(),NULL,78251),
(5973,1939,'Sugar and Spice',NULL,now(),NULL,78252),
(5974,5245,'Someone Like Me',NULL,now(),NULL,78253),
(5975,6088,'The Phoenix',NULL,now(),NULL,78254),
(5976,4982,'Rescue 911',NULL,now(),NULL,78256);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(5977,5504,'That\'s So Raven',NULL,now(),NULL,78258),
(5978,5760,'The Faculty',NULL,now(),NULL,78263),
(5979,9948,'Vic Reeves Big Night Out',NULL,now(),NULL,78264),
(5980,4387,'Masked Rider',NULL,now(),NULL,78265),
(5981,5603,'The Beast',NULL,now(),NULL,78266),
(5982,4668,'NightMan',NULL,now(),NULL,78267),
(5983,10182,'From These Roots',NULL,now(),NULL,78268),
(5984,5687,'The Colin Quinn Show',NULL,now(),NULL,78269),
(5985,3057,'CHiPs',NULL,now(),NULL,78270),
(5986,3030,'Chandler & Co.',NULL,now(),NULL,78271),
(5987,8217,'SM:TV Live',NULL,now(),NULL,78273),
(5988,4864,'Pop Idol',NULL,now(),NULL,78274),
(5989,3480,'EZ Streets',NULL,now(),NULL,78275),
(5990,5925,'The Legend of Zelda',NULL,now(),NULL,78276),
(5991,6155,'The Roy Rogers-Dale Evans Show',NULL,now(),NULL,78277),
(5992,1944,'Cobra',NULL,now(),NULL,78278),
(5993,3003,'Cash and Company',NULL,now(),NULL,78279),
(5994,1945,'The Cramp Twins',NULL,now(),NULL,78280),
(5995,5807,'The Gene Autry Show',NULL,now(),NULL,78281),
(5996,5035,'Rocket Power',NULL,now(),NULL,78282),
(5997,7321,'The Young Marrieds',NULL,now(),NULL,78283),
(5998,3659,'George and Martha',NULL,now(),NULL,78284),
(5999,6335,'This Is Your Life (US)',NULL,now(),NULL,78285),
(6000,1947,'Mind Your Language',NULL,now(),NULL,78286),
(6001,1948,'Not on Your Nellie',NULL,now(),NULL,78288),
(6002,2554,'All Creatures Great and Small',NULL,now(),NULL,78289),
(6003,6498,'Van der Valk',NULL,now(),NULL,78290),
(6004,6018,'The New Adventures of He-Man',NULL,now(),NULL,78291),
(6005,8746,'The Bell Telephone Hour',NULL,now(),NULL,78292),
(6006,5211,'Sisters',NULL,now(),NULL,78293),
(6007,3764,'Gunslinger',NULL,now(),NULL,78294),
(6008,6103,'The Powers of Matthew Star',NULL,now(),NULL,78295),
(6009,4413,'McClain\'s Law',NULL,now(),NULL,78296),
(6010,5905,'The Kids of Degrassi Street',NULL,now(),NULL,78297),
(6011,3277,'Degrassi Junior High',NULL,now(),NULL,78298),
(6012,70,'Emerald Point N.A.S.',NULL,now(),NULL,78299),
(6013,3276,'Degrassi High',NULL,now(),NULL,78300),
(6014,3278,'Degrassi Talks',NULL,now(),NULL,78301),
(6015,4288,'Lou Grant',NULL,now(),NULL,78302),
(6016,1950,'Battle Dome',NULL,now(),NULL,78303),
(6017,5825,'The Governor',NULL,now(),NULL,78304),
(6018,7288,'Zola Levitt',NULL,now(),NULL,78305),
(6019,4045,'Jem',NULL,now(),NULL,78306),
(6020,1955,'Different and the Same',NULL,now(),NULL,78307),
(6021,1956,'Square One TV',NULL,now(),NULL,78309),
(6022,3184,'CSI: Miami',NULL,now(),NULL,78310),
(6023,1957,'Meeting of Minds',NULL,now(),NULL,78311),
(6024,4175,'Lamb Chop\'s Play-Along',NULL,now(),NULL,78312),
(6025,9207,'Comedy Nation',NULL,now(),NULL,78313),
(6026,2973,'Captain Kangaroo',NULL,now(),NULL,78314),
(6027,1958,'The Puzzle Place',NULL,now(),NULL,78315),
(6028,3135,'The Conspiracy Zone',NULL,now(),NULL,78316),
(6029,2714,'Barnaby Jones',NULL,now(),NULL,78317),
(6030,2635,'A Prince Among Men',NULL,now(),NULL,78318),
(6031,7968,'R3',NULL,now(),NULL,78320),
(6032,13182,'The Man in Room 17',NULL,now(),NULL,78322),
(6033,4958,'Red Cap',NULL,now(),NULL,78323),
(6034,14970,'The Pathfinders',NULL,now(),NULL,78324),
(6035,1959,'Softly Softly',NULL,now(),NULL,78325),
(6036,12792,'Hunter\'s Walk',NULL,now(),NULL,78326),
(6037,4373,'Marker',NULL,now(),NULL,78327),
(6038,5112,'Scooby\'s All Star Laff-A-Lympics',NULL,now(),NULL,78328),
(6039,4319,'MacGruder and Loud',NULL,now(),NULL,78329),
(6040,3809,'Hawkeye and the Last of the Mohicans',NULL,now(),NULL,78331),
(6041,1960,'Top of the Pops (UK)',NULL,now(),NULL,78332),
(6042,10309,'New Love American Style',NULL,now(),NULL,78333),
(6043,17365,'Me and Mom',NULL,now(),NULL,78334),
(6044,3330,'Doctor in the House',NULL,now(),NULL,78335),
(6045,8925,'Legacy of Murder',NULL,now(),NULL,78336),
(6046,1961,'Combat Missions',NULL,now(),NULL,78337),
(6047,4352,'Making the Video',NULL,now(),NULL,78338),
(6048,17222,'Doctors Private Lives',NULL,now(),NULL,78339),
(6049,1962,'Truth Quest: California',NULL,now(),NULL,78340),
(6050,10532,'TV\'s Top Tunes',NULL,now(),NULL,78341),
(6051,3355,'Downtown',NULL,now(),NULL,78342),
(6052,18930,'Three on a Match',NULL,now(),NULL,78343),
(6053,71,'James at 15',NULL,now(),NULL,78344),
(6054,3724,'Good vs Evil',NULL,now(),NULL,78345),
(6055,3343,'Donny and Marie',NULL,now(),NULL,78347),
(6056,1963,'Young Talent Time',NULL,now(),NULL,78348),
(6057,18305,'Studio 5-B',NULL,now(),NULL,78352),
(6058,2455,'413 Hope St.',NULL,now(),NULL,78353),
(6059,5768,'The Famous Jett Jackson',NULL,now(),NULL,78354),
(6060,1964,'Chancer',NULL,now(),NULL,78355),
(6061,8581,'Fortunes of War',NULL,now(),NULL,78356),
(6062,1965,'The Return of Shelley',NULL,now(),NULL,78357),
(6063,4919,'Q.E.D.',NULL,now(),NULL,78358),
(6064,3319,'DiResta',NULL,now(),NULL,78359),
(6065,13205,'Animal Ark',NULL,now(),NULL,78360),
(6066,1966,'The Duchess of Duke Street',NULL,now(),NULL,78361),
(6067,1967,'Gloss',NULL,now(),NULL,78362),
(6068,1968,'Solo',NULL,now(),NULL,78363),
(6069,3953,'In a Heartbeat',NULL,now(),NULL,78364),
(6070,8787,'The Mistress',NULL,now(),NULL,78365),
(6071,8375,'Honey for Tea',NULL,now(),NULL,78366),
(6072,1969,'Zoom',NULL,now(),NULL,78367),
(6073,12679,'The Andromeda Breakthrough',NULL,now(),NULL,78368),
(6074,13116,'Katy',NULL,now(),NULL,78369),
(6075,23343,'The Forsyte Saga (1967)',NULL,now(),NULL,78370),
(6076,1971,'The Pallisers',NULL,now(),NULL,78371),
(6077,8483,'Leaving',NULL,now(),NULL,78372),
(6078,2754,'Beggars and Choosers',NULL,now(),NULL,78373),
(6079,8800,'My Good Friend',NULL,now(),NULL,78374),
(6080,3194,'Cyberchase',NULL,now(),NULL,78375),
(6081,4782,'Passion Cove',NULL,now(),NULL,78376),
(6082,4513,'Monica\'s Gang',NULL,now(),NULL,78378),
(6083,6065,'The Osbournes',NULL,now(),NULL,78379),
(6084,3053,'Chicken Soup for the Soul',NULL,now(),NULL,78382),
(6085,1975,'The Irish R. M.',NULL,now(),NULL,78383),
(6086,1976,'Black of Life',NULL,now(),NULL,78384),
(6087,10681,'Night Fever',NULL,now(),NULL,78387),
(6088,2597,'American Dreamer',NULL,now(),NULL,78388),
(6089,2968,'Candid Camera',NULL,now(),NULL,78389),
(6090,2627,'Another World',NULL,now(),NULL,78390),
(6091,2962,'Campion',NULL,now(),NULL,78391),
(6092,8425,'Three Up, Two Down',NULL,now(),NULL,78392),
(6093,1980,'Mystery and Imagination',NULL,now(),NULL,78393),
(6094,3676,'Ghost Stories (1997)',NULL,now(),NULL,78394),
(6095,5247,'Something is Out There',NULL,now(),NULL,78395),
(6096,3717,'Good Morning, Miami',NULL,now(),NULL,78396),
(6097,5974,'The Mask',NULL,now(),NULL,78398),
(6098,1982,'Medical Detectives',NULL,now(),NULL,78399),
(6099,1983,'Ghost Story',NULL,now(),NULL,78400),
(6100,2965,'Can\'t Hurry Love',NULL,now(),NULL,78402),
(6101,3458,'Eureeka\'s Castle',NULL,now(),NULL,78403),
(6102,1985,'Becoming',NULL,now(),NULL,78407),
(6103,1986,'Nearest and Dearest',NULL,now(),NULL,78408),
(6104,2696,'Baby Talk',NULL,now(),NULL,78409),
(6105,6089,'The Phoenix Team',NULL,now(),NULL,78410),
(6106,13398,'The Corridor People',NULL,now(),NULL,78411),
(6107,13170,'The Citadel',NULL,now(),NULL,78412),
(6108,6574,'What a Dummy',NULL,now(),NULL,78413),
(6109,15904,'Kentucky Jones',NULL,now(),NULL,78414),
(6110,72,'The United States Steel Hour',NULL,now(),NULL,78415),
(6111,10169,'People Do the Craziest Things',NULL,now(),NULL,78418),
(6112,5152,'Sesame Street',NULL,now(),NULL,78419),
(6113,5604,'The Beat',NULL,now(),NULL,78420),
(6114,3025,'ChalkZone',NULL,now(),NULL,78421),
(6115,4044,'Jefferson Drum',NULL,now(),NULL,78422),
(6116,1989,'The Marshal of Gunsight Pass',NULL,now(),NULL,78423),
(6117,3069,'Citizen Reno',NULL,now(),NULL,78426),
(6118,5104,'Scarecrow and Mrs. King',NULL,now(),NULL,78427),
(6119,3345,'Dooley Gardens',NULL,now(),NULL,78428),
(6120,2494,'Adam Adamant Lives',NULL,now(),NULL,78429),
(6121,6174,'The Secret Diary of Desmond Pfeiffer',NULL,now(),NULL,78430),
(6122,4998,'Rhona',NULL,now(),NULL,78431),
(6123,5851,'The High Life (1996)',NULL,now(),NULL,78432),
(6124,7921,'Cool McCool',NULL,now(),NULL,78433),
(6125,4863,'Popeye',NULL,now(),NULL,78435),
(6126,1991,'Bad Dog',NULL,now(),NULL,78437),
(6127,8651,'A Bit of a Do',NULL,now(),NULL,78438),
(6128,8370,'Ain\'t Misbehavin\'',NULL,now(),NULL,78439),
(6129,13158,'Mr Rose',NULL,now(),NULL,78440),
(6130,3961,'In Living Color',NULL,now(),NULL,78441),
(6131,3233,'Danny Phantom',NULL,now(),NULL,78443),
(6132,4584,'My Life as a Teenage Robot',NULL,now(),NULL,78444),
(6133,4208,'Law of the Plainsman',NULL,now(),NULL,78445),
(6134,13222,'The Devil\'s Crown',NULL,now(),NULL,78447),
(6135,1994,'Chances',NULL,now(),NULL,78448),
(6136,17530,'The Johnny Cash Show',NULL,now(),NULL,78450),
(6137,1995,'Super Chicken',NULL,now(),NULL,78451),
(6138,5567,'The Amazing Spider-Man',NULL,now(),NULL,78453),
(6139,1996,'Make Your Own Kind of Music',NULL,now(),NULL,78455),
(6140,10796,'My Generation',NULL,now(),NULL,78456),
(6141,1997,'Beat-Club',NULL,now(),NULL,78458),
(6142,5192,'Sidekicks',NULL,now(),NULL,78459),
(6143,6333,'This is David Lander',NULL,now(),NULL,78460),
(6144,6038,'The New Odd Couple',NULL,now(),NULL,78463),
(6145,8621,'Too Much Sun',NULL,now(),NULL,78464),
(6146,5960,'The Magician\'s House',NULL,now(),NULL,78466),
(6147,6588,'Whiplash',NULL,now(),NULL,78467),
(6148,1999,'Man Without a Gun',NULL,now(),NULL,78468),
(6149,2000,'Northwest Passage',NULL,now(),NULL,78469),
(6150,6684,'Young Dan\'l Boone',NULL,now(),NULL,78470),
(6151,17081,'Life Stories',NULL,now(),NULL,78472),
(6152,73,'The Beachcombers',NULL,now(),NULL,78473),
(6153,7455,'The New Perry Mason',NULL,now(),NULL,78474),
(6154,16502,'Most Deadly Game',NULL,now(),NULL,78475),
(6155,2001,'The Fear',NULL,now(),NULL,78476),
(6156,5157,'Shades of L.A.',NULL,now(),NULL,78477),
(6157,10986,'The Round Table',NULL,now(),NULL,78478),
(6158,9,'Too Close for Comfort',NULL,now(),NULL,78479),
(6159,23082,'The Magic Hour',NULL,now(),NULL,78480),
(6160,2002,'Joking Apart',NULL,now(),NULL,78481),
(6161,15670,'The Family Holvak',NULL,now(),NULL,78482),
(6162,16673,'The Good Life (1971)',NULL,now(),NULL,78483),
(6163,2003,'Ultimate Albums',NULL,now(),NULL,78484),
(6164,4711,'Oliver Beene',NULL,now(),NULL,78485),
(6165,2006,'Crane',NULL,now(),NULL,78486),
(6166,8755,'Up Rising',NULL,now(),NULL,78488),
(6167,2669,'Attack of The Killer Tomatoes',NULL,now(),NULL,78489),
(6168,9889,'This Morning with Richard Not Judy',NULL,now(),NULL,78491),
(6169,5819,'The Goldbergs',NULL,now(),NULL,78492),
(6170,2791,'Big Brother\'s Little Brother',NULL,now(),NULL,78493),
(6171,2806,'Bill Nye: The Science Guy',NULL,now(),NULL,78494),
(6172,22299,'Songs for Sale',NULL,now(),NULL,78496),
(6173,2009,'Chalk',NULL,now(),NULL,78497),
(6174,3040,'Chase',NULL,now(),NULL,78498),
(6175,2010,'Mr Dressup',NULL,now(),NULL,78499),
(6176,4885,'Pretty Soldier Sailor Moon',NULL,now(),NULL,78500),
(6177,6287,'The Weird Al Show',NULL,now(),NULL,78501),
(6178,2011,'Destiny Ridge',NULL,now(),NULL,78502),
(6179,5812,'The George Carlin Show',NULL,now(),NULL,78503),
(6180,2012,'Roundhouse',NULL,now(),NULL,78504),
(6181,4633,'Ned Blessing: The Story of My Life and Times',NULL,now(),NULL,78505),
(6182,17808,'POWW: Powerful Women of Wrestling',NULL,now(),NULL,78506),
(6183,3697,'GLOW: Gorgeous Ladies of Wrestling',NULL,now(),NULL,78507),
(6184,6250,'The Torkelsons',NULL,now(),NULL,78509),
(6185,2013,'Eden',NULL,now(),NULL,78510),
(6186,3523,'Fat Albert and the Cosby Kids',NULL,now(),NULL,78511),
(6187,2014,'The Ultimate Revenge',NULL,now(),NULL,78512),
(6188,5853,'The Hitchhiker',NULL,now(),NULL,78513),
(6189,4865,'Popples',NULL,now(),NULL,78515),
(6190,5548,'The Adventures of Teddy Ruxpin',NULL,now(),NULL,78517),
(6191,5320,'Starcom: The U.S. Space Force',NULL,now(),NULL,78518),
(6192,2016,'Dinosaucers',NULL,now(),NULL,78519),
(6193,2017,'Camp Candy',NULL,now(),NULL,78520),
(6194,6598,'Whose Line Is It Anyway? (UK)',NULL,now(),NULL,78521),
(6195,6298,'The Wizard of Oz',NULL,now(),NULL,78522),
(6196,2977,'Captain N and the New Super Mario World',NULL,now(),NULL,78523),
(6197,5558,'The All-New Popeye Hour',NULL,now(),NULL,78524),
(6198,2018,'CD:UK',NULL,now(),NULL,78525),
(6199,4173,'La Femme Nikita',NULL,now(),NULL,78527),
(6200,8704,'A Gentleman\'s Club',NULL,now(),NULL,78528),
(6201,13432,'Sergeant Cork',NULL,now(),NULL,78531),
(6202,8689,'Colonel Trumper\'s Private War',NULL,now(),NULL,78532),
(6203,3631,'GBH',NULL,now(),NULL,78534),
(6204,5552,'The Agatha Christie Hour',NULL,now(),NULL,78535),
(6205,6631,'Wodehouse Playhouse',NULL,now(),NULL,78536),
(6206,2020,'Ghost Squad',NULL,now(),NULL,78537),
(6207,8979,'The Aphrodite Inheritance',NULL,now(),NULL,78538),
(6208,6502,'Vengeance Unlimited',NULL,now(),NULL,78539),
(6209,3098,'Codename',NULL,now(),NULL,78540),
(6210,3547,'Fired Up',NULL,now(),NULL,78541),
(6211,7291,'Yanks Go Home',NULL,now(),NULL,78542),
(6212,6522,'Voltron: The Third Dimension',NULL,now(),NULL,78543),
(6213,450,'3 Friends and Jerry',NULL,now(),NULL,78545),
(6214,4881,'Presidio Med',NULL,now(),NULL,78546),
(6215,4221,'Less Than Perfect',NULL,now(),NULL,78547),
(6216,4494,'Mister Sterling',NULL,now(),NULL,78548),
(6217,16985,'Those Amazing Animals',NULL,now(),NULL,78549),
(6218,451,'The Miraculous Mellops',NULL,now(),NULL,78550),
(6219,5707,'The Cuckoo Waltz',NULL,now(),NULL,78552),
(6220,2761,'Benson',NULL,now(),NULL,78553),
(6221,2567,'Ally',NULL,now(),NULL,78554),
(6222,5833,'The Grimleys',NULL,now(),NULL,78555),
(6223,9477,'Fair Exchange',NULL,now(),NULL,78558),
(6224,7447,'William',NULL,now(),NULL,78559),
(6225,3089,'Clone High, USA',NULL,now(),NULL,78560),
(6226,3826,'Heavy Gear',NULL,now(),NULL,78561),
(6227,3872,'Hitz',NULL,now(),NULL,78562),
(6228,454,'Monkey',NULL,now(),NULL,78563),
(6229,455,'The Chamber',NULL,now(),NULL,78564),
(6230,6095,'The Pitts',NULL,now(),NULL,78565),
(6231,4904,'Project UFO',NULL,now(),NULL,78566),
(6232,3770,'H.G. Wells\' Invisible Man',NULL,now(),NULL,78567),
(6233,5728,'The Division',NULL,now(),NULL,78568),
(6234,456,'Tinsel Town',NULL,now(),NULL,78569),
(6235,5867,'The Imaginatively-Titled Punt & Dennis Show',NULL,now(),NULL,78570),
(6236,5875,'The Insiders',NULL,now(),NULL,78571),
(6237,6558,'Welcome Back, Kotter',NULL,now(),NULL,78572),
(6238,458,'Hey Landlord',NULL,now(),NULL,78573),
(6239,4661,'Nick Freno: Licensed Teacher',NULL,now(),NULL,78575),
(6240,5419,'Survivor (UK)',NULL,now(),NULL,78576),
(6241,459,'The Judy Garland Show',NULL,now(),NULL,78577),
(6242,3576,'Ford Star Jubilee',NULL,now(),NULL,78578),
(6243,3517,'Fantasy Island (1998)',NULL,now(),NULL,78580),
(6244,3312,'Dilbert',NULL,now(),NULL,78581),
(6245,16674,'The Brian Keith Show',NULL,now(),NULL,78582),
(6246,3837,'Here and Now',NULL,now(),NULL,78584),
(6247,8984,'Assignment Foreign Legion',NULL,now(),NULL,78585),
(6248,5542,'The Adventures of Sir Lancelot',NULL,now(),NULL,78586),
(6249,460,'The Book Group',NULL,now(),NULL,78588),
(6250,6029,'The New Dick Van Dyke Show',NULL,now(),NULL,78589),
(6251,461,'Home (1983)',NULL,now(),NULL,78590),
(6252,4893,'Princesses',NULL,now(),NULL,78591),
(6253,3406,'Dweebs',NULL,now(),NULL,78593),
(6254,463,'The Outcasts',NULL,now(),NULL,78594),
(6255,3502,'Family Album',NULL,now(),NULL,78595),
(6256,4239,'Life with Bonnie',NULL,now(),NULL,78596),
(6257,7700,'Arcade',NULL,now(),NULL,78597),
(6258,2586,'America\'s Greatest Pets',NULL,now(),NULL,78598),
(6259,5636,'The Bonnie Hunt Show',NULL,now(),NULL,78600),
(6260,5662,'The Building',NULL,now(),NULL,78601),
(6261,464,'A Horseman Riding By',NULL,now(),NULL,78602),
(6262,10896,'It Takes Two',NULL,now(),NULL,78603),
(6263,465,'The Last Train',NULL,now(),NULL,78604),
(6264,467,'Introducing Gary Petty',NULL,now(),NULL,78605),
(6265,468,'The Games (Australia)',NULL,now(),NULL,78606),
(6266,469,'Full Frontal',NULL,now(),NULL,78607),
(6267,5255,'Sons of Thunder',NULL,now(),NULL,78608),
(6268,4649,'Newman and Baddiel in Pieces',NULL,now(),NULL,78609),
(6269,5385,'Style and Substance',NULL,now(),NULL,78610),
(6270,5163,'Shane',NULL,now(),NULL,78612),
(6271,2766,'Best of the West',NULL,now(),NULL,78613),
(6272,470,'Shotgun Slade',NULL,now(),NULL,78617),
(6273,4276,'Lonesome Dove: The Series',NULL,now(),NULL,78618),
(6274,5317,'Stagecoach West',NULL,now(),NULL,78620),
(6275,4250,'Linda Green',NULL,now(),NULL,78621),
(6276,7862,'Albion Market',NULL,now(),NULL,78622),
(6277,5824,'The Good Life (1994)',NULL,now(),NULL,78623),
(6278,2447,'24Seven',NULL,now(),NULL,78624),
(6279,4419,'McMillan and Wife',NULL,now(),NULL,78625),
(6280,74,'The Good Guys',NULL,now(),NULL,78626),
(6281,5632,'The Bold Ones: The Lawyers',NULL,now(),NULL,78627),
(6282,6410,'Transformers Armada',NULL,now(),NULL,78628),
(6283,5635,'The Bold Ones: The Senator',NULL,now(),NULL,78629),
(6284,5634,'The Bold Ones: The Protectors',NULL,now(),NULL,78630),
(6285,5633,'The Bold Ones: The New Doctors',NULL,now(),NULL,78631),
(6286,4421,'MDs',NULL,now(),NULL,78632),
(6287,6561,'Welcome To Paradox',NULL,now(),NULL,78633),
(6288,16663,'The Ugliest Girl in Town',NULL,now(),NULL,78634),
(6289,471,'Second Time Around (1974)',NULL,now(),NULL,78635),
(6290,8712,'The Glums',NULL,now(),NULL,78636),
(6291,5197,'Significant Others (1998)',NULL,now(),NULL,78637),
(6292,9056,'Troubles and Strife',NULL,now(),NULL,78638),
(6293,9029,'Singles',NULL,now(),NULL,78639),
(6294,8433,'Only When I Laugh',NULL,now(),NULL,78640),
(6295,472,'Keep It in the Family',NULL,now(),NULL,78641),
(6296,5108,'Schoolhouse Rock!',NULL,now(),NULL,78642),
(6297,15845,'Sarge',NULL,now(),NULL,78643),
(6298,5283,'Spawn',NULL,now(),NULL,78645),
(6299,3818,'Health and Efficiency',NULL,now(),NULL,78647),
(6300,6526,'VR Troopers',NULL,now(),NULL,78648),
(6301,5599,'Barbary Coast',NULL,now(),NULL,78649),
(6302,8169,'Harbourmaster',NULL,now(),NULL,78651),
(6303,473,'Pop Years',NULL,now(),NULL,78654),
(6304,2484,'Ace of Wands',NULL,now(),NULL,78655),
(6305,5528,'The Adventures of Hiram Holliday',NULL,now(),NULL,78656),
(6306,2503,'The Adventures of Don Quick',NULL,now(),NULL,78657),
(6307,474,'The Adventures of Skippy',NULL,now(),NULL,78658),
(6308,5551,'The Adventures of William Tell',NULL,now(),NULL,78659),
(6309,2514,'Against the Law',NULL,now(),NULL,78661),
(6310,2518,'Agony Again',NULL,now(),NULL,78662),
(6311,2531,'Alexei Sayle\'s Stuff',NULL,now(),NULL,78663),
(6312,2553,'All at No. 20',NULL,now(),NULL,78664),
(6313,2555,'All Dogs Go to Heaven',NULL,now(),NULL,78665),
(6314,2558,'All in Good Faith',NULL,now(),NULL,78666),
(6315,2629,'Anthony Ant',NULL,now(),NULL,78667),
(6316,475,'Arabian Knights',NULL,now(),NULL,78668),
(6317,2642,'Archie\'s Weird Mysteries',NULL,now(),NULL,78669),
(6318,6229,'The Survivors',NULL,now(),NULL,78670),
(6319,5598,'The Baker Street Boys',NULL,now(),NULL,78671),
(6320,2706,'Bananaman',NULL,now(),NULL,78672),
(6321,2742,'Beat Route',NULL,now(),NULL,78673),
(6322,2760,'Benji, Zax and the Alien Prince',NULL,now(),NULL,78674),
(6323,2765,'Bert D\'Angelo Superstar',NULL,now(),NULL,78675),
(6324,2799,'Big Meg, Little Meg',NULL,now(),NULL,78676),
(6325,3995,'Ironside',NULL,now(),NULL,78677),
(6326,2794,'Bigfoot and Wildboy',NULL,now(),NULL,78678),
(6327,2805,'Bill & Ted\'s Excellent Adventures',NULL,now(),NULL,78679),
(6328,2809,'Birdland',NULL,now(),NULL,78680),
(6329,2813,'Blackfly',NULL,now(),NULL,78681),
(6330,2820,'Blackstar',NULL,now(),NULL,78682),
(6331,2843,'Blue Light',NULL,now(),NULL,78683),
(6332,2848,'Blue Skies (1988)',NULL,now(),NULL,78684),
(6333,2861,'Bob the Builder',NULL,now(),NULL,78685),
(6334,2491,'Adam-12',NULL,now(),NULL,78686),
(6335,2898,'Brats of the Lost Nebula',NULL,now(),NULL,78687),
(6336,2911,'Bringing up Jack',NULL,now(),NULL,78689),
(6337,5651,'The Broad Side',NULL,now(),NULL,78690),
(6338,2938,'Bureau of Alien Detectors',NULL,now(),NULL,78692),
(6339,2942,'Butch Patterson: Private Dick',NULL,now(),NULL,78693),
(6340,4427,'Medical Center',NULL,now(),NULL,78694),
(6341,3079,'Class Act',NULL,now(),NULL,78695),
(6342,476,'Mother Love',NULL,now(),NULL,78696),
(6343,2596,'American Dream',NULL,now(),NULL,78697),
(6344,2954,'Cadillacs and Dinosaurs',NULL,now(),NULL,78699),
(6345,477,'Cafe Americain',NULL,now(),NULL,78700),
(6346,2972,'Captain Caveman & the Teen Angels',NULL,now(),NULL,78701),
(6347,15691,'This Is The Life',NULL,now(),NULL,78702),
(6348,2974,'Captain Midnight',NULL,now(),NULL,78703),
(6349,2983,'Captain Star',NULL,now(),NULL,78704),
(6350,2986,'Captain Zep - Space Detective',NULL,now(),NULL,78705),
(6351,3021,'Centennial',NULL,now(),NULL,78706),
(6352,3045,'Chef!',NULL,now(),NULL,78707),
(6353,3070,'Citizen Smith',NULL,now(),NULL,78708),
(6354,75,'Tales of the Unexpected',NULL,now(),NULL,78710),
(6355,3111,'Colonel March of Scotland Yard',NULL,now(),NULL,78711),
(6356,3116,'Come Back Mrs. Noah',NULL,now(),NULL,78712),
(6357,3118,'Comedy Lab',NULL,now(),NULL,78713),
(6358,3122,'Coming Soon',NULL,now(),NULL,78714),
(6359,3124,'Commando Cody',NULL,now(),NULL,78715),
(6360,3126,'Committed (2001)',NULL,now(),NULL,78716),
(6361,5699,'The Count of Monte Cristo',NULL,now(),NULL,78717),
(6362,3148,'Counterstrike (UK)',NULL,now(),NULL,78718),
(6363,5700,'The Country Mouse & the City Mouse Adventures',NULL,now(),NULL,78719),
(6364,4395,'Matt Houston',NULL,now(),NULL,78721),
(6365,3186,'Cubix',NULL,now(),NULL,78722),
(6366,4997,'Rhoda',NULL,now(),NULL,78723),
(6367,6032,'The New Gidget',NULL,now(),NULL,78724),
(6368,480,'The New Phil Silvers Show',NULL,now(),NULL,78725),
(6369,6278,'The Wanderer',NULL,now(),NULL,78726),
(6370,6527,'W.E.B.',NULL,now(),NULL,78727),
(6371,6556,'Weird-Ohs',NULL,now(),NULL,78728),
(6372,6564,'Wham! Bam! Strawberry Jam!',NULL,now(),NULL,78729),
(6373,5608,'The Ben Stiller Show',NULL,now(),NULL,78730),
(6374,2687,'A Whole New Ballgame',NULL,now(),NULL,78731),
(6375,6606,'WildC.A.T.S.',NULL,now(),NULL,78732),
(6376,6664,'Wycliffe',NULL,now(),NULL,78733),
(6377,6675,'Yearbook',NULL,now(),NULL,78734),
(6378,6323,'The Young Rebels',NULL,now(),NULL,78735),
(6379,6693,'Yvon of the Yukon',NULL,now(),NULL,78736),
(6380,22595,'Zorro (1981)',NULL,now(),NULL,78737),
(6381,6708,'Zorro (1997)',NULL,now(),NULL,78738),
(6382,6709,'Zorro and Son',NULL,now(),NULL,78739),
(6383,5113,'Scooby and Scrappy-Doo',NULL,now(),NULL,78740),
(6384,4965,'Regular Joe',NULL,now(),NULL,78741),
(6385,481,'Take the High Road',NULL,now(),NULL,78742),
(6386,6350,'Thundarr the Barbarian',NULL,now(),NULL,78743),
(6387,482,'Spitting Image',NULL,now(),NULL,78744),
(6388,76,'The Kids from Room 402',NULL,now(),NULL,78745),
(6389,483,'The David Essex Showcase',NULL,now(),NULL,78746),
(6390,18901,'Body Language',NULL,now(),NULL,78747),
(6391,484,'Boys from the Blackstuff',NULL,now(),NULL,78748),
(6392,3158,'Cracker',NULL,now(),NULL,78749),
(6393,3077,'Clarence',NULL,now(),NULL,78750),
(6394,8732,'His Lordship Entertains',NULL,now(),NULL,78751),
(6395,18850,'Hot Potato',NULL,now(),NULL,78752),
(6396,8720,'Hark at Barker',NULL,now(),NULL,78753),
(6397,485,'Not Only... But Also',NULL,now(),NULL,78754),
(6398,8772,'The Magnificent Evans',NULL,now(),NULL,78755),
(6399,8967,'The Ronnie Barker Playhouse',NULL,now(),NULL,78756),
(6400,8897,'The Frost Report',NULL,now(),NULL,78757),
(6401,9319,'Foreign Affairs',NULL,now(),NULL,78758),
(6402,6036,'The New Mickey Mouse Club',NULL,now(),NULL,78760),
(6403,2626,'Another Life',NULL,now(),NULL,78761),
(6404,487,'Stella Street',NULL,now(),NULL,78762),
(6405,488,'The Home and Family Show',NULL,now(),NULL,78763),
(6406,4994,'Revealed with Jules Asner',NULL,now(),NULL,78764),
(6407,4340,'Maid Marian and Her Merry Men',NULL,now(),NULL,78765),
(6408,7310,'You Only Live Once',NULL,now(),NULL,78766),
(6409,4423,'Me, You and Him',NULL,now(),NULL,78767),
(6410,6790,'They Think It\'s All Over',NULL,now(),NULL,78768),
(6411,9717,'Paul Merton - The Series',NULL,now(),NULL,78770),
(6412,8607,'Terry and Julian',NULL,now(),NULL,78771),
(6413,10,'Bob & Carol & Ted & Alice',NULL,now(),NULL,78772),
(6414,4232,'Life as We Know It (2001)',NULL,now(),NULL,78775),
(6415,491,'Steve Donovan, Western Marshall',NULL,now(),NULL,78776),
(6416,492,'Record of Lodoss War',NULL,now(),NULL,78777),
(6417,493,'Cos',NULL,now(),NULL,78779),
(6418,9819,'The Jacksons',NULL,now(),NULL,78780),
(6419,5963,'The Man and the Challenge',NULL,now(),NULL,78781),
(6420,495,'Tales of the Texas Rangers',NULL,now(),NULL,78784),
(6421,2957,'Caitlin\'s Way',NULL,now(),NULL,78785),
(6422,5682,'The Chris Rock Show',NULL,now(),NULL,78787),
(6423,496,'TV Nation',NULL,now(),NULL,78788),
(6424,16726,'Empire (1984)',NULL,now(),NULL,78789),
(6425,3726,'Goosebumps',NULL,now(),NULL,78790),
(6426,9900,'The Mallens',NULL,now(),NULL,78791),
(6427,5719,'The Days and Nights of Molly Dodd',NULL,now(),NULL,78792),
(6428,5321,'Star Cops',NULL,now(),NULL,78793),
(6429,497,'Zoboomafoo',NULL,now(),NULL,78794),
(6430,14870,'Shannon',NULL,now(),NULL,78795),
(6431,4280,'Lord Peter Wimsey',NULL,now(),NULL,78796),
(6432,4307,'Lucan',NULL,now(),NULL,78797),
(6433,10131,'Hagen',NULL,now(),NULL,78798),
(6434,498,'Russian Roulette',NULL,now(),NULL,78799),
(6435,9913,'Skinnamarink TV',NULL,now(),NULL,78800),
(6436,4553,'Muppets Tonight',NULL,now(),NULL,78801),
(6437,499,'Jack of Hearts',NULL,now(),NULL,78802),
(6438,500,'Wild & Crazy Kids',NULL,now(),NULL,78803),
(6439,6717,'Inked',NULL,now(),NULL,78805),
(6440,2379,'A Bear\'s Tail',NULL,now(),NULL,78806),
(6441,6197,'The Smoking Room',NULL,now(),NULL,78808),
(6442,6538,'Wanted',NULL,now(),NULL,78809),
(6443,4640,'Nero Wolfe (2001)',NULL,now(),NULL,78811),
(6444,7177,'Miami Ink',NULL,now(),NULL,78816),
(6445,6818,'30 Days',NULL,now(),NULL,78818),
(6446,3504,'Family Business',NULL,now(),NULL,78819),
(6447,4652,'New Tricks',NULL,now(),NULL,78823),
(6448,4688,'North Shore',NULL,now(),NULL,78824),
(6449,6605,'Wildboyz',NULL,now(),NULL,78825),
(6450,2280,'John Safran Vs God',NULL,now(),NULL,78833),
(6451,6827,'The Girls Next Door',NULL,now(),NULL,78834),
(6452,1008,'Comedy Now',NULL,now(),NULL,79623),
(6453,17512,'Parlamentet',NULL,now(),NULL,79625),
(6454,3590,'Foyle\'s War',NULL,now(),NULL,78837),
(6455,5352,'Still Game',NULL,now(),NULL,78838),
(6456,4062,'John Doe',NULL,now(),NULL,78839),
(6457,6718,'Criss Angel Mindfreak',NULL,now(),NULL,78841),
(6458,6802,'Pimp My Ride (UK)',NULL,now(),NULL,78842),
(6459,8577,'MTV\'s The \'70s House',NULL,now(),NULL,78843),
(6460,5359,'Strange',NULL,now(),NULL,78844),
(6461,5162,'Shameless',NULL,now(),NULL,78846),
(6462,8148,'MTV Video Music Awards',NULL,now(),NULL,78847),
(6463,5387,'Sue Thomas: F.B.Eye',NULL,now(),NULL,78849),
(6464,2386,'What the Ancients Did For Us',NULL,now(),NULL,78850),
(6465,5253,'Son of the Beach',NULL,now(),NULL,78853),
(6466,2148,'Descendants of Darkness',NULL,now(),NULL,78855),
(6467,3688,'Girls Bravo',NULL,now(),NULL,78856),
(6468,4620,'Naruto',NULL,now(),NULL,78857),
(6469,5339,'Starved',NULL,now(),NULL,78858),
(6470,3310,'Digging for the Truth',NULL,now(),NULL,78863),
(6471,8132,'The Living Planet',NULL,now(),NULL,78864),
(6472,4127,'Killer Instinct',NULL,now(),NULL,78865),
(6473,630,'The Croc Files',NULL,now(),NULL,78867),
(6474,3642,'Game Over',NULL,now(),NULL,78871);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(6475,1340,'Seven Wonders of the Industrial World',NULL,now(),NULL,78873),
(6476,9591,'Connections',NULL,now(),NULL,78875),
(6477,3740,'Greg the Bunny',NULL,now(),NULL,78877),
(6478,3572,'Fooly Cooly (FLCL)',NULL,now(),NULL,78878),
(6479,12836,'Starship Operators',NULL,now(),NULL,78884),
(6480,8265,'Tweeny Witches',NULL,now(),NULL,78885),
(6481,4094,'Just Legal',NULL,now(),NULL,78887),
(6482,4458,'Spooks',NULL,now(),NULL,78890),
(6483,6640,'Wonder Showzen',NULL,now(),NULL,78891),
(6484,4673,'Nighty Night',NULL,now(),NULL,78893),
(6485,1370,'Absolute Power',NULL,now(),NULL,78894),
(6486,2515,'Agatha Christie\'s Marple',NULL,now(),NULL,78895),
(6487,4033,'Jake 2.0',NULL,now(),NULL,78898),
(6488,983,'Fruits Basket',NULL,now(),NULL,78900),
(6489,3573,'Footballers\' Wives',NULL,now(),NULL,78905),
(6490,1042,'Angelic Layer',NULL,now(),NULL,78909),
(6491,2195,'World Shut Your Mouth',NULL,now(),NULL,78911),
(6492,22612,'Pure Pwnage',NULL,now(),NULL,78912),
(6493,1443,'Full Metal Panic Fumoffu',NULL,now(),NULL,78915),
(6494,1043,'Gatekeepers',NULL,now(),NULL,78918),
(6495,2237,'Genshiken',NULL,now(),NULL,78919),
(6496,6807,'Chef at Home',NULL,now(),NULL,78921),
(6497,2583,'Amazing Stories',NULL,now(),NULL,76149),
(6498,3317,'Dinotopia',NULL,now(),NULL,78929),
(6499,7958,'The Chronicles of Narnia',NULL,now(),NULL,78930),
(6500,9648,'Mike Reid',NULL,now(),NULL,78934),
(6501,7946,'UFC Unleashed',NULL,now(),NULL,78937),
(6502,11079,'Going Tribal',NULL,now(),NULL,78939),
(6503,2252,'Space',NULL,now(),NULL,78942),
(6504,12166,'State of the Planet',NULL,now(),NULL,78943),
(6505,7865,'The Ascent of Man',NULL,now(),NULL,78944),
(6506,7548,'The Human Body',NULL,now(),NULL,78945),
(6507,7497,'Walking with Cavemen',NULL,now(),NULL,78946),
(6508,3695,'Glory Days',NULL,now(),NULL,78892),
(6509,5831,'The Grid',NULL,now(),NULL,78950),
(6510,4670,'Night Stalker',NULL,now(),NULL,78952),
(6511,5077,'S-CRY-Ed',NULL,now(),NULL,78953),
(6512,5270,'So You Think You Can Dance',NULL,now(),NULL,78956),
(6513,7138,'Mind of Mencia',NULL,now(),NULL,78958),
(6514,17214,'Blood+',NULL,now(),NULL,78960),
(6515,12766,'Shakugan no Shana',NULL,now(),NULL,78962),
(6516,4303,'Love Soup',NULL,now(),NULL,78964),
(6517,15676,'Heia Tufte',NULL,now(),NULL,78970),
(6518,12826,'Speed Grapher',NULL,now(),NULL,78972),
(6519,10673,'Divine Design',NULL,now(),NULL,78977),
(6520,6883,'Catscratch',NULL,now(),NULL,78979),
(6521,5832,'The Grim Adventures of Billy and Mandy',NULL,now(),NULL,78980),
(6522,10088,'Legend of Earthsea',NULL,now(),NULL,78984),
(6523,19417,'The Elegant Universe',NULL,now(),NULL,78985),
(6524,10298,'Mai-Otome',NULL,now(),NULL,78986),
(6525,12167,'The Life of Birds',NULL,now(),NULL,78989),
(6526,12168,'Wildlife Specials',NULL,now(),NULL,78992),
(6527,11202,'The Private Life of Plants',NULL,now(),NULL,78993),
(6528,4236,'The Life of Mammals',NULL,now(),NULL,78994),
(6529,10878,'The Worst Jobs in History',NULL,now(),NULL,78995),
(6530,6740,'Fifth Gear',NULL,now(),NULL,78998),
(6531,23031,'Russia\'s War: Blood Upon The Snow',NULL,now(),NULL,78999),
(6532,16019,'Densha Otoko',NULL,now(),NULL,79001),
(6533,9821,'Around The World In 80 Treasures',NULL,now(),NULL,79002),
(6534,6735,'Rocket Man',NULL,now(),NULL,79003),
(6535,583,'Crank Yankers',NULL,now(),NULL,79009),
(6536,20384,'Diggnation',NULL,now(),NULL,79013),
(6537,6737,'Funland',NULL,now(),NULL,79014),
(6538,9841,'Spoons',NULL,now(),NULL,79015),
(6539,6748,'Jenna\'s American Sex Star',NULL,now(),NULL,79016),
(6540,2292,'Squidbillies',NULL,now(),NULL,79017),
(6541,1204,'Stratos 4',NULL,now(),NULL,79022),
(6542,16764,'World Series of Poker',NULL,now(),NULL,79028),
(6543,6789,'ShakespeaRe-Told',NULL,now(),NULL,79029),
(6544,12383,'This Ugly Yet Beautiful World',NULL,now(),NULL,79032),
(6545,10089,'Captain Future',NULL,now(),NULL,79033),
(6546,4339,'Mahoromatic',NULL,now(),NULL,79035),
(6547,8222,'The Masterminds',NULL,now(),NULL,79038),
(6548,14785,'Britain\'s Next Top Model',NULL,now(),NULL,79040),
(6549,6780,'Man Stroke Woman',NULL,now(),NULL,79041),
(6550,8147,'He is my Master',NULL,now(),NULL,79043),
(6551,7987,'Honey and Clover',NULL,now(),NULL,79044),
(6552,12183,'Gunparade Orchestra',NULL,now(),NULL,79047),
(6553,6624,'Wire in the Blood',NULL,now(),NULL,79054),
(6554,3663,'Getbackers',NULL,now(),NULL,79056),
(6555,12827,'Spiral',NULL,now(),NULL,79059),
(6556,1138,'Onegai Teacher',NULL,now(),NULL,79062),
(6557,5344,'Static Shock',NULL,now(),NULL,79067),
(6558,3634,'G.I. Joe Sigma Six',NULL,now(),NULL,79069),
(6559,6723,'My Fair Brady',NULL,now(),NULL,79070),
(6560,2676,'Australia\'s Next Top Model',NULL,now(),NULL,79073),
(6561,6714,'Miss Seventeen',NULL,now(),NULL,79074),
(6562,3931,'Hunter X Hunter',NULL,now(),NULL,79076),
(6563,8131,'Mind, Body & Kick Ass Moves',NULL,now(),NULL,79079),
(6564,20959,'The Sex Files',NULL,now(),NULL,79080),
(6565,11300,'Sexual Secrets',NULL,now(),NULL,79085),
(6566,12953,'The Sex Inspectors',NULL,now(),NULL,79087),
(6567,5093,'Samurai Champloo',NULL,now(),NULL,79089),
(6568,11052,'Jericho (2005)',NULL,now(),NULL,79092),
(6569,8090,'Brothers & Sisters',NULL,now(),NULL,79506),
(6570,10232,'Property Ladder',NULL,now(),NULL,79098),
(6571,17748,'Miracle Planet',NULL,now(),NULL,79100),
(6572,12185,'Guyver The Bioboosted Armor',NULL,now(),NULL,79104),
(6573,2363,'Haibane Renmei',NULL,now(),NULL,79105),
(6574,21891,'Nurse Witch Komugi',NULL,now(),NULL,79109),
(6575,12849,'Suzuka',NULL,now(),NULL,79111),
(6576,13593,'The Beiderbecke Affair',NULL,now(),NULL,79114),
(6577,8445,'Xenosaga: The Animation',NULL,now(),NULL,79115),
(6578,6731,'Homewrecker',NULL,now(),NULL,79117),
(6579,6871,'Four Kings',NULL,now(),NULL,79118),
(6580,12559,'Tribe',NULL,now(),NULL,79119),
(6581,785,'Judge John Deed',NULL,now(),NULL,79120),
(6582,18320,'Full Circle with Michael Palin',NULL,now(),NULL,79122),
(6583,952,'The Wrong Coast',NULL,now(),NULL,79124),
(6584,7312,'Global Frequency',NULL,now(),NULL,79125),
(6585,6873,'South Beach',NULL,now(),NULL,79130),
(6586,8287,'Class of the Titans',NULL,now(),NULL,79133),
(6587,1195,'Tantei Gakuen Q',NULL,now(),NULL,79134),
(6588,12184,'Gun X Sword',NULL,now(),NULL,79135),
(6589,11009,'Fantastic Children',NULL,now(),NULL,79136),
(6590,10042,'Capeta',NULL,now(),NULL,79139),
(6591,7954,'Koi Kaze',NULL,now(),NULL,79142),
(6592,18960,'My Boy Jack',NULL,now(),NULL,81021),
(6593,1403,'Star Ocean EX',NULL,now(),NULL,79145),
(6594,6528,'W.I.T.C.H.',NULL,now(),NULL,79154),
(6595,6809,'The Ghost Squad',NULL,now(),NULL,79155),
(6596,21934,'NCIS: Los Angeles',NULL,now(),NULL,95441),
(6597,6868,'12 oz. Mouse',NULL,now(),NULL,79159),
(6598,3794,'Harry and His Bucket Full of Dinosaurs',NULL,now(),NULL,79160),
(6599,2106,'Divergence Eve',NULL,now(),NULL,79161),
(6600,13762,'Engine',NULL,now(),NULL,79162),
(6601,20885,'Home to Stay',NULL,now(),NULL,79163),
(6602,17447,'Home to Go',NULL,now(),NULL,79164),
(6603,2116,'Dirty Sanchez',NULL,now(),NULL,79165),
(6604,13535,'The Clangers',NULL,now(),NULL,79166),
(6605,13578,'The Joy of Painting',NULL,now(),NULL,79167),
(6606,5276,'Space Ghost Coast to Coast',NULL,now(),NULL,79171),
(6607,5100,'Saved by the Bell',NULL,now(),NULL,79172),
(6608,6236,'The Three Stooges',NULL,now(),NULL,79173),
(6609,6793,'Life in the Undergrowth',NULL,now(),NULL,79185),
(6610,7364,'Mischief',NULL,now(),NULL,79190),
(6611,7942,'High Stakes Poker',NULL,now(),NULL,79191),
(6612,8146,'Karin',NULL,now(),NULL,79192),
(6613,6875,'Perfect Hair Forever',NULL,now(),NULL,79196),
(6614,1448,'World Idol',NULL,now(),NULL,79198),
(6615,3701,'Godiva\'s',NULL,now(),NULL,79200),
(6616,6858,'Survivorman',NULL,now(),NULL,79203),
(6617,1268,'The Worst Week of My Life',NULL,now(),NULL,79204),
(6618,8024,'Air Master',NULL,now(),NULL,79205),
(6619,2198,'Stroker and Hoop',NULL,now(),NULL,79207),
(6620,457,'Saddle Rash',NULL,now(),NULL,79208),
(6621,20777,'Neat',NULL,now(),NULL,79210),
(6622,12737,'UFC All Access',NULL,now(),NULL,79211),
(6623,4626,'The Dog Whisperer',NULL,now(),NULL,79212),
(6624,7175,'Titty Bang Bang',NULL,now(),NULL,79213),
(6625,19487,'TransGeneration',NULL,now(),NULL,79218),
(6626,6893,'Flavor of Love',NULL,now(),NULL,79221),
(6627,7162,'Peppa Pig',NULL,now(),NULL,79222),
(6628,12506,'Miami Guns',NULL,now(),NULL,79225),
(6629,8007,'Paradise Kiss',NULL,now(),NULL,79233),
(6630,8552,'Aishiteruze Baby',NULL,now(),NULL,79235),
(6631,10787,'El Hazard: The Wanderers',NULL,now(),NULL,79241),
(6632,5115,'Scout\'s Safari',NULL,now(),NULL,79247),
(6633,6292,'The Wiggles',NULL,now(),NULL,79251),
(6634,2662,'Astro Boy (2004)',NULL,now(),NULL,79252),
(6635,7133,'King of Vegas',NULL,now(),NULL,79253),
(6636,6867,'Cheerleader Nation',NULL,now(),NULL,79258),
(6637,12937,'Traffik',NULL,now(),NULL,79672),
(6638,7468,'Hotel Babylon',NULL,now(),NULL,79260),
(6639,7892,'Free Ride',NULL,now(),NULL,79261),
(6640,7082,'Black. White.',NULL,now(),NULL,79262),
(6641,5105,'Scare Tactics',NULL,now(),NULL,79263),
(6642,11187,'Grand Designs',NULL,now(),NULL,79264),
(6643,7078,'Project Catwalk',NULL,now(),NULL,79265),
(6644,6946,'Sons & Daughters',NULL,now(),NULL,79249),
(6645,2526,'Ai yori Aoshi',NULL,now(),NULL,79267),
(6646,20934,'The Thirsty Traveler',NULL,now(),NULL,79272),
(6647,6715,'The Colbert Report',NULL,now(),NULL,79274),
(6648,3372,'Dragon Ball GT',NULL,now(),NULL,79275),
(6649,6835,'In Search of Shakespeare',NULL,now(),NULL,79277),
(6650,5082,'Sabrina\'s Secret Life',NULL,now(),NULL,79278),
(6651,5243,'So Little Time',NULL,now(),NULL,79279),
(6652,7955,'Teachers (US)',NULL,now(),NULL,79280),
(6653,16910,'Simply Quilts',NULL,now(),NULL,79281),
(6654,7856,'Johnny and The Bomb',NULL,now(),NULL,79284),
(6655,3327,'Doc Martin',NULL,now(),NULL,79285),
(6656,4625,'Nathan Barley',NULL,now(),NULL,79286),
(6657,8405,'Kick Ass Miracles',NULL,now(),NULL,79287),
(6658,7500,'American Inventor',NULL,now(),NULL,79288),
(6659,3638,'Galaxy Angel',NULL,now(),NULL,79289),
(6660,6930,'What About Brian',NULL,now(),NULL,79290),
(6661,13300,'The Box of Delights',NULL,now(),NULL,79291),
(6662,3333,'Doctor Who Confidential',NULL,now(),NULL,79298),
(6663,23048,'Vietnam: A Television History',NULL,now(),NULL,79301),
(6664,7038,'Moral Orel',NULL,now(),NULL,79302),
(6665,10158,'Japanorama',NULL,now(),NULL,79303),
(6666,3214,'Damage Control',NULL,now(),NULL,79305),
(6667,8521,'The World At War',NULL,now(),NULL,79309),
(6668,6635,'Wolf Lake',NULL,now(),NULL,79312),
(6669,6995,'Top Chef',NULL,now(),NULL,79313),
(6670,10790,'10.5 Apocalypse',NULL,now(),NULL,79314),
(6671,9464,'Perfect Disaster',NULL,now(),NULL,79315),
(6672,7096,'Hannah Montana',NULL,now(),NULL,79317),
(6673,7885,'Survival of the Richest',NULL,now(),NULL,79318),
(6674,1306,'National Geographic Explorer',NULL,now(),NULL,79319),
(6675,1781,'Dr. Terrible\'s House of Horrible',NULL,now(),NULL,79322),
(6676,6938,'Windfall',NULL,now(),NULL,79324),
(6677,10926,'Lucky Louie',NULL,now(),NULL,79326),
(6678,17127,'SvÃƒÂ­nasÃƒÂºpan',NULL,now(),NULL,79333),
(6679,7013,'Treasure Hunters',NULL,now(),NULL,79336),
(6680,8098,'Aquaman',NULL,now(),NULL,79338),
(6681,8053,'In Case of Emergency',NULL,now(),NULL,79686),
(6682,12681,'Work Out',NULL,now(),NULL,79341),
(6683,12376,'Eureka Seven',NULL,now(),NULL,79360),
(6684,12320,'It\'s a Big Big World',NULL,now(),NULL,79368),
(6685,4072,'JoJo\'s Circus',NULL,now(),NULL,79370),
(6686,8099,'The Class',NULL,now(),NULL,79372),
(6687,5319,'Stanley',NULL,now(),NULL,79373),
(6688,12165,'The Power Of Nightmares',NULL,now(),NULL,79374),
(6689,12988,'The Business',NULL,now(),NULL,79375),
(6690,6163,'The Save-Ums',NULL,now(),NULL,79376),
(6691,10663,'The Wonder Pets!',NULL,now(),NULL,79377),
(6692,12058,'Billable Hours',NULL,now(),NULL,79379),
(6693,11248,'Happy Hour',NULL,now(),NULL,79381),
(6694,21746,'Innocent Venus',NULL,now(),NULL,79382),
(6695,12671,'Prehistoric Park',NULL,now(),NULL,79383),
(6696,7141,'Canada\'s Next Top Model',NULL,now(),NULL,79385),
(6697,12011,'Drop Dead Gorgeous',NULL,now(),NULL,79386),
(6698,8490,'Shark',NULL,now(),NULL,79388),
(6699,10098,'Nightmares & Dreamscapes: From the Stories of Stephen King',NULL,now(),NULL,79389),
(6700,12817,'Feasting on Asphalt',NULL,now(),NULL,79393),
(6701,7913,'Legion of Super Heroes',NULL,now(),NULL,79394),
(6702,13782,'Dallas Cowboys Cheerleaders: Making the Team',NULL,now(),NULL,79395),
(6703,12077,'Korgoth of Barbaria',NULL,now(),NULL,79396),
(6704,17350,'The Eyes of Nye',NULL,now(),NULL,79398),
(6705,7769,'Who Wants to Be a Superhero?',NULL,now(),NULL,79399),
(6706,6754,'Minoriteam',NULL,now(),NULL,79402),
(6707,3574,'Footballers\' Wives - Extra Time',NULL,now(),NULL,79405),
(6708,11051,'Gene Simmons Family Jewels',NULL,now(),NULL,79406),
(6709,19718,'Mike Bassett: Manager',NULL,now(),NULL,79407),
(6710,11230,'Angela\'s Eyes',NULL,now(),NULL,79408),
(6711,12128,'Dane Cook\'s Tourgasm',NULL,now(),NULL,79416),
(6712,8293,'Bridezillas',NULL,now(),NULL,79417),
(6713,10753,'Sinchronicity',NULL,now(),NULL,79419),
(6714,19757,'NextWorld',NULL,now(),NULL,82754),
(6715,6820,'The Presidents',NULL,now(),NULL,79425),
(6716,13473,'Patent Bending',NULL,now(),NULL,79427),
(6717,7956,'The Underground',NULL,now(),NULL,79428),
(6718,7205,'Bagpuss',NULL,now(),NULL,79430),
(6719,21883,'IsÃƒÂ¤nmaan toivot',NULL,now(),NULL,79433),
(6720,6922,'Walking with Monsters',NULL,now(),NULL,79437),
(6721,10303,'Ippatsu Kikimusume',NULL,now(),NULL,79445),
(6722,1221,'Najica: Blitz Tactics',NULL,now(),NULL,79447),
(6723,7100,'Trouble Chocolate',NULL,now(),NULL,79453),
(6724,8021,'Uta Kata',NULL,now(),NULL,79454),
(6725,1412,'Karakuri Zoushi Ayatsuri Sakon',NULL,now(),NULL,79457),
(6726,7965,'Samurai Gun',NULL,now(),NULL,79459),
(6727,1218,'Aura Battler Dunbine',NULL,now(),NULL,79465),
(6728,18330,'Le Chevalier D\'Eon',NULL,now(),NULL,79470),
(6729,15156,'Salaryman Kintaro',NULL,now(),NULL,79471),
(6730,12481,'Demashitaa Powerpuff Girls Z',NULL,now(),NULL,79472),
(6731,1541,'Kazemakase Tsukikage Ran',NULL,now(),NULL,79476),
(6732,6836,'Everyday Italian',NULL,now(),NULL,79482),
(6733,13400,'Night Head Genesis',NULL,now(),NULL,79484),
(6734,11215,'30 Rock',NULL,now(),NULL,79488),
(6735,10408,'America\'s Got Talent',NULL,now(),NULL,79490),
(6736,10791,'How Do You Solve a Problem Like Maria',NULL,now(),NULL,79493),
(6737,12793,'Tabloid Wars',NULL,now(),NULL,79494),
(6738,2677,'Australian Idol',NULL,now(),NULL,79496),
(6739,12818,'Celebrity Duets',NULL,now(),NULL,79497),
(6740,20971,'Smart Woman Survival Guide',NULL,now(),NULL,79500),
(6741,19658,'Wow! Wow! Wubbzy!',NULL,now(),NULL,79507),
(6742,13474,'Intelligence',NULL,now(),NULL,79508),
(6743,8152,'Day Break',NULL,now(),NULL,79509),
(6744,7991,'I Pity the Fool',NULL,now(),NULL,79512),
(6745,18767,'Best of the Worst (UK)',NULL,now(),NULL,79514),
(6746,6822,'Skyland',NULL,now(),NULL,79516),
(6747,11239,'Saxondale',NULL,now(),NULL,79517),
(6748,2210,'Now and Then, Here and There',NULL,now(),NULL,79520),
(6749,18378,'Code Geass',NULL,now(),NULL,79525),
(6750,14756,'Ghost Hunt',NULL,now(),NULL,79527),
(6751,8456,'Help Me Help You',NULL,now(),NULL,79529),
(6752,14830,'Torchwood Declassified',NULL,now(),NULL,79537),
(6753,13319,'A Haunting',NULL,now(),NULL,79535),
(6754,19618,'Monarchy',NULL,now(),NULL,80833),
(6755,6890,'The Hills',NULL,now(),NULL,79550),
(6756,18755,'The Unpleasant World of Penn and Teller',NULL,now(),NULL,79553),
(6757,1509,'Garth Marenghi\'s Darkplace',NULL,now(),NULL,79554),
(6758,7234,'Dog Bites Man',NULL,now(),NULL,79555),
(6759,7074,'8 Out of 10 Cats',NULL,now(),NULL,79556),
(6760,3618,'Frontier House',NULL,now(),NULL,79558),
(6761,6719,'The Adam Carolla Project',NULL,now(),NULL,79559),
(6762,6720,'Too Late With Adam Carolla',NULL,now(),NULL,79560),
(6763,12636,'Metalocalypse',NULL,now(),NULL,79563),
(6764,8179,'The Real Hustle',NULL,now(),NULL,79565),
(6765,8134,'Boys Be...',NULL,now(),NULL,79566),
(6766,23693,'Megacities',NULL,now(),NULL,79569),
(6767,18842,'Louis Theroux\'s Weird Weekends',NULL,now(),NULL,79570),
(6768,13553,'America (Alistair Cooke)',NULL,now(),NULL,79571),
(6769,19991,'Boer zoekt vrouw',NULL,now(),NULL,79572),
(6770,13922,'Lead Balloon',NULL,now(),NULL,79573),
(6771,6742,'Category 7: The End of the World',NULL,now(),NULL,79575),
(6772,4029,'Jack of All Trades',NULL,now(),NULL,79577),
(6773,12222,'VeggieTales',NULL,now(),NULL,79578),
(6774,9151,'Twenty Good Years',NULL,now(),NULL,79584),
(6775,6892,'Storm Stories',NULL,now(),NULL,79585),
(6776,3700,'God, The Devil and Bob',NULL,now(),NULL,79594),
(6777,12907,'Damekko Doubutsu',NULL,now(),NULL,79593),
(6778,5711,'The D.A.',NULL,now(),NULL,79596),
(6779,12989,'Viva PiÃƒÂ±ata',NULL,now(),NULL,79598),
(6780,10418,'10 Items Or Less',NULL,now(),NULL,79599),
(6781,10417,'My Boys',NULL,now(),NULL,79600),
(6782,11227,'Big Day',NULL,now(),NULL,79602),
(6783,15352,'Wilfred',NULL,now(),NULL,80331),
(6784,22245,'Trick It Out',NULL,now(),NULL,79606),
(6785,6797,'The F Word',NULL,now(),NULL,79609),
(6786,19147,'In Search of the Trojan War',NULL,now(),NULL,79613),
(6787,12170,'Lost Worlds, Vanished Lives',NULL,now(),NULL,79616),
(6788,8843,'The Victoria\'s Secret Fashion Show',NULL,now(),NULL,79619),
(6789,18373,'Borettslaget',NULL,now(),NULL,79628),
(6790,16853,'Etaten',NULL,now(),NULL,79629),
(6791,14691,'Is it Real?',NULL,now(),NULL,79630),
(6792,20048,'Raymann is laat',NULL,now(),NULL,79632),
(6793,16631,'Ramsay\'s Boiling Point',NULL,now(),NULL,79633),
(6794,21852,'The Day The Universe Changed',NULL,now(),NULL,79634),
(6795,8159,'The Game',NULL,now(),NULL,79636),
(6796,14688,'The Lost Room',NULL,now(),NULL,79637),
(6797,14567,'Gokusen',NULL,now(),NULL,79638),
(6798,14328,'Engineering An Empire',NULL,now(),NULL,79642),
(6799,11228,'The Innocence Project',NULL,now(),NULL,79643),
(6800,17505,'The Amazing Race Asia',NULL,now(),NULL,79646),
(6801,5345,'Stella',NULL,now(),NULL,79649),
(6802,8346,'Faceless',NULL,now(),NULL,79650),
(6803,16695,'Tilbake til 80 tallet',NULL,now(),NULL,79651),
(6804,7081,'Mock the Week',NULL,now(),NULL,79761),
(6805,13041,'Respectable',NULL,now(),NULL,79696),
(6806,17090,'Crayon Shin-Chan',NULL,now(),NULL,79654),
(6807,11255,'Geneshaft',NULL,now(),NULL,79853),
(6808,7693,'Legend of the Condor Hero',NULL,now(),NULL,79658),
(6809,11174,'BBC Documentaries',NULL,now(),NULL,79660),
(6810,8399,'Zipang',NULL,now(),NULL,79712),
(6811,14983,'Anthony Bourdain: No Reservations',NULL,now(),NULL,79668),
(6812,12753,'Strictly Confidential',NULL,now(),NULL,79664),
(6813,15433,'Claymore',NULL,now(),NULL,79995),
(6814,6286,'The Weekenders',NULL,now(),NULL,79669),
(6815,7108,'The Thick of It',NULL,now(),NULL,79843),
(6816,14550,'Tripping Over',NULL,now(),NULL,79695),
(6817,12438,'Ouran High School Host Club',NULL,now(),NULL,79713),
(6818,6778,'Australian Princess',NULL,now(),NULL,79683),
(6819,12926,'Zero no Tsukaima',NULL,now(),NULL,79719),
(6820,6782,'Broken News',NULL,now(),NULL,79694),
(6821,14681,'The Secret Millionaire',NULL,now(),NULL,79687),
(6822,12340,'How To Look Good Naked',NULL,now(),NULL,79688),
(6823,10537,'Thank God You\'re Here (Aus)',NULL,now(),NULL,79697),
(6824,14927,'Grease: You\'re the One that I Want',NULL,now(),NULL,79699),
(6825,17785,'Everest Beyond the Limit',NULL,now(),NULL,79693),
(6826,15033,'Yamato Nadeshiko Shichi Henge',NULL,now(),NULL,79764),
(6827,8069,'Dark Kingdom: The Dragon King',NULL,now(),NULL,79704),
(6828,8536,'The Chaser\'s War on Everything',NULL,now(),NULL,79706),
(6829,13365,'The Sarah Jane Adventures',NULL,now(),NULL,79708),
(6830,15023,'Poker After Dark',NULL,now(),NULL,79715),
(6831,12799,'Simoun',NULL,now(),NULL,79754),
(6832,7594,'Inside Rugby',NULL,now(),NULL,79803),
(6833,1835,'Conan the Adventurer',NULL,now(),NULL,79716),
(6834,20183,'UrsÃƒÂ¤kta rÃƒÂ¶ran (Vi bygger om)',NULL,now(),NULL,79717),
(6835,2300,'The First 48',NULL,now(),NULL,81275),
(6836,18326,'UFO Hunters',NULL,now(),NULL,81265),
(6837,18502,'Pastewka',NULL,now(),NULL,80094),
(6838,12890,'The Third',NULL,now(),NULL,79794),
(6839,6277,'The Waltons',NULL,now(),NULL,71226),
(6840,15165,'MI High',NULL,now(),NULL,95901),
(6841,22370,'Walk on By: The Story of Popular Song',NULL,now(),NULL,81260),
(6842,18053,'Prism Ark',NULL,now(),NULL,93491),
(6843,15522,'Kingdom',NULL,now(),NULL,80201),
(6844,6899,'Beyond the Break',NULL,now(),NULL,79756),
(6845,15062,'Dancelife',NULL,now(),NULL,79757),
(6846,7360,'The Dresden Files',NULL,now(),NULL,79759),
(6847,12612,'Time Trumpet',NULL,now(),NULL,79812),
(6848,3628,'Futurama',NULL,now(),NULL,73871),
(6849,12898,'Tokko',NULL,now(),NULL,79774),
(6850,3519,'Farscape',NULL,now(),NULL,70522),
(6851,1402,'Real Bout High School',NULL,now(),NULL,79781),
(6852,7033,'Sausage Factory',NULL,now(),NULL,79788),
(6853,15072,'Bam\'s Unholy Union',NULL,now(),NULL,79791),
(6854,4106,'Kathy Griffin: My Life on the D-List',NULL,now(),NULL,79795),
(6855,12925,'Zegapain',NULL,now(),NULL,79797),
(6856,19745,'Britain From Above',NULL,now(),NULL,82678),
(6857,4631,'Ned\'s Declassified School Survival Guide',NULL,now(),NULL,79821),
(6858,15675,'Torpedo',NULL,now(),NULL,79804),
(6859,13011,'Charlie Brooker\'s Screenwipe',NULL,now(),NULL,79818),
(6860,17627,'Say Yes To The Dress',NULL,now(),NULL,80836),
(6861,12226,'The Amazing Screw-On Head',NULL,now(),NULL,79826),
(6862,2229,'Dante\'s Cove',NULL,now(),NULL,79832),
(6863,6783,'Dragons\' Den',NULL,now(),NULL,79836),
(6864,8594,'Aria the Animation',NULL,now(),NULL,79839),
(6865,2062,'First Person',NULL,now(),NULL,79844),
(6866,8102,'The Winner',NULL,now(),NULL,79852),
(6867,16918,'24 Hour Design',NULL,now(),NULL,79848),
(6868,15228,'Are You Smarter Than a 5th Grader?',NULL,now(),NULL,79851),
(6869,9557,'The Late Show',NULL,now(),NULL,79870),
(6870,7090,'The Emperor\'s New School',NULL,now(),NULL,79855),
(6871,12290,'How To Start Your Own Country',NULL,now(),NULL,79913),
(6872,9165,'Andy Barker, P.I.',NULL,now(),NULL,79868),
(6873,12056,'Gintama',NULL,now(),NULL,79895),
(6874,1470,'The Dave Gorman Collection',NULL,now(),NULL,79907),
(6875,10257,'Drive',NULL,now(),NULL,79871),
(6876,2349,'According To Bex',NULL,now(),NULL,79914),
(6877,17544,'House Of Cards',NULL,now(),NULL,79861),
(6878,15247,'The Wedding Bells',NULL,now(),NULL,79867),
(6879,8347,'Raines',NULL,now(),NULL,79869),
(6880,6751,'Supernova',NULL,now(),NULL,79873),
(6881,15004,'The Riches',NULL,now(),NULL,79876),
(6882,14221,'Race to Dakar',NULL,now(),NULL,79899),
(6883,7484,'Root of all Evil',NULL,now(),NULL,79877),
(6884,12656,'Utawarerumono',NULL,now(),NULL,79880),
(6885,13022,'Kamichu!',NULL,now(),NULL,79881),
(6886,12219,'Harukanaru Toki no Naka de - Hachiyou Shou',NULL,now(),NULL,79882),
(6887,18035,'Kill it, Cook it, Eat it',NULL,now(),NULL,79909),
(6888,15091,'Thieves Like Us',NULL,now(),NULL,79979),
(6889,14962,'The Surreal Life: Fame Games',NULL,now(),NULL,79904),
(6890,15069,'Tim and Eric Awesome Show, Great Job!',NULL,now(),NULL,79905),
(6891,15098,'Benidorm',NULL,now(),NULL,79912),
(6892,15439,'Human Giant',NULL,now(),NULL,79992),
(6893,14219,'Saul of the Mole Men',NULL,now(),NULL,79906),
(6894,3542,'Fillmore!',NULL,now(),NULL,79910),
(6895,7957,'This American Life',NULL,now(),NULL,79928),
(6896,11010,'Frisky Dingo',NULL,now(),NULL,79930),
(6897,12334,'Jyu Oh Sei',NULL,now(),NULL,79931),
(6898,17647,'Wood Works',NULL,now(),NULL,79933),
(6899,8344,'October Road',NULL,now(),NULL,79934),
(6900,17040,'Die ProSieben MÃƒÂ¤rchenstunde',NULL,now(),NULL,79942),
(6901,15609,'Devil May Cry',NULL,now(),NULL,80224),
(6902,12169,'The Trials of Life',NULL,now(),NULL,79947),
(6903,15280,'Debbie Does Dallas Again',NULL,now(),NULL,79949),
(6904,5763,'The Fall Guy',NULL,now(),NULL,71932),
(6905,12285,'A 4x4 Is Born',NULL,now(),NULL,79958),
(6906,9568,'It\'s Not Easy Being Green',NULL,now(),NULL,79963),
(6907,15240,'Dice Undisputed',NULL,now(),NULL,79964),
(6908,2122,'The Trap Door',NULL,now(),NULL,80057),
(6909,8558,'Android Ana Maico 2010',NULL,now(),NULL,80076),
(6910,21186,'How Music Works with Howard Goodall',NULL,now(),NULL,79976),
(6911,16921,'Jamie at Home',NULL,now(),NULL,79978),
(6912,8491,'Freak Show',NULL,now(),NULL,79983),
(6913,15387,'Thank God You\'re Here (US)',NULL,now(),NULL,79991),
(6914,15474,'Heroic Age',NULL,now(),NULL,79999),
(6915,15843,'History\'s Strongest Disciple Kenichi',NULL,now(),NULL,80000),
(6916,18316,'Nodame Cantabile (2007)',NULL,now(),NULL,80005),
(6917,7886,'Secrets of a Small Town',NULL,now(),NULL,80171),
(6918,4659,'Wild \'N Out',NULL,now(),NULL,80018),
(6919,6745,'The Surgeon',NULL,now(),NULL,80023),
(6920,18391,'We Can Be Heroes',NULL,now(),NULL,80024),
(6921,765,'Fat Cow Motel',NULL,now(),NULL,80025),
(6922,6721,'The Showbiz Show with David Spade',NULL,now(),NULL,80027),
(6923,13667,'Kvarteret Skatan',NULL,now(),NULL,80035),
(6924,7229,'Driving Force',NULL,now(),NULL,80036),
(6925,12182,'Gun Frontier',NULL,now(),NULL,80043),
(6926,15480,'Derren Brown: Trick or Treat',NULL,now(),NULL,80045),
(6927,12353,'Kamisama Kazoku',NULL,now(),NULL,80059),
(6928,3298,'Dexter\'s Laboratory',NULL,now(),NULL,77992),
(6929,5058,'Roughnecks: Starship Troopers Chronicles',NULL,now(),NULL,78067),
(6930,2145,'Saikano',NULL,now(),NULL,79798),
(6931,3226,'Danger Mouse',NULL,now(),NULL,76308),
(6932,3482,'F/X: The Series',NULL,now(),NULL,73771),
(6933,5296,'Spider-Man (1981)',NULL,now(),NULL,77686),
(6934,13026,'M jak Milosc',NULL,now(),NULL,80079),
(6935,15490,'Koutetsu Sangokushi',NULL,now(),NULL,80081),
(6936,5080,'Saber Rider and the Star Sheriffs',NULL,now(),NULL,70525),
(6937,2505,'The Adventures of the Galaxy Rangers',NULL,now(),NULL,77772),
(6938,2899,'BraveStarr',NULL,now(),NULL,78688),
(6939,5111,'Scooby-Doo Where Are You!',NULL,now(),NULL,78260),
(6940,3492,'Falcon Beach',NULL,now(),NULL,81397),
(6941,5266,'South Park',NULL,now(),NULL,75897),
(6942,2979,'Captain Planet and the Planeteers',NULL,now(),NULL,74142),
(6943,6193,'The Six Million Dollar Man',NULL,now(),NULL,77294),
(6944,1673,'Daktari',NULL,now(),NULL,77094),
(6945,3362,'Dr. Quinn, Medicine Woman',NULL,now(),NULL,78095),
(6946,3146,'Count Duckula',NULL,now(),NULL,76307),
(6947,3400,'Dumb and Dumber',NULL,now(),NULL,76199),
(6948,5785,'The Flintstones',NULL,now(),NULL,76171),
(6949,3644,'Garfield & Friends',NULL,now(),NULL,77054),
(6950,3682,'Gilligan\'s Island',NULL,now(),NULL,77345),
(6951,14600,'Klovn',NULL,now(),NULL,79269),
(6952,12670,'Aoki Densetsu Shoot',NULL,now(),NULL,79439),
(6953,4776,'Parker Lewis Can\'t Lose',NULL,now(),NULL,78471),
(6954,5741,'The Dukes of Hazzard',NULL,now(),NULL,77748),
(6955,3056,'Chip \'N Dale Rescue Rangers',NULL,now(),NULL,75477),
(6956,21254,'En Fyra FÃƒÂ¶r Tre',NULL,now(),NULL,80495),
(6957,16880,'Tengen Toppa Gurren Lagann',NULL,now(),NULL,80096),
(6958,6121,'The Real Ghostbusters',NULL,now(),NULL,71824),
(6959,22156,'R. I. S.',NULL,now(),NULL,80097),
(6960,17637,'Phenomenon',NULL,now(),NULL,80715),
(6961,3940,'I am Weasel',NULL,now(),NULL,78272),
(6962,4315,'M*A*S*H',NULL,now(),NULL,70994),
(6963,15510,'Superstorm',NULL,now(),NULL,80099),
(6964,858,'James Bond, Jr.',NULL,now(),NULL,71681),
(6965,4447,'Men In Black: The Series',NULL,now(),NULL,77743),
(6966,4484,'Mission: Impossible',NULL,now(),NULL,77231),
(6967,4485,'Mission: Impossible (1988)',NULL,now(),NULL,72762),
(6968,4492,'Mister Ed',NULL,now(),NULL,78045),
(6969,4551,'Mummies Alive!',NULL,now(),NULL,73721),
(6970,16623,'Stromberg',NULL,now(),NULL,79656),
(6971,3245,'Darkwing Duck',NULL,now(),NULL,75475),
(6972,15465,'Ruddy Hell! It\'s Harry And Paul',NULL,now(),NULL,80101);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(6973,3737,'Green Green Grass',NULL,now(),NULL,80102),
(6974,7191,'The Wombles',NULL,now(),NULL,80104),
(6975,11210,'Top Design',NULL,now(),NULL,80107),
(6976,15335,'Rush Hour',NULL,now(),NULL,80108),
(6977,15496,'Bokurano',NULL,now(),NULL,80111),
(6978,8442,'Little People, Big World',NULL,now(),NULL,80112),
(6979,823,'Arjuna',NULL,now(),NULL,80113),
(6980,10749,'New Street Law',NULL,now(),NULL,80375),
(6981,7218,'The Real Housewives of Orange County',NULL,now(),NULL,80117),
(6982,16783,'FÃƒÂ³stbrÃƒÂ¦ÃƒÂ°ur',NULL,now(),NULL,80118),
(6983,2870,'Bones',NULL,now(),NULL,75682),
(6984,344,'Iron Man',NULL,now(),NULL,76278),
(6985,6570,'What\'s New Scooby-Doo?',NULL,now(),NULL,80119),
(6986,6107,'The Pretender',NULL,now(),NULL,70704),
(6987,7925,'A Picture of Britain',NULL,now(),NULL,80128),
(6988,2744,'Beauty and the Beast (1987)',NULL,now(),NULL,75004),
(6989,5952,'The Love Boat',NULL,now(),NULL,76954),
(6990,4972,'Remington Steele',NULL,now(),NULL,78189),
(6991,19813,'Blade of the Immortal',NULL,now(),NULL,82498),
(6992,17962,'Food Safari',NULL,now(),NULL,80132),
(6993,18588,'Classic Albums',NULL,now(),NULL,80133),
(6994,11149,'The Jake Effect',NULL,now(),NULL,80135),
(6995,5907,'The King of Queens',NULL,now(),NULL,73641),
(6996,6736,'Egypt',NULL,now(),NULL,80137),
(6997,7482,'Autopsy Life and Death',NULL,now(),NULL,80139),
(6998,4461,'Miami Vice',NULL,now(),NULL,77098),
(6999,16653,'Schillerstrasse',NULL,now(),NULL,79834),
(7000,18615,'Playa Del Sol',NULL,now(),NULL,80143),
(7001,15660,'Durham County',NULL,now(),NULL,80144),
(7002,15363,'Blue Dragon',NULL,now(),NULL,80147),
(7003,10200,'Halfway Home',NULL,now(),NULL,80149),
(7004,12408,'Serial Experiments Lain',NULL,now(),NULL,78814),
(7005,17339,'Michael Palin\'s New Europe',NULL,now(),NULL,80589),
(7006,15560,'Tack gode gud',NULL,now(),NULL,80155),
(7007,14106,'The Mitchell and Webb Situation',NULL,now(),NULL,80156),
(7008,10473,'Totally Doctor Who',NULL,now(),NULL,80160),
(7009,8570,'The Micallef Program',NULL,now(),NULL,80163),
(7010,12723,'That Mitchell and Webb Look',NULL,now(),NULL,80165),
(7011,2209,'NY-LON (UK)',NULL,now(),NULL,80167),
(7012,15534,'The Real Wedding Crashers',NULL,now(),NULL,80168),
(7013,12525,'Shura no Toki',NULL,now(),NULL,80173),
(7014,10306,'On the Lot',NULL,now(),NULL,80174),
(7015,8170,'Hidden Palms',NULL,now(),NULL,80182),
(7016,16495,'I Bet You',NULL,now(),NULL,80184),
(7017,17902,'Kaamelott',NULL,now(),NULL,79175),
(7018,12114,'The Naked Chef',NULL,now(),NULL,80185),
(7019,15405,'Ronni Ancona & Co.',NULL,now(),NULL,80186),
(7020,15137,'Future Car',NULL,now(),NULL,80187),
(7021,3532,'Fawlty Towers',NULL,now(),NULL,75932),
(7022,4054,'Jim Henson\'s The Storyteller: Greek Myths',NULL,now(),NULL,77710),
(7023,6597,'Whose Line Is It Anyway?',NULL,now(),NULL,76808),
(7024,15575,'Pirate Master',NULL,now(),NULL,80188),
(7025,15287,'Bullrun',NULL,now(),NULL,80189),
(7026,18446,'Uti vÃƒÂ¥r hage',NULL,now(),NULL,78931),
(7027,6649,'World Poker Tour',NULL,now(),NULL,80192),
(7028,14535,'Rob & Big',NULL,now(),NULL,80193),
(7029,15151,'Shear Genius',NULL,now(),NULL,80197),
(7030,7280,'8th and Ocean',NULL,now(),NULL,80200),
(7031,12123,'MTV Movie Awards',NULL,now(),NULL,80203),
(7032,13453,'The Starter Wife',NULL,now(),NULL,80207),
(7033,7104,'The Janice Dickinson Modeling Agency',NULL,now(),NULL,80209),
(7034,14654,'Jozi-H',NULL,now(),NULL,80210),
(7035,790,'SoulTaker',NULL,now(),NULL,80211),
(7036,7279,'So NoTORIous',NULL,now(),NULL,80212),
(7037,14612,'John from Cincinnati',NULL,now(),NULL,80215),
(7038,2608,'Andromeda',NULL,now(),NULL,77544),
(7039,15554,'National Bingo Night',NULL,now(),NULL,80217),
(7040,14981,'Bindi, the Jungle Girl',NULL,now(),NULL,80218),
(7041,16419,'Fight Girls',NULL,now(),NULL,80221),
(7042,3747,'Growing Pains',NULL,now(),NULL,70967),
(7043,1159,'Sea of Souls',NULL,now(),NULL,80225),
(7044,3627,'Funky Cops',NULL,now(),NULL,80227),
(7045,10261,'Seven Periods With Mr. Gormsby',NULL,now(),NULL,80228),
(7046,10117,'Touch Me, I\'m Karen Taylor',NULL,now(),NULL,80229),
(7047,16394,'Building the Future',NULL,now(),NULL,80230),
(7048,15594,'Army Wives',NULL,now(),NULL,80231),
(7049,15473,'Seto no Hanayome',NULL,now(),NULL,80236),
(7050,15493,'Shining Tears X Wind',NULL,now(),NULL,80237),
(7051,15364,'Toward the Terra',NULL,now(),NULL,80241),
(7052,8277,'Desert Punk',NULL,now(),NULL,80242),
(7053,15722,'Paul Merton in China',NULL,now(),NULL,80243),
(7054,7492,'The Shining',NULL,now(),NULL,80245),
(7055,15527,'Lil\' Bush',NULL,now(),NULL,80246),
(7056,9945,'TÃƒÂ¼rkisch fÃƒÂ¼r AnfÃƒÂ¤nger',NULL,now(),NULL,80247),
(7057,10807,'Young Dracula',NULL,now(),NULL,80248),
(7058,16753,'Berlin, Berlin',NULL,now(),NULL,80249),
(7059,11110,'Destination Truth',NULL,now(),NULL,80251),
(7060,15936,'The Academy',NULL,now(),NULL,80253),
(7061,593,'Defenders of the Earth',NULL,now(),NULL,70824),
(7062,2575,'Always Greener',NULL,now(),NULL,80255),
(7063,16595,'Kick',NULL,now(),NULL,80256),
(7064,14611,'Holmes on Homes',NULL,now(),NULL,80258),
(7065,5945,'The Loop',NULL,now(),NULL,75275),
(7066,6534,'Walker, Texas Ranger',NULL,now(),NULL,73756),
(7067,6592,'Who\'s the Boss?',NULL,now(),NULL,76587),
(7068,6517,'Viper',NULL,now(),NULL,71098),
(7069,12786,'Dog Borstal',NULL,now(),NULL,80259),
(7070,15312,'Heartland (2007)',NULL,now(),NULL,80260),
(7071,15588,'The Ex-Wives Club',NULL,now(),NULL,80261),
(7072,15591,'Shaq\'s Big Challenge',NULL,now(),NULL,80262),
(7073,16467,'Hey Paula',NULL,now(),NULL,80269),
(7074,13032,'Oban Star-Racers',NULL,now(),NULL,80131),
(7075,15489,'Kishin Taisen Gigantic Formula',NULL,now(),NULL,80278),
(7076,15587,'Age of Love',NULL,now(),NULL,80279),
(7077,17160,'Beirut to Bosnia',NULL,now(),NULL,81078),
(7078,12932,'The Tom and Jerry Comedy Show',NULL,now(),NULL,83056),
(7079,6434,'True Life',NULL,now(),NULL,71949),
(7080,15345,'Saving Planet Earth',NULL,now(),NULL,80283),
(7081,3738,'Green Wing',NULL,now(),NULL,78899),
(7082,15332,'The Big Gay Sketch Show',NULL,now(),NULL,80286),
(7083,4136,'Kingpin',NULL,now(),NULL,70364),
(7084,4563,'Murphy\'s Law',NULL,now(),NULL,73425),
(7085,5447,'Taken',NULL,now(),NULL,78224),
(7086,15029,'Code Monkeys',NULL,now(),NULL,80287),
(7087,6576,'What I Like About You',NULL,now(),NULL,78462),
(7088,1508,'Fireflies',NULL,now(),NULL,80288),
(7089,4038,'Jamie\'s Kitchen',NULL,now(),NULL,80290),
(7090,3581,'Forever Eden',NULL,now(),NULL,80291),
(7091,8240,'Unan1mous (UK)',NULL,now(),NULL,80292),
(7092,19558,'Glutton for Punishment',NULL,now(),NULL,80295),
(7093,10091,'Creature Comforts (US)',NULL,now(),NULL,80298),
(7094,19276,'Edel & Starck',NULL,now(),NULL,79784),
(7095,15022,'Rob and Amber: Against the Odds',NULL,now(),NULL,80299),
(7096,22894,'The Lair',NULL,now(),NULL,80300),
(7097,9030,'Return to Lonesome Dove',NULL,now(),NULL,80306),
(7098,15533,'Tyler Perry\'s House of Payne',NULL,now(),NULL,80311),
(7099,16482,'Don\'t Forget the Lyrics',NULL,now(),NULL,80314),
(7100,18965,'Higurashi no Naku Koro Ni Kai',NULL,now(),NULL,80322),
(7101,15311,'Frank TV',NULL,now(),NULL,80873),
(7102,23315,'Zombie Loan',NULL,now(),NULL,80330),
(7103,2162,'The Assistant',NULL,now(),NULL,80332),
(7104,20227,'Temptation Island (UK)',NULL,now(),NULL,80335),
(7105,2367,'Double The Fist',NULL,now(),NULL,80336),
(7106,7042,'Dallas SWAT',NULL,now(),NULL,80341),
(7107,5773,'The Farm',NULL,now(),NULL,80343),
(7108,16763,'Fonejacker',NULL,now(),NULL,80350),
(7109,16964,'Human Weapon',NULL,now(),NULL,80353),
(7110,16701,'Build it Bigger',NULL,now(),NULL,80355),
(7111,14819,'The Paul Daniels Magic Show',NULL,now(),NULL,80354),
(7112,17315,'The New Inventors',NULL,now(),NULL,80358),
(7113,16626,'The Two Coreys',NULL,now(),NULL,80360),
(7114,5796,'The Fugitive',NULL,now(),NULL,80361),
(7115,15593,'State of Mind',NULL,now(),NULL,80364),
(7116,3149,'Coupling',NULL,now(),NULL,78131),
(7117,15705,'The Singing Bee',NULL,now(),NULL,80365),
(7118,18243,'Grand Star',NULL,now(),NULL,80366),
(7119,16870,'The Visit',NULL,now(),NULL,80371),
(7120,15714,'Cavemen',NULL,now(),NULL,80378),
(7121,2874,'Boomtown',NULL,now(),NULL,78619),
(7122,16803,'Food Network Challenge',NULL,now(),NULL,80384),
(7123,15751,'Cane',NULL,now(),NULL,80385),
(7124,10119,'American Hot Rod',NULL,now(),NULL,80389),
(7125,6916,'All-Star Kitchen Makeover',NULL,now(),NULL,80390),
(7126,7829,'IGPX',NULL,now(),NULL,80391),
(7127,17010,'Heroes Unmasked',NULL,now(),NULL,80393),
(7128,14697,'Nynne',NULL,now(),NULL,80465),
(7129,2953,'Cadfael',NULL,now(),NULL,77894),
(7130,16443,'Back To The Grind',NULL,now(),NULL,80394),
(7131,16562,'The Bronx Is Burning',NULL,now(),NULL,80396),
(7132,15226,'Bizarre Foods',NULL,now(),NULL,80399),
(7133,20197,'Rebelde',NULL,now(),NULL,80401),
(7134,18080,'Full Frys',NULL,now(),NULL,80408),
(7135,16624,'Dr. Psycho',NULL,now(),NULL,79941),
(7136,1830,'The Armando Iannucci Shows',NULL,now(),NULL,80410),
(7137,10792,'Not Going Out',NULL,now(),NULL,80411),
(7138,7064,'The Ronnie Johns Half Hour',NULL,now(),NULL,80412),
(7139,16567,'I Hate My 30\'s',NULL,now(),NULL,80413),
(7140,8396,'Masters of Science Fiction',NULL,now(),NULL,80415),
(7141,12057,'American Body Shop',NULL,now(),NULL,80419),
(7142,15227,'Fort Alpha',NULL,now(),NULL,80420),
(7143,7049,'Messiah',NULL,now(),NULL,80425),
(7144,2819,'Black Sheep Squadron',NULL,now(),NULL,77912),
(7145,15764,'Back to You',NULL,now(),NULL,80426),
(7146,7496,'The Sky At Night',NULL,now(),NULL,80428),
(7147,6819,'Coast',NULL,now(),NULL,80429),
(7148,4757,'Over There',NULL,now(),NULL,72045),
(7149,16987,'The Rock Life',NULL,now(),NULL,80432),
(7150,15310,'The Bill Engvall Show',NULL,now(),NULL,80434),
(7151,4192,'Las Vegas',NULL,now(),NULL,72229),
(7152,15597,'Gavin & Stacey',NULL,now(),NULL,80435),
(7153,15420,'Roman\'s Empire',NULL,now(),NULL,80436),
(7154,17001,'Bounty Girls - Miami',NULL,now(),NULL,80437),
(7155,15503,'scarred',NULL,now(),NULL,80439),
(7156,1476,'Future Boy Conan',NULL,now(),NULL,80444),
(7157,5264,'South of Nowhere',NULL,now(),NULL,80446),
(7158,3394,'DuckTales',NULL,now(),NULL,75931),
(7159,6480,'Unhappily Ever After',NULL,now(),NULL,77604),
(7160,15860,'The Trap',NULL,now(),NULL,80447),
(7161,15497,'Kaze no Stigma',NULL,now(),NULL,80449),
(7162,15501,'Jon & Kate Plus 8',NULL,now(),NULL,80450),
(7163,15359,'Once Upon a Time... Man',NULL,now(),NULL,79849),
(7164,5272,'Space: Above and Beyond',NULL,now(),NULL,76509),
(7165,15605,'Sola',NULL,now(),NULL,80452),
(7166,3552,'First Wave',NULL,now(),NULL,74476),
(7167,20217,'Hem till MidgÃƒÂ¥rd',NULL,now(),NULL,80453),
(7168,17021,'Hotel CÃƒÂ¦sar',NULL,now(),NULL,80455),
(7169,1142,'Gravitation',NULL,now(),NULL,80456),
(7170,2708,'Band Of Brothers',NULL,now(),NULL,74205),
(7171,6012,'The Net',NULL,now(),NULL,78090),
(7172,16984,'The Pick-up Artist',NULL,now(),NULL,80458),
(7173,11150,'Tell Me You Love Me',NULL,now(),NULL,80462),
(7174,15711,'Carpoolers',NULL,now(),NULL,80463),
(7175,4562,'Murder Rooms',NULL,now(),NULL,80464),
(7176,13467,'High Chaparall',NULL,now(),NULL,80467),
(7177,16930,'L.A. Ink',NULL,now(),NULL,80469),
(7178,18505,'Shigurui',NULL,now(),NULL,80471),
(7179,15763,'Anchorwoman',NULL,now(),NULL,80474),
(7180,4194,'Late Night with Conan O\'Brien',NULL,now(),NULL,76737),
(7181,12583,'Trinity Blood',NULL,now(),NULL,78967),
(7182,11229,'The Knights of Prosperity',NULL,now(),NULL,79339),
(7183,3745,'Grosse Pointe',NULL,now(),NULL,80479),
(7184,17228,'City Homicide',NULL,now(),NULL,80480),
(7185,15745,'New Amsterdam',NULL,now(),NULL,80481),
(7186,23920,'Billy Connolly\'s World Tour of Australia',NULL,now(),NULL,80484),
(7187,20228,'Last Man Standing (BBC3)',NULL,now(),NULL,80485),
(7188,17022,'Slacker Cats',NULL,now(),NULL,80486),
(7189,16988,'Kitchen Criminals',NULL,now(),NULL,80487),
(7190,3371,'Dragon Ball',NULL,now(),NULL,76666),
(7191,3410,'E-Ring',NULL,now(),NULL,75532),
(7192,16414,'Buy Me',NULL,now(),NULL,80492),
(7193,7767,'Six Degrees',NULL,now(),NULL,79397),
(7194,4093,'Justice League Unlimited',NULL,now(),NULL,76320),
(7195,4128,'Kim Possible',NULL,now(),NULL,78259),
(7196,2344,'Gantz',NULL,now(),NULL,78916),
(7197,17189,'Kokey',NULL,now(),NULL,80499),
(7198,12696,'Ancient Rome The Rise and Fall of an Empire',NULL,now(),NULL,80505),
(7199,18500,'Nordkalotten 365',NULL,now(),NULL,81256),
(7200,17323,'Summer Heights High',NULL,now(),NULL,80506),
(7201,5516,'The A-Team',NULL,now(),NULL,77904),
(7202,17085,'The Restaurant UK',NULL,now(),NULL,80507),
(7203,16923,'Rick Stein\'s Mediterranean Escapes',NULL,now(),NULL,80508),
(7204,20968,'The Naked Archaeologist',NULL,now(),NULL,80509),
(7205,8391,'Grownups',NULL,now(),NULL,80511),
(7206,18332,'HjÃƒÂ¤lp!',NULL,now(),NULL,80161),
(7207,7436,'Who Do You Think You Are?',NULL,now(),NULL,80522),
(7208,7549,'Massive Nature',NULL,now(),NULL,80529),
(7209,17024,'Newport Harbor: The Real Orange County',NULL,now(),NULL,75028),
(7210,8226,'Happy Tree Friends',NULL,now(),NULL,80534),
(7211,15915,'Nashville',NULL,now(),NULL,80535),
(7212,6713,'Run\'s House',NULL,now(),NULL,81249),
(7213,17103,'Lobster Wars',NULL,now(),NULL,80537),
(7214,14686,'Svensson Svensson',NULL,now(),NULL,80454),
(7215,22307,'The Bourne Identity',NULL,now(),NULL,80540),
(7216,12971,'The Minor Accomplishments of Jackie Woodman',NULL,now(),NULL,80544),
(7217,14760,'Expedition borneo',NULL,now(),NULL,80545),
(7218,16593,'Sea Patrol',NULL,now(),NULL,80546),
(7219,17500,'Myggan',NULL,now(),NULL,80550),
(7220,15241,'Born Survivor: Bear Grylls',NULL,now(),NULL,80551),
(7221,15769,'Kitchen Nightmares',NULL,now(),NULL,80552),
(7222,17700,'Manswers',NULL,now(),NULL,80556),
(7223,10682,'Now Get Out Of That',NULL,now(),NULL,80557),
(7224,10798,'After You\'ve Gone',NULL,now(),NULL,80558),
(7225,10115,'TV Heaven, Telly Hell',NULL,now(),NULL,80559),
(7226,8870,'Duty Free',NULL,now(),NULL,80560),
(7227,8608,'Terry & June',NULL,now(),NULL,80561),
(7228,9972,'The Adventure Game',NULL,now(),NULL,80562),
(7229,14148,'Emperor of the Sea',NULL,now(),NULL,80569),
(7230,21857,'A Cook On The Wild Side',NULL,now(),NULL,80574),
(7231,12771,'Escape to River Cottage',NULL,now(),NULL,80575),
(7232,10911,'Man to Man With Dean Learner',NULL,now(),NULL,80576),
(7233,18952,'My New Best Friend',NULL,now(),NULL,80577),
(7234,9855,'The Smell Of Reeves And Mortimer',NULL,now(),NULL,80579),
(7235,15792,'Kid Nation',NULL,now(),NULL,80580),
(7236,12739,'Lilies',NULL,now(),NULL,80582),
(7237,3175,'Crossing Jordan',NULL,now(),NULL,74922),
(7238,15761,'Online Nation',NULL,now(),NULL,80584),
(7239,4050,'Jeremy Clarkson\'s Motorworld',NULL,now(),NULL,80586),
(7240,5088,'Saiyuki',NULL,now(),NULL,80587),
(7241,17215,'Outnumbered',NULL,now(),NULL,80592),
(7242,15001,'Really Big Things',NULL,now(),NULL,80597),
(7243,17200,'Life of Ryan',NULL,now(),NULL,80598),
(7244,5298,'Spider-Man and His Amazing Friends',NULL,now(),NULL,71880),
(7245,18271,'Superbook',NULL,now(),NULL,80604),
(7246,1726,'Aaagh! It\'s the Mr. Hell Show!',NULL,now(),NULL,80832),
(7247,10629,'Kyle XY',NULL,now(),NULL,76143),
(7248,7990,'Friday Night Lights',NULL,now(),NULL,79337),
(7249,4964,'ReGenesis',NULL,now(),NULL,74948),
(7250,11184,'Green Green',NULL,now(),NULL,80608),
(7251,13095,'Ross Kemp On Gangs',NULL,now(),NULL,80609),
(7252,17511,'Boston Tea Party',NULL,now(),NULL,80585),
(7253,16618,'Rock of Love with Bret Michaels',NULL,now(),NULL,80610),
(7254,8051,'Angel Heart',NULL,now(),NULL,80596),
(7255,17397,'The Whistleblowers',NULL,now(),NULL,80615),
(7256,4686,'Northern Exposure',NULL,now(),NULL,71383),
(7257,716,'Jam',NULL,now(),NULL,80617),
(7258,12580,'Most Evil',NULL,now(),NULL,80743),
(7259,11926,'Bush Mechanics',NULL,now(),NULL,80734),
(7260,3541,'Fighting Foodons',NULL,now(),NULL,80623),
(7261,14812,'Beer Nutz',NULL,now(),NULL,80627),
(7262,14813,'Three Sheets',NULL,now(),NULL,80628),
(7263,7106,'Balderdash & Piffle',NULL,now(),NULL,80630),
(7264,6002,'The Mysterious Cities of Gold',NULL,now(),NULL,78783),
(7265,17884,'Kjetil og Kjartan Show',NULL,now(),NULL,80633),
(7266,11163,'Supergroup',NULL,now(),NULL,80634),
(7267,19588,'Jurassic Fight Club',NULL,now(),NULL,82642),
(7268,8249,'Vanished',NULL,now(),NULL,79332),
(7269,21033,'Race to Mars',NULL,now(),NULL,80638),
(7270,7125,'Hyperdrive',NULL,now(),NULL,79182),
(7271,16457,'Tycoon',NULL,now(),NULL,80639),
(7272,6604,'Wife Swap (UK)',NULL,now(),NULL,80640),
(7273,12157,'Solitary',NULL,now(),NULL,80641),
(7274,2083,'The Blue Planet',NULL,now(),NULL,74372),
(7275,8258,'Frontline (PBS)',NULL,now(),NULL,80646),
(7276,14692,'Strutter',NULL,now(),NULL,80651),
(7277,3613,'Friday Night with Jonathan Ross',NULL,now(),NULL,80652),
(7278,14661,'American Gangster',NULL,now(),NULL,80655),
(7279,17353,'Commando On the Front Line',NULL,now(),NULL,80657),
(7280,8288,'Mahoujin Guru Guru',NULL,now(),NULL,80659),
(7281,3299,'Dharma & Greg',NULL,now(),NULL,72406),
(7282,15181,'The Graham Norton Show',NULL,now(),NULL,80660),
(7283,4835,'Planet of the Apes',NULL,now(),NULL,73565),
(7284,17531,'I Want To Look Like A High School Cheerleader Again',NULL,now(),NULL,80662),
(7285,17520,'Last One Standing',NULL,now(),NULL,80664),
(7286,17538,'The Next Iron Chef',NULL,now(),NULL,80665),
(7287,18242,'Tracey Ullman\'s State of the Union',NULL,now(),NULL,81646),
(7288,6860,'Sugar Rush',NULL,now(),NULL,78889),
(7289,17026,'About a Girl',NULL,now(),NULL,80670),
(7290,15762,'Life Is Wild',NULL,now(),NULL,80671),
(7291,15505,'Ladette to Lady',NULL,now(),NULL,81836),
(7292,17838,'Night Wizard',NULL,now(),NULL,80676),
(7293,17846,'Rental Magica',NULL,now(),NULL,80677),
(7294,16184,'Labyrinth',NULL,now(),NULL,80678),
(7295,5418,'Survivor',NULL,now(),NULL,76733),
(7296,6834,'The Dark Side of Modern Love',NULL,now(),NULL,80682),
(7297,17540,'Americas Most Smartest Model',NULL,now(),NULL,80687),
(7298,16496,'Dr. Steve-O',NULL,now(),NULL,80689),
(7299,12199,'Meerkat Manor',NULL,now(),NULL,80692),
(7300,15758,'Viva Laughlin',NULL,now(),NULL,80694),
(7301,18435,'Stupid Stupid Man',NULL,now(),NULL,80695),
(7302,17629,'Newstopia',NULL,now(),NULL,80696),
(7303,9626,'The Mark Thomas Comedy Product',NULL,now(),NULL,80697),
(7304,17767,'Le Bureau',NULL,now(),NULL,80698),
(7305,19398,'Flikken Maastricht',NULL,now(),NULL,80701),
(7306,22553,'AidA',NULL,now(),NULL,80704),
(7307,15592,'Side Order of Life',NULL,now(),NULL,80705),
(7308,15016,'Wired Science',NULL,now(),NULL,80708),
(7309,18142,'NÃƒÂ¦turvaktin',NULL,now(),NULL,80711),
(7310,1567,'Great Hotels',NULL,now(),NULL,80712),
(7311,17340,'Schmidt & Pocher',NULL,now(),NULL,80716),
(7312,7215,'Doctor Snuggles',NULL,now(),NULL,80719),
(7313,20688,'Pulp Sport',NULL,now(),NULL,80720),
(7314,3587,'Foster\'s Home for Imaginary Friends',NULL,now(),NULL,79323),
(7315,15176,'Wizards of Waverly Place',NULL,now(),NULL,80726),
(7316,3548,'Firefly',NULL,now(),NULL,78874),
(7317,16137,'Britain\'s Got Talent',NULL,now(),NULL,80727),
(7318,6724,'Breaking Bonaduce',NULL,now(),NULL,80724),
(7319,17638,'Keeping Up with the Kardashians',NULL,now(),NULL,80725),
(7320,15351,'The Whitest Kids U Know',NULL,now(),NULL,80728),
(7321,17438,'Long Way Down',NULL,now(),NULL,80731),
(7322,18052,'Ghost Hound',NULL,now(),NULL,80733),
(7323,17863,'The Librarians',NULL,now(),NULL,80738),
(7324,13063,'If...',NULL,now(),NULL,80739),
(7325,2361,'Distant Shores',NULL,now(),NULL,80740),
(7326,6911,'The Gadget Show',NULL,now(),NULL,80747),
(7327,10101,'Panorama',NULL,now(),NULL,80748),
(7328,13419,'Rogue Traders',NULL,now(),NULL,80749),
(7329,9303,'The Mark Steel Lectures',NULL,now(),NULL,80750),
(7330,3255,'Dawson\'s Creek',NULL,now(),NULL,72584),
(7331,18311,'Hipp Hipp',NULL,now(),NULL,80758),
(7332,17521,'The Peter Serafinowicz Show',NULL,now(),NULL,80760),
(7333,7963,'Watchdog',NULL,now(),NULL,80762),
(7334,8531,'Waterloo Road',NULL,now(),NULL,80763),
(7335,17868,'Hej Rymden!',NULL,now(),NULL,80764),
(7336,8329,'The Lakes',NULL,now(),NULL,80776),
(7337,883,'Brain PowerD',NULL,now(),NULL,71750),
(7338,1022,'Comic Party',NULL,now(),NULL,80787),
(7339,1207,'Patlabor',NULL,now(),NULL,80789),
(7340,1028,'Steel Angel Kurumi',NULL,now(),NULL,80790),
(7341,12986,'100%',NULL,now(),NULL,80794),
(7342,12278,'100 HÃƒÂ¶jdare',NULL,now(),NULL,80026),
(7343,15674,'Extra Extra!',NULL,now(),NULL,80798),
(7344,14442,'FrÃƒÂ¥ga Anders och MÃƒÂ¥ns',NULL,now(),NULL,80799),
(7345,18073,'Labyrint',NULL,now(),NULL,80801),
(7346,16693,'Balls of Steel',NULL,now(),NULL,80804),
(7347,18067,'Kodomo no Jikan',NULL,now(),NULL,80811),
(7348,15931,'009-1',NULL,now(),NULL,80814),
(7349,14683,'The Fairies',NULL,now(),NULL,80816),
(7350,12666,'Pani Poni Dash',NULL,now(),NULL,80818),
(7351,8387,'Zoids Genesis',NULL,now(),NULL,80820),
(7352,14978,'The Land Before Time',NULL,now(),NULL,80822),
(7353,12873,'Tactical Roar',NULL,now(),NULL,80823),
(7354,15789,'The Next Great American Band',NULL,now(),NULL,80827),
(7355,17807,'Monster Quest',NULL,now(),NULL,80828),
(7356,19736,'howard stern on demand',NULL,now(),NULL,80834),
(7357,18078,'Echo Beach',NULL,now(),NULL,81200),
(7358,6799,'Sensitive Skin',NULL,now(),NULL,80838),
(7359,15100,'Party Animals',NULL,now(),NULL,80843),
(7360,17246,'Storm Hawks',NULL,now(),NULL,80845),
(7361,17855,'Sold',NULL,now(),NULL,80847),
(7362,8437,'Attention Scum',NULL,now(),NULL,80848),
(7363,14748,'Naruto Shippuuden',NULL,now(),NULL,79824),
(7364,12693,'My Friends Tigger and Pooh',NULL,now(),NULL,80850),
(7365,17572,'Project Runway Canada',NULL,now(),NULL,80851),
(7366,23811,'En Ãƒâ€žngels TÃƒÂ¥lamod',NULL,now(),NULL,80858),
(7367,23635,'Rederiet',NULL,now(),NULL,80859),
(7368,17316,'Tim Gunn\'s Guide to Style',NULL,now(),NULL,80860),
(7369,15064,'Ray Mears Wild Food',NULL,now(),NULL,80862),
(7370,17915,'Britz',NULL,now(),NULL,80863),
(7371,19114,'Monkey Life',NULL,now(),NULL,80864),
(7372,18323,'America\'s Ballroom Challenge',NULL,now(),NULL,80866),
(7373,4674,'Nikki',NULL,now(),NULL,80874),
(7374,17943,'Rise of the Video Game',NULL,now(),NULL,80877),
(7375,11114,'Dragon Hunters',NULL,now(),NULL,79666),
(7376,4247,'Lilo & Stitch: The Series',NULL,now(),NULL,71624),
(7377,6376,'Tom and Jerry',NULL,now(),NULL,72860),
(7378,9709,'TusenbrÃƒÂ¶der',NULL,now(),NULL,80887),
(7379,18826,'Ganges',NULL,now(),NULL,80892),
(7380,10799,'The Omid Djalili Show',NULL,now(),NULL,80893),
(7381,13028,'Five Days',NULL,now(),NULL,80895),
(7382,17293,'Nigella Express',NULL,now(),NULL,80899),
(7383,18042,'Ef - A Tale of Memories',NULL,now(),NULL,80904),
(7384,17842,'Live at the Apollo',NULL,now(),NULL,80905),
(7385,15728,'Miss Guided',NULL,now(),NULL,81550),
(7386,1507,'Saint Seiya',NULL,now(),NULL,72775),
(7387,17253,'Xavier: Renegade Angel',NULL,now(),NULL,80923),
(7388,11613,'Cory in the House',NULL,now(),NULL,80925),
(7389,14553,'Twin of Brothers',NULL,now(),NULL,80931),
(7390,1795,'The Road Runner Show',NULL,now(),NULL,77600),
(7391,5923,'The Legend of Tarzan',NULL,now(),NULL,76600),
(7392,12042,'The Real Football Factories',NULL,now(),NULL,80940),
(7393,15187,'Bionic Woman',NULL,now(),NULL,80370),
(7394,15562,'Bill Moyers Journal',NULL,now(),NULL,80942),
(7395,11043,'Final Fantasy: Unlimited',NULL,now(),NULL,78913),
(7396,17991,'Snoop Dogg\'s Father Hood',NULL,now(),NULL,80943),
(7397,20266,'Ride with the Devil',NULL,now(),NULL,80947),
(7398,17008,'Manshow',NULL,now(),NULL,80951),
(7399,16823,'Rikets RÃƒÂ¸st',NULL,now(),NULL,80952),
(7400,16694,'The Julekalender',NULL,now(),NULL,80954),
(7401,11185,'Tsunami: The Aftermath',NULL,now(),NULL,80963),
(7402,17329,'Yo Gabba Gabba',NULL,now(),NULL,80964),
(7403,15099,'Bonkers ',NULL,now(),NULL,80966),
(7404,17683,'Fanny Hill',NULL,now(),NULL,80967),
(7405,15567,'Holby Blue',NULL,now(),NULL,80969),
(7406,17872,'Arrange me a Marriage',NULL,now(),NULL,80970),
(7407,2693,'Babylon 5',NULL,now(),NULL,70726),
(7408,17492,'The Life and Times of Vivienne Vyle',NULL,now(),NULL,80976),
(7409,8114,'Kidnapped',NULL,now(),NULL,79331),
(7410,12144,'Kenran Butoh Sai: The Mars Daybreak',NULL,now(),NULL,80978),
(7411,12837,'The Story of Saiunkoku',NULL,now(),NULL,80979),
(7412,6733,'I Shouldn\'t Be Alive',NULL,now(),NULL,80983),
(7413,6815,'Natural World',NULL,now(),NULL,80987),
(7414,12721,'Stephen Fry: The Secret Life of the Manic Depressive',NULL,now(),NULL,80988),
(7415,17516,'The Nature of Britain',NULL,now(),NULL,81000),
(7416,8151,'Hungry Heart Wild Striker',NULL,now(),NULL,81003),
(7417,3604,'Freddie',NULL,now(),NULL,79276),
(7418,13519,'Rent-A-Goalie',NULL,now(),NULL,81006),
(7419,12309,'Ichigo Mashimaro',NULL,now(),NULL,81007),
(7420,5155,'Sex and the City',NULL,now(),NULL,76648),
(7421,18046,'Clash of the Choirs',NULL,now(),NULL,81010),
(7422,656,'Flame of Recca',NULL,now(),NULL,81011),
(7423,17604,'A Shot at Love with Tila Tequila',NULL,now(),NULL,81012),
(7424,9569,'The Amazing Mrs Pritchard',NULL,now(),NULL,81014),
(7425,10292,'The Street',NULL,now(),NULL,81015),
(7426,13647,'The Time of Your Life',NULL,now(),NULL,81016),
(7427,15124,'True Dare Kiss',NULL,now(),NULL,81017),
(7428,17348,'Torn',NULL,now(),NULL,81018),
(7429,17982,'Oliver Twist',NULL,now(),NULL,81020),
(7430,17694,'Half Broken Things',NULL,now(),NULL,81023),
(7431,1411,'Gravion',NULL,now(),NULL,81024),
(7432,5323,'Stargate: Infinity',NULL,now(),NULL,70852),
(7433,4599,'Mysterious Ways',NULL,now(),NULL,78786),
(7434,6936,'The Black Donnellys',NULL,now(),NULL,79647),
(7435,6407,'Traffic',NULL,now(),NULL,73509),
(7436,16920,'The Cosmos A Beginner\'s Guide',NULL,now(),NULL,81034),
(7437,16374,'Transformers Animated',NULL,now(),NULL,81036),
(7438,15607,'Dennou Coil',NULL,now(),NULL,81037),
(7439,19444,'Try Before You Die',NULL,now(),NULL,81038),
(7440,15749,'Babylon Fields',NULL,now(),NULL,81039),
(7441,2764,'Berserk',NULL,now(),NULL,73752),
(7442,18096,'Smash Lab',NULL,now(),NULL,81047),
(7443,3185,'CSI: NY',NULL,now(),NULL,73696),
(7444,8022,'Tactics',NULL,now(),NULL,81052),
(7445,2522,'Airline (2004)',NULL,now(),NULL,81059),
(7446,16612,'Live Earth',NULL,now(),NULL,81064),
(7447,12294,'The War of the World',NULL,now(),NULL,81065),
(7448,3608,'Free for All',NULL,now(),NULL,79597),
(7449,9072,'Alexei Sayle\'s Merry Go Round',NULL,now(),NULL,81070),
(7450,13312,'Star Stories',NULL,now(),NULL,81071),
(7451,6746,'Dubai Dreams',NULL,now(),NULL,81075),
(7452,18453,'Sleek Geeks',NULL,now(),NULL,81082),
(7453,5876,'The Invaders',NULL,now(),NULL,73535),
(7454,18047,'American Gladiators (2008)',NULL,now(),NULL,81088),
(7455,18099,'Tiger Team',NULL,now(),NULL,81089),
(7456,6738,'The Alice',NULL,now(),NULL,82011),
(7457,15483,'El Cazador de la Bruja',NULL,now(),NULL,81096),
(7458,17922,'Shrek The Halls',NULL,now(),NULL,81101),
(7459,15911,'Ookiku Furikabutte',NULL,now(),NULL,81108),
(7460,8262,'Daphne in the Brilliant Blue',NULL,now(),NULL,81113),
(7461,18157,'The Border',NULL,now(),NULL,81114),
(7462,12436,'Magical Girl Lyrical Nanoha',NULL,now(),NULL,81115),
(7463,17809,'Ghost Hunters International',NULL,now(),NULL,81120),
(7464,18143,'jPod',NULL,now(),NULL,81122),
(7465,18163,'Sophie',NULL,now(),NULL,81123),
(7466,1133,'RahXephon',NULL,now(),NULL,72720),
(7467,16522,'The States',NULL,now(),NULL,81125),
(7468,12778,'Little Miss Jocelyn',NULL,now(),NULL,81127),
(7469,17772,'Fairy Tales',NULL,now(),NULL,81132),
(7470,22259,'The Blue and the Gray',NULL,now(),NULL,81133);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(7471,2287,'School Rumble',NULL,now(),NULL,79194),
(7472,15265,'100 Greatest Discoveries',NULL,now(),NULL,81134),
(7473,5092,'Samurai 7',NULL,now(),NULL,79127),
(7474,8398,'Jimmy\'s Farm',NULL,now(),NULL,81135),
(7475,17865,'Monarchy - The Royal Family at Work',NULL,now(),NULL,81136),
(7476,10114,'Derren Brown: Trick of the Mind',NULL,now(),NULL,74365),
(7477,17795,'Visions of the Future',NULL,now(),NULL,81142),
(7478,18170,'True Tears',NULL,now(),NULL,81144),
(7479,18088,'Honest',NULL,now(),NULL,81146),
(7480,2277,'Battle Programmer Shirase',NULL,now(),NULL,81147),
(7481,7975,'Wild at Heart',NULL,now(),NULL,81390),
(7482,2594,'American Dad!',NULL,now(),NULL,73141),
(7483,17682,'Capturing Mary',NULL,now(),NULL,81310),
(7484,3266,'Dead Ringers',NULL,now(),NULL,81152),
(7485,6773,'Nature',NULL,now(),NULL,81157),
(7486,18087,'Moving Wallpaper',NULL,now(),NULL,81160),
(7487,18019,'Never Better',NULL,now(),NULL,81162),
(7488,9826,'Set Of Six',NULL,now(),NULL,81163),
(7489,17939,'Super WHY!',NULL,now(),NULL,81164),
(7490,10918,'Comanche Moon',NULL,now(),NULL,81165),
(7491,2722,'Batman Beyond',NULL,now(),NULL,75417),
(7492,21190,'G String Divas',NULL,now(),NULL,81167),
(7493,3430,'Elfen Lied',NULL,now(),NULL,75941),
(7494,6633,'Wolf\'s Rain',NULL,now(),NULL,79060),
(7495,8052,'Walking the Bible',NULL,now(),NULL,81171),
(7496,18379,'George of the Jungle',NULL,now(),NULL,81177),
(7497,18268,'Spice and Wolf',NULL,now(),NULL,81178),
(7498,18210,'City of Vice',NULL,now(),NULL,81180),
(7499,17896,'Fearless Planet',NULL,now(),NULL,81181),
(7500,1145,'Forever Summer With Nigella',NULL,now(),NULL,81192),
(7501,17715,'Russell Brand\'s Ponderland',NULL,now(),NULL,81193),
(7502,15450,'Hello! My Lady',NULL,now(),NULL,81194),
(7503,15469,'Witch Yoo Hee',NULL,now(),NULL,81198),
(7504,14226,'Love Story in Harvard',NULL,now(),NULL,81199),
(7505,23958,'An Aussie Goes Bolly',NULL,now(),NULL,81201),
(7506,18103,'Rabbit Fall',NULL,now(),NULL,81202),
(7507,9746,'Across The River to Motor City',NULL,now(),NULL,81203),
(7508,18525,'The Real Housewives of New York City',NULL,now(),NULL,84669),
(7509,608,'Gladiators',NULL,now(),NULL,81208),
(7510,709,'Asylum',NULL,now(),NULL,81211),
(7511,5172,'Sheep In The Big City',NULL,now(),NULL,81212),
(7512,17845,'Gangland',NULL,now(),NULL,81213),
(7513,14959,'Al Murray\'s Happy Hour',NULL,now(),NULL,81214),
(7514,959,'Chobits',NULL,now(),NULL,72070),
(7515,12801,'Solty Rei',NULL,now(),NULL,79282),
(7516,642,'John Safran\'s Music Jamboree',NULL,now(),NULL,81220),
(7517,6990,'The Flumps',NULL,now(),NULL,81222),
(7518,15025,'The Palace',NULL,now(),NULL,81225),
(7519,17959,'The Moment of Truth',NULL,now(),NULL,81226),
(7520,12891,'Tide Line Blue',NULL,now(),NULL,81228),
(7521,8438,'Brass',NULL,now(),NULL,81229),
(7522,1035,'Harry Hill\'s TV Burp',NULL,now(),NULL,72404),
(7523,18166,'Celebrity Rehab with Dr.Drew',NULL,now(),NULL,81230),
(7524,18267,'Gunslinger Girl: Il Teatrino',NULL,now(),NULL,81233),
(7525,23718,'Triumph of the Nerds: The Rise of Accidental Empires',NULL,now(),NULL,81237),
(7526,17776,'Don\'t Tell the Bride',NULL,now(),NULL,81243),
(7527,5576,'The Angry Beavers',NULL,now(),NULL,76386),
(7528,5128,'seaQuest DSV',NULL,now(),NULL,76022),
(7529,3841,'Hex',NULL,now(),NULL,75125),
(7530,20010,'WordGirl',NULL,now(),NULL,83660),
(7531,8530,'Pros vs Joes',NULL,now(),NULL,81252),
(7532,1794,'Adrian Mole: The Cappuccino Years',NULL,now(),NULL,81254),
(7533,5631,'The Bold and the Beautiful',NULL,now(),NULL,79838),
(7534,19801,'Switch Reloaded',NULL,now(),NULL,81051),
(7535,17489,'Ray Mears\' Extreme Survival',NULL,now(),NULL,81261),
(7536,4174,'Laguna Beach',NULL,now(),NULL,81271),
(7537,18344,'Relocation Relocation',NULL,now(),NULL,81272),
(7538,6340,'Thomas The Tank Engine & Friends',NULL,now(),NULL,78949),
(7539,6908,'A Cook\'s Tour',NULL,now(),NULL,81276),
(7540,6759,'The Comedians of Comedy',NULL,now(),NULL,81277),
(7541,16914,'Dinner: Impossible',NULL,now(),NULL,81278),
(7542,17916,'London Ink',NULL,now(),NULL,81279),
(7543,16024,'Dragon Zakura',NULL,now(),NULL,81282),
(7544,16628,'Welcome to the Captain',NULL,now(),NULL,81289),
(7545,22344,'Derren Brown: The System',NULL,now(),NULL,81290),
(7546,17681,'Joe\'s Palace',NULL,now(),NULL,81311),
(7547,22427,'Word World',NULL,now(),NULL,81312),
(7548,5981,'The Mighty Boosh',NULL,now(),NULL,74302),
(7549,4355,'Malcolm in the Middle',NULL,now(),NULL,73838),
(7550,1943,'Miss Marple',NULL,now(),NULL,81401),
(7551,17692,'The History Of The World Backwards',NULL,now(),NULL,81315),
(7552,17880,'Angelo\'s',NULL,now(),NULL,81316),
(7553,11447,'A River Somewhere',NULL,now(),NULL,81317),
(7554,3095,'Coach',NULL,now(),NULL,76905),
(7555,10884,'Wheeler Dealers',NULL,now(),NULL,81320),
(7556,13610,'Voyages of Discovery',NULL,now(),NULL,84104),
(7557,18165,'Make Me A Supermodel',NULL,now(),NULL,81333),
(7558,4943,'Ramsay\'s Kitchen Nightmares',NULL,now(),NULL,74584),
(7559,12981,'A Woman\'s Guide to Adultery',NULL,now(),NULL,81336),
(7560,17212,'Mr Men',NULL,now(),NULL,81339),
(7561,1104,'Kimba the White Lion',NULL,now(),NULL,81341),
(7562,18064,'Parking Wars',NULL,now(),NULL,81343),
(7563,6949,'Ikkitousen',NULL,now(),NULL,80158),
(7564,13548,'Nigella\'s Christmas Kitchen',NULL,now(),NULL,81347),
(7565,18487,'SketchNews',NULL,now(),NULL,81284),
(7566,5292,'Spicks and Specks',NULL,now(),NULL,81358),
(7567,18109,'Fight Quest',NULL,now(),NULL,81359),
(7568,10316,'Gear Fighter Dendoh',NULL,now(),NULL,81437),
(7569,18374,'Farmen',NULL,now(),NULL,81364),
(7570,17027,'Great British Journeys',NULL,now(),NULL,81370),
(7571,19204,'Wie is de mol?',NULL,now(),NULL,81391),
(7572,4677,'Nip Tuck',NULL,now(),NULL,72201),
(7573,4255,'Little Britain',NULL,now(),NULL,72135),
(7574,2506,'Aeon Flux',NULL,now(),NULL,78660),
(7575,13950,'The Genius of Photography',NULL,now(),NULL,81373),
(7576,18442,'Freezing',NULL,now(),NULL,81376),
(7577,14448,'Virtues Of Harmony II',NULL,now(),NULL,81377),
(7578,23019,'Baseball',NULL,now(),NULL,81381),
(7579,2392,'Planetes',NULL,now(),NULL,75796),
(7580,15346,'The Last Enemy',NULL,now(),NULL,81383),
(7581,8474,'\'Til Death',NULL,now(),NULL,79384),
(7582,19073,'Flodder',NULL,now(),NULL,81385),
(7583,3451,'ER',NULL,now(),NULL,70761),
(7584,18434,'Being Human',NULL,now(),NULL,81386),
(7585,8026,'What the Romans did for us',NULL,now(),NULL,81393),
(7586,8027,'What the Tudors and Stuarts did for us',NULL,now(),NULL,81395),
(7587,8028,'What the Victorians did for us',NULL,now(),NULL,81396),
(7588,18412,'Dawn...',NULL,now(),NULL,81402),
(7589,11054,'Graven',NULL,now(),NULL,81405),
(7590,9343,'Gas',NULL,now(),NULL,81407),
(7591,18003,'Paranormal State',NULL,now(),NULL,81408),
(7592,15537,'Victoria\'s Empire',NULL,now(),NULL,81409),
(7593,21751,'The Charlie Rose Show',NULL,now(),NULL,81410),
(7594,10246,'10 Days That Unexpectedly Changed America',NULL,now(),NULL,81412),
(7595,18286,'Head Case',NULL,now(),NULL,81414),
(7596,7469,'Child of Our Time',NULL,now(),NULL,81416),
(7597,17812,'The Art Of Spain',NULL,now(),NULL,81418),
(7598,23852,'How London Was Built',NULL,now(),NULL,81421),
(7599,17897,'Quarterlife',NULL,now(),NULL,81424),
(7600,20129,'Shugo Chara!',NULL,now(),NULL,81427),
(7601,12081,'Girl\'s High',NULL,now(),NULL,81426),
(7602,2404,'Top Buzzer',NULL,now(),NULL,81429),
(7603,4895,'Prison Break',NULL,now(),NULL,75340),
(7604,15326,'Mullvaden',NULL,now(),NULL,81434),
(7605,12507,'Attack No.1',NULL,now(),NULL,82380),
(7606,10578,'Great Teacher Onizuka',NULL,now(),NULL,80100),
(7607,12806,'Modern Toss',NULL,now(),NULL,81439),
(7608,13391,'The owl service',NULL,now(),NULL,81442),
(7609,17719,'Good Game',NULL,now(),NULL,81443),
(7610,10804,'Roman Mysteries',NULL,now(),NULL,81444),
(7611,14504,'Riches and Stitches',NULL,now(),NULL,81448),
(7612,15787,'Unhitched',NULL,now(),NULL,81452),
(7613,17029,'School Days',NULL,now(),NULL,80478),
(7614,14817,'The Truth About Food',NULL,now(),NULL,81457),
(7615,3637,'Galactica 1980',NULL,now(),NULL,71170),
(7616,14997,'Wild Caribbean',NULL,now(),NULL,81471),
(7617,3908,'House',NULL,now(),NULL,73255),
(7618,23353,'Inside MMA',NULL,now(),NULL,81473),
(7619,10920,'UFO Princess Valkyrie',NULL,now(),NULL,81477),
(7620,6966,'Thief',NULL,now(),NULL,74334),
(7621,17723,'Myself Yourself',NULL,now(),NULL,81484),
(7622,21660,'Crazy Rulers of the World',NULL,now(),NULL,81486),
(7623,18532,'Crime 360',NULL,now(),NULL,81487),
(7624,18186,'Persona - Trinity Soul',NULL,now(),NULL,81490),
(7625,23743,'Big Brother After Dark',NULL,now(),NULL,81491),
(7626,15237,'The Spectacular Spider-Man',NULL,now(),NULL,81494),
(7627,13686,'Liberty! The American Revolution',NULL,now(),NULL,81495),
(7628,18416,'El SÃƒÂ­ndrome de Ulises',NULL,now(),NULL,81497),
(7629,17828,'Chowder',NULL,now(),NULL,81500),
(7630,18750,'Tougher in Alaska',NULL,now(),NULL,101061),
(7631,14367,'Anne of Green Gables',NULL,now(),NULL,81954),
(7632,5214,'Six Feet Under',NULL,now(),NULL,75450),
(7633,18517,'Here come the Newlyweds',NULL,now(),NULL,81501),
(7634,6069,'The Outer Limits (1995)',NULL,now(),NULL,72224),
(7635,15767,'Canterbury\'s Law',NULL,now(),NULL,81502),
(7636,2075,'Shorties Watchin\' Shorties',NULL,now(),NULL,81503),
(7637,19532,'Peking Express',NULL,now(),NULL,81505),
(7638,19234,'Alles Atze',NULL,now(),NULL,80706),
(7639,18277,'C/o Segemyhr',NULL,now(),NULL,80791),
(7640,1170,'FortySomething',NULL,now(),NULL,80898),
(7641,22125,'Ein Herz und eine Seele',NULL,now(),NULL,81303),
(7642,5898,'The Joe Schmo Show',NULL,now(),NULL,81512),
(7643,4693,'Nowhere Man',NULL,now(),NULL,72933),
(7644,18498,'Autofil',NULL,now(),NULL,80950),
(7645,20122,'Kuroshitsuji',NULL,now(),NULL,83996),
(7646,18148,'David Nolande',NULL,now(),NULL,80958),
(7647,17007,'Post Mortem',NULL,now(),NULL,81379),
(7648,18404,'Lewis Black\'s Root of All Evil',NULL,now(),NULL,81516),
(7649,5410,'Supernatural',NULL,now(),NULL,78901),
(7650,18472,'The Story Of India',NULL,now(),NULL,81523),
(7651,4755,'Overhaulin\'',NULL,now(),NULL,74618),
(7652,15775,'The Return of Jezebel James',NULL,now(),NULL,81525),
(7653,10436,'Robin Hood',NULL,now(),NULL,79479),
(7654,8440,'Floris',NULL,now(),NULL,80318),
(7655,19290,'Nova ScienceNOW',NULL,now(),NULL,81526),
(7656,18497,'The Fixer',NULL,now(),NULL,81499),
(7657,12589,'The Herbs',NULL,now(),NULL,81528),
(7658,7840,'La Petite Vie',NULL,now(),NULL,80516),
(7659,7832,'Les Bougon',NULL,now(),NULL,81531),
(7660,21036,'Blueprint for Disaster',NULL,now(),NULL,81533),
(7661,6348,'Threshold',NULL,now(),NULL,75600),
(7662,2532,'ALF',NULL,now(),NULL,78020),
(7663,16356,'Mad Men',NULL,now(),NULL,80337),
(7664,4061,'Joey',NULL,now(),NULL,72920),
(7665,12930,'H2O: Just Add Water',NULL,now(),NULL,81061),
(7666,13372,'FETCH! with Ruff Ruffman',NULL,now(),NULL,81552),
(7667,18483,'Funny Movie',NULL,now(),NULL,81553),
(7668,18362,'Free Radio',NULL,now(),NULL,81558),
(7669,10272,'Great British Menu',NULL,now(),NULL,81561),
(7670,16771,'Border Security',NULL,now(),NULL,81563),
(7671,1579,'Something in the Air',NULL,now(),NULL,81573),
(7672,8574,'George of the Jungle (1967)',NULL,now(),NULL,81574),
(7673,17490,'MEGA DISASTERS',NULL,now(),NULL,81575),
(7674,18987,'World of Quest',NULL,now(),NULL,81576),
(7675,18481,'Ax Men',NULL,now(),NULL,81578),
(7676,15096,'The Snow Queen',NULL,now(),NULL,81579),
(7677,14762,'Come Dine With Me',NULL,now(),NULL,81580),
(7678,18581,'The Diets That Time Forgot',NULL,now(),NULL,81581),
(7679,23001,'Supersize Vs Superskinny',NULL,now(),NULL,81582),
(7680,21053,'Willie\'s Wonky Chocolate Factory',NULL,now(),NULL,81585),
(7681,14506,'Handy Manny',NULL,now(),NULL,81587),
(7682,17687,'Crime Investigation Australia',NULL,now(),NULL,81588),
(7683,15105,'Dangerous',NULL,now(),NULL,81589),
(7684,18008,'East West 101',NULL,now(),NULL,81590),
(7685,12936,'The Second Coming',NULL,now(),NULL,81595),
(7686,15582,'Cooking in the Danger Zone',NULL,now(),NULL,81596),
(7687,15245,'Fimbles',NULL,now(),NULL,81602),
(7688,23144,'Around the World in 80 Gardens',NULL,now(),NULL,81603),
(7689,7607,'The World Game',NULL,now(),NULL,81604),
(7690,13216,'Lipstick on Your Collar',NULL,now(),NULL,81605),
(7691,17969,'The No. 1 Ladies Detective Agency',NULL,now(),NULL,81606),
(7692,18636,'Daily Cooks Challenge',NULL,now(),NULL,81607),
(7693,18822,'So You Think You Can Dance Scandinavia',NULL,now(),NULL,81609),
(7694,5791,'The Forsyte Saga (2002)',NULL,now(),NULL,81610),
(7695,4475,'Millennium',NULL,now(),NULL,70940),
(7696,8081,'Jekyll',NULL,now(),NULL,80222),
(7697,22208,'Formula 1',NULL,now(),NULL,81084),
(7698,17464,'The War (Ken Burns)',NULL,now(),NULL,80620),
(7699,11169,'The Line of Beauty',NULL,now(),NULL,81615),
(7700,17843,'The 800 Million Pound Railway Station',NULL,now(),NULL,81616),
(7701,6638,'Wonderfalls',NULL,now(),NULL,78845),
(7702,6389,'Total Recall 2070',NULL,now(),NULL,76459),
(7703,2409,'Whistler',NULL,now(),NULL,79325),
(7704,7978,'Paul McKenna will Make you Thin',NULL,now(),NULL,81617),
(7705,6554,'Weeds',NULL,now(),NULL,74845),
(7706,20066,'Minami-Ke',NULL,now(),NULL,81463),
(7707,14895,'Marco Polo (1982)',NULL,now(),NULL,81625),
(7708,11153,'Twin Spica',NULL,now(),NULL,81628),
(7709,18451,'OdwrÃƒÂ³ceni',NULL,now(),NULL,81630),
(7710,10421,'Die Schatzinsel',NULL,now(),NULL,81629),
(7711,18480,'Recipe for Success UK',NULL,now(),NULL,81632),
(7712,18607,'He Kills Coppers',NULL,now(),NULL,81634),
(7713,6749,'Afterlife',NULL,now(),NULL,79021),
(7714,6177,'The Secret Life Of Us',NULL,now(),NULL,81636),
(7715,18645,'Teenage Kicks',NULL,now(),NULL,81638),
(7716,4279,'Loonatics Unleashed',NULL,now(),NULL,75815),
(7717,11451,'Quest',NULL,now(),NULL,81641),
(7718,20642,'R.I.S. Delitti Imperfetti',NULL,now(),NULL,81642),
(7719,10419,'The Company',NULL,now(),NULL,80424),
(7720,4923,'Queen of Swords',NULL,now(),NULL,75440),
(7721,22755,'The Good Wife',NULL,now(),NULL,95451),
(7722,18642,'Tiger - Spy in the Jungle',NULL,now(),NULL,81652),
(7723,6705,'Zorro',NULL,now(),NULL,74683),
(7724,22136,'The Shogun',NULL,now(),NULL,81655),
(7725,11165,'Wolverine and the X-Men',NULL,now(),NULL,82870),
(7726,18366,'Randy Jackson Presents: America\'s Best Dance Crew',NULL,now(),NULL,81659),
(7727,13059,'Real Stories',NULL,now(),NULL,81660),
(7728,5721,'The Dead Zone',NULL,now(),NULL,77955),
(7729,4162,'Kung Fu',NULL,now(),NULL,73069),
(7730,19315,'Sverige Pussas Och Kramas',NULL,now(),NULL,81665),
(7731,5782,'The Flashing Blade',NULL,now(),NULL,81666),
(7732,17970,'The Passion',NULL,now(),NULL,81667),
(7733,9744,'The Murdoch Mysteries',NULL,now(),NULL,81670),
(7734,17912,'Spy',NULL,now(),NULL,81672),
(7735,20796,'Roes',NULL,now(),NULL,81686),
(7736,17704,'Ross Kemp In Afghanistan',NULL,now(),NULL,81325),
(7737,8538,'R.I.S Police Scientifique',NULL,now(),NULL,81685),
(7738,4685,'Normal, Ohio',NULL,now(),NULL,81696),
(7739,19364,'Gillend Naar Huis',NULL,now(),NULL,81697),
(7740,18269,'Deadline',NULL,now(),NULL,81699),
(7741,14532,'Gentle Crack Down II',NULL,now(),NULL,81700),
(7742,2660,'As Told By Ginger',NULL,now(),NULL,81705),
(7743,8145,'Smith',NULL,now(),NULL,79346),
(7744,18824,'Ekipa',NULL,now(),NULL,81707),
(7745,4482,'MISSING',NULL,now(),NULL,72155),
(7746,17684,'Jackanory Junior',NULL,now(),NULL,81713),
(7747,18679,'The Wall',NULL,now(),NULL,81715),
(7748,18665,'Dan Cruickshank\'s Adventures in Architecture',NULL,now(),NULL,81716),
(7749,12825,'Tekkaman Blade',NULL,now(),NULL,81723),
(7750,7934,'Figure 17',NULL,now(),NULL,81725),
(7751,21196,'Firsthand',NULL,now(),NULL,81726),
(7752,21676,'Bokura ga Ita',NULL,now(),NULL,81730),
(7753,12892,'To Heart',NULL,now(),NULL,81766),
(7754,4442,'Meitantei Holmes',NULL,now(),NULL,73082),
(7755,12899,'Tonagura',NULL,now(),NULL,81735),
(7756,12952,'Property Ladder UK',NULL,now(),NULL,81738),
(7757,8230,'Cosmowarrior Zero',NULL,now(),NULL,81741),
(7758,11057,'Final Approach',NULL,now(),NULL,80083),
(7759,18567,'Special A',NULL,now(),NULL,81752),
(7760,10813,'In The Night Garden',NULL,now(),NULL,81749),
(7761,23916,'FÃƒÂ­sica o quÃƒÂ­mica ',NULL,now(),NULL,81748),
(7762,3630,'G-Spot',NULL,now(),NULL,81757),
(7763,15732,'Oprah\'s Big Give',NULL,now(),NULL,81761),
(7764,16257,'Liar Game',NULL,now(),NULL,81763),
(7765,16365,'Tokyo Tower',NULL,now(),NULL,81764),
(7766,18187,'Kimi ga Aruji de Shitsuji ga Ore de',NULL,now(),NULL,81702),
(7767,12893,'To Heart 2',NULL,now(),NULL,81768),
(7768,15771,'K-Ville',NULL,now(),NULL,80461),
(7769,18470,'Pucca',NULL,now(),NULL,81774),
(7770,19640,'Shipwrecked Battle of the Islands',NULL,now(),NULL,80347),
(7771,10103,'Shipwrecked',NULL,now(),NULL,81776),
(7772,6716,'Close to Home (2005)',NULL,now(),NULL,75128),
(7773,1446,'Mirage of Blaze',NULL,now(),NULL,81780),
(7774,15476,'Touka Gettan',NULL,now(),NULL,83022),
(7775,18676,'Headcases',NULL,now(),NULL,81783),
(7776,18722,'Night of Too Many Stars',NULL,now(),NULL,81786),
(7777,4051,'Jericho (1966)',NULL,now(),NULL,77482),
(7778,8250,'Men in Trees',NULL,now(),NULL,79366),
(7779,17804,'River Cottage: Gone Fishing',NULL,now(),NULL,81800),
(7780,18744,'Canal Road',NULL,now(),NULL,81804),
(7781,19815,'Draadstaal',NULL,now(),NULL,81805),
(7782,19579,'Amatsuki',NULL,now(),NULL,81808),
(7783,18550,'Allison & Lillia',NULL,now(),NULL,81809),
(7784,18553,'Kure-nai',NULL,now(),NULL,81810),
(7785,18461,'Dis/Connected',NULL,now(),NULL,81813),
(7786,18707,'The Complainers',NULL,now(),NULL,81814),
(7787,7682,'The Food Lovers Guide To Australia',NULL,now(),NULL,81818),
(7788,17789,'Secret Recipes',NULL,now(),NULL,81819),
(7789,7639,'Bush Tucker Man',NULL,now(),NULL,81820),
(7790,3316,'Dinosaurs',NULL,now(),NULL,76461),
(7791,6668,'Xena: Warrior Princess',NULL,now(),NULL,77799),
(7792,13074,'Two Twisted',NULL,now(),NULL,81824),
(7793,7576,'Collectors',NULL,now(),NULL,81833),
(7794,7107,'Up In Town',NULL,now(),NULL,81839),
(7795,18699,'Ben 10: Alien Force',NULL,now(),NULL,81841),
(7796,12677,'Phineas and Ferb',NULL,now(),NULL,81848),
(7797,14721,'Twentyfourseven',NULL,now(),NULL,81850),
(7798,7513,'Jamie Kennedy\'s Blowin\' Up',NULL,now(),NULL,81851),
(7799,5514,'The 4400',NULL,now(),NULL,73507),
(7800,5260,'Soul Food',NULL,now(),NULL,81878),
(7801,2463,'7th Heaven',NULL,now(),NULL,73928),
(7802,4400,'Max Headroom',NULL,now(),NULL,75361),
(7803,18690,'Big Bigger Biggest',NULL,now(),NULL,81863),
(7804,19167,'Kenny Starfighter',NULL,now(),NULL,81864),
(7805,20386,'Muchachada Nui',NULL,now(),NULL,81873),
(7806,16698,'Guarding the Queen',NULL,now(),NULL,81879),
(7807,15912,'Princess Resurrection',NULL,now(),NULL,81877),
(7808,18697,'All Star Mr & Mrs',NULL,now(),NULL,81883),
(7809,7127,'The Friday Night Project',NULL,now(),NULL,81884),
(7810,20646,'Pasila',NULL,now(),NULL,81869),
(7811,744,'Angel Sanctuary ',NULL,now(),NULL,81886),
(7812,7021,'Decoding The Past',NULL,now(),NULL,81889),
(7813,15015,'China From The Inside',NULL,now(),NULL,81891),
(7814,18706,'Am I Normal?',NULL,now(),NULL,81897),
(7815,20690,'Real Drive',NULL,now(),NULL,81907),
(7816,18737,'Carrier',NULL,now(),NULL,81913),
(7817,19141,'The Alaska Experiment',NULL,now(),NULL,81911),
(7818,18747,'The Invisibles',NULL,now(),NULL,81919),
(7819,15812,'Farmer Wants a Wife',NULL,now(),NULL,81929),
(7820,20793,'Roar (2007)',NULL,now(),NULL,81930),
(7821,12523,'Mushishi',NULL,now(),NULL,79214),
(7822,12396,'Akagi',NULL,now(),NULL,81935),
(7823,8190,'Inspector Rex',NULL,now(),NULL,80840),
(7824,15390,'Air Gear',NULL,now(),NULL,79796),
(7825,2519,'Ah! My Goddess',NULL,now(),NULL,78920),
(7826,19950,'Speed Racer: The Next Generation',NULL,now(),NULL,81944),
(7827,22248,'Mijn Restaurant',NULL,now(),NULL,81946),
(7828,2520,'Air America',NULL,now(),NULL,76860),
(7829,2525,'Airwolf',NULL,now(),NULL,73200),
(7830,13373,'Lost Worlds',NULL,now(),NULL,80685),
(7831,12413,'The Dudesons',NULL,now(),NULL,81955),
(7832,2125,'Kino\'s Journey',NULL,now(),NULL,81960),
(7833,3626,'Full Metal Panic!',NULL,now(),NULL,78914),
(7834,2175,'Kaleido Star',NULL,now(),NULL,74739),
(7835,8348,'Bondi Rescue',NULL,now(),NULL,81967),
(7836,16447,'Weaponology',NULL,now(),NULL,81968),
(7837,3605,'Freddy\'s Nightmares',NULL,now(),NULL,71521),
(7838,16765,'Dogfights',NULL,now(),NULL,80257),
(7839,17485,'GSG 9',NULL,now(),NULL,81971),
(7840,13554,'Busou Renkin',NULL,now(),NULL,79524),
(7841,15484,'Sisters Of Wellber',NULL,now(),NULL,81983),
(7842,3064,'CI5: The New Professionals',NULL,now(),NULL,81990),
(7843,18263,'Kaiba',NULL,now(),NULL,81991),
(7844,6589,'White Collar Blue',NULL,now(),NULL,82010),
(7845,18809,'Midnight Man',NULL,now(),NULL,81996),
(7846,18570,'Library War',NULL,now(),NULL,81999),
(7847,5854,'The Hitchhiker\'s Guide to the Galaxy',NULL,now(),NULL,78142),
(7848,2031,'The Twelve Kingdoms',NULL,now(),NULL,74125),
(7849,9371,'Ni Hao, Kai-Lan',NULL,now(),NULL,82005),
(7850,6160,'Sailor',NULL,now(),NULL,82006),
(7851,19509,'Fire 000',NULL,now(),NULL,82047),
(7852,9824,'BlackJack',NULL,now(),NULL,82015),
(7853,1549,'No Angels',NULL,now(),NULL,82016),
(7854,1893,'North Square',NULL,now(),NULL,82017),
(7855,18039,'Wild China',NULL,now(),NULL,82018),
(7856,12727,'Ring Ni Kakero',NULL,now(),NULL,82028),
(7857,2135,'Sakura Wars',NULL,now(),NULL,82029),
(7858,17378,'Secret Diary of a Call Girl',NULL,now(),NULL,80621),
(7859,1495,'The Hoobs',NULL,now(),NULL,82032),
(7860,3235,'Daria',NULL,now(),NULL,77996),
(7861,2825,'Bleach',NULL,now(),NULL,74796),
(7862,15219,'Ancient Discoveries',NULL,now(),NULL,82052),
(7863,18283,'Germany\'s Next Topmodel',NULL,now(),NULL,82053),
(7864,15914,'Lovely Complex',NULL,now(),NULL,80901),
(7865,6743,'Forensic Investigators',NULL,now(),NULL,82058),
(7866,18462,'Out Of The Question',NULL,now(),NULL,82059),
(7867,12663,'Air',NULL,now(),NULL,79101),
(7868,15488,'Nagasarete Airantou',NULL,now(),NULL,82076),
(7869,18746,'Glamour Girls',NULL,now(),NULL,82077),
(7870,18827,'Gladiators (2008)',NULL,now(),NULL,82078),
(7871,19058,'Ranking the stars',NULL,now(),NULL,82081),
(7872,5334,'Star Trek: Deep Space Nine',NULL,now(),NULL,72073),
(7873,21012,'1000 Ways To Die',NULL,now(),NULL,82083),
(7874,21942,'Hung',NULL,now(),NULL,82091),
(7875,19277,'KDD - Kriminaldauerdienst',NULL,now(),NULL,82064),
(7876,18098,'I\'d Do Anything',NULL,now(),NULL,82102),
(7877,3379,'Drak Pack',NULL,now(),NULL,82100),
(7878,18013,'Dress My Nest',NULL,now(),NULL,82104),
(7879,19578,'Blassreiter',NULL,now(),NULL,82116),
(7880,4846,'Point Pleasant',NULL,now(),NULL,73745),
(7881,18488,'Deal or No Deal',NULL,now(),NULL,82121),
(7882,18251,'Celebrity Circus (US)',NULL,now(),NULL,82123),
(7883,988,'George Shrinks',NULL,now(),NULL,82130),
(7884,18865,'Living Lohan',NULL,now(),NULL,82125),
(7885,18955,'House Guest',NULL,now(),NULL,82126),
(7886,5086,'Sagwa, the Chinese Siamese Cat',NULL,now(),NULL,82129),
(7887,5353,'Still Standing',NULL,now(),NULL,70692),
(7888,19716,'The Gruen Transfer',NULL,now(),NULL,82135),
(7889,21234,'Studio Julmahuvi',NULL,now(),NULL,82136),
(7890,2637,'Aquaman (1968)',NULL,now(),NULL,82143),
(7891,19174,'River Cottage Spring',NULL,now(),NULL,82146),
(7892,18781,'Police Interceptors',NULL,now(),NULL,82148),
(7893,145,'The Silver Surfer',NULL,now(),NULL,72461),
(7894,20018,'The Secret Saturdays',NULL,now(),NULL,83290),
(7895,18942,'The Supersizers Go...',NULL,now(),NULL,82151),
(7896,15246,'Tin Man',NULL,now(),NULL,80913),
(7897,13094,'Terry Pratchett\'s Hogfather',NULL,now(),NULL,79988),
(7898,16292,'Sexy Voice and Robo',NULL,now(),NULL,82153),
(7899,6110,'The Professionals',NULL,now(),NULL,75386),
(7900,17411,'20th Century Battlefields',NULL,now(),NULL,82165),
(7901,19258,'Making Tracks',NULL,now(),NULL,82170),
(7902,2668,'Attachments',NULL,now(),NULL,82158),
(7903,2929,'Budgie',NULL,now(),NULL,82160),
(7904,15419,'Diamond Geezer',NULL,now(),NULL,82161),
(7905,6952,'The Knock',NULL,now(),NULL,82162),
(7906,12944,'The Golden Hour',NULL,now(),NULL,82163),
(7907,7005,'The Next Food Network Star',NULL,now(),NULL,82166),
(7908,19952,'Sid the Science Kid',NULL,now(),NULL,82931),
(7909,12257,'Homes Under the Hammer',NULL,now(),NULL,82177),
(7910,19135,'Great British Body',NULL,now(),NULL,82178),
(7911,6953,'Not Forgotten',NULL,now(),NULL,82182),
(7912,18708,'Age of Terror',NULL,now(),NULL,82184),
(7913,9883,'Bod',NULL,now(),NULL,82187),
(7914,22342,'Derren Brown: Messiah',NULL,now(),NULL,82189),
(7915,22343,'Derren Brown: The Heist',NULL,now(),NULL,82190),
(7916,18568,'Soul Eater',NULL,now(),NULL,81751),
(7917,15755,'Swingtown',NULL,now(),NULL,82201),
(7918,8360,'Location, Location, Location',NULL,now(),NULL,82195),
(7919,5145,'Seinfeld',NULL,now(),NULL,79169),
(7920,8492,'Virtua Fighter',NULL,now(),NULL,82206),
(7921,19194,'The Marvelous Misadventures of Flapjack',NULL,now(),NULL,82207),
(7922,2042,'D.N.Angel',NULL,now(),NULL,82211),
(7923,12798,'Shuffle!',NULL,now(),NULL,82213),
(7924,9330,'Fun at the Funeral Parlour',NULL,now(),NULL,82215),
(7925,5820,'The Golden Girls',NULL,now(),NULL,71292),
(7926,17524,'Pfarrer Braun',NULL,now(),NULL,82219),
(7927,18551,'Crystal Blaze',NULL,now(),NULL,82223),
(7928,19208,'She\'s Got the Look',NULL,now(),NULL,82220),
(7929,4089,'Just Cause',NULL,now(),NULL,82222),
(7930,18959,'Wipeout (US)',NULL,now(),NULL,82226),
(7931,18958,'I Survived a Japanese Game Show',NULL,now(),NULL,82228),
(7932,4624,'Nashville Star',NULL,now(),NULL,82229),
(7933,19164,'When We Left Earth: The NASA Missions',NULL,now(),NULL,82232),
(7934,6607,'Wild Card',NULL,now(),NULL,79094),
(7935,18866,'Denise Richards: It\'s Complicated',NULL,now(),NULL,82237),
(7936,19562,'Nigella Bites',NULL,now(),NULL,80282),
(7937,21615,'The Drug Years',NULL,now(),NULL,82251),
(7938,13498,'500 Nations',NULL,now(),NULL,82253),
(7939,15224,'Pinks',NULL,now(),NULL,82265),
(7940,18529,'Fear Itself',NULL,now(),NULL,82120),
(7941,7894,'Human Crossing',NULL,now(),NULL,82267),
(7942,10983,'Roots: The Next Generations',NULL,now(),NULL,82270),
(7943,23986,'Domestic Blitz',NULL,now(),NULL,82279),
(7944,20602,'Keuringsdienst van Waarde',NULL,now(),NULL,82280),
(7945,19039,'Ray Mears Goes Walkabout',NULL,now(),NULL,82284),
(7946,6810,'Days that shook the World',NULL,now(),NULL,82285),
(7947,12567,'The Big Knights',NULL,now(),NULL,82286),
(7948,17607,'Street Customs',NULL,now(),NULL,82292),
(7949,18864,'Black Gold',NULL,now(),NULL,82297),
(7950,19109,'Legally Blonde The Musical: The Search for Elle Woods',NULL,now(),NULL,82295),
(7951,9301,'Storyville',NULL,now(),NULL,82300),
(7952,19333,'Pretty Handsome',NULL,now(),NULL,82301),
(7953,16940,'Inside American Jail',NULL,now(),NULL,82302),
(7954,13468,'Grattis vÃƒÂ¤rlden',NULL,now(),NULL,82303),
(7955,1832,'World of Pub',NULL,now(),NULL,82305),
(7956,16606,'Flipping Out',NULL,now(),NULL,82309),
(7957,18265,'Minami-ke Okawari',NULL,now(),NULL,82312),
(7958,12453,'Magikano ',NULL,now(),NULL,82313),
(7959,4004,'It\'s Always Sunny in Philadelphia',NULL,now(),NULL,75805),
(7960,18764,'Kekkaishi',NULL,now(),NULL,80921),
(7961,12385,' Kurau Phantom Memory',NULL,now(),NULL,80817),
(7962,11145,'R.O.D. - The TV',NULL,now(),NULL,82319),
(7963,19138,'Alexei Sayle\'s Liverpool',NULL,now(),NULL,82321),
(7964,14062,'Searchin\' for My Polestar',NULL,now(),NULL,82323),
(7965,2481,'According to Jim',NULL,now(),NULL,75926),
(7966,11063,'Stand Up!!',NULL,now(),NULL,82326),
(7967,14091,'Water Boys',NULL,now(),NULL,82330),
(7968,14092,'Wonderful Life',NULL,now(),NULL,82331);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(7969,14099,'Yamato Nadeshiko',NULL,now(),NULL,82334),
(7970,8038,'The Alan Clark Diaries',NULL,now(),NULL,82335),
(7971,16296,'Shiawase ni Naritai!',NULL,now(),NULL,82324),
(7972,16333,'You\'re Under Arrest',NULL,now(),NULL,82337),
(7973,20925,'Carla Cametti PD',NULL,now(),NULL,84472),
(7974,3413,'E\'s Otherwise',NULL,now(),NULL,75856),
(7975,15012,'Mega Builders',NULL,now(),NULL,80993),
(7976,13835,'Jyoou no Kyoushitsu',NULL,now(),NULL,82350),
(7977,16131,'Kekkon Dekinai Otoko',NULL,now(),NULL,82352),
(7978,16103,'Time Limit Detective',NULL,now(),NULL,82349),
(7979,280,'Capitol Critters',NULL,now(),NULL,75559),
(7980,23700,'The Living Edens',NULL,now(),NULL,82356),
(7981,9247,'Dare to Believe',NULL,now(),NULL,82357),
(7982,9096,'The All New Harry Hill Show',NULL,now(),NULL,82358),
(7983,22990,'It\'ll be Alright on the Night',NULL,now(),NULL,82360),
(7984,10349,'Families at War',NULL,now(),NULL,82362),
(7985,2873,'Booker',NULL,now(),NULL,74654),
(7986,6470,'Undeclared',NULL,now(),NULL,73557),
(7987,3415,'Earth 2',NULL,now(),NULL,75799),
(7988,2599,'American Gothic',NULL,now(),NULL,76594),
(7989,17934,'Ultimate Factories',NULL,now(),NULL,82374),
(7990,18769,'Zettai Kareshi',NULL,now(),NULL,82376),
(7991,3702,'Godzilla: The Series',NULL,now(),NULL,76061),
(7992,23390,'Bunnytown',NULL,now(),NULL,82381),
(7993,9533,'Fantastic Four (2006)',NULL,now(),NULL,79498),
(7994,5872,'The Incredible Hulk (1996)',NULL,now(),NULL,77155),
(7995,15727,'Eli Stone',NULL,now(),NULL,81025),
(7996,12193,'North and South (2006)',NULL,now(),NULL,79321),
(7997,15530,'Fat Guy Stuck In Internet',NULL,now(),NULL,82388),
(7998,5337,'Star Trek: The Next Generation',NULL,now(),NULL,71470),
(7999,15927,'After the Catch',NULL,now(),NULL,82391),
(8000,14722,'I Love New York',NULL,now(),NULL,82397),
(8001,10244,'Hard Knocks (2001)',NULL,now(),NULL,82402),
(8002,19383,'Factory',NULL,now(),NULL,82404),
(8003,7927,'The Tudors',NULL,now(),NULL,79925),
(8004,11417,'Secrets Of',NULL,now(),NULL,82406),
(8005,11191,'Grand Designs Abroad',NULL,now(),NULL,82408),
(8006,21619,'Everything You Need to Know',NULL,now(),NULL,82414),
(8007,19439,'Burying Brian',NULL,now(),NULL,82419),
(8008,5012,'Riptide',NULL,now(),NULL,77826),
(8009,11131,'Living With Modernism',NULL,now(),NULL,82420),
(8010,19325,'Marco\'s Great British Feast',NULL,now(),NULL,82421),
(8011,18837,'The Secret Life of the American Teenager',NULL,now(),NULL,82422),
(8012,15687,'The Baby Borrowers',NULL,now(),NULL,82423),
(8013,11161,'Ett Herrans Liv',NULL,now(),NULL,80795),
(8014,16435,'Speeders',NULL,now(),NULL,82427),
(8015,6455,'Two Guys and a Girl',NULL,now(),NULL,73178),
(8016,4009,'It\'s Like, You Know...',NULL,now(),NULL,78533),
(8017,19330,'Gok\'s Fashion Fix',NULL,now(),NULL,82432),
(8018,6060,'The Office (UK)',NULL,now(),NULL,78107),
(8019,6408,'Trailer Park Boys',NULL,now(),NULL,70566),
(8020,5044,'Rodney',NULL,now(),NULL,73915),
(8021,19346,'By Royal Appointment',NULL,now(),NULL,82436),
(8022,18805,'Grand Designs Today',NULL,now(),NULL,82437),
(8023,1941,'Angel Links',NULL,now(),NULL,82443),
(8024,9843,'Stand Up',NULL,now(),NULL,82445),
(8025,6258,'The Twilight Zone',NULL,now(),NULL,73587),
(8026,17209,'A Bucket O\' French And Saunders',NULL,now(),NULL,82446),
(8027,19621,'Project Runway Australia',NULL,now(),NULL,82448),
(8028,14220,'Wild Thing I Love You',NULL,now(),NULL,85574),
(8029,7029,'The New Animated Adventures of Flash Gordon',NULL,now(),NULL,82452),
(8030,5597,'The Bad News Bears',NULL,now(),NULL,71613),
(8031,19366,'Chinese Food Made Easy',NULL,now(),NULL,82454),
(8032,17689,'Bonekickers',NULL,now(),NULL,82456),
(8033,19461,'The Hollowmen',NULL,now(),NULL,82460),
(8034,3625,'Full Metal Challenge',NULL,now(),NULL,82462),
(8035,14272,'Phoenix',NULL,now(),NULL,82463),
(8036,13759,'Rallarsving',NULL,now(),NULL,78926),
(8037,18454,'Norske Talenter',NULL,now(),NULL,82464),
(8038,18717,'How Art Made the World',NULL,now(),NULL,82469),
(8039,2759,'Belle and Sebastian',NULL,now(),NULL,78614),
(8040,19272,'Tribal Wives',NULL,now(),NULL,82474),
(8041,19279,'Psychic Kids: Children of the Paranormal',NULL,now(),NULL,82476),
(8042,16327,'Most Shocking',NULL,now(),NULL,82481),
(8043,19473,'The Conspiracy Files',NULL,now(),NULL,82482),
(8044,6207,'The Soup',NULL,now(),NULL,82483),
(8045,19573,'Strike Witches',NULL,now(),NULL,82484),
(8046,18752,'Greatest American Dog',NULL,now(),NULL,82485),
(8047,3972,'The Inspector Lynley Mysteries',NULL,now(),NULL,78774),
(8048,3237,'Dark Angel',NULL,now(),NULL,76148),
(8049,17608,'Are You Smarter than a Canadian 5th Grader',NULL,now(),NULL,82489),
(8050,4360,'Manchild',NULL,now(),NULL,78255),
(8051,19356,'Brooke Knows Best',NULL,now(),NULL,82491),
(8052,15498,'iCarly',NULL,now(),NULL,82493),
(8053,15320,'Tori & Dean',NULL,now(),NULL,82499),
(8054,19185,'Hurl',NULL,now(),NULL,82500),
(8055,18169,'The Convention Crasher',NULL,now(),NULL,82501),
(8056,19371,'Lab Rats (2008)',NULL,now(),NULL,82502),
(8057,6289,'The West Wing',NULL,now(),NULL,72521),
(8058,19329,'Criminal Justice',NULL,now(),NULL,82512),
(8059,12587,'The Chaser Decides',NULL,now(),NULL,82521),
(8060,23788,'Jeremy Clarkson Meets the Neighbours',NULL,now(),NULL,82529),
(8061,8083,'Petrolheads',NULL,now(),NULL,82530),
(8062,19499,'Eating With the Enemy',NULL,now(),NULL,82533),
(8063,9138,'Bruiser',NULL,now(),NULL,82534),
(8064,19477,'Jimmy Doherty\'s Farming Heroes',NULL,now(),NULL,82536),
(8065,19543,'Wrecked',NULL,now(),NULL,82537),
(8066,19359,'CCTV Cities',NULL,now(),NULL,82539),
(8067,12258,'Insider',NULL,now(),NULL,82542),
(8068,19498,'Step Up to the Plate',NULL,now(),NULL,82544),
(8069,9310,'FAQ U',NULL,now(),NULL,82545),
(8070,17893,'Lily Allen and Friends',NULL,now(),NULL,82547),
(8071,16424,'Would I Lie to You?',NULL,now(),NULL,82548),
(8072,18957,'High School Musical: Get in the Picture',NULL,now(),NULL,82550),
(8073,13343,'The Adventure of English',NULL,now(),NULL,82553),
(8074,19341,'The Gong Show with Dave Attell',NULL,now(),NULL,82568),
(8075,12714,'Equator',NULL,now(),NULL,82558),
(8076,19438,'Send in the Dogs',NULL,now(),NULL,82559),
(8077,11029,'Texas Ranch House',NULL,now(),NULL,82565),
(8078,19442,'Harley Street',NULL,now(),NULL,82573),
(8079,19340,'Reality Bites Back',NULL,now(),NULL,82576),
(8080,19122,'9MM SÃƒÂ£o Paulo',NULL,now(),NULL,82578),
(8081,13247,'Emma (1972)',NULL,now(),NULL,82579),
(8082,5732,'Doodlebops',NULL,now(),NULL,82588),
(8083,8231,'Future GPX Cyber Formula',NULL,now(),NULL,82595),
(8084,13395,'River Police',NULL,now(),NULL,82585),
(8085,22363,'On Air',NULL,now(),NULL,82590),
(8086,16274,'Sapuri',NULL,now(),NULL,82591),
(8087,16318,'Slow Dance',NULL,now(),NULL,82593),
(8088,17923,'Thank God You\'re Here (UK)',NULL,now(),NULL,82597),
(8089,16761,'The Andromeda Strain (2008)',NULL,now(),NULL,81843),
(8090,16255,'Proposal Daisakusen',NULL,now(),NULL,82602),
(8091,16117,'Karei naru Ichizoku',NULL,now(),NULL,82603),
(8092,14301,'On Screen!',NULL,now(),NULL,82605),
(8093,12742,'Castle',NULL,now(),NULL,82607),
(8094,19318,'Snog Marry Avoid?',NULL,now(),NULL,82609),
(8095,7565,'Media Watch',NULL,now(),NULL,82616),
(8096,401,'The Strangerers',NULL,now(),NULL,82614),
(8097,1212,'City Hunter',NULL,now(),NULL,72877),
(8098,19554,'Food Detectives',NULL,now(),NULL,82619),
(8099,2464,'8 Simple Rules',NULL,now(),NULL,78461),
(8100,11123,'Gardeners\' World',NULL,now(),NULL,82623),
(8101,19605,'Rogue Restaurants',NULL,now(),NULL,82625),
(8102,17678,'Burn Up',NULL,now(),NULL,82626),
(8103,15389,'Hitohira',NULL,now(),NULL,83405),
(8104,13496,'Muteki Kanban Musume',NULL,now(),NULL,82628),
(8105,12026,'Buildings That Shaped Britain',NULL,now(),NULL,82632),
(8106,3397,'Due South',NULL,now(),NULL,75138),
(8107,18407,'Golden Goal',NULL,now(),NULL,82648),
(8108,4481,'Misfits of Science',NULL,now(),NULL,76951),
(8109,64,'Keeping Up Appearances',NULL,now(),NULL,78079),
(8110,23174,'Churchill\'s Bodyguard',NULL,now(),NULL,82651),
(8111,17885,'De syv dÃƒÂ¸dssyndene',NULL,now(),NULL,81269),
(8112,2240,'Someday\'s Dreamers',NULL,now(),NULL,82653),
(8113,15589,'Sunset Tan',NULL,now(),NULL,82655),
(8114,10934,'Assy McGee',NULL,now(),NULL,82661),
(8115,19549,'Grill It! with Bobby Flay',NULL,now(),NULL,82954),
(8116,18594,'SÃƒÂ¥ ska det lÃƒÂ¥ta',NULL,now(),NULL,82663),
(8117,10337,'Kannazuki No Miko',NULL,now(),NULL,82665),
(8118,2748,'Becker',NULL,now(),NULL,77725),
(8119,13497,'D.Gray-Man',NULL,now(),NULL,79635),
(8120,4832,'Pippi Longstocking',NULL,now(),NULL,78508),
(8121,3465,'Everybody Hates Chris',NULL,now(),NULL,75914),
(8122,19238,'Bully & Rick',NULL,now(),NULL,82117),
(8123,19384,'Spooks: Code 9',NULL,now(),NULL,82670),
(8124,1493,'Full Moon wo Sagashite',NULL,now(),NULL,82677),
(8125,14990,'Nuclear Secrets',NULL,now(),NULL,83053),
(8126,4129,'Kindred: The Embraced',NULL,now(),NULL,75758),
(8127,19572,'Nogizaka Haruka no Himitsu',NULL,now(),NULL,82688),
(8128,1111,'The Water Margin',NULL,now(),NULL,72675),
(8129,19362,'Shonen Onmyouji',NULL,now(),NULL,82697),
(8130,3322,'Disney\'s Hercules',NULL,now(),NULL,72163),
(8131,19130,'Scallywagga',NULL,now(),NULL,82702),
(8132,7659,'Batfink',NULL,now(),NULL,82703),
(8133,16613,'room 401',NULL,now(),NULL,82705),
(8134,3615,'Friday the 13th: The Series',NULL,now(),NULL,73804),
(8135,2425,'The Law of Ueki',NULL,now(),NULL,76031),
(8136,19639,'Lost Land of the Jaguar',NULL,now(),NULL,82715),
(8137,18554,'Itazura na Kiss',NULL,now(),NULL,82735),
(8138,6755,'Speaking in Tongues',NULL,now(),NULL,82743),
(8139,8678,'Full Metal Panic: The Second Raid',NULL,now(),NULL,78924),
(8140,16346,'5ive Days to Midnight',NULL,now(),NULL,79546),
(8141,15619,'Gossip Girl',NULL,now(),NULL,80547),
(8142,6185,'The Shield',NULL,now(),NULL,78261),
(8143,5324,'Stargate Atlantis',NULL,now(),NULL,70851),
(8144,8115,'Studio 60 on the Sunset Strip',NULL,now(),NULL,79343),
(8145,4780,'Party of Five',NULL,now(),NULL,72632),
(8146,2610,'Angel',NULL,now(),NULL,71035),
(8147,3983,'Into the West',NULL,now(),NULL,78828),
(8148,9091,'The Adam And Joe Show',NULL,now(),NULL,82756),
(8149,17202,'Snabbgrabbar med Raske Menn',NULL,now(),NULL,82748),
(8150,5795,'The Fresh Prince of Bel-Air',NULL,now(),NULL,76738),
(8151,13277,'All About George',NULL,now(),NULL,82757),
(8152,17751,'Blunder',NULL,now(),NULL,82758),
(8153,15122,'Comedy Map Of Britain',NULL,now(),NULL,82759),
(8154,12715,'Fear Stress And Anger',NULL,now(),NULL,82763),
(8155,12777,'I\'m With Stupid',NULL,now(),NULL,82766),
(8156,12439,'Jane Hall',NULL,now(),NULL,82767),
(8157,17398,'Meet The Natives',NULL,now(),NULL,82769),
(8158,9771,'Rich Hall\'s Fishing Show',NULL,now(),NULL,82771),
(8159,7953,'Rich Hall\'s Cattle Drive',NULL,now(),NULL,82772),
(8160,19527,'The Kevin Bishop Show',NULL,now(),NULL,82775),
(8161,11206,'The Message',NULL,now(),NULL,82776),
(8162,17317,'Distretto di Polizia',NULL,now(),NULL,82778),
(8163,19712,'Through The Dragon\'s Eye',NULL,now(),NULL,82782),
(8164,18758,'Wagaya no Oinari-sama',NULL,now(),NULL,82803),
(8165,10752,'Jane Eyre (2006)',NULL,now(),NULL,79582),
(8166,4465,'Middlemarch',NULL,now(),NULL,71908),
(8167,17979,'Sense And Sensibility (2008)',NULL,now(),NULL,81517),
(8168,22368,'The DL Chronicles',NULL,now(),NULL,82795),
(8169,19755,'Maestro',NULL,now(),NULL,82797),
(8170,20334,'Vegas Confessions',NULL,now(),NULL,82799),
(8171,15615,'Life',NULL,now(),NULL,80352),
(8172,19582,'Chi\'s Sweet Home',NULL,now(),NULL,81844),
(8173,4683,'No Opportunity Wasted',NULL,now(),NULL,82808),
(8174,19322,'Revealed',NULL,now(),NULL,82813),
(8175,18832,'America\'s Toughest Jobs',NULL,now(),NULL,82816),
(8176,19839,'Masterchef The Professionals',NULL,now(),NULL,82817),
(8177,19837,'Packed to the Rafters',NULL,now(),NULL,82820),
(8178,7158,'Life on Mars',NULL,now(),NULL,79177),
(8179,5335,'Star Trek: Enterprise',NULL,now(),NULL,73893),
(8180,6993,'Torchwood',NULL,now(),NULL,79511),
(8181,20349,'Batman: The Brave and the Bold',NULL,now(),NULL,82824),
(8182,19838,'How Not To Live Your Life',NULL,now(),NULL,82825),
(8183,14202,'The Invisible Man (1984)',NULL,now(),NULL,82827),
(8184,6848,'Yumeria',NULL,now(),NULL,82829),
(8185,18482,'Bakugan Battle Brawlers',NULL,now(),NULL,82832),
(8186,19820,'Xam`d: Lost Memories',NULL,now(),NULL,82834),
(8187,3283,'Dempsey and Makepeace',NULL,now(),NULL,78044),
(8188,3601,'Freaks and Geeks',NULL,now(),NULL,76321),
(8189,6330,'Third Watch',NULL,now(),NULL,73107),
(8190,19918,'The Wrong Door',NULL,now(),NULL,82845),
(8191,21851,'Ayakashi',NULL,now(),NULL,82847),
(8192,1358,'Slings and Arrows',NULL,now(),NULL,79874),
(8193,7849,'The Moomins',NULL,now(),NULL,82850),
(8194,18709,'Chaotic',NULL,now(),NULL,82852),
(8195,19636,'Telepathy Shoujo Ran',NULL,now(),NULL,82885),
(8196,14329,'Discovery: Atlas',NULL,now(),NULL,82854),
(8197,8019,' Ninja Nonsense',NULL,now(),NULL,82856),
(8198,2593,'American Chopper',NULL,now(),NULL,73104),
(8199,2609,'Andy Richter Controls the Universe',NULL,now(),NULL,77290),
(8200,2434,'101 Dalmatians: The Series',NULL,now(),NULL,73719),
(8201,8141,'Jericho (2006)',NULL,now(),NULL,79330),
(8202,6767,'Sleeper Cell',NULL,now(),NULL,74843),
(8203,15422,'Damages',NULL,now(),NULL,80367),
(8204,5835,'The Guardian',NULL,now(),NULL,82866),
(8205,5766,'The Famous Five (1978)',NULL,now(),NULL,70412),
(8206,8219,'Cinderella Boy',NULL,now(),NULL,82873),
(8207,19269,'Hole in the Wall',NULL,now(),NULL,82880),
(8208,19905,'Rush',NULL,now(),NULL,82881),
(8209,5038,'Rocko\'s Modern Life',NULL,now(),NULL,75729),
(8210,11213,'Galaxy Railways',NULL,now(),NULL,82883),
(8211,1046,'Tokyo Underground',NULL,now(),NULL,72435),
(8212,19851,'Hairspray: The School Musical',NULL,now(),NULL,82894),
(8213,22131,'Nerdz',NULL,now(),NULL,82897),
(8214,18729,'Mutual Friends',NULL,now(),NULL,82899),
(8215,18977,'Privileged',NULL,now(),NULL,82902),
(8216,19906,'Feasting on Waves',NULL,now(),NULL,82906),
(8217,19886,'CelebAir',NULL,now(),NULL,82933),
(8218,18410,'Samurai Girl',NULL,now(),NULL,82914),
(8219,16247,'Diners, Drive-ins and Dives',NULL,now(),NULL,82918),
(8220,19843,'The Children',NULL,now(),NULL,82920),
(8221,18361,'Doctor\'s Diary',NULL,now(),NULL,82926),
(8222,19960,'The Principal\'s Office',NULL,now(),NULL,82932),
(8223,19979,'Very Small Business',NULL,now(),NULL,82935),
(8224,21083,'Tank Overhaul',NULL,now(),NULL,82951),
(8225,6909,'Barefoot Contessa',NULL,now(),NULL,82953),
(8226,295,'Jason and the Argonauts',NULL,now(),NULL,82943),
(8227,18106,'Some Assembly Required',NULL,now(),NULL,82957),
(8228,6000,'The Muppet Show',NULL,now(),NULL,72476),
(8229,15958,'How We Built Britain',NULL,now(),NULL,83218),
(8230,12853,'Princess Nikki',NULL,now(),NULL,82978),
(8231,11278,'Women Of The Sun',NULL,now(),NULL,82989),
(8232,20922,'Renovation Nation',NULL,now(),NULL,82994),
(8233,20155,'Guy Off the Hook',NULL,now(),NULL,82999),
(8234,19941,'Massive',NULL,now(),NULL,83055),
(8235,18776,'Fleksnes Fataliteter',NULL,now(),NULL,80313),
(8236,20059,'Vem kan slÃƒÂ¥ Filip och Fredrik',NULL,now(),NULL,82996),
(8237,20260,'Kenny\'s World',NULL,now(),NULL,83004),
(8238,16574,'Wanna Bet',NULL,now(),NULL,83007),
(8239,15114,'Skins',NULL,now(),NULL,79773),
(8240,11571,'Labor in Power',NULL,now(),NULL,83011),
(8241,19042,'Voetbalvrouwen',NULL,now(),NULL,82497),
(8242,17840,'Frontline World',NULL,now(),NULL,85237),
(8243,19808,'Antique Bakery',NULL,now(),NULL,83013),
(8244,22173,'East Of Eden',NULL,now(),NULL,83014),
(8245,20078,'Surf & Rescue: Dewey Beach',NULL,now(),NULL,83016),
(8246,23596,'The Outdoor Room',NULL,now(),NULL,83017),
(8247,11071,'Notorious',NULL,now(),NULL,83019),
(8248,8537,'Patapata Hikousen no Bouken ',NULL,now(),NULL,83021),
(8249,19197,'Date My Ex: Jo & Slade',NULL,now(),NULL,83026),
(8250,19284,'Valentine',NULL,now(),NULL,83028),
(8251,20068,'So You Think You Can Dance Canada',NULL,now(),NULL,83034),
(8252,19435,'New York Goes To Hollywood',NULL,now(),NULL,83038),
(8253,19930,'Accidental Heroes',NULL,now(),NULL,83042),
(8254,8089,'Ugly Betty',NULL,now(),NULL,79352),
(8255,21791,'The Fashion Show',NULL,now(),NULL,83047),
(8256,17307,'Jail',NULL,now(),NULL,83051),
(8257,21600,'Sexcetera',NULL,now(),NULL,83060),
(8258,8494,'Justice (2006)',NULL,now(),NULL,79345),
(8259,22841,'Wild Africa',NULL,now(),NULL,83064),
(8260,12213,'Pryde of the X-Men',NULL,now(),NULL,83070),
(8261,20039,'What To Eat Now',NULL,now(),NULL,83072),
(8262,23730,'The Planners are Coming',NULL,now(),NULL,83080),
(8263,18684,'Kath & Kim (US)',NULL,now(),NULL,83092),
(8264,19819,'My Family\'s Got Guts',NULL,now(),NULL,83094),
(8265,15844,'Rena Rama Rolf',NULL,now(),NULL,79684),
(8266,20095,'Jamie\'s Ministry Of Food',NULL,now(),NULL,83271),
(8267,7838,'Un gars, une fille',NULL,now(),NULL,82985),
(8268,3811,'He-Man and the Masters of the Universe',NULL,now(),NULL,73014),
(8269,18409,'Charterfeber',NULL,now(),NULL,83100),
(8270,5024,'RoboCop',NULL,now(),NULL,75981),
(8271,23112,'Hanazakari no kimitachi e',NULL,now(),NULL,83104),
(8272,1187,'Touch',NULL,now(),NULL,83105),
(8273,19985,'No Heroics',NULL,now(),NULL,83108),
(8274,1851,'From the Earth to the Moon',NULL,now(),NULL,77831),
(8275,13818,'1 Litre of Tears',NULL,now(),NULL,83111),
(8276,23796,'The World\'s Strictest Parents',NULL,now(),NULL,83115),
(8277,4317,'M.A.S.K.',NULL,now(),NULL,71117),
(8278,12054,'Ginga Densetsu Weed',NULL,now(),NULL,83126),
(8279,20001,'Evelien',NULL,now(),NULL,82863),
(8280,6747,'Strictly Come Dancing',NULL,now(),NULL,83127),
(8281,7543,'Play School (1966)',NULL,now(),NULL,83129),
(8282,20215,'I ditt ansikte',NULL,now(),NULL,83075),
(8283,13697,'Spaceballs: The Animated Series',NULL,now(),NULL,83132),
(8284,20056,'Hole in the Wall (UK)',NULL,now(),NULL,83133),
(8285,12743,'Ashita no Joe',NULL,now(),NULL,81232),
(8286,2261,'Biker Build-Off',NULL,now(),NULL,83137),
(8287,5374,'Street Time',NULL,now(),NULL,83154),
(8288,12775,'Rob Brydon\'s Annually Retentive',NULL,now(),NULL,83139),
(8289,19656,'Evolve',NULL,now(),NULL,83141),
(8290,23115,'Iljimae',NULL,now(),NULL,83142),
(8291,21903,'Mantracker',NULL,now(),NULL,83153),
(8292,19970,'The Sex Education Show',NULL,now(),NULL,83169),
(8293,21817,'Richard Hammond\'s Engineering Connections',NULL,now(),NULL,83165),
(8294,15895,'War Of Money',NULL,now(),NULL,83166),
(8295,14230,'Lovers In Paris',NULL,now(),NULL,83167),
(8296,14108,'Alone In Love',NULL,now(),NULL,83171),
(8297,18932,'Opportunity Knocks',NULL,now(),NULL,83176),
(8298,10793,'Jam and Jerusalem',NULL,now(),NULL,83177),
(8299,3600,'Freakazoid!',NULL,now(),NULL,72875),
(8300,18380,'Luksusfellen',NULL,now(),NULL,83194),
(8301,7777,'Exposed...',NULL,now(),NULL,83200),
(8302,18575,'The Suite Life on Deck',NULL,now(),NULL,83207),
(8303,19423,'Francesco\'s Mediterranean Voyage',NULL,now(),NULL,83212),
(8304,20038,'Somebodies',NULL,now(),NULL,83220),
(8305,23123,'Only Connect',NULL,now(),NULL,83221),
(8306,2810,'Birds of Prey',NULL,now(),NULL,77746),
(8307,12089,'A Very British Coup',NULL,now(),NULL,83223),
(8308,20110,'James May\'s Big Ideas',NULL,now(),NULL,83227),
(8309,3449,'Entourage',NULL,now(),NULL,74543),
(8310,19650,'Little Britain USA',NULL,now(),NULL,83232),
(8311,20324,'Top Gear Australia',NULL,now(),NULL,83234),
(8312,19810,'Ryoko\'s Case File',NULL,now(),NULL,83236),
(8313,13657,'Earthfasts',NULL,now(),NULL,84989),
(8314,15343,'Stargate Universe',NULL,now(),NULL,83237),
(8315,20052,'Place of execution',NULL,now(),NULL,83242),
(8316,3129,'Complete Savages',NULL,now(),NULL,73827),
(8317,3672,'Ghostbusters',NULL,now(),NULL,77055),
(8318,7828,'Alas Smith and Jones',NULL,now(),NULL,83246),
(8319,18066,'Dragonaut - The Resonance',NULL,now(),NULL,81072),
(8320,12318,'Sherlock Hound',NULL,now(),NULL,83248),
(8321,19436,'The Cho Show',NULL,now(),NULL,83251),
(8322,20573,'XIII',NULL,now(),NULL,83258),
(8323,17496,'Bamboo Blade',NULL,now(),NULL,80673),
(8324,17377,'Roomservice',NULL,now(),NULL,81795),
(8325,19428,'Ibiza',NULL,now(),NULL,83264),
(8326,21,'Trapper John, M.D.',NULL,now(),NULL,73350),
(8327,20118,'Toradora!',NULL,now(),NULL,83277),
(8328,11223,'Glass Fleet',NULL,now(),NULL,83280),
(8329,5175,'Sherlock Holmes (1984)',NULL,now(),NULL,77728),
(8330,6816,'Iconoclasts',NULL,now(),NULL,83282),
(8331,18110,'The New Detectives',NULL,now(),NULL,83283),
(8332,20167,'Coming of Age (2008)',NULL,now(),NULL,83284),
(8333,7174,'It\'s Me or the Dog',NULL,now(),NULL,83288),
(8334,2354,'The Beiderbecke Tapes',NULL,now(),NULL,83291),
(8335,19229,'Total Drama Island',NULL,now(),NULL,83294),
(8336,11177,'Shingu: Secret of the Stellar Wars',NULL,now(),NULL,83295),
(8337,5937,'The Littles',NULL,now(),NULL,71665),
(8338,13518,'The Complete Guide to Parenting',NULL,now(),NULL,83298),
(8339,19287,'Easy Money',NULL,now(),NULL,83300),
(8340,5023,'Robin of Sherwood',NULL,now(),NULL,75366),
(8341,15529,'Superjail',NULL,now(),NULL,83306),
(8342,19285,'In Harms Way',NULL,now(),NULL,83307),
(8343,6982,'Papa Beaver\'s Story Time',NULL,now(),NULL,83310),
(8344,19796,'Pacific Abyss',NULL,now(),NULL,83311),
(8345,19565,'Trexx and Flipside',NULL,now(),NULL,83313),
(8346,17089,'Gooische vrouwen',NULL,now(),NULL,83010),
(8347,20119,'Casshern Sins',NULL,now(),NULL,83316),
(8348,20120,'Hyakko',NULL,now(),NULL,83317),
(8349,20128,'Linebarrels of Iron',NULL,now(),NULL,83321),
(8350,20131,'To Aru Majutsu no Index',NULL,now(),NULL,83322),
(8351,4614,'Naked Josh',NULL,now(),NULL,83323),
(8352,20139,'Hero Ships',NULL,now(),NULL,83325),
(8353,16970,'Kimora: Life in the Fab Lane',NULL,now(),NULL,83326),
(8354,3868,'History\'s Mysteries',NULL,now(),NULL,83330),
(8355,3506,'Family Guy',NULL,now(),NULL,75978),
(8356,20632,'The Merrick & Rosso Show',NULL,now(),NULL,83337),
(8357,13178,'Ace Of Cakes',NULL,now(),NULL,83338),
(8358,15152,'Magic',NULL,now(),NULL,83343),
(8359,8411,'Standoff',NULL,now(),NULL,79347),
(8360,16348,'1,000 Places to See Before You Die',NULL,now(),NULL,83346),
(8361,18516,'Freaky Eaters',NULL,now(),NULL,83349),
(8362,19697,'Down and Dirty with Jim Norton',NULL,now(),NULL,83355),
(8363,20189,'Chaos head',NULL,now(),NULL,83358),
(8364,20133,'Ga-Rei: Zero',NULL,now(),NULL,83359),
(8365,20190,'Tytania',NULL,now(),NULL,83364),
(8366,22229,'The Conquerors',NULL,now(),NULL,83365),
(8367,20146,'ef - a tale of melodies',NULL,now(),NULL,83370),
(8368,8470,'Windy Tales',NULL,now(),NULL,83371),
(8369,12039,'The Thorn Birds',NULL,now(),NULL,83375),
(8370,18821,'Beautiful People (2008)',NULL,now(),NULL,83377),
(8371,15718,'Women\'s Murder Club',NULL,now(),NULL,80649),
(8372,8459,'Notes from the Underbelly',NULL,now(),NULL,80129),
(8373,18177,'Wonderland',NULL,now(),NULL,83382),
(8374,16008,'CHANGE',NULL,now(),NULL,83386),
(8375,16477,'Krecek v nocni kosili',NULL,now(),NULL,83389),
(8376,20225,'Stephen Fry in America',NULL,now(),NULL,83391),
(8377,3294,'Desperate Housewives',NULL,now(),NULL,73800),
(8378,18808,'Sunshine (2008)',NULL,now(),NULL,83396),
(8379,6132,'The Ren and Stimpy Show',NULL,now(),NULL,76088),
(8380,3986,'Invasion',NULL,now(),NULL,75588),
(8381,15744,'Reaper',NULL,now(),NULL,80345),
(8382,12716,'Galapagos',NULL,now(),NULL,80688),
(8383,20177,'British Style Genius',NULL,now(),NULL,83402),
(8384,19321,'Top Gear Xtra',NULL,now(),NULL,83407),
(8385,20188,'Kurozuka',NULL,now(),NULL,83408),
(8386,11790,'First Australians',NULL,now(),NULL,83409),
(8387,3271,'DearS',NULL,now(),NULL,79236),
(8388,3183,'CSI: Crime Scene Investigation',NULL,now(),NULL,72546),
(8389,6521,'Voltron: Defender of the Universe',NULL,now(),NULL,78646),
(8390,19501,'Lama Gezocht',NULL,now(),NULL,83256),
(8391,11234,'Gash Bell!',NULL,now(),NULL,83421),
(8392,3188,'Curb Your Enthusiasm',NULL,now(),NULL,76203),
(8393,20204,'Dead Set',NULL,now(),NULL,83427),
(8394,8065,'The Sarah Silverman Program',NULL,now(),NULL,79789),
(8395,20162,'Unbreakable',NULL,now(),NULL,83431),
(8396,1631,'Video Girl Ai',NULL,now(),NULL,79777),
(8397,15716,'Pushing Daisies',NULL,now(),NULL,80351),
(8398,20163,'The Story of Maths',NULL,now(),NULL,83443),
(8399,19660,'Iditarod: Toughest Race on Earth',NULL,now(),NULL,83442),
(8400,367,'Triangle',NULL,now(),NULL,83439),
(8401,3739,'Greetings From Tucson',NULL,now(),NULL,83448),
(8402,19666,'Prototype This',NULL,now(),NULL,83452),
(8403,18161,'Vratice se rode',NULL,now(),NULL,83459),
(8404,20787,'River Cottage Autumn',NULL,now(),NULL,83461),
(8405,1878,'Thunderbirds',NULL,now(),NULL,77964),
(8406,5333,'Star Trek',NULL,now(),NULL,77526),
(8407,2611,'Angelina Ballerina',NULL,now(),NULL,83466),
(8408,18280,'Alan Carr\'s Celebrity Ding Dong',NULL,now(),NULL,83468),
(8409,8092,'The Nine',NULL,now(),NULL,79380),
(8410,15319,'Californication',NULL,now(),NULL,80349),
(8411,22582,'Sinatra',NULL,now(),NULL,83477),
(8412,23518,'Pranked',NULL,now(),NULL,111091),
(8413,15541,'Space Academy',NULL,now(),NULL,83490),
(8414,20090,'Dagvaktin',NULL,now(),NULL,83138),
(8415,16449,'Arabela',NULL,now(),NULL,80745),
(8416,16431,'Britain\'s Got More Talent',NULL,now(),NULL,83497),
(8417,17699,'Storm Chasers',NULL,now(),NULL,83498),
(8418,6190,'The Simpsons',NULL,now(),NULL,71663),
(8419,8331,'Thin Ice',NULL,now(),NULL,83502),
(8420,7462,'Los Serrano',NULL,now(),NULL,83503),
(8421,19818,'Het Schnitzelparadijs',NULL,now(),NULL,83507),
(8422,18238,'Maisy',NULL,now(),NULL,79361),
(8423,19355,'Hulk Hogan\'s Celebrity Championship Wrestling',NULL,now(),NULL,83508),
(8424,5348,'Stephen King\'s Kingdom Hospital',NULL,now(),NULL,78951),
(8425,19387,'Britain\'s Missing Top Model',NULL,now(),NULL,83510),
(8426,19708,'The Jaquie Brown Diaries',NULL,now(),NULL,83511),
(8427,20322,'Bogan Pride',NULL,now(),NULL,83512),
(8428,9745,'Blood Ties',NULL,now(),NULL,79875),
(8429,3293,'Desmond\'s',NULL,now(),NULL,83516),
(8430,8171,'Traveler',NULL,now(),NULL,79846),
(8431,20329,'Miss Naked Beauty',NULL,now(),NULL,83518),
(8432,9988,'All Over the Shop',NULL,now(),NULL,83519),
(8433,14573,'Man vs. Wild',NULL,now(),NULL,79545),
(8434,19721,'The Tony Rock Project',NULL,now(),NULL,83524),
(8435,19797,'The Cup',NULL,now(),NULL,83527),
(8436,2719,'Batman',NULL,now(),NULL,77871),
(8437,9376,'Best Student Council',NULL,now(),NULL,83536),
(8438,20222,'Wired',NULL,now(),NULL,83540),
(8439,8738,'Home James!',NULL,now(),NULL,83541),
(8440,1951,'In Sickness and In Health',NULL,now(),NULL,83543),
(8441,4057,'Joan of Arcadia',NULL,now(),NULL,72277),
(8442,20077,'Rita Rocks',NULL,now(),NULL,83549),
(8443,15313,'Saving Grace',NULL,now(),NULL,80381),
(8444,4417,'McLeod\'s Daughters',NULL,now(),NULL,76532),
(8445,12691,'The Arabian Nights Adventures of Sinbad',NULL,now(),NULL,83566),
(8446,18032,'Clannad',NULL,now(),NULL,80644),
(8447,18460,'Rock Rivals',NULL,now(),NULL,83570),
(8448,10780,'Little Einsteins',NULL,now(),NULL,78978),
(8449,6202,'The Smurfs',NULL,now(),NULL,75719),
(8450,18145,'Westenwind',NULL,now(),NULL,83583),
(8451,20338,'Ghost Adventures',NULL,now(),NULL,83929),
(8452,4498,'Mobile Suit Gundam',NULL,now(),NULL,76547),
(8453,20547,'Swift and Shift Couriers',NULL,now(),NULL,83596),
(8454,20685,'Argumental',NULL,now(),NULL,83597),
(8455,20178,'Paul Merton in India',NULL,now(),NULL,83600),
(8456,1134,'Generator Gawl',NULL,now(),NULL,81169),
(8457,7365,'Deputy Dawg',NULL,now(),NULL,83638),
(8458,19758,'S1ngle',NULL,now(),NULL,83589),
(8459,12412,'Mazinger Z',NULL,now(),NULL,83609),
(8460,21884,'Party Down',NULL,now(),NULL,83616),
(8461,12127,'The Good Witch of the West',NULL,now(),NULL,83619),
(8462,16959,'Living With Ed',NULL,now(),NULL,83624),
(8463,9358,'Grumpy Old Men',NULL,now(),NULL,83625),
(8464,2680,'Avatar: The Last Airbender',NULL,now(),NULL,74852),
(8465,5962,'The Magnificent Seven',NULL,now(),NULL,75093),
(8466,17971,'Apparitions',NULL,now(),NULL,83734);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(8467,8015,'Mouse',NULL,now(),NULL,83635),
(8468,15671,'The Winds of War',NULL,now(),NULL,83637),
(8469,21144,'The American Future: A History, by Simon Schama',NULL,now(),NULL,83640),
(8470,23255,'Soundstage',NULL,now(),NULL,83641),
(8471,20945,'Canada\'s Worst Driver',NULL,now(),NULL,83642),
(8472,20944,'Canada\'s Worst Handyman',NULL,now(),NULL,83644),
(8473,15342,'Acceptable.TV',NULL,now(),NULL,83645),
(8474,18296,'Little Dorrit',NULL,now(),NULL,83649),
(8475,20216,'Paris Hilton\'s My New BFF',NULL,now(),NULL,83654),
(8476,20375,'Neil Morrissey\'s Risky Business',NULL,now(),NULL,83665),
(8477,647,'BTX',NULL,now(),NULL,70978),
(8478,18439,'Fredsstyrkan',NULL,now(),NULL,81826),
(8479,16248,'Big Medicine',NULL,now(),NULL,83659),
(8480,14745,'D Motor',NULL,now(),NULL,83581),
(8481,19184,'The Unusuals',NULL,now(),NULL,83661),
(8482,20328,'Britannia High',NULL,now(),NULL,83662),
(8483,6367,'Titus',NULL,now(),NULL,73809),
(8484,18751,'In Plain Sight',NULL,now(),NULL,82155),
(8485,14676,'Casualty 1907',NULL,now(),NULL,83679),
(8486,20399,'Whale Wars',NULL,now(),NULL,83682),
(8487,20179,'Greatest Cities of the World with Griff Rhys Jones',NULL,now(),NULL,83411),
(8488,8010,'Rizelmine',NULL,now(),NULL,83692),
(8489,3471,'Extras',NULL,now(),NULL,75660),
(8490,19609,'Legend of the Seeker',NULL,now(),NULL,82672),
(8491,18976,'stylista',NULL,now(),NULL,83705),
(8492,19951,'The Xtacles',NULL,now(),NULL,83707),
(8493,15719,'Cashmere Mafia',NULL,now(),NULL,81077),
(8494,18244,'Less Than Kind',NULL,now(),NULL,83710),
(8495,18504,'Harald Schmidt',NULL,now(),NULL,80591),
(8496,21978,'Genius',NULL,now(),NULL,83714),
(8497,23944,'World War II: Behind Closed Doors',NULL,now(),NULL,83718),
(8498,16228,'Nodame Cantabile',NULL,now(),NULL,83722),
(8499,3887,'Home Improvement',NULL,now(),NULL,77537),
(8500,12255,'Francesco\'s Italy: Top to Toe',NULL,now(),NULL,83725),
(8501,2654,'As If',NULL,now(),NULL,83728),
(8502,1817,'Doctors',NULL,now(),NULL,83729),
(8503,19647,'Estate of Panic',NULL,now(),NULL,83730),
(8504,7559,'Catalyst',NULL,now(),NULL,83731),
(8505,4505,'Modern Marvels',NULL,now(),NULL,71697),
(8506,13550,'Lost Cities of the Ancients',NULL,now(),NULL,83741),
(8507,23902,'Romanzo Criminale',NULL,now(),NULL,83742),
(8508,20173,'Dear Green Place',NULL,now(),NULL,83745),
(8509,20308,'cha$e',NULL,now(),NULL,83747),
(8510,1873,'Hey Dad...!',NULL,now(),NULL,77934),
(8511,21579,'Hero Corp',NULL,now(),NULL,83750),
(8512,12823,'Space Adventure Cobra',NULL,now(),NULL,82971),
(8513,4605,'MythBusters',NULL,now(),NULL,73388),
(8514,20251,'Danger Men',NULL,now(),NULL,83754),
(8515,6824,'Matroesjka\'s',NULL,now(),NULL,83756),
(8516,4432,'Medium',NULL,now(),NULL,73265),
(8517,20506,'The Barristers',NULL,now(),NULL,83773),
(8518,6206,'The Sopranos',NULL,now(),NULL,75299),
(8519,22251,'Holocaust',NULL,now(),NULL,83780),
(8520,20538,'A History of Scotland',NULL,now(),NULL,83785),
(8521,15438,'Adventures In Hollyhood',NULL,now(),NULL,83786),
(8522,5865,'The Hunger',NULL,now(),NULL,76238),
(8523,4705,'Odyssey 5',NULL,now(),NULL,70694),
(8524,18063,'Black Lagoon',NULL,now(),NULL,79604),
(8525,20419,'Clone',NULL,now(),NULL,83800),
(8526,12367,'KIBA',NULL,now(),NULL,79483),
(8527,20495,'The Devil\'s Whore',NULL,now(),NULL,83795),
(8528,20422,'The Last Millionaire',NULL,now(),NULL,83801),
(8529,20082,'The Dark Side of Fame with Piers Morgan',NULL,now(),NULL,83809),
(8530,14977,'Little Mosque on the Prairie',NULL,now(),NULL,79701),
(8531,4925,'Queer as Folk',NULL,now(),NULL,75367),
(8532,18572,'High School Confidential',NULL,now(),NULL,83813),
(8533,13031,'Afro Samurai',NULL,now(),NULL,79755),
(8534,11127,'Unreported World',NULL,now(),NULL,83817),
(8535,16924,'The City Gardener',NULL,now(),NULL,83818),
(8536,6857,'Rollergirls',NULL,now(),NULL,83819),
(8537,15673,'Kodenavn Hunter',NULL,now(),NULL,79805),
(8538,17871,'Ben\'s Zoo',NULL,now(),NULL,83820),
(8539,13318,'The Path to 9/11',NULL,now(),NULL,83821),
(8540,2069,'The Koala Brothers',NULL,now(),NULL,74327),
(8541,19586,'Iron Man: Armored Adventures ',NULL,now(),NULL,83826),
(8542,2990,'CarnivÃƒÂ le',NULL,now(),NULL,70860),
(8543,14810,'Wall Street Warriors',NULL,now(),NULL,83830),
(8544,2747,'Beck: Mongolian Chop Squad',NULL,now(),NULL,75838),
(8545,19890,'Extreme Fishing With Robson Green',NULL,now(),NULL,83834),
(8546,19273,'Jews',NULL,now(),NULL,83841),
(8547,4153,'Kojak (2005)',NULL,now(),NULL,75737),
(8548,8506,'Runaway',NULL,now(),NULL,79353),
(8549,21173,'Innocent Love',NULL,now(),NULL,83847),
(8550,20539,'How Stuff Works',NULL,now(),NULL,83851),
(8551,4037,'Jakers! The Adventures of Piggley Winks',NULL,now(),NULL,82131),
(8552,14143,'7 Vidas',NULL,now(),NULL,83853),
(8553,3378,'Drake and Josh',NULL,now(),NULL,79357),
(8554,21902,'The Elephant Princess',NULL,now(),NULL,83865),
(8555,20530,'Hospital Heroes',NULL,now(),NULL,83867),
(8556,9443,'Suburban Shootout',NULL,now(),NULL,80549),
(8557,18127,'Richard Hammond Meets Evel Knievel ',NULL,now(),NULL,83871),
(8558,6086,'The Persuaders',NULL,now(),NULL,76362),
(8559,5433,'Swiss Toni',NULL,now(),NULL,83875),
(8560,5340,'Star Wars: Clone Wars',NULL,now(),NULL,72244),
(8561,20482,'D.L. Hughley Breaks the News',NULL,now(),NULL,83879),
(8562,20526,'Extreme Loggers',NULL,now(),NULL,83881),
(8563,16881,'Big Ideas for a Small Planet',NULL,now(),NULL,83883),
(8564,18216,'Merlin (1998)',NULL,now(),NULL,83269),
(8565,18549,'Vampire Knight',NULL,now(),NULL,81816),
(8566,18249,'Life After People',NULL,now(),NULL,83897),
(8567,19840,'Lost in Austen',NULL,now(),NULL,82934),
(8568,9171,'Carrott Confidential',NULL,now(),NULL,83903),
(8569,3102,'Cold Case',NULL,now(),NULL,72167),
(8570,6728,'Masters of Horror',NULL,now(),NULL,79066),
(8571,20382,'Gemini Division',NULL,now(),NULL,83909),
(8572,12726,'1990',NULL,now(),NULL,83913),
(8573,18175,'Wallander (UK)',NULL,now(),NULL,83925),
(8574,15528,'The Drinky Crow Show',NULL,now(),NULL,83931),
(8575,7000,'Dreamspaces',NULL,now(),NULL,83935),
(8576,20145,'Macademi Wasshoi!',NULL,now(),NULL,83940),
(8577,2078,'We Are History',NULL,now(),NULL,83945),
(8578,2776,'Beverly Hills, 90210',NULL,now(),NULL,74936),
(8579,20580,'Unexplained with George Noory',NULL,now(),NULL,83951),
(8580,17977,'Mistresses',NULL,now(),NULL,81126),
(8581,15357,'Once Upon a Time... The Explorers',NULL,now(),NULL,83005),
(8582,2664,'At Home with the Braithwaites',NULL,now(),NULL,83968),
(8583,15581,'The Museum',NULL,now(),NULL,83971),
(8584,19173,'I Love Money',NULL,now(),NULL,83973),
(8585,22135,'Railway Walks',NULL,now(),NULL,83986),
(8586,4049,'Jeremiah',NULL,now(),NULL,78820),
(8587,10100,'Saved',NULL,now(),NULL,79401),
(8588,3874,'Hogan\'s Heroes',NULL,now(),NULL,71635),
(8589,6639,'Wonderland (2000)',NULL,now(),NULL,84004),
(8590,20717,'Man V Food',NULL,now(),NULL,84006),
(8591,17091,'locked up Abroad',NULL,now(),NULL,84014),
(8592,20552,'Oceans',NULL,now(),NULL,84017),
(8593,19907,'Better Off Ted',NULL,now(),NULL,84021),
(8594,14640,'Frosty the Snowman',NULL,now(),NULL,84032),
(8595,14629,'Rudolph the Red-Nosed Reindeer',NULL,now(),NULL,84033),
(8596,14632,'Santa Claus is Comin\' to Town',NULL,now(),NULL,84034),
(8597,14627,'The Little Drummer Boy',NULL,now(),NULL,84035),
(8598,15417,'Sweet Baby James',NULL,now(),NULL,84036),
(8599,19520,'Shatner\'s Raw Nerve',NULL,now(),NULL,84037),
(8600,20089,'Celebrity Juice',NULL,now(),NULL,84041),
(8601,4956,'Recess',NULL,now(),NULL,71780),
(8602,20531,'Catastrophe',NULL,now(),NULL,84047),
(8603,13976,'Orange Days',NULL,now(),NULL,84051),
(8604,6493,'V',NULL,now(),NULL,76354),
(8605,15504,'Ashes to Ashes',NULL,now(),NULL,81253),
(8606,15513,'The Football Years',NULL,now(),NULL,84057),
(8607,2620,'Animaniacs',NULL,now(),NULL,72879),
(8608,15155,'Flavor of Love Girls: Charm School',NULL,now(),NULL,84065),
(8609,17675,'Independent Lens',NULL,now(),NULL,84082),
(8610,19128,'The Last Templar',NULL,now(),NULL,84067),
(8611,10109,'War And Remembrance',NULL,now(),NULL,84070),
(8612,13077,'Amerika',NULL,now(),NULL,84073),
(8613,16982,'A Pillow Case of Mystery',NULL,now(),NULL,84074),
(8614,20648,'Country House Rescue',NULL,now(),NULL,84079),
(8615,15717,'Dirty Sexy Money',NULL,now(),NULL,80593),
(8616,20149,'Big Daddy\'s House',NULL,now(),NULL,84091),
(8617,3646,'Gargoyles',NULL,now(),NULL,72885),
(8618,5427,'Sweating Bullets',NULL,now(),NULL,74988),
(8619,3404,'Dungeons & Dragons',NULL,now(),NULL,77129),
(8620,22443,'America\'s Game',NULL,now(),NULL,84099),
(8621,21999,'Mega Movers',NULL,now(),NULL,84103),
(8622,993,'William and Mary',NULL,now(),NULL,84105),
(8623,20545,'50 Cent: The Money and The Power',NULL,now(),NULL,84110),
(8624,20337,'Extreme Trains',NULL,now(),NULL,84127),
(8625,9134,'Victory at Sea',NULL,now(),NULL,84130),
(8626,20516,'Manhunters: Fugitive Task Force',NULL,now(),NULL,84133),
(8627,20142,'Parents of the Band',NULL,now(),NULL,84134),
(8628,1055,'Doggy Fizzle Televizzle',NULL,now(),NULL,84137),
(8629,10947,'The State Within',NULL,now(),NULL,79548),
(8630,14606,'Dan Rather Reports',NULL,now(),NULL,84143),
(8631,14579,'.hack//Roots',NULL,now(),NULL,79297),
(8632,949,'.hack//SIGN',NULL,now(),NULL,79099),
(8633,20123,'Akane-Iro ni Somaru Saka',NULL,now(),NULL,83315),
(8634,8555,'Ghost in the Shell: Stand Alone Complex',NULL,now(),NULL,73749),
(8635,19129,'Momma\'s Boys',NULL,now(),NULL,84145),
(8636,19012,'Secret Millionaire',NULL,now(),NULL,84146),
(8637,2568,'Ally McBeal',NULL,now(),NULL,72116),
(8638,4545,'Mr. Terrific',NULL,now(),NULL,74905),
(8639,18038,'Mobile Suit Gundam 00',NULL,now(),NULL,80675),
(8640,19672,'The Real Housewives of Atlanta',NULL,now(),NULL,84159),
(8641,19619,'Skip Beat !',NULL,now(),NULL,83653),
(8642,4926,'Queer as Folk (UK)',NULL,now(),NULL,75625),
(8643,7053,'Heist',NULL,now(),NULL,79255),
(8644,22439,'Arthur C. Clarke\'s Mysterious World',NULL,now(),NULL,84175),
(8645,20125,'Yozakura Quartet',NULL,now(),NULL,83588),
(8646,20121,'Shikabane Hime: Aka',NULL,now(),NULL,83363),
(8647,4203,'Law & Order: Criminal Intent',NULL,now(),NULL,71489),
(8648,17900,'Meet The Magoons',NULL,now(),NULL,84179),
(8649,20278,'Spain: On The Road Again',NULL,now(),NULL,84181),
(8650,23872,'Monster Moves',NULL,now(),NULL,93681),
(8651,19632,'Tim and Eric Nite Live',NULL,now(),NULL,84189),
(8652,1186,'Kamikaze Kaitou Jeanne',NULL,now(),NULL,72832),
(8653,1630,'Ranma Ã‚Â½',NULL,now(),NULL,76932),
(8654,7167,'The Replacements',NULL,now(),NULL,84192),
(8655,19575,'Sekirei',NULL,now(),NULL,82807),
(8656,10802,'Sorted',NULL,now(),NULL,84207),
(8657,19263,'Martin Lawrence Presents 1st Amendment Stand-Up',NULL,now(),NULL,84209),
(8658,18167,'Rosario + Vampire',NULL,now(),NULL,81184),
(8659,10931,'Traffic Cops',NULL,now(),NULL,84218),
(8660,10018,'Bargain Hunt',NULL,now(),NULL,84223),
(8661,22707,'Bargain Hunt Famous Finds',NULL,now(),NULL,84224),
(8662,20783,'Crooked House',NULL,now(),NULL,84228),
(8663,3171,'Criminal Minds',NULL,now(),NULL,75710),
(8664,21038,'Casualty 1909',NULL,now(),NULL,101341),
(8665,10622,'Future Weapons',NULL,now(),NULL,80037),
(8666,17764,'Goshushosama Ninomiya-kun',NULL,now(),NULL,82167),
(8667,16700,'Into Alaska With Jeff Corwin',NULL,now(),NULL,84231),
(8668,12115,'Jesus of Nazareth',NULL,now(),NULL,84235),
(8669,19231,'Otto - Die Serie',NULL,now(),NULL,81882),
(8670,8044,'The IT Crowd',NULL,now(),NULL,79216),
(8671,4257,'Little House on the Prairie',NULL,now(),NULL,77921),
(8672,20132,'Kemeko Deluxe!',NULL,now(),NULL,83360),
(8673,18070,'Kateikyoushi Hitman Reborn!',NULL,now(),NULL,80975),
(8674,6342,'Those Who Hunt Elves',NULL,now(),NULL,72447),
(8675,20947,'Race Car Driver',NULL,now(),NULL,84247),
(8676,8949,'Porterhouse Blue',NULL,now(),NULL,84249),
(8677,7043,'Campus Ladies',NULL,now(),NULL,84258),
(8678,19698,'Trust Me (2009)',NULL,now(),NULL,84262),
(8679,20126,'Tales of the Abyss',NULL,now(),NULL,83335),
(8680,20626,'Spectacle: Elvis Costello With...',NULL,now(),NULL,84270),
(8681,14365,'Jane Eyre',NULL,now(),NULL,84274),
(8682,20692,'M.K. 22',NULL,now(),NULL,84279),
(8683,16877,'Baldwin Hills',NULL,now(),NULL,84281),
(8684,20620,'The City (2008)',NULL,now(),NULL,84284),
(8685,15197,'Badly Dubbed Porn',NULL,now(),NULL,84286),
(8686,7449,'Webdreams',NULL,now(),NULL,84287),
(8687,1060,'Kiddy Grade',NULL,now(),NULL,72471),
(8688,16825,'To Love and Die',NULL,now(),NULL,84291),
(8689,5827,'The Greatest American Hero',NULL,now(),NULL,77267),
(8690,5452,'Tales of the Gold Monkey',NULL,now(),NULL,76815),
(8691,1398,'Oruchuban Ebichu',NULL,now(),NULL,79147),
(8692,20797,'Bromance',NULL,now(),NULL,84313),
(8693,7976,'Bodyshock',NULL,now(),NULL,84314),
(8694,3331,'Doctor Who',NULL,now(),NULL,76107),
(8695,4536,'Mr. Bean',NULL,now(),NULL,76316),
(8696,14022,'Sekai no chuushin de, ai wo sakebu',NULL,now(),NULL,84319),
(8697,13902,'Love Revolution',NULL,now(),NULL,84320),
(8698,14069,'Tokyo Love Story',NULL,now(),NULL,84322),
(8699,17092,'Oggy and the Cockroaches',NULL,now(),NULL,82956),
(8700,19724,'Studio DC - Almost Live',NULL,now(),NULL,84328),
(8701,20773,'Around the World in 80 Faiths',NULL,now(),NULL,84336),
(8702,21906,'Candy Girls',NULL,now(),NULL,85881),
(8703,17545,'Dirty Jobs (Australia)',NULL,now(),NULL,84350),
(8704,20835,'Total Wipeout',NULL,now(),NULL,84352),
(8705,5212,'Sitting Ducks',NULL,now(),NULL,84354),
(8706,20836,'Eurovision: Your Country Needs You',NULL,now(),NULL,84357),
(8707,13863,'Kimi wa petto',NULL,now(),NULL,84360),
(8708,21568,'Master Blasters',NULL,now(),NULL,84362),
(8709,20491,'Rude Tube',NULL,now(),NULL,84364),
(8710,16284,'Tough Gig',NULL,now(),NULL,84366),
(8711,20542,'Ruben vs Sophie',NULL,now(),NULL,84367),
(8712,21699,'The Girls of Hedsor Hall',NULL,now(),NULL,85232),
(8713,2358,'Da Capo',NULL,now(),NULL,84187),
(8714,19374,'Above Suspicion',NULL,now(),NULL,84373),
(8715,2649,'Arrested Development',NULL,now(),NULL,72173),
(8716,20677,'Britney: For The Record',NULL,now(),NULL,84387),
(8717,17676,'The Diary of Anne Frank',NULL,now(),NULL,84390),
(8718,17023,'Bar Bender',NULL,now(),NULL,84398),
(8719,20870,'Warzone',NULL,now(),NULL,84402),
(8720,20998,'Daddy\'s Girls (2009)',NULL,now(),NULL,84412),
(8721,20818,'Being Erica',NULL,now(),NULL,84414),
(8722,17995,'Medea',NULL,now(),NULL,84417),
(8723,5221,'Sleepwalkers',NULL,now(),NULL,76502),
(8724,20871,'Oz & James Drink To Britain',NULL,now(),NULL,84423),
(8725,14531,'Just Love',NULL,now(),NULL,84427),
(8726,13752,'Doctor',NULL,now(),NULL,84430),
(8727,20833,'Wild Roses',NULL,now(),NULL,84449),
(8728,20619,'Homeland Security USA',NULL,now(),NULL,84450),
(8729,20071,'Chocolate News',NULL,now(),NULL,83449),
(8730,18371,'Chandon Pictures',NULL,now(),NULL,84451),
(8731,12554,'The Mighty B!',NULL,now(),NULL,84452),
(8732,8541,'Black Cat',NULL,now(),NULL,78987),
(8733,10255,'Seconds From Disaster',NULL,now(),NULL,79772),
(8734,20627,'True Beauty',NULL,now(),NULL,84460),
(8735,20154,'Z-Rock',NULL,now(),NULL,84462),
(8736,20623,'Meet the Browns',NULL,now(),NULL,84467),
(8737,18970,'13 Fear Is Real',NULL,now(),NULL,84468),
(8738,18741,'Under One Roof (2008)',NULL,now(),NULL,84471),
(8739,907,'Children of the Stones',NULL,now(),NULL,84475),
(8740,12945,'The Lost Prince',NULL,now(),NULL,84482),
(8741,20910,'Life of Riley (2008)',NULL,now(),NULL,84483),
(8742,20911,'Victorian Farm',NULL,now(),NULL,84484),
(8743,10461,'Masterchef',NULL,now(),NULL,84489),
(8744,21097,'Personal Justice',NULL,now(),NULL,84493),
(8745,20727,'Rick & Steve: The Happiest Gay Couple in All the World',NULL,now(),NULL,80520),
(8746,18967,'The Mentalist',NULL,now(),NULL,82459),
(8747,20914,'Plus One',NULL,now(),NULL,84500),
(8748,21830,'Half Moon Investigations',NULL,now(),NULL,84498),
(8749,19875,'Oud Geld',NULL,now(),NULL,82098),
(8750,18775,'Sordid Lives: The Series',NULL,now(),NULL,84503),
(8751,1193,'Azumanga Daioh',NULL,now(),NULL,79077),
(8752,20645,'Beehive',NULL,now(),NULL,84507),
(8753,12066,'Cowboy Bebop',NULL,now(),NULL,76885),
(8754,4053,'Jim Henson\'s The Storyteller',NULL,now(),NULL,77747),
(8755,15491,'Darker than Black',NULL,now(),NULL,80042),
(8756,5304,'SpongeBob SquarePants',NULL,now(),NULL,75886),
(8757,12418,'Little Women',NULL,now(),NULL,84523),
(8758,3479,'EyeShield 21',NULL,now(),NULL,79183),
(8759,18936,'Game Show in My Head',NULL,now(),NULL,84525),
(8760,21885,'Spartacus',NULL,now(),NULL,84528),
(8761,20289,'Rookies',NULL,now(),NULL,84533),
(8762,996,'Devil Hunter Yohko',NULL,now(),NULL,84537),
(8763,3453,'Vision of Escaflowne',NULL,now(),NULL,84539),
(8764,12875,'Tales of Eternia',NULL,now(),NULL,84540),
(8765,20226,'Howie Do It',NULL,now(),NULL,84550),
(8766,20414,'Conversations With A Serial Killer',NULL,now(),NULL,84551),
(8767,21069,'Tool Academy',NULL,now(),NULL,84552),
(8768,20561,'White Album',NULL,now(),NULL,84553),
(8769,21829,'One Minute Wonders',NULL,now(),NULL,84555),
(8770,20993,'Unforgiven',NULL,now(),NULL,84557),
(8771,13570,'Food Jammers',NULL,now(),NULL,84568),
(8772,10824,'Mickey Mouse Clubhouse',NULL,now(),NULL,79854),
(8773,13362,'Around the World in 80 Days (1989)',NULL,now(),NULL,84583),
(8774,21652,'RuPaul\'s Drag Race',NULL,now(),NULL,85002),
(8775,20953,'Maria+Holic',NULL,now(),NULL,84591),
(8776,18786,'A Girls Guide  to 21st Century Sex',NULL,now(),NULL,79534),
(8777,15519,'Living Lahaina',NULL,now(),NULL,84604),
(8778,23170,'Svarte Penger, Hvite LÃƒÂ¸gner',NULL,now(),NULL,84587),
(8779,8638,'Nadja',NULL,now(),NULL,84609),
(8780,7877,'The Virgin Queen',NULL,now(),NULL,84614),
(8781,23459,'One outs',NULL,now(),NULL,84616),
(8782,5271,'Space: 1999',NULL,now(),NULL,76366),
(8783,6279,'The War at Home',NULL,now(),NULL,78902),
(8784,1147,'Silverwing',NULL,now(),NULL,84619),
(8785,18802,'The Philanthropist',NULL,now(),NULL,84644),
(8786,21197,'Southland',NULL,now(),NULL,84645),
(8787,20996,'Legally Mad',NULL,now(),NULL,84646),
(8788,3602,'Freakylinks',NULL,now(),NULL,84649),
(8789,20663,'Superstars of Dance',NULL,now(),NULL,84656),
(8790,21068,'Treasure Quest',NULL,now(),NULL,84657),
(8791,3509,'Family Matters',NULL,now(),NULL,76732),
(8792,16575,'Rad Girls',NULL,now(),NULL,84659),
(8793,6951,'The New Tomorrow',NULL,now(),NULL,84661),
(8794,23851,'Chuggington',NULL,now(),NULL,85034),
(8795,20821,'Hunter (2009)',NULL,now(),NULL,84673),
(8796,21103,'Million Dollar Traders',NULL,now(),NULL,84674),
(8797,20987,'RideBack',NULL,now(),NULL,84576),
(8798,21104,'Big Chef Takes on Little Chef',NULL,now(),NULL,84687),
(8799,11822,'The Dream with Roy and HG',NULL,now(),NULL,84691),
(8800,20958,'Asu no Yoichi!',NULL,now(),NULL,84573),
(8801,20467,'The Department Store',NULL,now(),NULL,84703),
(8802,13110,'Vanity Fair',NULL,now(),NULL,84704),
(8803,9222,'The Camomile Lawn',NULL,now(),NULL,84706),
(8804,20969,'Forbidden Science',NULL,now(),NULL,84708),
(8805,22423,'Noble House',NULL,now(),NULL,82200),
(8806,12419,'Dogtanian and the Three Muskehounds',NULL,now(),NULL,74972),
(8807,11168,'Paul Merton\'s Silent Clowns',NULL,now(),NULL,84732),
(8808,20912,'Total Emergency',NULL,now(),NULL,84736),
(8809,2445,'24',NULL,now(),NULL,76290),
(8810,13701,'At Home Dad',NULL,now(),NULL,84742),
(8811,15988,'Attention Please',NULL,now(),NULL,84745),
(8812,13893,'long vacation',NULL,now(),NULL,84747),
(8813,13874,'Love & Fight',NULL,now(),NULL,84753),
(8814,16204,'Moto Kare',NULL,now(),NULL,84755),
(8815,17061,'Kamen Rider Dragon Knight',NULL,now(),NULL,84758),
(8816,3326,'Doc',NULL,now(),NULL,84765),
(8817,22744,'Zeni Geba',NULL,now(),NULL,84778),
(8818,20830,'The Phone',NULL,now(),NULL,84783),
(8819,13895,'Last Christmas',NULL,now(),NULL,84816),
(8820,12078,'CNN Presents',NULL,now(),NULL,84797),
(8821,10276,'Cromartie High School',NULL,now(),NULL,76057),
(8822,13882,'Kou Kou Kyoushi (2003)',NULL,now(),NULL,84817),
(8823,13684,'Aijou Ippon',NULL,now(),NULL,84804),
(8824,14045,'Summer Snow',NULL,now(),NULL,84810),
(8825,13806,'Heaven\'s Coins',NULL,now(),NULL,84815),
(8826,14032,'It was sudden, like a storm...',NULL,now(),NULL,84814),
(8827,13804,'Hitotsu Yane no Shita',NULL,now(),NULL,84820),
(8828,13680,'Ace wo Nerae!',NULL,now(),NULL,84821),
(8829,13702,'Beautiful Life',NULL,now(),NULL,84822),
(8830,13706,'Boku dake no Madonna',NULL,now(),NULL,84823),
(8831,8005,'H2',NULL,now(),NULL,84824),
(8832,3493,'Falcon Crest',NULL,now(),NULL,77109),
(8833,16061,'Hanayome wa Yakudoshi!',NULL,now(),NULL,84825),
(8834,13823,'Itoshi Kimi e',NULL,now(),NULL,84826),
(8835,20926,'Binbo Danshi',NULL,now(),NULL,84827),
(8836,13955,'A Sleeping Forest',NULL,now(),NULL,84828),
(8837,13993,'Pride',NULL,now(),NULL,84831),
(8838,15309,'Mobile',NULL,now(),NULL,84832),
(8839,20296,'Real Chance of Love',NULL,now(),NULL,84833),
(8840,13866,'Kisarazu Cat\'s Eye',NULL,now(),NULL,84836),
(8841,20543,'Combat Chefs',NULL,now(),NULL,84840),
(8842,12669,'Petopeto-san',NULL,now(),NULL,84841),
(8843,12171,'The First Eden',NULL,now(),NULL,84843),
(8844,17752,'Million Dollar Password',NULL,now(),NULL,84844),
(8845,13844,'Tokio',NULL,now(),NULL,84855),
(8846,2733,'Baywatch',NULL,now(),NULL,70874),
(8847,5957,'The L Word',NULL,now(),NULL,72477),
(8848,17199,'America at a Crossroads',NULL,now(),NULL,84877),
(8849,20671,'The Detonators',NULL,now(),NULL,84879),
(8850,4596,'My So-Called Life',NULL,now(),NULL,78644),
(8851,12348,'Kage Kara Mamoru!',NULL,now(),NULL,84890),
(8852,12152,'Rave Master',NULL,now(),NULL,72973),
(8853,5116,'Scrapped Princess',NULL,now(),NULL,79645),
(8854,6061,'The Office (US)',NULL,now(),NULL,73244),
(8855,5481,'Teenage Mutant Ninja Turtles (1987)',NULL,now(),NULL,74582),
(8856,10561,'Merlin\'s Apprentice',NULL,now(),NULL,84895),
(8857,18686,'Playing Shakespeare',NULL,now(),NULL,84909),
(8858,19833,'Yu-Gi-Oh! 5D\'s',NULL,now(),NULL,84911),
(8859,4375,'Married ... with Children',NULL,now(),NULL,76385),
(8860,4204,'Law & Order: Special Victims Unit',NULL,now(),NULL,75692),
(8861,2254,'The Planets',NULL,now(),NULL,75062),
(8862,3332,'Doctor Who (2005)',NULL,now(),NULL,78804),
(8863,21721,'RAW Comedy Club',NULL,now(),NULL,84920),
(8864,21622,'Moses Jones',NULL,now(),NULL,84925),
(8865,21527,'Whitechapel',NULL,now(),NULL,84926),
(8866,14162,'Full House (Korean)',NULL,now(),NULL,84928),
(8867,23386,'Roary the Racing Car',NULL,now(),NULL,84800),
(8868,1811,'Slayers',NULL,now(),NULL,77681),
(8869,21686,'Parks And Recreation',NULL,now(),NULL,84912),
(8870,18387,'Wildside (Australia)',NULL,now(),NULL,84936),
(8871,2882,'Boston Legal',NULL,now(),NULL,74058),
(8872,3741,'Grey\'s Anatomy',NULL,now(),NULL,73762),
(8873,21528,'Doctors and Nurses at War',NULL,now(),NULL,84939),
(8874,11083,'Countryfile',NULL,now(),NULL,84945),
(8875,10191,'xxxHOLiC',NULL,now(),NULL,79283),
(8876,22606,'Treme',NULL,now(),NULL,84946),
(8877,23561,'Boardwalk Empire',NULL,now(),NULL,84947),
(8878,23107,'Nicholas Nickleby',NULL,now(),NULL,84949),
(8879,17987,'Vuurzee',NULL,now(),NULL,84952),
(8880,16421,'Cold Blood',NULL,now(),NULL,84953),
(8881,1400,'Amazing Nurse Nanako',NULL,now(),NULL,84957),
(8882,6257,'The Trouble with Normal',NULL,now(),NULL,75606),
(8883,19770,'Sonny With a Chance',NULL,now(),NULL,84963),
(8884,21586,'The Old Guys',NULL,now(),NULL,84986),
(8885,22756,'Eastwick',NULL,now(),NULL,95291),
(8886,4338,'Magnum, P.I.',NULL,now(),NULL,74380),
(8887,23794,'Lava-Lava!',NULL,now(),NULL,84995),
(8888,21889,'Iran and the West',NULL,now(),NULL,85000),
(8889,3750,'Great Teacher Onizuka (1999)',NULL,now(),NULL,79071),
(8890,5325,'Stargate SG-1',NULL,now(),NULL,72449),
(8891,20628,'Chopped',NULL,now(),NULL,85019),
(8892,20565,'The Chopping Block',NULL,now(),NULL,85020),
(8893,6628,'Without a Trace',NULL,now(),NULL,77963),
(8894,21696,'The CollegeHumor Show',NULL,now(),NULL,85023),
(8895,21646,'Hows Your News',NULL,now(),NULL,85026),
(8896,21834,'In the Motherhood',NULL,now(),NULL,85027),
(8897,18580,'John Adams',NULL,now(),NULL,81547),
(8898,21779,'Customs',NULL,now(),NULL,85030),
(8899,21698,'Nitro Circus',NULL,now(),NULL,85031),
(8900,3416,'Earth: Final Conflict',NULL,now(),NULL,71784),
(8901,21695,'Rob Dyrdek\'s Fantasy Factory',NULL,now(),NULL,85038),
(8902,20913,'Cowboy Builders',NULL,now(),NULL,85047),
(8903,18559,'Macross Frontier',NULL,now(),NULL,81086),
(8904,21637,'Free Agents',NULL,now(),NULL,85055),
(8905,20802,'Important Things With Demetri Martin',NULL,now(),NULL,85061),
(8906,21677,'How Britain was Built',NULL,now(),NULL,85058),
(8907,8469,'Wind: A breath of Heart',NULL,now(),NULL,85064),
(8908,13445,'Terminator: The Sarah Connor Chronicles',NULL,now(),NULL,80344),
(8909,12287,'Love Love?',NULL,now(),NULL,85065),
(8910,12071,'Meine Liebe',NULL,now(),NULL,85066),
(8911,17720,'Gardening Australia',NULL,now(),NULL,85074),
(8912,21608,'Nature\'s Great Events',NULL,now(),NULL,85083),
(8913,21529,'Minder (2009)',NULL,now(),NULL,85088),
(8914,16213,'My Boss, My Hero',NULL,now(),NULL,85091),
(8915,21700,'T.I.\'s Road to Redemption',NULL,now(),NULL,85095),
(8916,12829,'Blizzard Ã¢â‚¬â€œ Race to the Pole',NULL,now(),NULL,85096),
(8917,22041,'Aussie Ladette To Lady',NULL,now(),NULL,85104),
(8918,19583,'From G\'s To Gents',NULL,now(),NULL,85107),
(8919,11718,'Little House on the Prairie (2005) ',NULL,now(),NULL,85109),
(8920,825,'Band Of Gold',NULL,now(),NULL,85115),
(8921,21943,'Building the Olympic Dream',NULL,now(),NULL,85382),
(8922,6697,'Mobile Suit Zeta Gundam',NULL,now(),NULL,71535),
(8923,3420,'Ed',NULL,now(),NULL,73902),
(8924,6024,'The New Adventures of Winnie the Pooh',NULL,now(),NULL,78442),
(8925,21665,'Heli-Loggers',NULL,now(),NULL,85331),
(8926,3624,'FullMetal Alchemist',NULL,now(),NULL,75579),
(8927,1308,'Grass',NULL,now(),NULL,85127),
(8928,6319,'The Young Indiana Jones Chronicles',NULL,now(),NULL,72452),
(8929,19400,'Vendetta',NULL,now(),NULL,85131),
(8930,3347,'Do over',NULL,now(),NULL,85136),
(8931,21723,'The Victorians',NULL,now(),NULL,85141),
(8932,14033,'Shota no Sushi',NULL,now(),NULL,85152),
(8933,20135,'Aaron Stone',NULL,now(),NULL,85156),
(8934,4146,'Knight Rider',NULL,now(),NULL,77216),
(8935,19809,'Birdy the Mighty Decode',NULL,now(),NULL,82884),
(8936,21868,'Dogg After Dark',NULL,now(),NULL,85172),
(8937,19654,'Discovery Project Earth',NULL,now(),NULL,84305),
(8938,22054,'Stewart Lee\'s Comedy Vehicle',NULL,now(),NULL,85535),
(8939,12839,'Ichigo 100%',NULL,now(),NULL,80759),
(8940,22436,'Masada',NULL,now(),NULL,85185),
(8941,17322,'Man Made',NULL,now(),NULL,85188),
(8942,20799,'Delocated',NULL,now(),NULL,85190),
(8943,9139,'Hellsing',NULL,now(),NULL,71278),
(8944,12308,'Moomin',NULL,now(),NULL,85198),
(8945,2537,'Alias',NULL,now(),NULL,75930),
(8946,3114,'Columbo',NULL,now(),NULL,70369),
(8947,18198,'Law & Order: UK',NULL,now(),NULL,85228),
(8948,21024,'The Summit',NULL,now(),NULL,85219),
(8949,12549,'Space Battleship Yamato',NULL,now(),NULL,85230),
(8950,12046,'Gilgamesh',NULL,now(),NULL,79306),
(8951,15133,'The Naked Brothers Band',NULL,now(),NULL,85234),
(8952,15358,'Once Upon a Time... Life',NULL,now(),NULL,79158),
(8953,2628,'Ant and Dec\'s Saturday Night Takeaway',NULL,now(),NULL,85241),
(8954,7595,'Hot Pursuit (2006)',NULL,now(),NULL,85244),
(8955,23888,'World\'s Most Amazing Videos',NULL,now(),NULL,85252),
(8956,17671,'Rain Shadow',NULL,now(),NULL,85255),
(8957,8266,'Rose of Versailles',NULL,now(),NULL,80504),
(8958,16462,'Bylo nÃƒÂ¡s pÃ„â€ºt',NULL,now(),NULL,85184),
(8959,8205,'One Piece',NULL,now(),NULL,81797),
(8960,10435,'The Smith & Jones Sketchbook',NULL,now(),NULL,85262),
(8961,7194,'Academy Awards',NULL,now(),NULL,85269),
(8962,17190,'Dumped',NULL,now(),NULL,91141),
(8963,12662,'True Blood',NULL,now(),NULL,82283),
(8964,11092,'Saturday Kitchen',NULL,now(),NULL,85284);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(8965,3591,'Fraggle Rock',NULL,now(),NULL,76085),
(8966,21063,'The Tonight Show with Conan O\'Brien',NULL,now(),NULL,85285),
(8967,15138,'Primeval',NULL,now(),NULL,79809),
(8968,3812,'He-Man and the Masters of the Universe (2002)',NULL,now(),NULL,71179),
(8969,3464,'Everwood',NULL,now(),NULL,78465),
(8970,23969,'No Signal',NULL,now(),NULL,85293),
(8971,7936,'Lewis',NULL,now(),NULL,79864),
(8972,7926,'Dexter',NULL,now(),NULL,79349),
(8973,21598,'Boys and Girls Alone',NULL,now(),NULL,85307),
(8974,4628,'NCIS',NULL,now(),NULL,72108),
(8975,21887,'American Loggers',NULL,now(),NULL,85317),
(8976,15824,'Alice in Wonderland',NULL,now(),NULL,85318),
(8977,18739,'Grand Designs Trade Secrets',NULL,now(),NULL,85324),
(8978,16634,'The Tower',NULL,now(),NULL,85326),
(8979,18083,'Picture This',NULL,now(),NULL,85328),
(8980,17440,'Dinosaur King',NULL,now(),NULL,85329),
(8981,21969,'FM (2009)',NULL,now(),NULL,85330),
(8982,5336,'Star Trek: The Animated Series',NULL,now(),NULL,73566),
(8983,6691,'Yu-Gi-Oh! GX',NULL,now(),NULL,76113),
(8984,23638,'Marked',NULL,now(),NULL,111101),
(8985,22267,'Sex: How To Do Everything',NULL,now(),NULL,85341),
(8986,21878,'Monty Halls\' Great Escape',NULL,now(),NULL,85345),
(8987,18574,'Melodifestivalen',NULL,now(),NULL,85348),
(8988,8296,'Platinum Weddings',NULL,now(),NULL,85349),
(8989,130,'Spellbinder',NULL,now(),NULL,71894),
(8990,5524,'The Adventures of Brisco County, Jr.',NULL,now(),NULL,72364),
(8991,20843,'Late Night with Jimmy Fallon',NULL,now(),NULL,85355),
(8992,11709,'Good News Week',NULL,now(),NULL,85357),
(8993,5179,'She Spies',NULL,now(),NULL,78652),
(8994,1936,'Mother and Son',NULL,now(),NULL,85366),
(8995,5949,'The Lost World',NULL,now(),NULL,78457),
(8996,5056,'Roswell',NULL,now(),NULL,73965),
(8997,21572,'The Penguins of Madagascar',NULL,now(),NULL,85075),
(8998,21980,'Red Riding',NULL,now(),NULL,85387),
(8999,3260,'Deadliest Catch',NULL,now(),NULL,78957),
(9000,13040,'The Dark Side of Porn',NULL,now(),NULL,85394),
(9001,3296,'Detective Conan',NULL,now(),NULL,72454),
(9002,20933,'The Passionate Eye',NULL,now(),NULL,85423),
(9003,21232,'Orangutan Diary',NULL,now(),NULL,85420),
(9004,22001,'Haborer',NULL,now(),NULL,82687),
(9005,22625,'No Heroics (US)',NULL,now(),NULL,85426),
(9006,22022,'Melrose Place (2009)',NULL,now(),NULL,85427),
(9007,2332,'Steam Detectives',NULL,now(),NULL,85428),
(9008,22013,'Battles BC',NULL,now(),NULL,85432),
(9009,20245,'Breaking the Magician\'s Code: Magic\'s Biggest Secrets Finally Revealed',NULL,now(),NULL,85433),
(9010,18806,'Horne and Corden',NULL,now(),NULL,85440),
(9011,21896,'Darwin\'s Dangerous Idea',NULL,now(),NULL,85441),
(9012,23775,'Last Chance to See',NULL,now(),NULL,85453),
(9013,22520,'League of Super Evil',NULL,now(),NULL,85459),
(9014,22087,'Diplomatic Immunity',NULL,now(),NULL,85460),
(9015,4540,'Mr Men and Little Miss',NULL,now(),NULL,85462),
(9016,22454,'Empire Falls',NULL,now(),NULL,84737),
(9017,22040,'Warriors',NULL,now(),NULL,85467),
(9018,21840,'Al Murray\'s Multiple Personality Disorder',NULL,now(),NULL,85468),
(9019,18693,'The Crow Road',NULL,now(),NULL,85471),
(9020,19011,'Do Not Disturb',NULL,now(),NULL,82304),
(9021,2886,'Boy Meets Boy',NULL,now(),NULL,72866),
(9022,18691,'Step It Up and Dance',NULL,now(),NULL,81834),
(9023,5139,'Secret Agent Man',NULL,now(),NULL,85476),
(9024,14006,'Ruri no Shima',NULL,now(),NULL,84744),
(9025,19633,'Chimera',NULL,now(),NULL,85483),
(9026,19345,'Meebox',NULL,now(),NULL,85487),
(9027,4959,'Red Dwarf',NULL,now(),NULL,71326),
(9028,2087,'Serial Killers',NULL,now(),NULL,85494),
(9029,12794,'Alive',NULL,now(),NULL,85495),
(9030,4084,'Jumpers for Goalposts',NULL,now(),NULL,85505),
(9031,21678,'For the Love of Ray J',NULL,now(),NULL,85519),
(9032,15415,'Flight of the Conchords',NULL,now(),NULL,80252),
(9033,5301,'Spin City',NULL,now(),NULL,73153),
(9034,9809,'Scully',NULL,now(),NULL,85531),
(9035,8311,'Metal Armor Dragonar',NULL,now(),NULL,85536),
(9036,8239,'Maddigan\'s Quest',NULL,now(),NULL,85537),
(9037,19186,'Storm of the Century',NULL,now(),NULL,85538),
(9038,19316,'The Pacific',NULL,now(),NULL,85539),
(9039,22118,'Ballar av stÃƒÂ¥l',NULL,now(),NULL,85534),
(9040,21174,'Tenchijin',NULL,now(),NULL,85541),
(9041,13736,'What about Love?',NULL,now(),NULL,85542),
(9042,18355,'The Mr Men Show',NULL,now(),NULL,85545),
(9043,15622,'The Best Years',NULL,now(),NULL,85546),
(9044,11076,'Eurotrash',NULL,now(),NULL,85550),
(9045,21995,'WCG Ultimate Gamer',NULL,now(),NULL,85556),
(9046,5420,'Survivors',NULL,now(),NULL,78544),
(9047,5118,'Scrubs',NULL,now(),NULL,76156),
(9048,19955,'Imagination Movers',NULL,now(),NULL,85568),
(9049,22675,'New Zealand\'s Next Top Model',NULL,now(),NULL,85569),
(9050,12467,'Bad Lads Army',NULL,now(),NULL,85570),
(9051,15759,'Aliens In America',NULL,now(),NULL,80376),
(9052,21908,'Tough Love',NULL,now(),NULL,85581),
(9053,20259,'Guinea Pig',NULL,now(),NULL,85591),
(9054,18356,'Britain\'s Dream Homes',NULL,now(),NULL,85631),
(9055,22088,'Stormworld',NULL,now(),NULL,85651),
(9056,22316,'Chiefs',NULL,now(),NULL,85671),
(9057,6421,'Trial and Retribution',NULL,now(),NULL,72382),
(9058,21064,'The Exterminators',NULL,now(),NULL,87431),
(9059,12637,'Legends',NULL,now(),NULL,85831),
(9060,8140,'Burn Up! Scramble',NULL,now(),NULL,79048),
(9061,22106,'Easton Meets West',NULL,now(),NULL,85861),
(9062,12731,'Ray the animation',NULL,now(),NULL,85971),
(9063,19544,'Lemur Island',NULL,now(),NULL,85981),
(9064,2729,'Battlestar Galactica',NULL,now(),NULL,71173),
(9065,1168,'The Martian Chronicles',NULL,now(),NULL,72794),
(9066,2800,'The Big O',NULL,now(),NULL,73988),
(9067,1125,'Dual! Parallel Trouble Adventures',NULL,now(),NULL,79996),
(9068,4636,'Neon Genesis Evangelion',NULL,now(),NULL,70350),
(9069,15383,'Burn Notice',NULL,now(),NULL,80270),
(9070,19571,'Pam: Girl on the Loose',NULL,now(),NULL,86221),
(9071,18839,'Roommates',NULL,now(),NULL,86241),
(9072,3039,'Charmed',NULL,now(),NULL,70626),
(9073,22036,'Newswipe',NULL,now(),NULL,86451),
(9074,6360,'Time Trax',NULL,now(),NULL,72401),
(9075,22332,'Lawrence Leung\'s Choose Your Own Adventure',NULL,now(),NULL,86381),
(9076,6920,'Dirty Jobs',NULL,now(),NULL,78904),
(9077,3061,'Chrono Crusade',NULL,now(),NULL,73993),
(9078,9302,'Extraordinary People',NULL,now(),NULL,86541),
(9079,22116,'Bully Beatdown',NULL,now(),NULL,86551),
(9080,7747,'The Adventures of Tintin',NULL,now(),NULL,72668),
(9081,4240,'Life With Derek',NULL,now(),NULL,86611),
(9082,4568,'Mutant X',NULL,now(),NULL,76533),
(9083,4580,'My Gym Partner\'s a Monkey',NULL,now(),NULL,86681),
(9084,2927,'Buck Rogers in the 25th Century',NULL,now(),NULL,74523),
(9085,20647,'Metsolat',NULL,now(),NULL,86761),
(9086,22047,'Real Swiss Family Robinson',NULL,now(),NULL,86801),
(9087,20315,'Co-Ed Confidential',NULL,now(),NULL,86881),
(9088,6614,'Will & Grace',NULL,now(),NULL,71814),
(9089,6054,'The O.C.',NULL,now(),NULL,72164),
(9090,3779,'Hammer House of Horror',NULL,now(),NULL,76799),
(9091,22082,'Alan Whicker\'s Journey of a Lifetime',NULL,now(),NULL,86911),
(9092,8542,'Fate Stay Night',NULL,now(),NULL,79151),
(9093,17357,'Most Daring',NULL,now(),NULL,86991),
(9094,17870,'Earth: The Power of the Planet',NULL,now(),NULL,80865),
(9095,18933,'Eleventh Hour (US)',NULL,now(),NULL,83066),
(9096,23625,'Walk On The Wild Side',NULL,now(),NULL,87061),
(9097,22033,'Chris Moyles Quiz Night',NULL,now(),NULL,87071),
(9098,22063,'Budget TV',NULL,now(),NULL,87141),
(9099,2325,'$25 Million Dollar Hoax',NULL,now(),NULL,87211),
(9100,22090,'All the Small Things',NULL,now(),NULL,87231),
(9101,742,'Revolutionary Girl Utena',NULL,now(),NULL,79449),
(9102,19667,'Osbournes: Reloaded',NULL,now(),NULL,87281),
(9103,18772,'Living with the Dead',NULL,now(),NULL,87291),
(9104,17759,'Mary, Mungo & Midge',NULL,now(),NULL,87321),
(9105,9603,'Metal Mickey',NULL,now(),NULL,87331),
(9106,21706,'Ladies of Letters',NULL,now(),NULL,87341),
(9107,13135,'Oppenheimer (1980)',NULL,now(),NULL,87351),
(9108,19198,'The Rachel Zoe Project',NULL,now(),NULL,87421),
(9109,20028,'Verborgen Gebreken',NULL,now(),NULL,87521),
(9110,2462,'7 Days',NULL,now(),NULL,74540),
(9111,6109,'The Prisoner',NULL,now(),NULL,74805),
(9112,13024,'Australia\'s Got Talent',NULL,now(),NULL,87581),
(9113,5084,'Sabrina The Teenage Witch',NULL,now(),NULL,76876),
(9114,1321,'Insektors',NULL,now(),NULL,87591),
(9115,9172,'Carrott\'s Lib',NULL,now(),NULL,87611),
(9116,22059,'Basquash!',NULL,now(),NULL,87621),
(9117,5389,'Summerland',NULL,now(),NULL,72831),
(9118,13081,'The Incredible Journey of Mary Bryant',NULL,now(),NULL,87691),
(9119,5448,'Tales from the Crypt',NULL,now(),NULL,73029),
(9120,19286,'Surviving Suburbia',NULL,now(),NULL,87731),
(9121,5286,'Special Unit 2',NULL,now(),NULL,80707),
(9122,15913,'Lucky Star',NULL,now(),NULL,80177),
(9123,5355,'Stingray',NULL,now(),NULL,76251),
(9124,4847,'Agatha Christie\'s Poirot',NULL,now(),NULL,76133),
(9125,21217,'Afterworld',NULL,now(),NULL,82486),
(9126,22070,'Pandora Hearts',NULL,now(),NULL,87861),
(9127,6312,'The X-Files',NULL,now(),NULL,77398),
(9128,12172,'The Tribal Eye',NULL,now(),NULL,87911),
(9129,22794,'Madventures',NULL,now(),NULL,81621),
(9130,4554,'Murder, She Wrote',NULL,now(),NULL,78049),
(9131,22072,'Phantom: Requiem for the Phantom',NULL,now(),NULL,87931),
(9132,5877,'The Invisible Man',NULL,now(),NULL,74460),
(9133,22112,'Before Green Gables',NULL,now(),NULL,88041),
(9134,22701,'NYC Prep',NULL,now(),NULL,106751),
(9135,5769,'The Famous Teddy Z',NULL,now(),NULL,73408),
(9136,22121,'Tears to Tiara',NULL,now(),NULL,88101),
(9137,22676,'e2 Design',NULL,now(),NULL,88111),
(9138,14511,'The King Kong Show',NULL,now(),NULL,88131),
(9139,3616,'Friends',NULL,now(),NULL,79168),
(9140,22105,'Slap Up Party: Arad Senki',NULL,now(),NULL,88141),
(9141,20989,'Hetalia Axis Powers',NULL,now(),NULL,88161),
(9142,15536,'Flash Gordon (2007)',NULL,now(),NULL,80441),
(9143,6454,'Two and a Half Men',NULL,now(),NULL,72227),
(9144,22157,'The Speaker',NULL,now(),NULL,88221),
(9145,5047,'Rome',NULL,now(),NULL,73508),
(9146,22903,'Dragon Ball Kai',NULL,now(),NULL,88031),
(9147,19339,'Tabatha\'s Salon Takeover',NULL,now(),NULL,88321),
(9148,22117,'Natsu no Arashi!',NULL,now(),NULL,88391),
(9149,17657,'The Abbey',NULL,now(),NULL,88431),
(9150,22161,'Ristorante Paradiso',NULL,now(),NULL,88481),
(9151,1109,'Nadia of the Mysterious Seas',NULL,now(),NULL,81789),
(9152,6540,'War of the Worlds',NULL,now(),NULL,76616),
(9153,8322,'Psych',NULL,now(),NULL,79335),
(9154,18329,'Underbelly',NULL,now(),NULL,81346),
(9155,18402,'KrÃƒÂ¶d MÃƒÂ¤ndoon and the Flaming Sword of Fire',NULL,now(),NULL,88631),
(9156,22056,'Sengoku Basara',NULL,now(),NULL,87881),
(9157,8261,'The Apprentice: You\'re Fired!',NULL,now(),NULL,88771),
(9158,22159,'07-Ghost',NULL,now(),NULL,88651),
(9159,12519,'Higurashi no Naku Koro ni',NULL,now(),NULL,79682),
(9160,6239,'The Time Tunnel',NULL,now(),NULL,77268),
(9161,22113,'Guin Saga',NULL,now(),NULL,88881),
(9162,22115,'Tayutama: Kiss on my Deity',NULL,now(),NULL,88891),
(9163,3168,'Crime Story',NULL,now(),NULL,72619),
(9164,12751,'Sasuke',NULL,now(),NULL,88861),
(9165,22331,'Trawlers, Rigs & Rescue: North Sea',NULL,now(),NULL,89591),
(9166,21185,'One Way Out',NULL,now(),NULL,88941),
(9167,15149,'Rough Diamond',NULL,now(),NULL,88951),
(9168,22206,'Pretty Wicked',NULL,now(),NULL,88981),
(9169,6970,'Secret Smile',NULL,now(),NULL,88991),
(9170,7964,'\'Til Death Do Us Part',NULL,now(),NULL,89001),
(9171,6823,'Brotherhood',NULL,now(),NULL,79356),
(9172,5097,'Sanford and Son',NULL,now(),NULL,78193),
(9173,22058,'Asura Cryin\'',NULL,now(),NULL,87981),
(9174,22108,'Valkyria Chronicles',NULL,now(),NULL,87851),
(9175,22114,'Shangri-La',NULL,now(),NULL,87871),
(9176,20957,'Viper\'s Creed ',NULL,now(),NULL,84571),
(9177,11608,'Genesis Climber Mospeada',NULL,now(),NULL,89101),
(9178,18557,'Kamen no Maid Guy',NULL,now(),NULL,82086),
(9179,3044,'Cheers',NULL,now(),NULL,77623),
(9180,3635,'G4TV.COM',NULL,now(),NULL,89181),
(9181,21624,'Deadliest Warrior',NULL,now(),NULL,89211),
(9182,6832,'The Unit',NULL,now(),NULL,75707),
(9183,21636,'Van Vlees en Bloed',NULL,now(),NULL,84381),
(9184,1228,'You\'re Under Arrest!',NULL,now(),NULL,80054),
(9185,23644,'Mad Labs',NULL,now(),NULL,89531),
(9186,1821,'Janus',NULL,now(),NULL,89541),
(9187,22162,'Higashi no Eden',NULL,now(),NULL,88731),
(9188,6102,'The Powerpuff Girls',NULL,now(),NULL,76200),
(9189,14904,'Lincoln Heights',NULL,now(),NULL,79921),
(9190,15569,'Meadowlands',NULL,now(),NULL,80223),
(9191,22547,'Britain\'s Best Drives',NULL,now(),NULL,89641),
(9192,16423,'Lost World of the Raj',NULL,now(),NULL,89691),
(9193,614,'Initial D',NULL,now(),NULL,70900),
(9194,18830,'Cupid (2009)',NULL,now(),NULL,83615),
(9195,18411,'Leverage',NULL,now(),NULL,82339),
(9196,19295,'Lie to Me',NULL,now(),NULL,83602),
(9197,5027,'Robot Chicken',NULL,now(),NULL,75734),
(9198,20562,' Genji Monogatari Sennenki ',NULL,now(),NULL,89741),
(9199,4610,'My Wife and Kids',NULL,now(),NULL,70329),
(9200,1222,'Neo Ranga',NULL,now(),NULL,89771),
(9201,848,'Fancy Lala',NULL,now(),NULL,89781),
(9202,12848,'St. Luminous Mission High School',NULL,now(),NULL,89791),
(9203,22257,'The Hospital',NULL,now(),NULL,89821),
(9204,20976,'Kurokami The Animation',NULL,now(),NULL,84561),
(9205,22647,'The Real Housewives of New Jersey',NULL,now(),NULL,89831),
(9206,22287,'Pitchmen',NULL,now(),NULL,90001),
(9207,3677,'Ghost Whisperer',NULL,now(),NULL,78817),
(9208,4724,'One Tree Hill',NULL,now(),NULL,72158),
(9209,22341,'Patton 360',NULL,now(),NULL,90061),
(9210,4274,'Lois & Clark The New Adventures of Superman',NULL,now(),NULL,72468),
(9211,15298,'Sandglass',NULL,now(),NULL,90111),
(9212,7555,'Australian Story',NULL,now(),NULL,90161),
(9213,4698,'NYPD Blue',NULL,now(),NULL,73132),
(9214,22122,'Saki',NULL,now(),NULL,87841),
(9215,7979,'Epitafios',NULL,now(),NULL,82341),
(9216,15485,'Shinkyoku Soukai Polyphonica',NULL,now(),NULL,90341),
(9217,17698,'Final 24',NULL,now(),NULL,90351),
(9218,22110,'Shinkyoku Soukai Polyphonica Crimson S',NULL,now(),NULL,90361),
(9219,18677,'Melvyn Bragg\'s Travels In Written Britain',NULL,now(),NULL,90381),
(9220,3562,'Flipper (1964)',NULL,now(),NULL,70417),
(9221,5222,'Sliders',NULL,now(),NULL,76557),
(9222,19710,'Ultraviolet: Code 044',NULL,now(),NULL,90511),
(9223,8094,'Armored Trooper VOTOMS',NULL,now(),NULL,90521),
(9224,2743,'Beautiful People',NULL,now(),NULL,78851),
(9225,18979,'Sit Down, Shut Up',NULL,now(),NULL,90571),
(9226,22160,'Souten Kouro',NULL,now(),NULL,90641),
(9227,22840,'Dirt Game',NULL,now(),NULL,90681),
(9228,5947,'The Lost Islands',NULL,now(),NULL,90471),
(9229,21647,'Doing Davinci',NULL,now(),NULL,90731),
(9230,20191,'World\'s Toughest Fixes ',NULL,now(),NULL,90771),
(9231,686,'Walking with Beasts',NULL,now(),NULL,78854),
(9232,16471,'Ice Road Truckers',NULL,now(),NULL,80273),
(9233,21946,'Dennis VS Valerio',NULL,now(),NULL,90801),
(9234,19212,'Surviving History',NULL,now(),NULL,90811),
(9235,6504,'Veritas: The Quest',NULL,now(),NULL,70365),
(9236,22334,'Bara no nai Hanaya',NULL,now(),NULL,90891),
(9237,4909,'Psi Factor: Chronicles of the Paranormal',NULL,now(),NULL,72469),
(9238,2498,'Adventure Inc',NULL,now(),NULL,83647),
(9239,6353,'Thunder in Paradise',NULL,now(),NULL,78583),
(9240,18076,'Auschwitz: The Nazis & The Final Solution',NULL,now(),NULL,78968),
(9241,4921,'Quantum Leap',NULL,now(),NULL,72248),
(9242,11236,'Escape To The Country',NULL,now(),NULL,91171),
(9243,22333,'Reggie Perrin 2009',NULL,now(),NULL,91251),
(9244,7551,'The 7.30 Report',NULL,now(),NULL,91261),
(9245,1556,'Fur TV',NULL,now(),NULL,82513),
(9246,15189,'Journeyman',NULL,now(),NULL,80494),
(9247,5892,'The Jetsons',NULL,now(),NULL,76204),
(9248,4831,'Pinky and the Brain',NULL,now(),NULL,72880),
(9249,5347,'Stephen King\'s Golden Years',NULL,now(),NULL,71107),
(9250,5055,'Roseanne',NULL,now(),NULL,77068),
(9251,10335,'A Perfect Spy',NULL,now(),NULL,91731),
(9252,13640,'One Summer',NULL,now(),NULL,91641),
(9253,12689,'Princess Princess',NULL,now(),NULL,84028),
(9254,22319,'Daisy of Love',NULL,now(),NULL,91661),
(9255,20649,'Chris Ryan\'s Elite World Cops',NULL,now(),NULL,91741),
(9256,23409,'Love Shuffle',NULL,now(),NULL,91781),
(9257,10733,'The Age of Uncertainty',NULL,now(),NULL,91861),
(9258,2413,'Kyo Kara Maoh!',NULL,now(),NULL,91871),
(9259,6888,'The Story of God',NULL,now(),NULL,91891),
(9260,1407,'Dragon Half',NULL,now(),NULL,91911),
(9261,8172,'Heroes',NULL,now(),NULL,79501),
(9262,20601,'United States of Tara',NULL,now(),NULL,83463),
(9263,2859,'Bobobo-bo Bo-bobo',NULL,now(),NULL,79355),
(9264,17844,'Exes & Ohs',NULL,now(),NULL,80736),
(9265,23108,'Napoleon',NULL,now(),NULL,81712),
(9266,21023,'The Line',NULL,now(),NULL,74849),
(9267,12494,'The Melancholy of Haruhi Suzumiya',NULL,now(),NULL,79414),
(9268,17229,'Growing Up Creepie',NULL,now(),NULL,92341),
(9269,18834,'Merlin (2008)',NULL,now(),NULL,83123),
(9270,5240,'Special Ops Force',NULL,now(),NULL,72081),
(9271,5326,'Starhunter',NULL,now(),NULL,78821),
(9272,7488,'Murder 101',NULL,now(),NULL,92401),
(9273,22336,'Royal Pains',NULL,now(),NULL,92411),
(9274,22652,'Weaponizers',NULL,now(),NULL,92451),
(9275,12789,'Body Image',NULL,now(),NULL,92511),
(9276,22367,'Boy Meets Girl',NULL,now(),NULL,92521),
(9277,18558,'Kanokon',NULL,now(),NULL,81830),
(9278,24003,'The Pam Ann Show',NULL,now(),NULL,92651),
(9279,17964,'Insight (1999)',NULL,now(),NULL,92691),
(9280,12173,'Wildlife on One',NULL,now(),NULL,92711),
(9281,15561,'Cities of the Underworld',NULL,now(),NULL,80373),
(9282,18455,'When Were We Funniest?',NULL,now(),NULL,92741),
(9283,6677,'Yes Minister',NULL,now(),NULL,76829),
(9284,6622,'Winx Club',NULL,now(),NULL,74256),
(9285,22511,'Around the World in 80 Trades',NULL,now(),NULL,92801),
(9286,4466,'Midsomer Murders',NULL,now(),NULL,76846),
(9287,9277,'The Dame Edna Experience',NULL,now(),NULL,92971),
(9288,15274,'The Dame Edna Treatment',NULL,now(),NULL,92981),
(9289,10068,'Dame Edna\'s Neighbourhood Watch',NULL,now(),NULL,92991),
(9290,3417,'Earthworm Jim',NULL,now(),NULL,76993),
(9291,21070,'MoCap, LLC',NULL,now(),NULL,93291),
(9292,2692,'Baby Bob',NULL,now(),NULL,78066),
(9293,17373,'Property Virgins',NULL,now(),NULL,93351),
(9294,2615,'Animal Cops: Houston',NULL,now(),NULL,93361),
(9295,10770,'Animal Cops: Detroit',NULL,now(),NULL,93371),
(9296,19592,'Animal Cops: Philadelphia',NULL,now(),NULL,93381),
(9297,19226,'Animal Cops: South Africa',NULL,now(),NULL,93391),
(9298,22695,'Extreme Male Beauty',NULL,now(),NULL,93621),
(9299,18164,'Breaking Bad',NULL,now(),NULL,81189),
(9300,19841,'Sandhogs',NULL,now(),NULL,83052),
(9301,3725,'Goof Troop',NULL,now(),NULL,78250),
(9302,8004,'Pretty Cure',NULL,now(),NULL,84334),
(9303,23935,'Atashinchi no Danshi',NULL,now(),NULL,93791),
(9304,9530,'A Life in Pieces',NULL,now(),NULL,93861),
(9305,14191,'I\'m Sorry, I Love You',NULL,now(),NULL,93971),
(9306,6507,'Veronica Mars',NULL,now(),NULL,73730),
(9307,22592,'The Incredible Human Journey',NULL,now(),NULL,93951),
(9308,3941,'I Dream of Jeannie',NULL,now(),NULL,77388),
(9309,18740,'The Cleveland Show',NULL,now(),NULL,93991),
(9310,16055,'Haken no Hinkaku',NULL,now(),NULL,94001),
(9311,5781,'The Flash',NULL,now(),NULL,78650),
(9312,18388,'Fringe',NULL,now(),NULL,82066),
(9313,13754,'Double Kitchen',NULL,now(),NULL,94061),
(9314,10040,'Candidate for Goddess',NULL,now(),NULL,79778),
(9315,22373,'Martin Clunes: Islands Of Britain',NULL,now(),NULL,94141),
(9316,3414,'Early Edition',NULL,now(),NULL,74295),
(9317,15526,'Katie and Peter: The Next Chapter',NULL,now(),NULL,94211),
(9318,18370,'10 Years Younger',NULL,now(),NULL,94241),
(9319,5685,'The Closer',NULL,now(),NULL,74875),
(9320,18333,'Black Blood Brothers',NULL,now(),NULL,79412),
(9321,11610,'Life on Mars (US)',NULL,now(),NULL,82289),
(9322,10226,'Bands on the Run',NULL,now(),NULL,94371),
(9323,2109,'Texhnolyze',NULL,now(),NULL,74493),
(9324,13338,'Quatermass II',NULL,now(),NULL,94421),
(9325,13336,'Quatermass and the Pit',NULL,now(),NULL,94431),
(9326,13337,'The Quatermass Experiment',NULL,now(),NULL,77791),
(9327,1841,'Quatermass',NULL,now(),NULL,94441),
(9328,15576,'Private Practice',NULL,now(),NULL,80542),
(9329,3261,'Dead Like Me',NULL,now(),NULL,72129),
(9330,14838,'Tatort',NULL,now(),NULL,83214),
(9331,15221,'Best Evidence',NULL,now(),NULL,100691),
(9332,3683,'Gilmore Girls',NULL,now(),NULL,76568),
(9333,5227,'Smallville',NULL,now(),NULL,72218),
(9334,22021,'Day One',NULL,now(),NULL,94531),
(9335,21844,'Trauma',NULL,now(),NULL,94541),
(9336,22586,'Parenthood (2009)',NULL,now(),NULL,94551),
(9337,22587,'Mercy',NULL,now(),NULL,94561),
(9338,22734,'100 Questions',NULL,now(),NULL,94581),
(9339,21531,'Epicurious',NULL,now(),NULL,94591),
(9340,18531,'Flashpoint',NULL,now(),NULL,82438),
(9341,22580,'Recruits',NULL,now(),NULL,94631),
(9342,5343,'State of Play',NULL,now(),NULL,80383),
(9343,17974,'Lark Rise to Candleford',NULL,now(),NULL,81161),
(9344,15150,'Cranford Chronicles',NULL,now(),NULL,80965),
(9345,2930,'Buffy the Vampire Slayer',NULL,now(),NULL,70327),
(9346,16083,'The Universe',NULL,now(),NULL,80198),
(9347,10344,'Devilman',NULL,now(),NULL,94701),
(9348,14583,'Rooftop Room Cat',NULL,now(),NULL,104341),
(9349,4672,'Night Visions',NULL,now(),NULL,94731),
(9350,23310,'Wainwright Walks Coast to Coast',NULL,now(),NULL,94751),
(9351,2868,'Bonanza',NULL,now(),NULL,73378),
(9352,22699,'Human Target (2009)',NULL,now(),NULL,94801),
(9353,22712,'Past Life',NULL,now(),NULL,94821),
(9354,22674,'Sons of Tucson',NULL,now(),NULL,94831),
(9355,22735,'Brothers (2009)',NULL,now(),NULL,94841),
(9356,22613,'Moving On',NULL,now(),NULL,94941),
(9357,22814,'V (2009)',NULL,now(),NULL,94971),
(9358,22757,'The Forgotten',NULL,now(),NULL,94981),
(9359,22626,'Cougar Town',NULL,now(),NULL,94991),
(9360,22813,'Hank (2009)',NULL,now(),NULL,95001),
(9361,22622,'Modern Family',NULL,now(),NULL,95011),
(9362,20678,'The Middle',NULL,now(),NULL,95021),
(9363,22812,'The Deep End',NULL,now(),NULL,95031),
(9364,21843,'Happy Town',NULL,now(),NULL,95041),
(9365,19187,'Star Wars - The Clone Wars',NULL,now(),NULL,83268),
(9366,2636,'A Pup Named Scooby-Doo',NULL,now(),NULL,73546),
(9367,15713,'Samantha Who?',NULL,now(),NULL,80680),
(9368,6181,'The Sentinel',NULL,now(),NULL,71156),
(9369,3918,'How I Met Your Mother',NULL,now(),NULL,75760),
(9370,18202,'Hope Springs',NULL,now(),NULL,95311),
(9371,4706,'Off Centre',NULL,now(),NULL,76816),
(9372,6523,'Voyagers!',NULL,now(),NULL,74579),
(9373,3765,'Gunslinger Girl',NULL,now(),NULL,78812),
(9374,22754,'Three Rivers',NULL,now(),NULL,95461),
(9375,22771,'Accidentally On Purpose',NULL,now(),NULL,95471),
(9376,23114,'Life UneXpected',NULL,now(),NULL,95511),
(9377,18820,'Ian Hislop Goes Off the Rails',NULL,now(),NULL,95541),
(9378,22546,'Keep It in the Family (2009)',NULL,now(),NULL,95661),
(9379,22838,'Mumbai Calling',NULL,now(),NULL,95681),
(9380,18403,'The Goode Family',NULL,now(),NULL,95711),
(9381,19651,'Nurse Jackie',NULL,now(),NULL,95731),
(9382,21648,'Hawthorne',NULL,now(),NULL,95741),
(9383,22679,'Make it or Break it',NULL,now(),NULL,95751),
(9384,21702,'10 Things I Hate About You',NULL,now(),NULL,95761),
(9385,21886,'Drop Dead Diva',NULL,now(),NULL,95771),
(9386,21649,'Dark Blue',NULL,now(),NULL,95781),
(9387,21703,'Ruby & The Rockits',NULL,now(),NULL,95791),
(9388,22731,'Players (2009)',NULL,now(),NULL,95801),
(9389,4995,'Revelations',NULL,now(),NULL,73997),
(9390,13796,'Hero',NULL,now(),NULL,95821),
(9391,11117,'Angelmouse',NULL,now(),NULL,95851),
(9392,17703,'The Cook and the Chef',NULL,now(),NULL,95921),
(9393,24011,'Food Investigators',NULL,now(),NULL,95931),
(9394,6432,'Tru Calling',NULL,now(),NULL,72233),
(9395,23032,'Talkin\' \'Bout Your Generation',NULL,now(),NULL,96001),
(9396,1281,'Hustle',NULL,now(),NULL,73028),
(9397,4514,'Monk',NULL,now(),NULL,78490),
(9398,20621,'Jesse James Is a Dead Man',NULL,now(),NULL,96071),
(9399,13819,'IWGP',NULL,now(),NULL,96081),
(9400,13784,'Golden Bowl',NULL,now(),NULL,96111),
(9401,16368,'Tokyo Wankei',NULL,now(),NULL,96151),
(9402,16275,'Satorare',NULL,now(),NULL,96161),
(9403,13711,'Boku to Kanojo to Kanojo no Ikiru Michi',NULL,now(),NULL,96181),
(9404,18619,'Hollywood Residential',NULL,now(),NULL,96201),
(9405,11061,'Piece of Cake',NULL,now(),NULL,96211),
(9406,22678,'Obsessed',NULL,now(),NULL,96301),
(9407,684,'Snow Queen',NULL,now(),NULL,96281),
(9408,22692,'The Operation: Surgery Live',NULL,now(),NULL,96321),
(9409,6203,'The Snorks',NULL,now(),NULL,76305),
(9410,22682,'Diamonds (2009)',NULL,now(),NULL,96361),
(9411,4284,'Lost',NULL,now(),NULL,73739),
(9412,3287,'Dennis the Menace (1986)',NULL,now(),NULL,76423),
(9413,2548,'All-New Dennis the Menace',NULL,now(),NULL,72186),
(9414,23078,'Coin Locker Monogatari',NULL,now(),NULL,96441),
(9415,13912,'Majo no Jouken',NULL,now(),NULL,96481),
(9416,9702,'Ginga Nagareboshi Gin',NULL,now(),NULL,96491),
(9417,22689,'Ryusei no Kizuna',NULL,now(),NULL,96581),
(9418,7908,'The New Adventures of Old Christine',NULL,now(),NULL,75756),
(9419,6491,'Up the Elephant and Round the Castle',NULL,now(),NULL,96601),
(9420,22662,'4th and Long',NULL,now(),NULL,96621),
(9421,4167,'L.A. Dragnet',NULL,now(),NULL,70655),
(9422,22684,'Tattoo Highway',NULL,now(),NULL,96711),
(9423,23300,'The Threshold Of A Persona',NULL,now(),NULL,96721),
(9424,17526,'Urban Legends',NULL,now(),NULL,96731),
(9425,13797,'Haruka 17',NULL,now(),NULL,96781),
(9426,554,'Cardcaptor Sakura',NULL,now(),NULL,70668),
(9427,15614,'Chuck',NULL,now(),NULL,80348),
(9428,14227,'A Love To Kill',NULL,now(),NULL,96821),
(9429,19982,'Destroyed in Seconds',NULL,now(),NULL,84921),
(9430,22464,'Copper',NULL,now(),NULL,96901),
(9431,16289,'Seito Shokun!',NULL,now(),NULL,96951),
(9432,22902,'Natures Most Amazing Events',NULL,now(),NULL,96961),
(9433,7582,'rocKwiz',NULL,now(),NULL,97041),
(9434,6692,'Yu Yu Hakusho',NULL,now(),NULL,76665),
(9435,6233,'The Thin Blue Line',NULL,now(),NULL,76753),
(9436,20791,'Demons',NULL,now(),NULL,84347),
(9437,16398,'Watashitachi no Kyokasho',NULL,now(),NULL,97211),
(9438,2270,'Urusei Yatsura',NULL,now(),NULL,75113),
(9439,22634,'Expedition Africa',NULL,now(),NULL,97281),
(9440,18397,'The Cleaner',NULL,now(),NULL,82492),
(9441,3985,'Invader ZIM',NULL,now(),NULL,75545),
(9442,5378,'Stripperella',NULL,now(),NULL,78852),
(9443,6926,'Ben 10',NULL,now(),NULL,79567),
(9444,22044,'Der kleine Mann',NULL,now(),NULL,87091),
(9445,13494,'The Force: Behind the Line',NULL,now(),NULL,97311),
(9446,22484,'Jonas',NULL,now(),NULL,97331),
(9447,6620,'Wing Commander Academy',NULL,now(),NULL,76797),
(9448,17527,'Dragonfly TV',NULL,now(),NULL,97401),
(9449,6364,'Tiny Planets',NULL,now(),NULL,97421),
(9450,9192,'Candy Candy',NULL,now(),NULL,94321),
(9451,19354,'Scream Queens',NULL,now(),NULL,97441),
(9452,22029,'Mental',NULL,now(),NULL,93651),
(9453,22690,'Tosh.0',NULL,now(),NULL,97731),
(9454,14179,'Guardian Angel',NULL,now(),NULL,97891),
(9455,18297,'10 Days to War',NULL,now(),NULL,98041),
(9456,19262,'Virtuality',NULL,now(),NULL,98051),
(9457,16909,'Small Space Big Style',NULL,now(),NULL,98131),
(9458,22905,'Michael McIntyre\'s Comedy Roadshow',NULL,now(),NULL,98151),
(9459,9581,'Hi No Tori',NULL,now(),NULL,98161),
(9460,3267,'Deadwood',NULL,now(),NULL,72023),
(9461,18813,'90210',NULL,now(),NULL,82716),
(9462,15361,'Once Upon a Time... The Americas',NULL,now(),NULL,81661);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(9463,12641,'Mark Williams\' Big Bangs',NULL,now(),NULL,98291),
(9464,22936,' Empire of Cricket',NULL,now(),NULL,98301),
(9465,12984,'Trawlermen',NULL,now(),NULL,102551),
(9466,21899,'The Cougar',NULL,now(),NULL,98361),
(9467,22691,'Kendra',NULL,now(),NULL,98371),
(9468,4696,'Numb3rs',NULL,now(),NULL,73918),
(9469,16842,'Naked Camera',NULL,now(),NULL,98391),
(9470,15595,'The Kill Point',NULL,now(),NULL,80369),
(9471,6450,'Twin Peaks',NULL,now(),NULL,70533),
(9472,23801,'Mud, Sweat and Tractors - The Story of Agriculture',NULL,now(),NULL,98421),
(9473,22795,'Everest ER',NULL,now(),NULL,98451),
(9474,22683,'The Little Couple',NULL,now(),NULL,98481),
(9475,7571,'True Stories',NULL,now(),NULL,98491),
(9476,7871,'Noah\'s Arc',NULL,now(),NULL,79562),
(9477,22111,'Cross Game',NULL,now(),NULL,87901),
(9478,22909,'Property Snakes and Ladders',NULL,now(),NULL,98681),
(9479,22896,'Ross Kemp in Search of Pirates',NULL,now(),NULL,98691),
(9480,19649,'Top Chef Masters',NULL,now(),NULL,98821),
(9481,20277,'Time Warp',NULL,now(),NULL,82682),
(9482,22581,'New York Goes to Work',NULL,now(),NULL,98961),
(9483,6260,'The Twilight Zone (2002)',NULL,now(),NULL,78611),
(9484,8852,'Dear Ladies',NULL,now(),NULL,98971),
(9485,2417,'Magical Meow Meow Taruto',NULL,now(),NULL,75982),
(9486,22967,'Air Force Afghanistan',NULL,now(),NULL,99001),
(9487,22839,'Nature\'s Fury',NULL,now(),NULL,99021),
(9488,22680,'Hitched or Ditched',NULL,now(),NULL,99041),
(9489,22870,'Psychoville',NULL,now(),NULL,99101),
(9490,6760,'Bleak House',NULL,now(),NULL,73558),
(9491,2318,'Blackpool',NULL,now(),NULL,75350),
(9492,21955,'River Monsters',NULL,now(),NULL,99151),
(9493,22590,'The Birth of British Music',NULL,now(),NULL,99171),
(9494,4042,'Jeeves and Wooster',NULL,now(),NULL,76934),
(9495,14739,'Heidi (1978)',NULL,now(),NULL,99271),
(9496,2381,'Pingu',NULL,now(),NULL,75669),
(9497,13481,'Curious George (2006)',NULL,now(),NULL,79429),
(9498,13903,'Love Story (TBS)',NULL,now(),NULL,99431),
(9499,17816,'Dollhouse',NULL,now(),NULL,82046),
(9500,22824,'Who\'s Watching You',NULL,now(),NULL,99511),
(9501,11301,'Underground Ernie',NULL,now(),NULL,99541),
(9502,11361,'Spyforce',NULL,now(),NULL,99571),
(9503,2672,'Auf Wiedersehen Pet',NULL,now(),NULL,78287),
(9504,13801,'Hito Ni Yasashiku',NULL,now(),NULL,99621),
(9505,2203,'The Adventures of Mini Goddess',NULL,now(),NULL,79411),
(9506,13705,'Bijo ka Yajuu',NULL,now(),NULL,99631),
(9507,16175,'Kurosagi',NULL,now(),NULL,99641),
(9508,16128,'Kiken Na Aneki (Dangerous Sister)',NULL,now(),NULL,99681),
(9509,22911,'I\'m Running Sainsbury\'s',NULL,now(),NULL,99691),
(9510,14568,'Hana Yori Dango (2005)',NULL,now(),NULL,80700),
(9511,12125,'Gokusen (2004)',NULL,now(),NULL,81239),
(9512,6533,'Waking the Dead',NULL,now(),NULL,79053),
(9513,22957,'Alan Carr: Chatty Man',NULL,now(),NULL,99711),
(9514,14547,'Friends (2002)',NULL,now(),NULL,99721),
(9515,16085,'Hotelier',NULL,now(),NULL,99731),
(9516,22659,'Sterretje Gezocht',NULL,now(),NULL,99751),
(9517,13982,'Onsen e Ikou',NULL,now(),NULL,99791),
(9518,19385,'Occupation',NULL,now(),NULL,99801),
(9519,16216,'Nanisama!',NULL,now(),NULL,99811),
(9520,13935,'My Little Chef',NULL,now(),NULL,99831),
(9521,13936,'Narita Rikon',NULL,now(),NULL,99851),
(9522,13821,'Itazura Na Kiss (1996)',NULL,now(),NULL,99861),
(9523,13956,'News no Onna',NULL,now(),NULL,99871),
(9524,16225,'Ns\' Aoi',NULL,now(),NULL,99881),
(9525,13746,'Anego',NULL,now(),NULL,99891),
(9526,22960,'Personal Affairs',NULL,now(),NULL,99901),
(9527,19587,'Testees',NULL,now(),NULL,83339),
(9528,21888,'Hammertime',NULL,now(),NULL,99971),
(9529,19267,'Castle (2009)',NULL,now(),NULL,83462),
(9530,10833,'Scrapheap Challenge',NULL,now(),NULL,79903),
(9531,3746,'Grounded for Life',NULL,now(),NULL,78579),
(9532,23324,'The Take',NULL,now(),NULL,100002),
(9533,22657,'Undercover Boss',NULL,now(),NULL,100061),
(9534,15125,'George Gently',NULL,now(),NULL,80226),
(9535,3699,'Go, Diego, Go!',NULL,now(),NULL,79896),
(9536,11212,'Galaxy Express 999',NULL,now(),NULL,100171),
(9537,23299,'Sweetness In The Salt',NULL,now(),NULL,100201),
(9538,18424,'The Zoo',NULL,now(),NULL,100211),
(9539,4535,'Mr. & Mrs. Smith',NULL,now(),NULL,74945),
(9540,939,'A Mind to Kill',NULL,now(),NULL,100271),
(9541,9884,'The Buddah of Suburbia',NULL,now(),NULL,100301),
(9542,9305,'Lady Chatterley',NULL,now(),NULL,100311),
(9543,14145,'InuYasha',NULL,now(),NULL,71361),
(9544,4321,'Mad About You',NULL,now(),NULL,74340),
(9545,18984,'Bed of Roses',NULL,now(),NULL,100321),
(9546,21135,'Made in Spain',NULL,now(),NULL,100331),
(9547,22702,'Beyond A Joke',NULL,now(),NULL,100501),
(9548,20705,'Gardeners\' World Top Tips',NULL,now(),NULL,100461),
(9549,11242,'Private Parts',NULL,now(),NULL,100471),
(9550,6425,'Trigun',NULL,now(),NULL,77680),
(9551,1336,'GunGrave',NULL,now(),NULL,78963),
(9552,5414,'Surface',NULL,now(),NULL,78906),
(9553,17990,'House of Saddam',NULL,now(),NULL,82624),
(9554,19449,'Impact',NULL,now(),NULL,85137),
(9555,13089,'Two-A-Days: Hoover High',NULL,now(),NULL,100601),
(9556,21051,'Eastbound & Down',NULL,now(),NULL,82467),
(9557,2097,'Silversun',NULL,now(),NULL,100661),
(9558,23633,'James May on the Moon',NULL,now(),NULL,100571),
(9559,10097,'Cat\'s Eye',NULL,now(),NULL,79835),
(9560,2120,'Chorlton and the Wheelies',NULL,now(),NULL,100711),
(9561,6091,'The Pink Panther (1993)',NULL,now(),NULL,100741),
(9562,14208,'Jumong',NULL,now(),NULL,100771),
(9563,22910,'May Contain Nuts (2009)',NULL,now(),NULL,100831),
(9564,15251,'Shaun the Sheep',NULL,now(),NULL,79890),
(9565,6409,'Transformers',NULL,now(),NULL,72499),
(9566,2648,'Around the World in 80 Days',NULL,now(),NULL,73888),
(9567,15720,'Seven Ages of Rock',NULL,now(),NULL,82096),
(9568,23006,'Famous, Rich and Homeless',NULL,now(),NULL,100901),
(9569,20711,'I Can\'t Believe I\'m Still Single',NULL,now(),NULL,100911),
(9570,2976,'Captain N: The Game Master',NULL,now(),NULL,76792),
(9571,22948,'Swamp Loggers',NULL,now(),NULL,101671),
(9572,11570,'Mick Molloy Show',NULL,now(),NULL,100931),
(9573,12151,'Grenadier',NULL,now(),NULL,79132),
(9574,18689,'The Listener',NULL,now(),NULL,85358),
(9575,5897,'The Job',NULL,now(),NULL,79180),
(9576,14249,'My Girl',NULL,now(),NULL,100961),
(9577,14268,'Princess Hours',NULL,now(),NULL,100971),
(9578,22610,'Shark Tank',NULL,now(),NULL,100981),
(9579,17667,'Talking Heads',NULL,now(),NULL,101071),
(9580,3823,'Heathcliff and the Catillac Cats',NULL,now(),NULL,78514),
(9581,2812,'Black Books',NULL,now(),NULL,76924),
(9582,6594,'Whoopi',NULL,now(),NULL,72297),
(9583,23075,'On Thin Ice',NULL,now(),NULL,101411),
(9584,5668,'The Cape',NULL,now(),NULL,71394),
(9585,5625,'The Bionic Woman (1976)',NULL,now(),NULL,77259),
(9586,6459,'UFO',NULL,now(),NULL,75133),
(9587,13375,'Dawn French\'s Girls Who Do Comedy',NULL,now(),NULL,101711),
(9588,18762,'Verminators',NULL,now(),NULL,101801),
(9589,10043,'Movies That Shook The World',NULL,now(),NULL,101891),
(9590,18992,'Care Bears: Adventures in Care-A-Lot',NULL,now(),NULL,101901),
(9591,22982,'Aoi Hana',NULL,now(),NULL,102011),
(9592,12266,'28 Acts In 28 Minutes',NULL,now(),NULL,102081),
(9593,2543,'Alien Nation',NULL,now(),NULL,72358),
(9594,23298,'A Great Way to Care',NULL,now(),NULL,102211),
(9595,8610,'That\'s My Boy',NULL,now(),NULL,102221),
(9596,8911,'North and South',NULL,now(),NULL,81645),
(9597,2134,'Gunparade March',NULL,now(),NULL,79046),
(9598,10110,'P.O.V.',NULL,now(),NULL,102241),
(9599,5553,'The Agency',NULL,now(),NULL,78822),
(9600,5727,'The District',NULL,now(),NULL,73152),
(9601,8429,'Love on a Branch Line',NULL,now(),NULL,115411),
(9602,3161,'Crash',NULL,now(),NULL,83266),
(9603,13979,'Over Time',NULL,now(),NULL,102301),
(9604,3473,'Extreme Ghostbusters',NULL,now(),NULL,76552),
(9605,11816,'A Dog\'s Life',NULL,now(),NULL,102321),
(9606,22980,'Umineko no Naku Koro ni',NULL,now(),NULL,102361),
(9607,6814,'CMT Crossroads',NULL,now(),NULL,102381),
(9608,22329,'Bizarre ER',NULL,now(),NULL,102391),
(9609,15477,'Derrick',NULL,now(),NULL,102371),
(9610,23947,'Lingerie',NULL,now(),NULL,102441),
(9611,22983,'Needless',NULL,now(),NULL,102471),
(9612,23188,'Yoku Wakaru Gendai Mahou',NULL,now(),NULL,102481),
(9613,22987,'Element Hunters',NULL,now(),NULL,102491),
(9614,2730,'Battlestar Galactica (2003)',NULL,now(),NULL,73545),
(9615,22984,'Taishou Yakyuu Musume',NULL,now(),NULL,102541),
(9616,23719,'Team Schrick',NULL,now(),NULL,102581),
(9617,22988,'Canaan',NULL,now(),NULL,102591),
(9618,4546,'Mrs. Columbo',NULL,now(),NULL,102891),
(9619,22655,'Dating In The Dark',NULL,now(),NULL,102941),
(9620,23066,'Four Weddings',NULL,now(),NULL,103001),
(9621,23151,'That\'s Impossible!',NULL,now(),NULL,103011),
(9622,23088,'You Have Been Watching',NULL,now(),NULL,103021),
(9623,12379,'Kirarin Revolution',NULL,now(),NULL,103071),
(9624,22806,'Masterwork',NULL,now(),NULL,103101),
(9625,23309,'Getting On',NULL,now(),NULL,103151),
(9626,17005,'DinoSapien',NULL,now(),NULL,103191),
(9627,2533,'ALF: The Animated Series',NULL,now(),NULL,73041),
(9628,22704,'Great American Road Trip',NULL,now(),NULL,103241),
(9629,23187,'Tokyo Magnitude 8.0',NULL,now(),NULL,103321),
(9630,18683,'My Own Worst Enemy',NULL,now(),NULL,82122),
(9631,5290,'Spender',NULL,now(),NULL,103331),
(9632,6862,'Da Vinci\'s City Hall',NULL,now(),NULL,103341),
(9633,8614,'Thick as Thieves',NULL,now(),NULL,103351),
(9634,9693,'Gangsters',NULL,now(),NULL,103381),
(9635,12692,'Take Me',NULL,now(),NULL,103391),
(9636,11838,'A Dance to the Music of Time',NULL,now(),NULL,103451),
(9637,12108,'The Glittering Prizes',NULL,now(),NULL,103481),
(9638,16990,'Little Devil',NULL,now(),NULL,103501),
(9639,12887,'I\'m Gonna Be An Angel',NULL,now(),NULL,103571),
(9640,14431,'Ai No Uta!',NULL,now(),NULL,103621),
(9641,19743,'Aspe',NULL,now(),NULL,81915),
(9642,13269,'Five Children and It',NULL,now(),NULL,103701),
(9643,6007,'The Nanny',NULL,now(),NULL,75675),
(9644,5346,'Step by Step',NULL,now(),NULL,76095),
(9645,23510,'2 Months 2 Million',NULL,now(),NULL,103821),
(9646,21950,'Rocket Science',NULL,now(),NULL,103911),
(9647,8489,'Invincible',NULL,now(),NULL,104011),
(9648,5939,'The Lone Gunmen',NULL,now(),NULL,76245),
(9649,7884,'Warehouse 13',NULL,now(),NULL,84676),
(9650,1615,'Yes, Prime Minister',NULL,now(),NULL,76864),
(9651,23023,'Monday Monday',NULL,now(),NULL,104081),
(9652,24014,'Airways',NULL,now(),NULL,104171),
(9653,6344,'Three',NULL,now(),NULL,72547),
(9654,21582,'Renovation Realities',NULL,now(),NULL,104191),
(9655,5486,'TekWar',NULL,now(),NULL,76850),
(9656,23686,'Covert Affairs',NULL,now(),NULL,104281),
(9657,621,'\'orrible',NULL,now(),NULL,104301),
(9658,10924,'Ergo Proxy',NULL,now(),NULL,79228),
(9659,13482,'Anna Karenina',NULL,now(),NULL,104391),
(9660,22071,'K-ON!',NULL,now(),NULL,87501),
(9661,15985,'Anna-san no Omame',NULL,now(),NULL,104481),
(9662,8077,'Planet Earth',NULL,now(),NULL,79257),
(9663,16030,'Enka no Joou',NULL,now(),NULL,104521),
(9664,23190,'Home for Life',NULL,now(),NULL,104541),
(9665,6725,'Human Trafficking',NULL,now(),NULL,104551),
(9666,23125,'Defying Gravity',NULL,now(),NULL,104581),
(9667,22338,'Bored to Death',NULL,now(),NULL,104641),
(9668,23196,'Time Team America',NULL,now(),NULL,104691),
(9669,23191,'Rick Stein\'s Far Eastern Odyssey',NULL,now(),NULL,104701),
(9670,7349,'Big Brother\'s Big Mouth',NULL,now(),NULL,104711),
(9671,5312,'Spy Groove',NULL,now(),NULL,104721),
(9672,2633,'Anything but Love',NULL,now(),NULL,75081),
(9673,20939,'The Assistants',NULL,now(),NULL,104761),
(9674,16875,'Surf Patrol',NULL,now(),NULL,104771),
(9675,936,'The FBI Files',NULL,now(),NULL,104781),
(9676,23063,'Bite Me With Dr Mike',NULL,now(),NULL,104871),
(9677,2698,'Back to the Future',NULL,now(),NULL,75880),
(9678,16403,'YAOH',NULL,now(),NULL,104981),
(9679,16107,'Jigoku Shoujo (2006)',NULL,now(),NULL,80363),
(9680,23141,'As Seen on TV',NULL,now(),NULL,105001),
(9681,13617,'SKINT (2007)',NULL,now(),NULL,105011),
(9682,14103,'Yasashii Jikan',NULL,now(),NULL,105021),
(9683,14096,'Yankee Bokou ni Kaeru',NULL,now(),NULL,105041),
(9684,16405,'Yakusha Damashii',NULL,now(),NULL,105071),
(9685,14429,'Legend Of The Condor Heroes (1982)',NULL,now(),NULL,105081),
(9686,23027,'Princess Lover',NULL,now(),NULL,105131),
(9687,3407,'Dynasty',NULL,now(),NULL,71487),
(9688,3211,'Dallas',NULL,now(),NULL,77092),
(9689,7506,'Eureka',NULL,now(),NULL,79334),
(9690,21766,'The Vampire Diaries',NULL,now(),NULL,95491),
(9691,22328,'The Beautiful Life: TBL',NULL,now(),NULL,95501),
(9692,22791,'Ashita no Kita Yoshio',NULL,now(),NULL,105321),
(9693,22705,'Michael and Michael Have Issues',NULL,now(),NULL,105331),
(9694,23235,'Taking the Flak',NULL,now(),NULL,104531),
(9695,6221,'The Suite Life of Zack and Cody',NULL,now(),NULL,79316),
(9696,23443,'The 7pm Project',NULL,now(),NULL,105421),
(9697,23307,'Desperate Romantics',NULL,now(),NULL,105461),
(9698,23356,'The Colony',NULL,now(),NULL,105521),
(9699,18990,'3-2-1 Penguins!',NULL,now(),NULL,105581),
(9700,18935,'The Ex List',NULL,now(),NULL,83025),
(9701,1288,'.hack//Legend of the Twilight',NULL,now(),NULL,78815),
(9702,7477,'Superhuman',NULL,now(),NULL,105591),
(9703,18102,'Generation Kill',NULL,now(),NULL,82109),
(9704,23010,'Total Drama Action',NULL,now(),NULL,105601),
(9705,23308,'Wildest Dreams',NULL,now(),NULL,105661),
(9706,13961,'Ningen No Shoumei ',NULL,now(),NULL,105681),
(9707,13742,'Ame to Yume no Ato ni',NULL,now(),NULL,105791),
(9708,7887,'Lipstick Jungle',NULL,now(),NULL,80392),
(9709,16388,'Warui Yatsura',NULL,now(),NULL,105901),
(9710,23234,'The Rat Pack',NULL,now(),NULL,105911),
(9711,8539,'SCANDAL',NULL,now(),NULL,105941),
(9712,16778,'The Real Football Factories International',NULL,now(),NULL,80938),
(9713,9312,'Fast Forward (US)',NULL,now(),NULL,105971),
(9714,14030,'Sora Kara Furu Ichioku no Hoshi',NULL,now(),NULL,106031),
(9715,20978,'Chrome Shelled Regios',NULL,now(),NULL,84677),
(9716,2829,'Blind Justice',NULL,now(),NULL,73934),
(9717,23186,'Sora no Manimani',NULL,now(),NULL,106071),
(9718,16051,'Gal Circle',NULL,now(),NULL,106081),
(9719,16336,'Tiger & Dragon',NULL,now(),NULL,106111),
(9720,12393,'The Adventures of Peter Pan',NULL,now(),NULL,106121),
(9721,23422,'Live From the Artists Den',NULL,now(),NULL,106161),
(9722,13714,'Churasan',NULL,now(),NULL,106361),
(9723,13772,'Fugoh Keiji',NULL,now(),NULL,106371),
(9724,10910,'Life in Cold Blood',NULL,now(),NULL,81394),
(9725,18661,'The Paper',NULL,now(),NULL,106291),
(9726,16738,'Yogi\'s Gang',NULL,now(),NULL,106351),
(9727,16010,'Cupid no Itazura',NULL,now(),NULL,106391),
(9728,23311,'Rivers with Griff Rhys Jones',NULL,now(),NULL,106411),
(9729,22290,'The Beatles Anthology',NULL,now(),NULL,84780),
(9730,16073,'Himitsu no Hanazono',NULL,now(),NULL,106461),
(9731,16095,'Ima Ai ni Yukimasu',NULL,now(),NULL,106471),
(9732,22706,'The Storm',NULL,now(),NULL,106491),
(9733,16056,'Hanayome to Papa',NULL,now(),NULL,106501),
(9734,4760,'Oz',NULL,now(),NULL,70682),
(9735,13899,'Long Love Letter',NULL,now(),NULL,106531),
(9736,22614,'Blood, Sweat and Takeaways',NULL,now(),NULL,106581),
(9737,15989,'Bambino!',NULL,now(),NULL,84751),
(9738,22745,'Lock \'N Load',NULL,now(),NULL,106591),
(9739,4488,'Mission Top Secret',NULL,now(),NULL,77396),
(9740,16367,'Tokyo Friends',NULL,now(),NULL,106611),
(9741,16385,'Unfair',NULL,now(),NULL,106621),
(9742,20726,'Jotei',NULL,now(),NULL,106651),
(9743,16298,'Shimokita GLORY DAYS',NULL,now(),NULL,106671),
(9744,22076,'Samantha Brown\'s Great Weekends',NULL,now(),NULL,106741),
(9745,18262,'In Treatment',NULL,now(),NULL,81248),
(9746,6400,'Tracker',NULL,now(),NULL,76950),
(9747,22696,'Food Party',NULL,now(),NULL,106791),
(9748,22415,'More to Love',NULL,now(),NULL,106801),
(9749,7075,'The 10th Kingdom',NULL,now(),NULL,78886),
(9750,6105,'The Practice',NULL,now(),NULL,73226),
(9751,3516,'Fantasy Island',NULL,now(),NULL,77722),
(9752,4032,'JAG',NULL,now(),NULL,73710),
(9753,4590,'My Name Is Earl',NULL,now(),NULL,75397),
(9754,11224,'Rules of Engagement',NULL,now(),NULL,79842),
(9755,18174,'Sons of Anarchy',NULL,now(),NULL,82696),
(9756,17908,'Survivors (2008)',NULL,now(),NULL,83708),
(9757,18753,'The Inbetweeners',NULL,now(),NULL,81950),
(9758,18817,'The Middleman',NULL,now(),NULL,82230),
(9759,18969,'Worst Week',NULL,now(),NULL,82895),
(9760,18491,'Crusoe',NULL,now(),NULL,83226),
(9761,18968,'Gary Unmarried',NULL,now(),NULL,82916),
(9762,1867,'SilverHawks',NULL,now(),NULL,77918),
(9763,6294,'The Wild Wild West',NULL,now(),NULL,71637),
(9764,3342,'Donkey Kong Country',NULL,now(),NULL,71162),
(9765,3850,'Highlander',NULL,now(),NULL,75160),
(9766,2720,'The New Batman Adventures',NULL,now(),NULL,77084),
(9767,5072,'Rupert',NULL,now(),NULL,77287),
(9768,5294,'Spider-Man (1994)',NULL,now(),NULL,73750),
(9769,23397,'How the Other Half Live',NULL,now(),NULL,107041),
(9770,8773,'The Management',NULL,now(),NULL,107051),
(9771,2541,'Alienated',NULL,now(),NULL,107111),
(9772,16782,'Fastir LiÃƒÂ°ir Eins og Venjulega',NULL,now(),NULL,83736),
(9773,23394,'Rock \'n\' Roll Hotel',NULL,now(),NULL,107131),
(9774,20954,'Minami-ke Okaeri',NULL,now(),NULL,107241),
(9775,23028,'Kanamemo',NULL,now(),NULL,107301),
(9776,3311,'Digimon: Digital Monsters',NULL,now(),NULL,72241),
(9777,16312,'Shiroi Kyoto',NULL,now(),NULL,107401),
(9778,18107,'Wangan Midnight',NULL,now(),NULL,81658),
(9779,23496,'Rescue Special Ops',NULL,now(),NULL,107381),
(9780,12204,'Happy Seven',NULL,now(),NULL,107431),
(9781,8017,'Nanaka 6/17',NULL,now(),NULL,107441),
(9782,22849,'The T.O. Show',NULL,now(),NULL,107461),
(9783,23312,'Bang Goes The Theory',NULL,now(),NULL,107471),
(9784,7735,'Captain Pugwash',NULL,now(),NULL,107551),
(9785,23387,'Show Me Show Me',NULL,now(),NULL,107591),
(9786,4804,'Perry Mason',NULL,now(),NULL,70589),
(9787,667,'A Bit of Fry and Laurie',NULL,now(),NULL,71040),
(9788,23444,'Clash of the Gods',NULL,now(),NULL,107631),
(9789,23197,'The Ascent of Money (US)',NULL,now(),NULL,107661),
(9790,23305,'Lock N\' Load with R. Lee Ermey',NULL,now(),NULL,107701),
(9791,4522,'Monty Python\'s Flying Circus',NULL,now(),NULL,75853),
(9792,22999,'Web Soup',NULL,now(),NULL,107801),
(9793,13490,'How It\'s Made',NULL,now(),NULL,79485),
(9794,5626,'BlackAdder',NULL,now(),NULL,76736),
(9795,21742,'Baccano',NULL,now(),NULL,80421),
(9796,16652,'Alles ausser Sex',NULL,now(),NULL,81998),
(9797,10458,'Make My Day 2009',NULL,now(),NULL,107951),
(9798,4701,'Ocean Girl',NULL,now(),NULL,75750),
(9799,3520,'Fastlane',NULL,now(),NULL,78807),
(9800,5316,'Stacked',NULL,now(),NULL,75705),
(9801,6494,'V.I.P.',NULL,now(),NULL,74181),
(9802,23441,'Sex in the Ancient World',NULL,now(),NULL,108021),
(9803,3348,'Dora the Explorer',NULL,now(),NULL,74697),
(9804,16879,'Wide Angle',NULL,now(),NULL,108031),
(9805,23631,'Economy Gastronomy',NULL,now(),NULL,108141),
(9806,7232,'Squirrel Boy',NULL,now(),NULL,108151),
(9807,16446,'Angels in America',NULL,now(),NULL,79311),
(9808,23306,'Young, Dumb and Living Off Mum',NULL,now(),NULL,108171),
(9809,23396,'Classic Goldie',NULL,now(),NULL,108201),
(9810,15753,'Moonlight',NULL,now(),NULL,80512),
(9811,6362,'Timon & Pumbaa',NULL,now(),NULL,70559),
(9812,23185,'GA: Geijutsuka Art Design Class',NULL,now(),NULL,108401),
(9813,4335,'Magical Shopping Arcade Abenobashi',NULL,now(),NULL,73616),
(9814,16818,'The 808',NULL,now(),NULL,108491),
(9815,15878,'Benjamin Franklin',NULL,now(),NULL,108501),
(9816,12943,'All or Nothing at All',NULL,now(),NULL,108571),
(9817,20720,'White Collar',NULL,now(),NULL,108611),
(9818,22815,'There Goes The Neighborhood ',NULL,now(),NULL,108631),
(9819,3799,'Hart to Hart',NULL,now(),NULL,71308),
(9820,23515,'Drain the Ocean',NULL,now(),NULL,108651),
(9821,22844,'Megan Wants A Millionaire',NULL,now(),NULL,108661),
(9822,15710,'Big Shots',NULL,now(),NULL,80570),
(9823,8259,'Doodlez',NULL,now(),NULL,108761),
(9824,5497,'That \'70s Show',NULL,now(),NULL,73787),
(9825,22971,'Bollywood Hero',NULL,now(),NULL,108861),
(9826,19169,'The Beast (2009)',NULL,now(),NULL,83805),
(9827,4872,'Postman Pat',NULL,now(),NULL,73854),
(9828,11868,'The Cell',NULL,now(),NULL,109171),
(9829,23419,'D.I.E. Again',NULL,now(),NULL,109191),
(9830,13678,'All In',NULL,now(),NULL,109401),
(9831,17501,'Knight Rider (2008)',NULL,now(),NULL,81318),
(9832,7342,'Match of the Day',NULL,now(),NULL,109411),
(9833,4900,'Profiler',NULL,now(),NULL,74257),
(9834,23204,' Kourtney and Khloe Take Miami',NULL,now(),NULL,109491),
(9835,20572,'Swords: Life on the Line',NULL,now(),NULL,109531),
(9836,18994,'Dino Squad',NULL,now(),NULL,109541),
(9837,18995,'Horseland',NULL,now(),NULL,109551),
(9838,3380,'Drawn Together',NULL,now(),NULL,78831),
(9839,5360,'Strange Days at Blake Holsey High',NULL,now(),NULL,109741),
(9840,10610,'Hospital Central',NULL,now(),NULL,109781),
(9841,17102,'Der Lehrer',NULL,now(),NULL,109731),
(9842,10199,'The Naked Trucker & T-Bones Show',NULL,now(),NULL,109901),
(9843,23457,'Shaq vs.',NULL,now(),NULL,109921),
(9844,23232,'Reality Hell',NULL,now(),NULL,109951),
(9845,2574,'Alvin & the Chipmunks',NULL,now(),NULL,74724),
(9846,23100,'Popular Science\'s Future of',NULL,now(),NULL,109981),
(9847,990,'Ulysses 31',NULL,now(),NULL,72219),
(9848,22728,'Glenn Martin DDS',NULL,now(),NULL,110001),
(9849,24077,'Vertical City',NULL,now(),NULL,110011),
(9850,23550,'Benefit Busters',NULL,now(),NULL,110091),
(9851,11190,'Lovespring International',NULL,now(),NULL,110101),
(9852,23637,'Doctor Who Greatest Moments',NULL,now(),NULL,110141),
(9853,3993,'Iron Chef',NULL,now(),NULL,71991),
(9854,22286,'Models of the Runway',NULL,now(),NULL,110201),
(9855,22985,'Bakemonogatari',NULL,now(),NULL,102261),
(9856,23216,'Car Crime UK',NULL,now(),NULL,110281),
(9857,23354,'Archer (2009)',NULL,now(),NULL,110381),
(9858,6753,'Top Gear',NULL,now(),NULL,74608),
(9859,11955,'The Big Gig',NULL,now(),NULL,110411),
(9860,4543,'Mr. Show with Bob and David',NULL,now(),NULL,110441),
(9861,17080,'Hunter Hunted',NULL,now(),NULL,110671),
(9862,23694,'Caribbean Food Made Easy',NULL,now(),NULL,110561),
(9863,13901,'Love Generation',NULL,now(),NULL,82442),
(9864,15362,'Once Upon a Time... The Discoverers',NULL,now(),NULL,82342),
(9865,23626,'The Cube',NULL,now(),NULL,110861),
(9866,23978,'The Jesters',NULL,now(),NULL,110881),
(9867,23628,' The Hairy Bikers\' Food Tour of Britain',NULL,now(),NULL,110931),
(9868,17839,'To The Manor Bowen',NULL,now(),NULL,111001),
(9869,2307,'The World of Narue',NULL,now(),NULL,111071),
(9870,8415,'Rose Red',NULL,now(),NULL,84126),
(9871,6981,'Grimm\'s Fairy Tale Classics',NULL,now(),NULL,111131),
(9872,10099,'Lucy, The Daughter of the Devil',NULL,now(),NULL,79019),
(9873,18285,'The Guard',NULL,now(),NULL,81219),
(9874,4272,'Lock, Stock...',NULL,now(),NULL,78135),
(9875,23703,'Jamie\'s American Road Trip',NULL,now(),NULL,111161),
(9876,1375,'The Sleepover Club',NULL,now(),NULL,111171),
(9877,5500,'That\'s Life',NULL,now(),NULL,72926),
(9878,16104,'Jodan ja nai!',NULL,now(),NULL,111301),
(9879,11103,'Persons Unknown',NULL,now(),NULL,111381),
(9880,6296,'The Wire',NULL,now(),NULL,79126),
(9881,3391,'Duck Dodgers',NULL,now(),NULL,72269),
(9882,12143,'The Wedge',NULL,now(),NULL,111641),
(9883,22709,'How\'d You Get So Rich?',NULL,now(),NULL,111701),
(9884,23455,'Neighbourhood Watched',NULL,now(),NULL,111721),
(9885,14285,'Resurrection',NULL,now(),NULL,111761),
(9886,6734,'I Love The 80s 3-D',NULL,now(),NULL,111771),
(9887,13305,'Who Pays the Ferryman?',NULL,now(),NULL,111821),
(9888,2858,'Bob Morane',NULL,now(),NULL,78698),
(9889,16335,'Tatta Hitotsu no Koi',NULL,now(),NULL,111871),
(9890,12652,'The Prince of Tennis',NULL,now(),NULL,111881),
(9891,22207,'Hatsukoi Limited',NULL,now(),NULL,89311),
(9892,23420,'Hot Wheels Battle Force 5',NULL,now(),NULL,111961),
(9893,18569,'To Love Ru',NULL,now(),NULL,93701),
(9894,22826,'The Rod, Jane and Freddy Show',NULL,now(),NULL,112001),
(9895,6490,'Upstairs, Downstairs',NULL,now(),NULL,71000),
(9896,23630,'Future of Food',NULL,now(),NULL,112101),
(9897,18292,'Raising the Bar',NULL,now(),NULL,82340),
(9898,23708,'Wuthering Heights 2009',NULL,now(),NULL,112141),
(9899,23340,'Police Women of Broward County',NULL,now(),NULL,112161),
(9900,20681,'The Electric Company (2009)',NULL,now(),NULL,112181),
(9901,17931,'Shootout!',NULL,now(),NULL,112261),
(9902,6144,'The Rockford Files',NULL,now(),NULL,71219),
(9903,23783,'The Frankincense Trail',NULL,now(),NULL,112441),
(9904,19309,'Britain\'s Lost World',NULL,now(),NULL,112461),
(9905,24058,'Letter Bee',NULL,now(),NULL,112521),
(9906,23470,'On Tour with The Queen',NULL,now(),NULL,112561),
(9907,23436,'Surviving Disaster',NULL,now(),NULL,113321),
(9908,23777,'Lost Land of the Volcano',NULL,now(),NULL,112631),
(9909,3033,'Charlie & Lola',NULL,now(),NULL,75933),
(9910,11158,'G-On Riders',NULL,now(),NULL,112661),
(9911,22664,'Xtreme 4X4',NULL,now(),NULL,112701),
(9912,17765,'Genshiken 2',NULL,now(),NULL,112751),
(9913,23774,'Joanna Lumley: Catwoman',NULL,now(),NULL,112761),
(9914,18171,'Shattered Angels',NULL,now(),NULL,112781),
(9915,23431,'Deadly 60',NULL,now(),NULL,112801),
(9916,20186,'Escape from Scorpion Island',NULL,now(),NULL,112821),
(9917,23712,'Land Girls',NULL,now(),NULL,112871),
(9918,23413,'Adoption Diaries',NULL,now(),NULL,112881),
(9919,15435,'Mortified',NULL,now(),NULL,113081),
(9920,23399,'Grimefighters',NULL,now(),NULL,113101),
(9921,23779,'Nigel Slater\'s Simple Suppers',NULL,now(),NULL,113201),
(9922,23847,'Off The Hook',NULL,now(),NULL,113281),
(9923,23411,'Alone in the Wild',NULL,now(),NULL,113291),
(9924,23715,'Lunch Monkeys',NULL,now(),NULL,113301),
(9925,23840,'Derren Brown: The Events',NULL,now(),NULL,113451),
(9926,23639,'Nostradamus Effect',NULL,now(),NULL,113481),
(9927,12804,'Soul Link',NULL,now(),NULL,113571),
(9928,12325,'Kurogane Communication',NULL,now(),NULL,113591),
(9929,3923,'Huckleberry Finn and His Friends',NULL,now(),NULL,76129),
(9930,7516,'The Hairy Bikers\' Cookbook',NULL,now(),NULL,113641),
(9931,8829,'Oh No It\'s Selwyn Froggitt',NULL,now(),NULL,113661),
(9932,12698,'Celebrity Masterchef',NULL,now(),NULL,113701),
(9933,6613,'Wild West Tech',NULL,now(),NULL,113881),
(9934,23846,'Home Time',NULL,now(),NULL,113891),
(9935,22790,'The Jay Leno Show',NULL,now(),NULL,113921),
(9936,22027,'How the Earth Was Made',NULL,now(),NULL,85168),
(9937,18937,'Harper\'s Island',NULL,now(),NULL,84084),
(9938,22203,'Lockup',NULL,now(),NULL,114011),
(9939,23910,'The Cult',NULL,now(),NULL,114031),
(9940,22591,'South Pacific',NULL,now(),NULL,94231),
(9941,14768,'Heidi',NULL,now(),NULL,82927),
(9942,3249,'Dave the Barbarian',NULL,now(),NULL,79538),
(9943,2208,'Bakuretsu Tenshi',NULL,now(),NULL,74908),
(9944,6275,'The Visitor',NULL,now(),NULL,75564),
(9945,23939,'Big Trouble in Thailand',NULL,now(),NULL,114161),
(9946,1589,'Fury',NULL,now(),NULL,76761),
(9947,4802,'Perfect Strangers',NULL,now(),NULL,70570),
(9948,5278,'Space Island One',NULL,now(),NULL,77810),
(9949,23461,'MysteryQuest',NULL,now(),NULL,114241),
(9950,15068,'Jane Eyre (1983)',NULL,now(),NULL,86781),
(9951,2562,'Allo! Allo!',NULL,now(),NULL,73940),
(9952,23548,'The Troop',NULL,now(),NULL,114381),
(9953,1004,'Gerhard Reinke\'s Wanderlust',NULL,now(),NULL,114441),
(9954,23827,'30 Seconds',NULL,now(),NULL,114461),
(9955,23778,'The Big Food Fight',NULL,now(),NULL,114501),
(9956,18685,'Kings',NULL,now(),NULL,84068),
(9957,23176,'The Dating Guy',NULL,now(),NULL,114571),
(9958,23894,'Merlin Secrets And Magic',NULL,now(),NULL,114661),
(9959,23034,'The League',NULL,now(),NULL,114701),
(9960,23815,'Trinity (UK)',NULL,now(),NULL,114711);
INSERT INTO `tvrage` (`ID`, `rageID`, `releasetitle`, `description`, `createddate`, `imgdata`, `tvdbID`) 
VALUES 
(9961,21975,'Yellowstone',NULL,now(),NULL,85527),
(9962,18331,'Satisfaction',NULL,now(),NULL,81053),
(9963,5328,'Starman',NULL,now(),NULL,73794),
(9964,8511,'The Big Bang Theory',NULL,now(),NULL,80379),
(9965,16310,'Shinsengumi!',NULL,now(),NULL,117721),
(9966,13217,'Summer\'s Lease',NULL,now(),NULL,115351),
(9967,13442,'Get Lost!',NULL,now(),NULL,115361),
(9968,6608,'Wildfire',NULL,now(),NULL,77908),
(9969,2160,'Devil Lady',NULL,now(),NULL,83309),
(9970,11011,'Caprica',NULL,now(),NULL,85040),
(9971,22589,'Community',NULL,now(),NULL,94571),
(9972,21704,'Glee',NULL,now(),NULL,83610),
(9973,18151,'Hessie\'s Shed',NULL,now(),NULL,115561),
(9974,23909,'Costa\'s Garden Odyssey',NULL,now(),NULL,115581),
(9975,652,'The Adventures of Lano and Woodley',NULL,now(),NULL,115591),
(9976,19242,'Crocodile Shoes',NULL,now(),NULL,115671),
(9977,18401,'The Life and Times of Tim',NULL,now(),NULL,115801),
(9978,20404,'Download: The True Story of the Internet',NULL,now(),NULL,83083),
(9979,8676,'Avenger',NULL,now(),NULL,80642),
(9980,23943,'Restaurant In Our Living Room',NULL,now(),NULL,115911),
(9981,23837,'Atlantic Convoys: The War at Sea',NULL,now(),NULL,116081),
(9982,23817,'Nyan Koi!',NULL,now(),NULL,115971),
(9983,23818,'KÃƒÂ¤mpfer',NULL,now(),NULL,115991),
(9984,24051,'Seitokai no Ichizon',NULL,now(),NULL,116001),
(9985,15348,'Any Dream Will Do',NULL,now(),NULL,116051),
(9986,14880,'Olivers\' Travels',NULL,now(),NULL,116281),
(9987,23545,'Dinosaur Train',NULL,now(),NULL,116291),
(9988,23848,'Ross Noble\'s Australian Trip',NULL,now(),NULL,116331),
(9989,5338,'Star Trek: Voyager',NULL,now(),NULL,74550),
(9990,20798,'FlashForward',NULL,now(),NULL,84024),
(9991,22621,'Titan Maximum',NULL,now(),NULL,116421),
(9992,16047,'Ganbatte Ikimasshoi',NULL,now(),NULL,116431),
(9993,13811,'Hotman',NULL,now(),NULL,116441),
(9994,8889,'First of the Summer Wine',NULL,now(),NULL,116481),
(9995,9657,'Naked Video',NULL,now(),NULL,116541),
(9996,24029,'Celebrity Masterchef Australia',NULL,now(),NULL,116551),
(9997,23940,'Charlie Brooker\'s Gameswipe',NULL,now(),NULL,116651),
(9998,23866,'Ruth Watson\'s Hotel Rescue',NULL,now(),NULL,116681),
(9999,11089,'12 Books That Changed the World',NULL,now(),NULL,116751),
(10000,23941,'Return to A Pembrokeshire Farm',NULL,now(),NULL,116791),
(10001,23967,'Hungry Beast',NULL,now(),NULL,116801),
(10002,8392,'The Tales of Para Handy',NULL,now(),NULL,116821),
(10003,13245,'Madame Bovary',NULL,now(),NULL,116841),
(10004,23742,'What Katie Did Next',NULL,now(),NULL,116981),
(10005,23897,'Electric Dreams',NULL,now(),NULL,116991),
(10006,23925,'The Poker Star',NULL,now(),NULL,117101),
(10007,23844,'To Aru Kagaku no Railgun',NULL,now(),NULL,117151),
(10008,13717,'Shotgun Marriage',NULL,now(),NULL,117171),
(10009,13980,'Oyaji',NULL,now(),NULL,117181),
(10010,23845,'The Sacred Blacksmith',NULL,now(),NULL,117081),
(10011,7612,'The Adventures Of Long John Silver',NULL,now(),NULL,117261),
(10012,13676,'Beach Boys',NULL,now(),NULL,117271),
(10013,23709,'Tough Guy or Chicken',NULL,now(),NULL,117331),
(10014,15127,'Soulmate',NULL,now(),NULL,117321),
(10015,9997,'Americana',NULL,now(),NULL,117401),
(10016,23903,'Emma (2009)',NULL,now(),NULL,117411),
(10017,24044,'Valens Frokost-TV',NULL,now(),NULL,117431),
(10018,23341,'My Antonio',NULL,now(),NULL,117531),
(10019,23931,'Designing The Decades',NULL,now(),NULL,117561),
(10020,5706,'The Crow: Stairway to Heaven',NULL,now(),NULL,72850),
(10021,15669,'Star Wars: The Legacy Revealed',NULL,now(),NULL,117611),
(10022,23905,'Ghost Lab',NULL,now(),NULL,117651),
(10023,18274,'Tess of the D\'Urbervilles',NULL,now(),NULL,83099),
(10024,23238,'Secret Girlfriend',NULL,now(),NULL,117731);
